#!/pfrm2.0/bin/lua 

--[[
#### File: configUpdate.lua
#### Description: restore functions

#### Revisions:
01g,03Sep13,  nik fixed SPR 39803
01f,26aug13,  ksr fixed SPR 39226
01e,20may13,  su changes for generic traffic metering.
01d,02apr13,  su changes for vlan on wan - config update of vpn tables.
01d,28mar13,  sig fixed SPR 37446
01c,22mar13,  cg fixed SPR 37391
01b,06Dec'12, ksr fixed spr 35011
01a,28Nov'07, nsr written
]]--


--************* Requires *************
require "teamf1lualib/util"
require "teamf1lualib/db"

--************* Globals *************
local MAJOR_VERSION_DEFAULTS = ''
local MAJOR_VERSION_DATA_FLASH = ''

local TEAMF1_DEFAULTS_CONFIG="/pfrm2.0/etc/teamf1.cfg.defaults.ascii"
local TEAMF1_FLASH_CONFIG="/tmp/teamf1.cfg.ascii"
local TEAMF1_DATA_FLASH_CONFIG="/data/tmp/teamf1.cfg.ascii"

--************* Functions *************
function MAJOR_VERSION_DEFAULTS_FN(arg)
	local pfile = io.popen ([[grep -s "systemConfig\[1\]\[\"configMajorVer\"\]" /pfrm2.0/etc/teamf1.cfg.defaults.ascii | cut -d '"' -f4 ]])
	local value = pfile:read ("*number")
	pfile:close()
	return value or 0
end
function MAJOR_VERSION_DATA_FLASH_FN(arg)
	local pfile = io.popen ([[grep -s "systemConfig\[1\]\[\"configMajorVer\"\]" /data/tmp/teamf1.cfg.ascii | cut -d '"' -f4 ]])
	local value = pfile:read ("*number")
	pfile:close()
	return value or 0
end

MAJOR_VERSION_DEFAULTS = MAJOR_VERSION_DEFAULTS_FN()
MAJOR_VERSION_DATA_FLASH = MAJOR_VERSION_DATA_FLASH_FN()

function filterTable (t, f)
	assert (type (t) == "table")
	assert (type (f) == "function")
	local k, v
    local kk, vv
	local result = {}
	for k, v in pairs (t) do
        result[k]={}
	    for kk, vv in pairs (v) do
    	    if f (kk) then result [k][kk] = vv end
        end
  	end 
  	return result
end 

function tableAppend (table1, table2)
    if (table1 == nil or table2 == nil) then return table1 end
    local k,v
    local kk, vv
    for k, v in pairs (table2) do
        for kk,vv in pairs (v) do
            -- create new row if required
            if (not table1[k]) then
                table1[k]= {}
            end
            table1[k][kk] = vv;
        end -- for
    end -- for

    return table1
end

function tableAppendSSLVPNUserLoginPolicyDefinedAddress (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
			if (table[k1]["IPVersion"] ~= "2" and table[k1]["IPVersion"] ~= "10") then
				local ipAddress = table[k1]["Address"] or ""
				if (string.find(ipAddress, ":") == nil) then
					table1[k1]["IPVersion"] = "2"
				else
					table1[k1]["IPVersion"] = "10"
				end
			end
        end
    end 
    return table1
end

function tableAppendSSLVPNResourceObject (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
			if (table[k1]["AddressType"] ~= "1" and table[k1]["AddressType"] ~= "2") then
				local ipAddress = table[k1]["ObjectAddress"] or ""
				local ipAddress6 = table[k1]["ObjectAddress6"] or ""
				if (ipAddress ~= "") then
					table1[k1]["AddressType"] = "1"
				elseif (ipAddress6 ~= "") then
					table1[k1]["AddressType"] = "2"
				end
			end
        end
    end 
    return table1
end

function tableAppendSSLVPNClientRoute (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
			local ipAddress = table[k1]["DestinationNetwork"] or ""
			if (string.find(ipAddress, ":") == nil) then
				table1[k1]["IPVersion"] = "1"
			else
				table1[k1]["IPVersion"] = "2"
			end
        end
    end 
    return table1
end

function tableAppendRadiusClient (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    if (table[k1]["authtimeout"] == '30') then
            table1[k1]["authtimeout"] = '15'
        end
        if(table[k1]["authretries"] == '4') then
            table1[k1]["authretries"] = '3'
        end
    end
    end 
    return table1
end

--function to enable NAT traversal
function tableAppendIpsecIKEPolicy (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
			local natfield = table[k1]["NatTraversal"] or ""
			if (natfield == '0') then
				table1[k1]["NatTraversal"] = '1'
            end
        end
    end 
    return table1
end

--function to update localLogIfName for vlan on wan
function tableAppendIpsecIKEPolicy_vow (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end

			local localGw= table[k1]["LocalGateway"] or ""
	        if (localGw == '0') then
    		table1[k1]["localLogIfName"] = 'WAN1'
            elseif (localGw == '1') then
    		table1[k1]["localLogIfName"] = 'WAN2'
            elseif (localGw == '2') then
    		table1[k1]["localLogIfName"] = 'WAN3'
            elseif (localGw == '3') then
    		table1[k1]["localLogIfName"] = 'WAN4'
            end            
        end
    end 
    return table1
end

--function to update localLogIfName for vlan on wan
function tableAppendIpsecVPNPolicy (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end

			local localGw= table[k1]["LocalGateway"] or ""
            if (localGw == '1') then
    		table1[k1]["localLogIfName"] = 'WAN2'
            elseif (localGw == '2') then
    		table1[k1]["localLogIfName"] = 'WAN3'
            elseif (localGw == '3') then
    		table1[k1]["localLogIfName"] = 'WAN4'
            else
    		table1[k1]["localLogIfName"] = 'WAN1'
            end            

			local rolloverGw= table[k1]["RolloverGateway"] or ""
            if (rolloverGw == '1') then
    		table1[k1]["rolloverLogIfName"] = 'WAN2'
            elseif (rolloverGw == '2') then
    		table1[k1]["rolloverLogIfName"] = 'WAN3'
            elseif (rolloverGw == '3') then
    		table1[k1]["rolloverLogIfName"] = 'WAN4'
            else
    		table1[k1]["rolloverLogIfName"] = 'WAN1'
            end            
        end
    end 
    return table1
end

function tableAppendPortMgmt (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    if (table[k1]["PortName"] == "DedicatedWAN") then
		table1[k1]["PortName"] = 'WAN1'
            end
        end
    end 
    return table1
end

--defaultInit functions definition start{
--No majorVersion check required.
function tableAppendNimfMtu (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    local prevMtuValue = ''
	    if(table[k1]["LogicalIfName"] == 'WAN1')then
		prevMtuValue = table[k1]["MinMtu"] or ''
	    	if(prevMtuValue == '1200')then
			table1[k1]["MinMtu"] = '68'
		end
	    end
        end
    end 
    return table1
end

--No majorVersion check required. If there is no row for WAN2, insert a new row. This is for DMZ.
function tableAppend_tcpdump (table1, table)
    local wan2Entry = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    count = count + 1
	    if(table[k1]["LogicalIfName"] == 'WAN2')then
		wan2Entry = 'Y'
	    end
        end
    end 
    if(wan2Entry == 'N' )then
	count = count + 1
	table1[count] = {}
        table1[count]["tcpdumpEnabled"] = '0'
        table1[count]["LogicalIfName"] = 'WAN2'

        table[count] = {}
        table[count]["tcpdumpEnabled"] = '0'
        table[count]["LogicalIfName"] = 'WAN2'
    end
    return table1
end

--No majorVersion check required. If crontab table is configured for command='/pfrm2.0/bin/emailLogs /tmp/system.db',
-- Modify dayOfWeek to 0, if its set for 7 earlier.
-- Modify hour to 0, if its set for 12 earlier.
-- Modify hour to 12, if its set for 24 earlier.
function tableAppend_crontab (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    if(table1[k1]["command"] == '/pfrm2.0/bin/emailLogs /tmp/system.db')then
		if(table1[k1]["dayOfWeek"] == '7')then
			table1[k1]["dayOfWeek"] = '0'
		end
		if(table1[k1]["hour"] == '12')then
		    table1[k1]["hour"] = '0'
		end
		if(table1[k1]["hour"] == '24')then
		    table1[k1]["hour"] = '12'
		end
	    end
        end
    end 
    return table1
end
--No need of this function. Comment it for future purposes.
function tableAppendFwTrafficMeter(table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    -- in default case change the DayOfMonth to  32(the last day of the month)
	    table1[k1]["DayOfMonth"] = '32'
        end
    end 
    return table1
end
--No majorVersion check required. If row for interfaceName='eth0.4094' does not exist in ethernet table, insert it.
function tableAppend_ethernet(table1, table)
    local preDMZEntry = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    count = count + 1
	    if(table1[k1]["interfaceName"] == 'eth0.4094' ) then
	    	preDMZEntry = 'Y'
	    end
        end
    end 
    if(preDMZEntry == 'N')then
	count = count + 1
	table[count] = {}
	table[count]["interfaceName"] = 'eth0.4094'
	table[count]["poe"] = '0'
	table[count]["duplex"] = '1'
	table[count]["vlanId"] = '1'
	table[count]["vlanEnabled"] = '0'
	table[count]["trunkPort"] = '0'
	table[count]["macAddress"] = nil
	table[count]["edgePortStatus"] = nil
	table[count]["ethernetEdgePortCost"] = nil

	table1[count] = {}
	table1[count]["interfaceName"] = 'eth0.4094'
	table1[count]["poe"] = '0'
	table1[count]["duplex"] = '1'
	table1[count]["vlanId"] = '1'
	table1[count]["vlanEnabled"] = '0'
	table1[count]["trunkPort"] = '0'
	table1[count]["macAddress"] = nil
	table1[count]["edgePortStatus"] = nil
	table1[count]["ethernetEdgePortCost"] = nil
    end
    return table1
end

--No majorVersion check required. Update nil values with 0, for first upgrade.
function tableAppend_accessMgmt (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    if ( table1[k1]["ipMode"] == nil )then
		    table1[k1]["ipMode"] = '0' 
	    end
	    if ( table1[k1]["accessType6"] == nil )then
		    table1[k1]["accessType6"] = '0' 
	    end
	    if(table[k1]["accessMgmt6Enable"] == nil or table[k1]["accessMgmt6Enable"] == '') then
	    	if(table[k1]["port"] == '23') then
		    table1[k1]["accessMgmt6Enable"] = '0' 
		else
		    table1[k1]["accessMgmt6Enable"] = '1' 
		end
	    end
        end
    end 
    return table1
end
--No majorVersion check required. Update nil values with 0, for first upgrade.
function tableAppendDhcpc (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    if ( table1[k1]["clientId"] == nil )then
		table1[k1]["clientId"] = '0'
	    end
	    if ( table1[k1]["vendorClassId"] == nil )then
		table1[k1]["vendorClassId"] = '0'
	    end
        end
    end 
    return table1
end

--No majorVersion check required. If row2 is not present in ConfigPort table, insert it.
function tableAppendConfigPort(table1, table)
    local prevLANEntry = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    count = count + 1
	    if ( count == 2)then
	    	prevLANEntry = 'Y'
	    end
        end
    end 
    if(prevLANEntry == 'N')then
	count = count + 1 
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["LogicalIfName"] = 'LAN' 

	table[count] = {}
	table[count]["OID"] = nil
	table[count]["LogicalIfName"] = 'LAN' 
    end
    return table1
end

--Check for major version. Applicable, if defaults major version > 30 && data/tmp/ major version <= 30. Overwrite 'status' of all rows of 'interVLANRouting' to 0.
function tableAppend_interVLANRouting(table1, table)
    local count = 0 
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    count = count + 1 
	    if(count == 1)then
		if ( table1[k1]["status"] ~= '0' )then
			table1[k1]["status"] = '0' 
		end
	    end
        end
    end 
    return table1
end

--No majorVersion check required. If an entry for AddressFamily=10 not exist in IPMACEmailLogsConfig table, insert it.
function tableAppendIPMACEmailLogsConfig(table1, table)
    local prev6version = 'N'
    local count = 0 
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    count = count + 1 
	    if ( table1[k1]["ipVersion"] == '10' )then
		prev6version = 'Y'
	    end
        end
    end
    if(prev6version == 'N')then
	count = count + 1
	table[count]= {}
	table[count]["OID"] = nil
	table[count]["IPMACVoilationEmailLogsSet"] = '0'
	table[count]["ipVersion"] = '10'

	table1[count]= {}
	table1[count]["OID"] = nil
	table1[count]["IPMACVoilationEmailLogsSet"] = '0'
	table1[count]["ipVersion"] = '10'
    end 
    return table1
end

--No major version check required. Applicable to ONLY row no 1.
function tableAppendBlockSites(table1, table)
    local count = 0 
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    count = count + 1 
	    if(count == 1)then
		if ( table1[k1]["KeywordToBlock"] == 'dummy' or table1[k1]["KeywordToBlock"] == nil)then
			table1[k1]["KeywordToBlock"] = 'TF1BlockKeyword' 
		end
	    end
        end
    end 
    return table1
end

--logConfig is not configurable by user. No majorVersion check required.
function tableAppend_logConfig (table1, table)
    local prevSslFacility = 'N'
    local count = 0 
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end
	    count = count + 1 
	    if(table1[count]["facilityId"] == '18')then
		prevSslFacility = 'Y'
	    end
	    if(table1[k1]["facility"] == 'System')then
		table1[count]["emergency"] = '5'
		table1[count]["alert"] = '5'
		table1[count]["critical"] = '5'
		table1[count]["error"] = '5'
		table1[count]["warning"] = '5'
		table1[count]["notice"] = '5'
		table1[count]["information"] = '5'
		table1[count]["debug"] = '5'
	    end
        end
    end 

    if(prevSslFacility == 'N')then
	count = count + 1
	table[count] = {}
	table[count]["facility"] = 'Local1-SSLVPN'
	table[count]["facilityId"] = '18'
	table[count]["emergency"] = '1'
	table[count]["alert"] = '1'
	table[count]["critical"] = '1'
	table[count]["error"] = '1'
	table[count]["warning"] = '1'
	table[count]["notice"] = '1'
	table[count]["information"] = '1'
	table[count]["debug"] = '1'

	table1[count] = {}
	table1[count]["facility"] = 'Local1-SSLVPN'
	table1[count]["facilityId"] = '18'
	table1[count]["emergency"] = '1'
	table1[count]["alert"] = '1'
	table1[count]["critical"] = '1'
	table1[count]["error"] = '1'
	table1[count]["warning"] = '1'
	table1[count]["notice"] = '1'
	table1[count]["information"] = '1'
	table1[count]["debug"] = '1'
    end
    return table1
end

--No majorVersion check required. If entries for NFS-TCP or RPC-TCP does not exist in Services table, insert them.
function tableAppendServices2 (table1, table)
    local prevNFSTCPEntry = 'N'
    local prevRPCTCPEntry = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
	    end
	    count = count + 1 
	    if(table1[k1]["ServiceName"] == 'NFS-TCP')then
		prevNFSTCPEntry = 'Y'
	    end
	    if(table1[k1]["ServiceName"] == 'RPC-TCP')then
		prevRPCTCPEntry = 'Y'
	    end
        end
    end 
    if(prevNFSTCPEntry == 'N')then
	count = count + 1
	table1[count] = {}
	table1[count]["OID"] = ''
	table1[count]["ServiceName"] = 'NFS-TCP'
	table1[count]["Protocol"] = '6'
	table1[count]["DestinationPortStart"] = '2049'
	table1[count]["DestinationPortEnd"] = '0'
	table1[count]["TypeOfService"] = '0'
	table1[count]["IsDefault"] = '1'

	table[count] = {}
	table[count]["OID"] = ''
	table[count]["ServiceName"] = 'NFS-TCP'
	table[count]["Protocol"] = '6'
	table[count]["DestinationPortStart"] = '2049'
	table[count]["DestinationPortEnd"] = '0'
	table[count]["TypeOfService"] = '0'
	table[count]["IsDefault"] = '1'
    end
    if(prevRPCTCPEntry == 'N')then
	count = count + 1
	table1[count] = {}
	table1[count]["OID"] = ''
	table1[count]["ServiceName"] = 'RPC-TCP'
	table1[count]["Protocol"] = '6'
	table1[count]["DestinationPortStart"] = '111'
	table1[count]["DestinationPortEnd"] = '0'
	table1[count]["TypeOfService"] = '0'
	table1[count]["IsDefault"] = '1'

	table[count] = {}
	table[count]["OID"] = ''
	table[count]["ServiceName"] = 'RPC-TCP'
	table[count]["Protocol"] = '6'
	table[count]["DestinationPortStart"] = '111'
	table[count]["DestinationPortEnd"] = '0'
	table[count]["TypeOfService"] = '0'
	table[count]["IsDefault"] = '1'
    end
    return table1
end

--No majorVersion check required. If there are no rows for ICMPv6 in Services table, insert a new rows. We are checking for only one entry 'ICMPv6-TYPE-128' and inserting related new rows if its not found.
function tableAppendServices (table1, table)
    local icmpV6Entry = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
	    end
	    count = count + 1 
	    --seeing if one icmpv6 service is present then no need to update services table
	    if(table1[k1]["ServiceName"] == 'ICMPv6-TYPE-128')then
		icmpV6Entry = 'Y'
	    end
        end
    end 
    if(icmpV6Entry == 'N')then
--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-1' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '1' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-1' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '1' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 
--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-2' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '2' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-2' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '2' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 

--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-3' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '3' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-3' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '3' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 

--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-4' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '4' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-4' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '4' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 

--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-127' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '127' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-127' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '127' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 
--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-128' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '128' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-128' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '128' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 
--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-134' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '134' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-134' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '134' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 
--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-135' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '135' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-135' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '135' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 
--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-136' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '136' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-136' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '136' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 
--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-137' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '137' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-137' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '137' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 
--
	count = count + 1
	table[count] = {}
	table[count]["OID"] = nil
	table[count]["ServiceName"] = 'ICMPv6-TYPE-255' 
	table[count]["Protocol"] = '58' 
	table[count]["DestinationPortStart"] = '255' 
	table[count]["DestinationPortEnd"] = '0' 
	table[count]["TypeOfService"] = '0' 
	table[count]["IsDefault"] = '1' 
	
	table1[count] = {}
	table1[count]["OID"] = nil
	table1[count]["ServiceName"] = 'ICMPv6-TYPE-255' 
	table1[count]["Protocol"] = '58' 
	table1[count]["DestinationPortStart"] = '255' 
	table1[count]["DestinationPortEnd"] = '0' 
	table1[count]["TypeOfService"] = '0' 
	table1[count]["IsDefault"] = '1' 
    end
    return table1
end 

--No majorVersion check required. If row3 and row4 does not exist in ntpDefServers table, insert them.
function tableAppend_ntpDefServers(table1, table)
    local prevNtpEntry1= 'N'
    local prevNtpEntry2= 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
	    end
	    count = count + 1 
	    if(table1[k1]["serverName"] == '2.europe.pool.ntp.org' and count == 4)then
		prevNtpEntry2 = 'Y'
	    end
	    if(table1[k1]["serverName"] == '2.pool.ntp.org' and count == 3)then
		prevNtpEntry1 = 'Y'
	    end
        end
    end 
    if(prevNtpEntry1 == 'N')then
	count = count + 1
	table1[count] = {}
	table1[count]["serverName"] = '2.pool.ntp.org'

	table[count] = {}
	table[count]["serverName"] = '2.pool.ntp.org'
    end
    if(prevNtpEntry2 == 'N')then
	count = count + 1
	table1[count] = {}
	table1[count]["serverName"] = '2.europe.pool.ntp.org'

	table[count] = {}
	table[count]["serverName"] = '2.europe.pool.ntp.org'
    end
    return table1
end 

--No majorVersion check required. If there is no row for WAN1PPPOE6 in dhcpv6c, insert a new row.
function tableAppend_dhcpv6c (table1, table)
    local prevWAN1PPPOE6 = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
	    end
	    count = count + 1 
	    if(table1[k1]["LogicalIfName"] == 'WAN1PPPOE6')then
		prevWAN1PPPOE6 = 'Y'
	    end
        end
    end 
    if(prevWAN1PPPOE6 == 'N')then
	count = count + 1 
	table[count] = {}
	table[count]["LogicalIfName"] = 'WAN1PPPOE6'
	table[count]["isEnabled"] = '0'
	table[count]["statelessMode"] = '1'
	table[count]["requestPreferredAddress"] = '0'
	table[count]["preferredAddress"] = ''
	table[count]["preferredAddressPrefixLength"] = '0'
	table[count]["requestDNS"] = '1'
	table[count]["requestDNSSearchList"] = '0'
	table[count]["prefixDelegation"] = '0'
	table[count]["requestPreferredPrefix"] = '0'
	table[count]["preferredPrefix"] = ''
	table[count]["preferredPrefixPrefixLength"] = '0'
	table[count]["renewTime"] = '3600'
	table[count]["sendRapidCommit"] = '0'

	table1[count] = {}
	table1[count]["LogicalIfName"] = 'WAN1PPPOE6'
	table1[count]["isEnabled"] = '0'
	table1[count]["statelessMode"] = '1'
	table1[count]["requestPreferredAddress"] = '0'
	table1[count]["preferredAddress"] = ''
	table1[count]["preferredAddressPrefixLength"] = '0'
	table1[count]["requestDNS"] = '1'
	table1[count]["requestDNSSearchList"] = '0'
	table1[count]["prefixDelegation"] = '0'
	table1[count]["requestPreferredPrefix"] = '0'
	table1[count]["preferredPrefix"] = ''
	table1[count]["preferredPrefixPrefixLength"] = '0'
	table1[count]["renewTime"] = '3600'
	table1[count]["sendRapidCommit"] = '0'
    end
    return table1
end 

--No majorVersion check required. If  a row with ifTyepe 'FW_IPv6_MAC' is not present in interfaceStatsParams table, insert it.
function tableAppend_interfaceStatsParams (table1, table)
    local prevFW_IPv6_MACEntry = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
	    end
	    count = count + 1 
	    if(table1[k1]["ifType"] == 'FW_IPv6_MAC')then
		prevFW_IPv6_MACEntry = 'Y'
	    end
        end
    end 
    if(prevFW_IPv6_MACEntry == 'N')then
	count = count + 1 
	table[count] = {}
	table[count]["ifType"] = 'FW_IPv6_MAC'
	table[count]["refreshInterval"] = '10'
	table[count]["autoRefreshFlag"] = '1'

	table1[count] = {}
	table1[count]["ifType"] = 'FW_IPv6_MAC'
	table1[count]["refreshInterval"] = '10'
	table1[count]["autoRefreshFlag"] = '1'
    end
    return table1
end 

--No majorVersion check required. If row2 does not exist in dhcpv6s table, insert it.
function tableAppend_dhcpv6s(table1, table)
    local prevDMZEntry = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
	    end
	    count = count + 1 
	    if(table1[k1]["LogicalIfName"] == 'DMZ' and count == 2)then
		prevDMZEntry = 'Y'
	    end
        end
    end 
    if(prevDMZEntry == 'N')then
	count = count + 1 
	table[count] = {}
	table[count]["LogicalIfName"] = 'DMZ'
	table[count]["isEnabled"] = '0'
	table[count]["statelessMode"] = '1'
	table[count]["serverPreference"] = '255'
	table[count]["domainName"] = 'netgear.com'
	table[count]["useDNSServersFrom"] = '1'
	table[count]["primaryDNSServer"] = ''
	table[count]["secondaryDNSServer"] = ''
	table[count]["leaseTime"] = '86400'
	table[count]["prefixDelegation"] = '0'
	
	table1[count] = {}
	table1[count]["LogicalIfName"] = 'DMZ'
	table1[count]["isEnabled"] = '0'
	table1[count]["statelessMode"] = '1'
	table1[count]["serverPreference"] = '255'
	table1[count]["domainName"] = 'netgear.com'
	table1[count]["useDNSServersFrom"] = '1'
	table1[count]["primaryDNSServer"] = ''
	table1[count]["secondaryDNSServer"] = ''
	table1[count]["leaseTime"] = '86400'
	table1[count]["prefixDelegation"] = '0'
    end
    return table1
end 

--No majorVersion check required. If row2 does not exist in radvd table, insert it.
function tableAppend_radvd(table1, table)
    local prevRadvdDMZEntry = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
	    end
	    count = count + 1 
	    if(table1[k1]["LogicalIfName"] == 'DMZ' and count == 2)then
		prevRadvdDMZEntry= 'Y'
	    end
        end
    end 
    if(prevRadvdDMZEntry== 'N')then
	count = count + 1 
	table[count] = {}
	table[count]["isEnabled"] = '0'
	table[count]["useDHCP6sPrefixes"] = '0'
	table[count]["UnicastOnly"] = '0'
	table[count]["MaxRtrAdvInterval"] = '30'
	table[count]["AdvLinkMTU"] = '1500'
	table[count]["AdvManagedFlag"] = '0'
	table[count]["AdvOtherConfigFlag"] = '1'
	table[count]["AdvDefaultLifetime"] = '3600'
	table[count]["AdvDefaultPreference"] = '3'
	table[count]["LogicalIfName"] = 'DMZ'
	
	table1[count] = {}
	table1[count]["isEnabled"] = '0'
	table1[count]["useDHCP6sPrefixes"] = '0'
	table1[count]["UnicastOnly"] = '0'
	table1[count]["MaxRtrAdvInterval"] = '30'
	table1[count]["AdvLinkMTU"] = '1500'
	table1[count]["AdvManagedFlag"] = '0'
	table1[count]["AdvOtherConfigFlag"] = '1'
	table1[count]["AdvDefaultLifetime"] = '3600'
	table1[count]["AdvDefaultPreference"] = '3'
	table1[count]["LogicalIfName"] = 'DMZ'
    end
    return table1
end 

--No majorVersion check required. If an entry with LogicalIfName='DMZ' and AddressFamily=10 doesn't exist, then insert it. 
function tableAppend_ifStatic(table1, table)
    local prevIfstaticDMZEntry = 'N'
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
	    end
	    count = count + 1 
	    if(table1[k1]["LogicalIfName"] == 'DMZ' and table1[k1]["AddressFamily"] == '10')then
		prevIfstaticDMZEntry= 'Y'
	    end
        end
    end 
    if(prevIfstaticDMZEntry== 'N')then
	count = count + 1 
	table[count] = {}
	table[count]["OID"] = '0'
	table[count]["LogicalIfName"] = 'DMZ'
	table[count]["AddressFamily"] = '10'
	table[count]["IspName"] = '0'
	table[count]["StaticIp"] = '176::1'
	table[count]["NetMask"] = ''
	table[count]["PrefixLength"] = '64'
	table[count]["Gateway"] = ''
	table[count]["PrimaryDns"] = ''
	table[count]["SecondaryDns"] = ''
	
	table1[count] = {}
	table1[count]["OID"] = '0'
	table1[count]["LogicalIfName"] = 'DMZ'
	table1[count]["AddressFamily"] = '10'
	table1[count]["IspName"] = '0'
	table1[count]["StaticIp"] = '176::1'
	table1[count]["NetMask"] = ''
	table1[count]["PrefixLength"] = '64'
	table1[count]["Gateway"] = ''
	table1[count]["PrimaryDns"] = ''
	table1[count]["SecondaryDns"] = ''
    end
    return table1
end 

function tableAppend_BandWidthProfileStatus(table1, table)
    local count = 0
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
	    end
	    count = count + 1
	end
    end 
    if(#table1 == 0)then
	count = count + 1
	table1[count] = {}
	table1[count]["Status"] = '1'

	table[count] = {}
	table[count]["Status"] = '1'
    end

    return table1
end 

--defaultInit functions definition end}

function tableSubtract (table1, allKeys)
	local result = {}
    result = filterTable (table1, 
    function (k)
        local key, value
		for key, value in pairs(allKeys) do
            if value == k then 
                return true end
        end
      	return false
    end
    )
    return result;
end

--function to disable  TCP and UDP flood
function tableAppendAttackChecks (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                if (table[k1][kk1] ~= nil) then
                table1[k1][kk1] = table[k1][kk1]
		end
            end
			local tcp = table[k1]["TcpFlood"] or ""
			local udp = table[k1]["UdpFlood"] or ""
			if (tcp == '1') then
				table1[k1]["TcpFlood"] = '0'
            end
            if (udp == '1') then
				table1[k1]["UdpFlood"] = '0'
            end

        end
    end 
    return table1
end

--function to update wanLogicalIfName for vlan on wan
function tableAppendTrafficMeterConfig_vow (table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                table1[k1][kk1] = table[k1][kk1]
            end

			local wanId= table[k1]["WanId"] or ""
	        if (wanId == '1') then
    		table1[k1]["wanLogicalIfName"] = 'WAN1'
            elseif (wanId == '2') then
    		table1[k1]["wanLogicalIfName"] = 'WAN2'
            elseif (wanId == '3') then
    		table1[k1]["wanLogicalIfName"] = 'WAN3'
            elseif (wanId == '4') then
    		table1[k1]["wanLogicalIfName"] = 'WAN4'
            end            
        end
    end 
    return table1
end
-- function for xmlId changes introduced for CMS support
function xmlIdEnhancement(table1, table2, allKeys)
	if (table1 == nil and table2 == nil and allKeys == nil) then return {},{} end
	for kk,vv in pairs(allKeys) do
		for kkk,vvv in pairs(table1) do
			if(vv ~= nil and vv == "xmlId" and table1[kkk] ~= nil ) then 
				-- create a old table (table genrated from database conf), taking the new table(new conf)
				if (table2[kkk] == nil) then table2[kkk] = table1[kkk] end
				-- if old value == nil or '' then don't do anything
				if(table2[kkk][vv] == nil or table2[kkk][vv] == '') then
					if(table1[kkk][vv] ~= nil and table1[kkk][vv] ~= '') then -- new value is not equal to nil and not equal to ''
						table2[kkk][vv] = table1[kkk][vv]
					elseif (table1[kkk][vv] == nil or table1[kkk][vv] == '') then -- new value is equal to nil or ''
						table1[kkk][vv] = table2[kkk]["_ROWID_"] or kkk
						table2[kkk][vv] = table2[kkk]["_ROWID_"] or kkk
					end
				end
			end
		end -- for
	end -- for
	return table1, table2
end

function tableModify_FwDosAttackSettings(table1, table)
    for k1, v1 in pairs (table1) do
        if (table[k1] ~= nil) then
            for kk1,vv1 in pairs (v1) do
                if (table[k1][kk1] ~= nil) then
                    table1[k1][kk1] = table[k1][kk1]
                end
            end
	    -- in default case change the SynFloodDetectRate to  100
            local tcpFloodRate = table[k1]["SynFloodDetectRate"] or ""
            if (tcpFloodRate == '128') then
                table1[k1]["SynFloodDetectRate"] = '100'
            end
        end
    end 
    return table1
end

--************* Initial Code *************

local oldTable = {}
local table = {}

--************* Logic *************

db.connect (arg[1])

local SETTINGS_FILE = db.getAttribute("environment", "name", "TEAMF1_CFG_ASCII", "value")
local saveTables = db.getTable("saveTables", false)

dofile (SETTINGS_FILE) -- read old configuration

for k,v in pairs(saveTables) do
    local tableName = v ["tableName"]

    -- record the old table values
    oldTable[tableName] = util.getLuaVariable(tableName)
    
    -- clear glob. variables
    _G[tableName] = nil
end


dofile (arg[2]) -- read new configuration

for k,v in pairs(saveTables) do

    local tableName = v ["tableName"]
    local table1 = util.getLuaVariable(tableName)
    local table2 = oldTable[tableName]

    if (table1 == nil) then
        
        print ("configUpdate : no LUA variable : " .. tableName)
    else

        -- TODO : Handle systemConfig table properly
        if (table2 and tableName ~= "systemConfig") then

            -- make new table compatible for old values
            local allKeys = db.getColNames(tableName);
            for kk,vv in pairs(allKeys) do
                -- skip rowId column
                if (kk ~= "_ROWID_") then
                    for kkk,vvv in pairs(table1) do
                        -- if column required
                        if (not table1[kkk][vv] and table2[kkk] and table2[kkk][vv]) then 
                            table1[kkk][vv] = "";
                        end
                    end -- for
                end
            end -- for
       
            -- Make old table compatible for merge
            if (#table1 ~= 0) then
                table2 = tableSubtract (table2, allKeys) 
            end

            -- Update table with old values
            table1 = tableAppend (table1, table2)
			-- { xmlId  
			table1, table2 = xmlIdEnhancement(table1, table2, allKeys)
			-- xmlId }
        end
         --  Handle Services table updates
        if (tableName == "Services") then
            for i,j in pairs(table1) do
                if(j["ServiceName"] == "RCMD") then
                    j["DestinationPortStart"] = 514
                end
                if(j["ServiceName"] == "REXEC") then
                    j["DestinationPortStart"] = 512
                end
            end
        end

        if (table2 and tableName == "SSLVPNUserLoginPolicyDefinedAddress") then
            table1 = tableAppendSSLVPNUserLoginPolicyDefinedAddress (table1, table2)
        end
	--defaultsInit function call begin{
        if (table2 and tableName == "NimfMtu") then
            table1 = tableAppendNimfMtu (table1, table2)
        end
	
        if (table2 and tableName == "crontab") then
            table1 = tableAppend_crontab (table1, table2)
        end
        if (table2 and tableName == "dhcpv6c") then
            table1 = tableAppend_dhcpv6c (table1, table2)
        end

        if (table2 and tableName == "Services") then
            table1 = tableAppendServices (table1, table2)
            table1 = tableAppendServices2 (table1, table2)
        end

        if (table2 and tableName == "accessMgmt") then
            table1 = tableAppend_accessMgmt(table1, table2)
        end

        if (table2 and tableName == "BlockSites") then
            table1 = tableAppendBlockSites(table1, table2)
        end

        if (table2 and tableName == "AttackChecks") then
            if(tonumber(MAJOR_VERSION_DEFAULTS) > 53 and tonumber(MAJOR_VERSION_DATA_FLASH) <= 53 )then
                table1 = tableAppendAttackChecks(table1, table2)
            end
        end

        if (table2 and tableName == "interVLANRouting") then
	    if(tonumber(MAJOR_VERSION_DEFAULTS) > 30 and tonumber(MAJOR_VERSION_DATA_FLASH) <= 30)then
            	table1 = tableAppend_interVLANRouting(table1, table2)
	    end
        end
--[[
        if (table2 and tableName == "FwTrafficMeter") then
            table1 = tableAppendFwTrafficMeter(table1, table2)
        end
]]
        if (table2 and tableName == "ethernet") then
            table1 = tableAppend_ethernet(table1, table2)
        end

        if (table2 and tableName == "Dhcpc") then
            table1 = tableAppendDhcpc(table1, table2)
        end

        if (table2 and tableName == "ConfigPort") then
            table1 = tableAppendConfigPort(table1, table2)
        end

        if (table2 and tableName == "interfaceStatsParams") then
            table1 = tableAppend_interfaceStatsParams(table1, table2)
        end

        if (table2 and tableName == "ntpDefServers") then
            table1 = tableAppend_ntpDefServers(table1, table2)
        end
	
        if (table2 and tableName == "dhcpv6s") then
            table1 = tableAppend_dhcpv6s(table1, table2)
        end

        if (table2 and tableName == "radvd") then
            table1 = tableAppend_radvd(table1, table2)
        end

        if (table2 and tableName == "ifStatic") then
            table1 = tableAppend_ifStatic(table1, table2)
        end

        if (table2 and tableName == "IPMACEmailLogsConfig") then
            table1 = tableAppendIPMACEmailLogsConfig(table1, table2)
        end

        if (table2 and tableName == "logConfig") then
            table1 = tableAppend_logConfig(table1, table2)
        end


        if (table2 and tableName == "BandWidthProfileStatus") then
            table1 = tableAppend_BandWidthProfileStatus(table1, table2)
        end

	--defaultsInit function call end }			
        if (table2 and tableName == "SSLVPNResourceObject") then
            table1 = tableAppendSSLVPNResourceObject (table1, table2)
        end

        if (table2 and tableName == "SSLVPNClientRoute") then
            table1 = tableAppendSSLVPNClientRoute (table1, table2)
        end

        if (table2 and tableName == "radiusClient") then
            table1 = tableAppendRadiusClient (table1, table2)
        end

        if (table2 and tableName == "IpsecIKEPolicy") then
            table1 = tableAppendIpsecIKEPolicy (table1, table2)
        end
	if(tonumber(MAJOR_VERSION_DEFAULTS) > 61 and tonumber(MAJOR_VERSION_DATA_FLASH) <= 61)then
	    if (table2 and tableName == "PortMgmt") then
  		table1 = tableAppendPortMgmt (table1, table2)
	    end

        if (table2 and tableName == "IpsecVPNPolicy") then
            table1 = tableAppendIpsecVPNPolicy (table1, table2)
        end        

        if (table2 and tableName == "IpsecIKEPolicy") then
            table1 = tableAppendIpsecIKEPolicy_vow (table1, table2)
        end        

        if (table2 and tableName == "FwTrafficMeter") then
            table1 = tableAppendTrafficMeterConfig_vow (table1, table2)
        end        

        if (table2 and tableName == "TrafficCounters") then
            table1 = tableAppendTrafficMeterConfig_vow (table1, table2)
        end        

        if (table2 and tableName == "TrafficStatistics") then
            table1 = tableAppendTrafficMeterConfig_vow (table1, table2)
        end        
	end
    
        if (table2 and tableName == "FwDosAttackSettings") then
            table1 = tableModify_FwDosAttackSettings(table1, table2)
        end

-- Check table intergrity 
        for kk,vv in pairs (table1) do
            -- add columns for NOT NULL kind of fields
            for key,val in pairs (table1[1]) do
                -- if field is missing from a row
                if (not table1[kk][key]) then
                    -- use value from first row
                    table1[kk][key] = table1[1][key]
                end
            end
            -- delete rows from table if not needed
            if (table2 and not table2[kk]) then
                table1[kk] = nil
            end
        end
    
        -- Save new table 
        if (#table1 >= 0) then
            table[tableName] = table1;
        end
    end
end -- for


-- save configuration to file
 
local file = io.open(SETTINGS_FILE .. "", "wb")

for k,v in pairs(saveTables) do

    local tableName = v ["tableName"]
    db.saveTable(file, tableName, table[tableName], false, 1, 1)
end

db.close() -- close db handle
