--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### Feb 29, 2008
#### File: fwView.lua
#### Description: Firewall functions
#### Revisions:
01h 28aug13, ksr , fixed SPR 39327
01g 03dec12, nik , changes specific to fvs336gv2
01f 10jul12, nik , Fixed SPR 33375
01e 27Jun12, sig ,Fixed SPR 32995
01d 20Jun12, sig , Fixed spr 32570
01c 23May12, sig , fix spr 31445
01b 19Aug11, Smita P. Added fwView.wanModeSet () function
01a 18Nov10, Venu Babu .M. fwView.trafficMeterConfig () function modified. Added new function fwView.trafficMtr_ConfigGet()
]]--

--************* Requires *************

require "teamf1lualib/fwRules"
require "ifDevLib"

--************* Initial Code *************

--package wlNw
fwView = {}
ipv6FirewallRules = {}
DB_FILE_NAME="/tmp/system.db"
--************* Functions *************
-- Services Configuration
function fwView.serviceConfig (inputTable, rowid, operation)
	-- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    if(operation == "add") then
        local serName = inputTable["Services.ServiceName"]
	    if(db.existsRowWhere("Services", "ServiceName = '" .. serName .. "'")) then
            db.rollback()
            return "ERROR", "SERVICE_NAME_ALREADY_EXISTS"
        end
        if(db.existsRowWhere("serviceGroup", "serviceGroupName = '" .. serName .. "'")) then
            db.rollback()
            return "ERROR", "SERVICE_GRP_NAME_ALREADY_EXISTS"
        end
    end

    if (operation == "edit") then
        local ruleInUse = 0
        local ruleInUse6 = 0
        local ruleInUseProtocol = 0
        local ruleInUseNetwork = 0
        local service = inputTable["Services.ServiceName"]
        local serviceprotocol = inputTable["Services.Protocol"]
        local rows = db.existsRowWhere("FirewallRules", "ServiceName = '" .. service .. "'")
        local configRow = db.getRows("FirewallRules", "ServiceName", service)
        local rows6 = db.existsRowWhere("FirewallRules6", "ServiceName = '" .. service .. "'")
        local configRow6 = db.getRows("FirewallRules6", "ServiceName", service)
        local protocolRow = db.existsRowWhere("ProtocolBinding", "ServiceName = '" .. service .. "'")
        local configprotocolRow = db.getRows("ProtocolBinding", "ServiceName", service)
        local networkQosRow =db.existsRowWhere("NetworkQosProfile", "ServiceName = '" .. service .. "'")
        local confignetworkQosRow = db.getRows("NetworkQosProfile", "ServiceName", service)
    	for a,b in pairs(configRow) do
             ruleInUse=0;
	         if (b["FirewallRules.Status"] == "1") then
                ruleInUse=1;
                break;
             end
        end
        for a,b in pairs(configRow6) do
             ruleInUse6=0;
	         if (b["FirewallRules6.Status"] == "1") then
                ruleInUse6=1;
                break;
             end
        end
        if (rows or rows6) and ((ruleInUse == 1) or (ruleInUse6 == 1)) then
            db.rollback()
    		return "ERROR", "SERVICE_IN_USE"
        end
        for a,b in pairs(configprotocolRow) do
             ruleInUseProtocol=0;
	         if (b["ProtocolBinding.Enabled"] == "1" or b["ProtocolBinding.Enabled"] == "0") then
                ruleInUseProtocol=1;
                break;
             end
           
        end
        if (ruleInUseProtocol == 1 ) then
            db.rollback()
            return "ERROR", "PROTOCOL_IN_USE"
        end
        for a,b in pairs(confignetworkQosRow) do
             ruleInUseNetwork=0;
	         if (b["NetworkQosProfile.QosProfileStatus"] == "1" or b["NetworkQosProfile.QosProfileStatus"] == "0" ) then
                ruleInUseNetwork=1;
                break;
             end
        end
        if (ruleInUseNetwork == 1 ) then
            db.rollback()
            return "ERROR", "NETWORK_QOS_PROFILE_IN_USE"
        end
    end
    
    if(operation == "add") then
        local maxId = db.columnMaxValueWhere("Services", "xmlId", "IsDefault = 0")
        if(maxId ~= nil) then
            inputTable["Services.xmlId"] = tonumber (maxId) + 1
        else
            inputTable["Services.xmlId"] = 1
        end
    end

	-- config Service
	valid = fw.config(inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "SERVICE_CONFIG_FAILED"
	end
end

-- Services Delete
function fwView.deleteServices (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

	-- Check for Using Services
    for k,v in pairs(rowIds) do
        local service = db.getAttribute("Services", "_ROWID_", v, "ServiceName")
        local rows = db.existsRowWhere("FirewallRules", "ServiceName = '" .. service .. "'")
        local rows6 = db.existsRowWhere("FirewallRules6", "ServiceName = '" .. service .. "'")
        local protocolRow = db.existsRowWhere("ProtocolBinding", "ServiceName = '" .. service .. "'")
        local networkQosRow =db.existsRowWhere("NetworkQosProfile", "ServiceName = '" .. service .. "'")
        if (rows or rows6) then
            valid = false
            errStr = "SERVICE_IN_USE_DEL_RULE"
            break
        end
        if (protocolRow) then
            valid = false
            errStr = "PROTOCOL_IN_USE"
            break
        end
        if (networkQosRow) then
            valid = false
            errStr = "NETWORK_QOS_PROFILE_IN_USE"
            break
        end
    end

    if (valid) then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["Services._ROWID_"] = v;
            if (db.existsRow("Services","_ROWID_",v)) then
                valid = fw.config(inTable, "-1", "delete")
            end
            if (valid == false) then
                errStr = "SERVICE_DELETE_FAILED"
                break
            end
	    end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "SERVICE_DELETE_OK"
	else
		db.rollback()
		return "ERROR", errStr
	end

end

-- Schedules Configuration Get
function fwView.scheduleConfigGet (inputTable)

    local inTable = {}
    inTable["Schedules.ScheduleName"] = inputTable["Schedules.ScheduleName"]
    inTable["Schedules._ROWID_"] = inputTable["Schedules._ROWID_"]
    inTable["Schedules.AllDay"] = inputTable["Schedules.AllDay"]
    local days = tonumber(inputTable["Schedules.Days"])

    if (days ~= nil) then
        if (days == 127 or days == 128) then
           inTable["Schedules.allDays"] = "1"
           days = days - days
        else
           inTable["Schedules.allDays"] = "0"
        end

        if (days >= 64) then
            inTable["Schedules.sunday"] = "1"
            days = days - 64
        else
            inTable["Schedules.sunday"] = "0"
        end

        if (days >= 32) then
            inTable["Schedules.saturday"] = "1"
            days = days - 32
        else
            inTable["Schedules.saturday"] = "0"
        end

        if (days >= 16) then
            inTable["Schedules.friday"] = "1"
            days = days - 16
        else
            inTable["Schedules.friday"] = "0"
        end

        if (days >= 8) then
            inTable["Schedules.thursday"] = "1"
            days = days - 8
        else
            inTable["Schedules.thursday"] = "0"
        end

        if (days >= 4) then
            inTable["Schedules.wednesday"] = "1"
            days = days - 4
        else
            inTable["Schedules.wednesday"] = "0"
        end

        if (days >= 2) then
            inTable["Schedules.tuesday"] = "1"
            days = days - 2
        else
            inTable["Schedules.tuesday"] = "0"
        end

        if (days >= 1) then
            inTable["Schedules.monday"] = "1"
            days = days - 1
        else
            inTable["Schedules.monday"] = "0"
        end
    end

    local hrs = tonumber(inputTable["Schedules.StartTimeHours"])
    if (hrs ~= nil) then
        if (inputTable["Schedules.StartTimeHours"] == "0" and inputTable["Schedules.StartTimeMins"] == "0") then
            inTable["Schedules.startTimeHour"] = "12"
            inTable["Schedules.startTimeMin"] = "00"
            inTable["Schedules.startTimeMeridian"] = "0"
        elseif (inputTable["Schedules.StartTimeHours"] == "0") then
            inTable["Schedules.startTimeHour"] = "12"
            inTable["Schedules.startTimeMin"] = inputTable["Schedules.StartTimeMins"]
            inTable["Schedules.startTimeMeridian"] = "0"
        elseif (hrs > 12) then
            inTable["Schedules.startTimeHour"] = hrs - 12
            inTable["Schedules.startTimeMin"] = inputTable["Schedules.StartTimeMins"]
            inTable["Schedules.startTimeMeridian"] = "1"
        elseif (hrs == 12) then
            inTable["Schedules.startTimeHour"] = inputTable["Schedules.StartTimeHours"]
            inTable["Schedules.startTimeMin"] = inputTable["Schedules.StartTimeMins"]
            inTable["Schedules.startTimeMeridian"] = "1"
        else
            inTable["Schedules.startTimeHour"] = inputTable["Schedules.StartTimeHours"]
            inTable["Schedules.startTimeMin"] = inputTable["Schedules.StartTimeMins"]
            inTable["Schedules.startTimeMeridian"] = "0"
        end
    end

    hrs = tonumber(inputTable["Schedules.EndTimeHours"])
    if (hrs ~= nil) then
        if (inputTable["Schedules.EndTimeHours"] == "0" and inputTable["Schedules.EndTimeMins"] == "0") then
            inTable["Schedules.endTimeHour"] = "12"
            inTable["Schedules.endTimeMin"] = "00"
            inTable["Schedules.endTimeMeridian"] = "0"
        elseif (inputTable["Schedules.EndTimeHours"] == "0") then
            inTable["Schedules.endTimeHour"] = "12"
            inTable["Schedules.endTimeMin"] = inputTable["Schedules.EndTimeMins"]
            inTable["Schedules.endTimeMeridian"] = "0"
        elseif (hrs > 12) then
            inTable["Schedules.endTimeHour"] = hrs - 12
            inTable["Schedules.endTimeMin"] = inputTable["Schedules.EndTimeMins"]
            inTable["Schedules.endTimeMeridian"] = "1"
        elseif (hrs == 12) then
            inTable["Schedules.endTimeHour"] = inputTable["Schedules.EndTimeHours"]
            inTable["Schedules.endTimeMin"] = inputTable["Schedules.EndTimeMins"]
            inTable["Schedules.endTimeMeridian"] = "1"
        else
            inTable["Schedules.endTimeHour"] = inputTable["Schedules.EndTimeHours"]
            inTable["Schedules.endTimeMin"] = inputTable["Schedules.EndTimeMins"]
            inTable["Schedules.endTimeMeridian"] = "0"
        end
    end

    local startHH = inTable["Schedules.startTimeHour"]
    local startMM = inTable["Schedules.startTimeMin"]
    local endHH = inTable["Schedules.endTimeHour"]
    local endMM = inTable["Schedules.endTimeMin"]

    if (startHH ~= nil and string.len(startHH) == 1) then
        inTable["Schedules.startTimeHour"] = string.format ("%d%d",0,startHH)
    end
    if (startMM ~= nil and string.len(startMM) == 1) then
        inTable["Schedules.startTimeMin"] = string.format ("%d%d",0,startMM)
    end
    if (endHH ~= nil and string.len(endHH) == 1) then
        inTable["Schedules.endTimeHour"] = string.format ("%d%d",0,endHH)
    end
    if (endMM ~= nil and string.len(endMM) == 1) then
        inTable["Schedules.endTimeMin"] = string.format ("%d%d",0,endMM)
    end

    return inTable
end

-- Schedules Configuration
function fwView.schedulesConfig (inputTable, rowid, operation)
        -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                return "ACCESS_DENIED", "ADMIN_REQD"
        end

        db.beginTransaction() --begin transaction

        local valid = false
        local inTable = {}
        local days = 0
        if (inputTable["Schedules.monday"] == "1") then
            days = days + 1
        end
        if (inputTable["Schedules.tuesday"] == "1") then
            days = days + 2
        end
        if (inputTable["Schedules.wednesday"] == "1") then
            days = days + 4
        end
        if (inputTable["Schedules.thursday"] == "1") then
            days = days + 8
        end
        if (inputTable["Schedules.friday"] == "1") then
            days = days + 16
        end
        if (inputTable["Schedules.saturday"] == "1") then
            days = days + 32
        end
        if (inputTable["Schedules.sunday"] == "1") then
            days = days + 64
        end
        if (inputTable["Schedules.allDays"] == "1") then
            days = days + 127
        end

        inTable["Schedules.ScheduleName"] = inputTable["Schedules.ScheduleName"]
        inTable["Schedules.Days"] = days
        inTable["Schedules.AllDay"] = inputTable["Schedules.AllDay"]
        if (inputTable["Schedules.AllDay"] == "1") then
            inTable["Schedules.StartTimeHours"] = "0"
            inTable["Schedules.StartTimeMins"] = "0"
            inTable["Schedules.EndTimeHours"] = "23"
            inTable["Schedules.EndTimeMins"] = "59"
        else
            if (inputTable["Schedules.startTimeMeridian"] == "0") then
                if (inputTable["Schedules.startTimeHour"] == "12") then
                    inTable["Schedules.StartTimeHours"] = "0"
                else
                    inTable["Schedules.StartTimeHours"] = inputTable["Schedules.startTimeHour"]
                end
            else
                if (inputTable["Schedules.startTimeHour"] ~= "12") then
                    inTable["Schedules.StartTimeHours"] = inputTable["Schedules.startTimeHour"] + 12
                else
                    inTable["Schedules.StartTimeHours"] = inputTable["Schedules.startTimeHour"]
                end
            end

            if (inputTable["Schedules.endTimeMeridian"] == "0") then
                if (inputTable["Schedules.endTimeHour"] == "12") then
                    inTable["Schedules.EndTimeHours"] = "0"
                else
                    inTable["Schedules.EndTimeHours"] = inputTable["Schedules.endTimeHour"]
                end
            else
                if (inputTable["Schedules.endTimeHour"] ~= "12") then
                    inTable["Schedules.EndTimeHours"] = inputTable["Schedules.endTimeHour"] + 12
                else
                    inTable["Schedules.EndTimeHours"] = inputTable["Schedules.endTimeHour"]
                end
            end
            inTable["Schedules.StartTimeMins"] = inputTable["Schedules.startTimeMin"]
            inTable["Schedules.EndTimeMins"] = inputTable["Schedules.endTimeMin"]
        end

        valid = fw.config (inTable,  rowid, operation)

        -- return
        if (valid) then
                db.commitTransaction()
                return "OK", "STATUS_OK"
        else
                db.rollback()
                return "ERROR", "SCHEDULE_CONFIG_FAILED"
        end
end

-- Schedules Delete
function fwView.deleteSchedules (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

	-- Check for Using Schedules
    for k,v in pairs(rowIds) do
        local schedule = db.getAttribute("Schedules", "_ROWID_", v, "ScheduleName")
        local rows = db.existsRowWhere("FirewallRules", "ScheduleName = '" .. schedule .. "'")
        local rows6 = db.existsRowWhere("FirewallRules6", "ScheduleName = '" .. schedule .. "'")
        if (rows or rows6) then
            valid = false
            errStr = "SCHEDULE_IN_USE_DEL_RULE"
            break
        end
	end

    if (valid) then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["Schedules._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "SCHEDULE_DELETE_FAILED"
                break
            end
	    end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "SCHEDULE_DELETE_OK"
	else
		db.rollback()
		return "ERROR", errStr
	end

end

-- Setting Default Outbound Policy Configuration
function fwView.defaultPolicyConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

	-- config Service
	valid = fw.config(inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_DEFAULT_CONFIG_FAILED"
	end
end

-- Firewall Inbound Rules Configuration
function fwView.lwInConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

	-- config Lan Wan In bound Rule
    -- If router is in Classical Routing then Set Lan Users.
    -- Other wise we should use send to lan Server Option.
    if (inputTable["FirewallRules.DestinationAddressType"] ~= nil) then
        if (inputTable["FirewallRules.DestinationAddressType"] == "1") then
            inputTable["FirewallRules.DestinationAddressStart"] = util.joinFields(inputTable,"FirewallRules","destStartIpAddrbyte", ".",4)
        elseif (inputTable["FirewallRules.DestinationAddressType"] == "2") then
            inputTable["FirewallRules.DestinationAddressStart"] = util.joinFields(inputTable,"FirewallRules","destStartIpAddrbyte", ".",4)
            inputTable["FirewallRules.DestinationAddressEnd"] = util.joinFields(inputTable,"FirewallRules","destEndIpAddrbyte", ".",4)
        end
    else
        if (inputTable["FirewallRules.Action"] ~= "DROP") then
            inputTable["FirewallRules.DNATAddress"] = util.joinFields(inputTable,"FirewallRules","sendToSvrIpAddrbyte", ".",4)
        end
    end

    -- Setting WAN Destination IP Address
    if (inputTable["FirewallRules.DestinationPublicInterface"] ~= nil) then
        if (inputTable["FirewallRules.DestinationPublicInterface"] ~= "WAN") then
            inputTable["FirewallRules.DestinationPublicAddress"] = util.joinFields(inputTable,"FirewallRules","wanDestIpAddrbyte", ".",4)
        end
    end

    -- Setting WAN Users
    if (inputTable["FirewallRules.SourceAddressType"] == "1") then
        inputTable["FirewallRules.SourceAddressStart"] = util.joinFields(inputTable,"FirewallRules","sourceStartIpAddrbyte", ".",4)
    elseif (inputTable["FirewallRules.SourceAddressType"] == "2") then
        inputTable["FirewallRules.SourceAddressStart"] = util.joinFields(inputTable,"FirewallRules","sourceStartIpAddrbyte", ".",4)
        inputTable["FirewallRules.SourceAddressEnd"] = util.joinFields(inputTable,"FirewallRules","sourceEndIpAddrbyte", ".",4)
    end

    -- check if schedule Name is selected.
    if (inputTable["FirewallRules.Action"] == "DROP_BY_SCHED_MATCH" or
        inputTable["FirewallRules.Action"] == "ACCEPT_BY_SCHED_MATCH") then
        if (inputTable["FirewallRules.ScheduleName"] == nil) then
            db.rollback()
		    return "ERROR", "FW_SCHED_REQUIRE"
        end
    end

    valid = fw.config(inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_IN_RULE_CONFIG_FAILED"
	end
end

-- Firewall Outbound Rules Configuration
function fwView.lwOutConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

	-- config Lan Wan Outbound Rule
    if (inputTable["FirewallRules.SourceAddressType"] == "1") then
        inputTable["FirewallRules.SourceAddressStart"] = util.joinFields(inputTable,"FirewallRules","sourceStartIpAddrbyte", ".",4)
    elseif (inputTable["FirewallRules.SourceAddressType"] == "2") then
        inputTable["FirewallRules.SourceAddressStart"] = util.joinFields(inputTable,"FirewallRules","sourceStartIpAddrbyte", ".",4)
        inputTable["FirewallRules.SourceAddressEnd"] = util.joinFields(inputTable,"FirewallRules","sourceEndIpAddrbyte", ".",4)
    end

    -- Setting WAN Users
    if (inputTable["FirewallRules.DestinationAddressType"] == "1") then
        inputTable["FirewallRules.DestinationAddressStart"] = util.joinFields(inputTable,"FirewallRules","destStartIpAddrbyte", ".",4)
    elseif (inputTable["FirewallRules.DestinationAddressType"] == "2") then
        inputTable["FirewallRules.DestinationAddressStart"] = util.joinFields(inputTable,"FirewallRules","destStartIpAddrbyte", ".",4)
        inputTable["FirewallRules.DestinationAddressEnd"] = util.joinFields(inputTable,"FirewallRules","destEndIpAddrbyte", ".",4)
    end

    -- Setting NAT IP
    if (inputTable["FirewallRules.SNATAddressType"] ~= nil) then
        if (inputTable["FirewallRules.SNATAddressType"] == "1") then
            inputTable["FirewallRules.SNATAddress"] = util.joinFields(inputTable,"FirewallRules","natIpAddrbyte", ".",4)
        end
        inputTable["FirewallRules.SNATInterface"] = "WAN1"
    end

    -- check if schedule Name is selected.
    if (inputTable["FirewallRules.Action"] == "DROP_BY_SCHED_MATCH" or
        inputTable["FirewallRules.Action"] == "ACCEPT_BY_SCHED_MATCH") then
        if (inputTable["FirewallRules.ScheduleName"] == nil) then
            db.rollback()
		    return "ERROR", "FW_SCHED_REQUIRE"
        end
    end

    valid = fw.config(inputTable, rowid, operation)
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_OUT_RULE_CONFIG_FAILED"
	end
end

-- Modify Firewall Rules (Enable/Disable/Delete)
function fwView.modifyRules (rowIds, operation)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    if (operation == "enable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules.Status"] = "1";

            local bwTable = {}
            local bFlag = true
            local bStat = "1"
            bwTable = db.getRow ("firewallRules", "_ROWID_", v)
            if(bwTable["firewallRules.BandwidthProfileName"] ~= nil and bwTable["firewallRules.BandwidthProfileName"] ~= "" and bwTable["firewallRules.BandwidthProfileName"] ~= "NONE") then
                bStat = db.getAttribute ("BandwidthProfileStatus", "_ROWID_", "1", "Status")
                if(bStat == "0") then
                    if(db.existsRowWhere("BandWidthProfile","ProfileName ='"..bwTable["firewallRules.BandwidthProfileName"].."'")) then
                        valid = false
                        errStr = "ENABLE_BWPROF_STATUS"
                    else
                        valid = false
                        errStr = "BWPROF_DONOT_EXIST"
                    end
                end
            end

            valid = valid and fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = errStr or "FW_RULES_ENABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_RULES_ENABLED"
        end
    elseif (operation == "disable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules.Status"] = "0";
            valid = fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = "FW_RULES_DISABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_RULES_DISABLED"
        end
    else
        local priIds = {}
        local idx = 0
        for k,v in pairs(rowIds) do
            idx = idx + 1
            priIds[idx] = db.getAttribute ("FirewallRules", "_ROWID_", v, "PriorityId") or ''
            local inTable = {}
            inTable["FirewallRules._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "FW_RULES_DELETE_FAILED"
                break
            end
        end
        if (valid ~= false) then
            errStr = "FW_RULES_DELETED"
        end

        if (valid ~= false) then
            -- Re-arrange Rules priority
            local rulesPriority = ""
            local rulesPriConfig = db.getRow ("fwPriorityMap", "RuleType", "ALL")
            if (rulesPriConfig ~= nil) then
                rulesPriority = rulesPriConfig["fwPriorityMap.PriorityStr"] or ''
                if (rulesPriority ~= nil and rulesPriority ~= "") then
                    local ruleIds = util.split (rulesPriority, ",")
                    if (ruleIds ~= false) then
                        rulesPriority = ""
                        for k,v in pairs (ruleIds) do
                            local notInDeleteList = true
                            for m,n in pairs (priIds) do
                                if (v == n) then
                                    notInDeleteList = false
                                    break
                                end
                            end

                            if (notInDeleteList) then
                                rulesPriority = rulesPriority .. v .. ","
                            end
                        end

                       	rulesPriority = string.sub(rulesPriority, 1, string.len(rulesPriority) - 1) --shave last comma

                        rulesPriConfig["fwPriorityMap.PriorityStr"] = rulesPriority
                        DBTable = "fwPriorityMap"
                        valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap._ROWID_"], "edit")

                        if (valid) then
                        else
                            errStr = "FW_RULES_PRIORITY_CONFIG_FAILED"
                        end
                    else
                        valid = false
                        errStr = "FW_RULES_PRIORITY_CONFIG_FAILED"
                    end
                else
                    valid = false
                    errStr = "FW_RULES_PRIORITY_CONFIG_FAILED"
                end
            else
                valid = false
                errStr = "FW_RULES_PRIORITY_CONFIG_FAILED"
            end
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
        -- hardware offload /sppe update
        fwView.hwOffloadUpdate()
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Move Firewall rules
function fwView.moveRules (rowIds, position)
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    -- Collect priority Is
    local ruleIds = {}
    if (#rowIds == 0) then
        ruleIds["1"] = db.getAttribute ("FirewallRules", "_ROWID_", rowIds["1"], "PriorityId") or ''
    else
        local idx = 0
        for k,v in pairs(rowIds) do
            idx = idx + 1
            ruleIds[idx] = db.getAttribute ("FirewallRules", "_ROWID_", v, "PriorityId") or ''
        end
    end

    db.beginTransaction() --begin transactions
    local valid = false

    local rulesPriority = nil
    local rulesToMove = ""
    local rulesPriConfig = db.getRow ("fwPriorityMap", "RuleType", "ALL")
    if (rulesPriConfig ~= nil) then
        rulesPriority = rulesPriConfig["fwPriorityMap.PriorityStr"] or ''
    end

    if (rulesPriority ~= nil and rulesPriority ~= "") then
        local curRuleIds = util.split (rulesPriority, ",")
        if (curRuleIds ~= false) then
            -- Collect rules to be move
            if (#ruleIds == 0) then
                rulesToMove = ruleIds["1"] .. ","
            else
                for k,v in pairs(ruleIds) do
                    rulesToMove = rulesToMove .. v .. ","
                end
            end

            -- Rearrange priorities.
            rulesPriority = ""
            local skipInsert = false
            for k,v in pairs (curRuleIds) do
                local notInMoveList = true
                for m,n in pairs (ruleIds) do
                    if (v == n) then
                        notInMoveList = false
                        break
                    end
                end

                if (notInMoveList) then
                    rulesPriority = rulesPriority .. v .. ","
                end
            end

            -- Insert selected rules in teh position.
            if (rulesPriority == "") then
                rulesPriority = rulesToMove
            elseif (tonumber(position) == 1) then -- Move to First
                rulesPriority = rulesToMove .. "," .. rulesPriority
            elseif (tonumber(position) == #curRuleIds) then -- Move to Last
                rulesPriority = rulesPriority .. "," .. rulesToMove
            else
                local newRulesList = util.split (rulesPriority, ",")
                if (newRulesList ~= false) then
                    if (tonumber(position) >= #newRulesList) then
                        rulesPriority = rulesPriority .. "," .. rulesToMove
                    else
                        rulesPriority = ""
                        local idx = 0
                        for k,v in pairs (newRulesList) do
                            idx = idx + 1
                            if (idx == tonumber(position)) then
                                rulesPriority = rulesPriority .. "," .. rulesToMove
                            end
                            rulesPriority = rulesPriority .. v .. ","
                        end
                    end
                end
            end

          	rulesPriority = string.gsub(rulesPriority, ",,", ",") --shave double comma
           	rulesPriority = string.sub(rulesPriority, 1, string.len(rulesPriority) - 1) --shave last comma
        end
    end

    rulesPriConfig["fwPriorityMap.PriorityStr"] = rulesPriority
    DBTable = "fwPriorityMap"
    valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap._ROWID_"], "edit")

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "MOVE_OPERATION_FAILED"
    end
end

-- Modify IPv6 Firewall Rules (Enable/Disable/Delete)
function fwView.modifyIPv6Rules (rowIds, operation)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    if (operation == "enable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules6.Status"] = "1";
            valid = fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = "FW_RULES_ENABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_RULES_ENABLED"
        end
    elseif (operation == "disable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules6.Status"] = "0";
            valid = fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = "FW_RULES_DISABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_RULES_DISABLED"
        end
    else
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules6._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "FW_RULES_DELETE_FAILED"
                break
            end
        end
        if (valid ~= false) then
            errStr = "FW_RULES_DELETED"
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Port Triggering Configuration
function fwView.portTrgConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

	-- config Service
	valid = fw.config(inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_PORT_TRG_CONFIG_FAILED"
	end
end

-- Delete Port Triggering Rules
function fwView.deleteRules (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["PortTriggering._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_PORT_TRG_RULES_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_PORT_TRG_RULES_DELETED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- attack Checks Configuration
function fwView.attackChecksConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

   	valid = fw.config(inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_AC_CONFIG_FAILED"
	end
end

-- Content Filtering Configuration
function fwView.contentFileteringConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = true

    if (inputTable["ContentFiltering.Status"] == "1") then
        DBTable = "WebComponents"
       	valid = fw.config(inputTable, rowid, operation)
    end

    if (valid) then
        DBTable = "ContentFiltering"
       	valid = fw.config(inputTable, rowid, operation)
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_CF_CONFIG_FAILED"
	end
end

-- Blocked Keywords Configuration
function fwView.blockedKeywordConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

    if (operation == "add") then
        valid = fw.config(inputTable, rowid, operation)
     elseif (operation == "edit") then
        valid = fw.config(inputTable, rowid, operation)
     end

	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_KEYWORD_CONFIG_FAILED"
	end
end

-- Trusted Domains Configuration
function fwView.trustedDomainConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

   	valid = fw.config(inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_DOMAINS_CONFIG_FAILED"
	end
end

-- Trusted Domains Configuration
function fwView.enableEmailLogConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
        -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert("IPMACEmailLogsConfig", inputTable)
        elseif (operation == "edit") then
            return db.update("IPMACEmailLogsConfig", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("IPMACEmailLogsConfig", inputTable)
        end
    end
    return false
end

--  Mac Address filter Configuration
function fwView.macFilteringConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

   	valid = fw.config(inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_MACFILTERING_CONFIG_FAILED"
	end
end

--  ip MAcBinding Configuration
function fwView.ipMacConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
    local table = db.getTable("IPMACBinding")
    local RowId = 0
    for k,v in pairs (table) do
        RowId = RowId+1
        local tableRow = table [RowId]
        if (tableRow["IPMACBinding._ROWID_"] ~= rowid) then
            if(inputTable["IPMACBinding.ipVersion"]== tableRow["IPMACBinding.ipVersion"]) then
                if(inputTable["IPMACBinding.Name"] == tableRow["IPMACBinding.Name"]) then
                    db.rollback()
                    return "ERROR","FW_IPMACBINDING_CONFIG_DUPLICATE_NAME"
                elseif(inputTable["IPMACBinding.SourceMACAddress"] == 
                    tableRow["IPMACBinding.SourceMACAddress"]) then
                    db.rollback()
                    return "ERROR","FW_IPMACBINDING_CONFIG_DUPLICATE_MAC"
                elseif(inputTable["IPMACBinding.IPAddress"] == 
                    tableRow["IPMACBinding.IPAddress"]) then
                    db.rollback()
                    return "ERROR","FW_IPMACBINDING_CONFIG_DUPLICATE_IP"
                end
            end
        end
    end

    if (operation == "add") then
        maxId = db.columnMaxValueWhere("IPMACBinding", "xmlId", "Name IS NOT NULL")
        if (maxId == "" or maxId == nil) then
            maxId = -1
        end
        inputTable["IPMACBinding.xmlId"] = tonumber(maxId) + 1
    end
   	valid = fw.config(inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_IPMACBINDING_CONFIG_FAILED"
	end
end

-- Delete Blocked Keywords
function fwView.deleteKeywords (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["BlockSites._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_BLOCKED_KEYWORDS_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_BLOCKED_KEYWORDS_DELETED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Enable / Disable Blocked Keywords
function fwView.modifyKeywordStatus (rowIds, operation)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"
    local enableFlag = "1"
    if (operation == "enable") then
          enableFlag = "1"
    else
          enableFlag = "0"
    end

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["BlockSites.Status"] = enableFlag;
        valid = fw.config(inTable, v, "edit")
        if (valid == false) then
            errStr = "FW_KEYWORD_CONFIG_FAILED"
            break
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Delete Trusted Domains
function fwView.deleteDomains (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["TrustedDomains._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_TRUSTED_DOMAINS_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_TRUSTED_DOMAINS_DELETED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Delete Source MacFilterAddress
function fwView.deleteMac (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["SourceMACAddress._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_MAC_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_MAC_DELETED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Delete IpMacBindings
function fwView.deleteIpMac (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["IPMACBinding._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_IPMAC_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_IPMAC_DELETED"
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", errStr
    else
        db.rollback()
        return "ERROR", errStr
    end
end

function fwView.getMaxPriority ()
    -- Get All Firewall Rules to find the newly added rule.
    local ruleType = "RuleType = 'SECURE_INSECURE'"
    ruleType = ruleType .. " or RuleType = 'INSECURE_SECURE'"
    ruleType = ruleType .. " or RuleType = 'PUBLIC_SECURE'"
    ruleType = ruleType .. " or RuleType = 'SECURE_PUBLIC'"
    ruleType = ruleType .. " or RuleType = 'INSECURE_PUBLIC'"
    ruleType = ruleType .. " or RuleType = 'PUBLIC_INSECURE'"

    local cur = db.execute(string.format([[SELECT MAX(PriorityId) PriorityId FROM FirewallRules WHERE %s]], ruleType))

     -- check for rows
    local result = nil
    if (cur) then
        local row = cur:fetch ({}, "a")
        cur:close()
        if (row) then result = row["PriorityId"] or '' end
    end
    return result
end

-- Firewall Rules of Configuration
function fwView.firewallRulesConfig(inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    if (inputTable["FirewallRules.RuleType"] == nil) then
        inputTable["FirewallRules.RuleType"] = inputTable["FirewallRules.FromZoneType"] .. "_" .. inputTable["FirewallRules.ToZoneType"]
    else
        local pos = string.find(inputTable["FirewallRules.RuleType"], "_")
        inputTable["FirewallRules.FromZoneType"] = string.sub(inputTable["FirewallRules.RuleType"], 0, pos-1)
        inputTable["FirewallRules.ToZoneType"] = string.sub(inputTable["FirewallRules.RuleType"], pos+1)
    end
    if(inputTable["FirewallRules.BandwidthProfileName"] ~= nil and inputTable["FirewallRules.BandwidthProfileName"] ~= "") then
        local bStat = db.getAttribute ("BandwidthProfileStatus", "_ROWID_", "1", "Status")
        if(bStat == "0" and db.existsRow("BandWidthProfile", "ProfileName",inputTable["FirewallRules.BandwidthProfileName"])) then
            db.rollback()
            return "ERROR", "ENABLE_BWPROF_STATUS"
        end
    end

    local newPriority = 0
    if (valid and operation == "add") then
        local maxPriorityId = fwView.getMaxPriority ()
        if (maxPriorityId == nil or
            maxPriorityId == "" or
            maxPriorityId == "0") then
            newPriority = 1
        else
            newPriority = tonumber(maxPriorityId) + 1
        end

    if (newPriority == 4294967295) then
        newPriority = 1
            while (true) do
                local priorityRow = db.getRow ("FirewallRules", "PriorityId", newPriority)
                if (priorityRow == nil or priorityRow == "") then
                    break
                else
                    newPriority = newPriority + 1
                end
            end
        end
        inputTable["FirewallRules.PriorityId"] = newPriority
    end

    valid = fw.config(inputTable, rowid, operation)


    -- Add the added rule into priority list
    if (valid and operation == "add") then
        -- Get All Firewall Rules to find the newly added rule.
        local ruleType = "RuleType = 'SECURE_INSECURE'"
        ruleType = ruleType .. " or RuleType = 'INSECURE_SECURE'"
        ruleType = ruleType .. " or RuleType = 'PUBLIC_SECURE'"
        ruleType = ruleType .. " or RuleType = 'SECURE_PUBLIC'"
        ruleType = ruleType .. " or RuleType = 'INSECURE_PUBLIC'"
        ruleType = ruleType .. " or RuleType = 'PUBLIC_INSECURE'"
        local fwRules = db.getRowsWhere ("FirewallRules", ruleType)
        local rulesPriority = db.getAttribute ("fwPriorityMap", "RuleType", "ALL", "PriorityStr") or ''
        if (rulesPriority == "") then
            -- First Firewall Rule Added
            rulesPriority = newPriority
        else
            rulesPriority = rulesPriority .. "," .. newPriority
        end

        local rulesPriConfig = db.getRow ("fwPriorityMap", "RuleType", "ALL")
        if (rulesPriConfig ~= nil) then
            rulesPriConfig["fwPriorityMap.PriorityStr"] = rulesPriority
            DBTable = "fwPriorityMap"
            valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap._ROWID_"], "edit")
        else
            valid = false
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        -- hardware offload /sppe update
        fwView.hwOffloadUpdate()
        return "OK", "STATUS_OK" , newPriority
    else
        db.rollback()
        return "ERROR", "FW_CONFIG_FAILED"
    end
end

-- IPv6 Firewall Rules of Configuration
function fwView.ipv6FirewallRulesConfig(inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    if (inputTable["FirewallRules6.RuleType"] == nil) then
        inputTable["FirewallRules6.RuleType"] = inputTable["FirewallRules6.FromZoneType"] .. "_" .. inputTable["FirewallRules6.ToZoneType"]
    else
        local pos = string.find(inputTable["FirewallRules6.RuleType"], "_")
        inputTable["FirewallRules6.FromZoneType"] = string.sub(inputTable["FirewallRules6.RuleType"], 0, pos-1)
        inputTable["FirewallRules6.ToZoneType"] = string.sub(inputTable["FirewallRules6.RuleType"], pos+1)
    end
    local newPriority = 0
    if (valid and operation == "add") then
        local maxPriorityId = fwView.getMaxPriority_v6 ()
        if (maxPriorityId == nil or
            maxPriorityId == "" or
            maxPriorityId == "0") then
            newPriority = 1
        else
            newPriority = tonumber(maxPriorityId) + 1
        end

    if (newPriority == 4294967295) then
        newPriority = 1    
            while (true) do 
                local priorityRow = db.getRow ("FirewallRules6", "PriorityId", newPriority)
                if (priorityRow == nil or priorityRow == "") then
                    break
                else
                    newPriority = newPriority + 1
                end
            end
        end
        inputTable["FirewallRules6.PriorityId"] = newPriority
    end    
 
    valid = fw.config(inputTable, rowid, operation)

    -- Add the added rule into priority list
    if (valid and operation == "add") then
        -- Get All Firewall Rules to find the newly added rule.
        local ruleType = "RuleType = 'SECURE_INSECURE'"
        ruleType = ruleType .. " or RuleType = 'INSECURE_SECURE'"
        ruleType = ruleType .. " or RuleType = 'PUBLIC_SECURE'"
        ruleType = ruleType .. " or RuleType = 'SECURE_PUBLIC'"
        ruleType = ruleType .. " or RuleType = 'INSECURE_PUBLIC'"
        ruleType = ruleType .. " or RuleType = 'PUBLIC_INSECURE'"
        local fwRules = db.getRowsWhere ("FirewallRules", ruleType)
        local rulesPriority = db.getAttribute ("fwPriorityMap6", "RuleType", "ALL", "PriorityStr") or ''
        if (rulesPriority == "") then
            -- First Firewall Rule Added
            rulesPriority = newPriority
        else
            rulesPriority = rulesPriority .. "," .. newPriority
        end

        local rulesPriConfig = db.getRow ("fwPriorityMap6", "RuleType", "ALL")
        if (rulesPriConfig ~= nil) then
            rulesPriConfig["fwPriorityMap6.PriorityStr"] = rulesPriority
            DBTable = "fwPriorityMap6"
            valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap6._ROWID_"], "edit")
        else
            valid = false
        end
    end
      
    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	db.rollback()
	return "ERROR", "FW_CONFIG_FAILED"
    end
end

--- @description This function adds the ipv6 Firewall configuration
--
-- @param inputTable The following is the input provided by User :
--
-- @return "OK" on success, "ERROR" on failure
--
-- @see 
--
function ipv6FirewallRules.add (inputTable)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local addStatus = false

    if (inputTable["FirewallRules6.RuleType"] == nil) then
        inputTable["FirewallRules6.RuleType"] = inputTable["FirewallRules6.FromZoneType"] .. "_" .. inputTable["FirewallRules6.ToZoneType"]
    else
        local pos = string.find(inputTable["FirewallRules6.RuleType"], "_")
        inputTable["FirewallRules6.FromZoneType"] = string.sub(inputTable["FirewallRules6.RuleType"], 0, pos-1)
        inputTable["FirewallRules6.ToZoneType"] = string.sub(inputTable["FirewallRules6.RuleType"], pos+1)
    end

    -- add the settings in DB
    addStatus = fw.config(inputTable, "-1", "add")
    
    if (addStatus) then
        db.commitTransaction(true)
         return "OK", "STATUS_OK"
     else
        db.rollback()
        return "ERROR", "FW_CONFIG_FAILED"
     end
  
 end
 
--- @description This function edits the ipv6 Firewall rules configuration
--
-- @param inputTable The following is the input provided by User :
--
-- @param uniqueId
--
-- @return "OK" on success, "ERROR" on failure
--
-- @see 
--
function ipv6FirewallRules.edit (inputTable, uniqueId)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local editStatus = false

    -- edit the settings in DB
    editStatus = fw.config(inputTable, uniqueId, "edit")
   
    if (editStatus) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
   else
        db.rollback()
        return "ERROR", "FW_CONFIG_FAILED"
    end
end

--- @description This function gets the ipv6 Firewall configuration
--
-- @param uniqueId
--
-- @return Lua Table on success, nil on failure
--
-- @see 
--
function ipv6FirewallRules.get (uniqueId)
    local firewallRules6Table = {}
    if (uniqueId == "-1") then
        firewallRules6Table ["FirewallRules6"] = db.getRow ("FirewallRules6", "_ROWID_", uniqueId)
    else
        firewallRules6Table = db.getRow ("FirewallRules6", "_ROWID_", uniqueId)
    end

    firewallRules6Table ["Services"] = db.getTable ("Services")
    firewallRules6Table ["Schedules"] = db.getTable ("Schedules")
    -- show settings in DB
    return firewallRules6Table
end

--- @description This function deletes the ipv6 firewall rule
--
-- @param uniqueIds  Index denoting the location in table at which operation is 
-- to be performed
--
-- @return "OK" on success, "ERROR" on failure
--
-- @see 
--
function ipv6FirewallRules.delete (uniqueIds)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    for k,v in pairs(uniqueIds) do
        local inTable = {}
        inTable["FirewallRules6._ROWID_"] = v;
        deleteStatus = fw.config(inTable, v, "delete")
        if (deleteStatus == false) then
            break
        end
    end

    -- return
    if (deleteStatus) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "FW_CONFIG_FAILED"
    end

end

--- @description This function is used to enable an IPV6 Firewall Rule
--
-- @param uniqueId Index denoting the location in the table where operation is
-- to be performed
--
-- @return OK on success, ERROR on failure
--         Error Code - Description of the error
--
-- @see firewall6RuleDisable()
function firewall6RuleEnable (uniqueIds)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true

    for k,v in pairs(uniqueIds) do
        local inTable = {}
        inTable["FirewallRules6.Status"] = "1";
        ruleEnableStatus = fw.config (inTable, v, "edit")
        if (ruleEnableStatus == false) then
            break
        end
    end

    -- return
    if (ruleEnableStatus) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "FW_CONFIG_FAILED"
    end

end

--- @description This function is used to disable an IPV6 Firewall Rule
--
-- @param uniqueId Index denoting the location in the table where operation is
-- to be performed
--
-- @return OK on success, ERROR on failure
--         Error Code - Description of the error
--
-- @see firewall6RuleEnable()
function firewall6RuleDisable (uniqueIds)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true

    for k,v in pairs(uniqueIds) do
        local inTable = {}
        inTable["FirewallRules6.Status"] = "0";
        ruleDisableStatus = fw.config (inTable, v, "edit")
        if (ruleDisableStatus == false) then
            break
        end
    end

    -- return
	if (ruleDisableStatus) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_CONFIG_FAILED"
	end

end

--- @description This function is used to set the default outbound IPV6 Firewall Rule
--
-- @param inputTable
-- @param uniqueId Index denoting the location in the table where operation is
-- to be performed
--
-- @return OK on success, ERROR on failure
--         Error Code - Description of the error
--
function defaultPolicyEdit (inputTable, uniqueId)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local editDefaultPolicyStatus = false

	-- config Service
    rowIdIpv4 = inputTable ["FirewallRules.ROWID"]
    rowIdIpv6 = inputTable ["FirewallRules6.ROWID"]
    DBTable = "FirewallRules"
	editDefaultPolicyStatus = fw.config(inputTable, rowIdIpv4, "edit")
    DBTable = "FirewallRules6"
	editDefaultPolicyStatus = fw.config(inputTable, rowIdIpv6, "edit")
	
	-- return
	if (editDefaultPolicyStatus) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_DEFAULT_CONFIG_FAILED"
	end
end

--- @description This function is used to get the default outbound IPV6 Firewall Rule
--
-- @return Lua Table on success, nil on failure
--
function defaultPolicyGet ()
    local defaultPolicyTable = {}
    defaultPolicyTable ["FirewallRules"] = db.getRow ("FirewallRules", "RuleType", "DEFAULT_POLICY")
    defaultPolicyTable ["FirewallRules6"] = db.getRow ("FirewallRules6", "RuleType", "DEFAULT_POLICY")
    return defaultPolicyTable
end


--- @description This function will get all the custom rules, modify them to
--make it user-readable and returns the table. 
--
-- @return Lua Table on success, nil on failure
--
function ipv6FirewallRulesDisplay ()
--Need to get the rules which are of type SECURE_INSECURE, INSECURE_SECURE,
--PUBLIC_SECURE, PUBLIC_INSECURE, SECURE_PUBLIC, INSECURE_PUBLIC. Return this
--table
   local ruleArray = {}
	ruleArray[1] = "SECURE_INSECURE"
	ruleArray[2] = "INSECURE_SECURE"
	ruleArray[3] = "PUBLIC_SECURE"
	ruleArray[4] = "SECURE_PUBLIC"
	ruleArray[5] = "INSECURE_PUBLIC"
	ruleArray[6] = "PUBLIC_INSECURE"
    -- We presently don't support DMZ (PUBLIC) zone of IPv6. But added the rule
    -- types involving DMZ zone anyway, in case it is supported in future.
	local i = 0
	local j = 0
	local ruleTable = {}
	local newRows = {}
	for j=1, #ruleArray do
		if(db.existsRow ("FirewallRules6", "RuleType", ruleArray[j])) then
			newRows = db.getRows("FirewallRules6", "RuleType", ruleArray[j])
			for k,v in pairs(newRows) do
                table.insert(ruleTable, (#ruleTable + 1), newRows[k])
                -- for From Zone Type
                  if (ruleTable[#ruleTable] ["FirewallRules6.FromZoneType"] == "SECURE") then
                      ruleTable [#ruleTable]["FirewallRules6.FromZone"] = "LAN"
                  elseif (ruleTable[#ruleTable] ["FirewallRules6.FromZoneType"] == "INSECURE") then
                      ruleTable [#ruleTable]["FirewallRules6.FromZone"] = "WAN"
                  end

                  -- for To Zone Type
                if (ruleTable[#ruleTable] ["FirewallRules6.ToZoneType"] == "SECURE") then
                      ruleTable [#ruleTable]["FirewallRules6.ToZone"] = "LAN"
                  elseif (ruleTable[#ruleTable] ["FirewallRules6.ToZoneType"] == "INSECURE") then
                      ruleTable [#ruleTable]["FirewallRules6.ToZone"] = "WAN"
                  end

                 -- for Rule Status
                 if (ruleTable[#ruleTable] ["FirewallRules6.Status"] == "0") then
                      ruleTable [#ruleTable]["FirewallRules6.ruleStatus"] = "Disabled"
                  elseif (ruleTable[#ruleTable] ["FirewallRules6.Status"] == "1") then
                      ruleTable [#ruleTable]["FirewallRules6.ruleStatus"] = "Enabled"
                  end

                 -- for Source Addresses
                 if (ruleTable[#ruleTable] ["FirewallRules6.SourceAddressStart"] == nil) then
                 elseif (ruleTable[#ruleTable] ["FirewallRules6.SourceAddressEnd"] == nil) then
                     ruleTable [#ruleTable]["FirewallRules6.SourceHosts"] = ruleTable[#ruleTable] ["FirewallRules6.SourceAddressStart"]
                 else
                     ruleTable [#ruleTable]["FirewallRules6.SourceHosts"] = ruleTable[#ruleTable] ["FirewallRules6.SourceAddressStart"]..":"..ruleTable[#ruleTable] ["FirewallRules6.SourceAddressEnd"]
                 end

                 --For Destination Addresses
                 if (ruleTable[#ruleTable] ["FirewallRules6.DestinationAddressStart"] == nil) then
                 elseif (ruleTable[#ruleTable] ["FirewallRules6.DestinationAddressEnd"] == nil) then
                     ruleTable [#ruleTable]["FirewallRules6.DestinationHosts"] = ruleTable[#ruleTable] ["FirewallRules6.DestinationAddressStart"]
                 else
                     ruleTable [#ruleTable]["FirewallRules6.DestinationHosts"] = ruleTable[#ruleTable] ["FirewallRules6.DestinationAddressStart"]..":"..ruleTable[#ruleTable] ["FirewallRules6.DestinationAddressEnd"]
                 end

                 -- For Logs
                 if (ruleTable[#ruleTable] ["FirewallRules6.LogLevel"] == "1") then
                      ruleTable [#ruleTable]["FirewallRules6.Loglevel"] = "Never"
                  elseif (ruleTable[#ruleTable] ["FirewallRules6.LogLevel"] == "2") then
                      ruleTable [#ruleTable]["FirewallRules6.Loglevel"] = "Always"
                  end

                 -- For Action
                 if (ruleTable[#ruleTable] ["FirewallRules6.Action"] == "ACCEPT") then
                      ruleTable [#ruleTable]["FirewallRules6.displayAction"] = "Allow Always"
                  elseif (ruleTable[#ruleTable] ["FirewallRules6.Action"] == "DROP") then
                      ruleTable [#ruleTable]["FirewallRules6.displayAction"] = "Block Always"
                  elseif (ruleTable[#ruleTable] ["FirewallRules6.Action"] == "ACCEPT_BY_SCHED_MATCH") then
                      ruleTable [#ruleTable]["FirewallRules6.displayAction"] = "Allow By Schedule"
                  elseif (ruleTable[#ruleTable] ["FirewallRules6.Action"] == "DROP_BY_SCHED_MATCH") then
                      ruleTable [#ruleTable]["FirewallRules6.displayAction"] = "Block By Schedule"
                  end
			end
		end
	end 

    return ruleTable
end


-- Routing Mode (NAT/Classical Routing) Configuration
function fwView.nwModeConfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true   
    local nat = db.getAttribute ("NatTable","_ROWID_","1","Enable")
    if (inputTable["NatTable.Enable"] ~= nat) then
        valid = fwView.priorityMapUpdate()
    end
    DBTable = "NatTable"
    if (valid) then
        valid = fw.config(inputTable, rowid, operation)
    end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "ROUTING_MODE_CONFIG_FAILED"
    end
end

-- WAN Mode Configuration for SRX55308 (Routing Mode (NAT/Classical Routing) Configuration + Wan Mode)
function fwView.wanModeSet (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true

    local nwModeTable = {}
    nwModeTable["NatTable.Enable"] = inputTable["NatTable.Enable"]
    local nat = db.getAttribute ("NatTable","_ROWID_","1","Enable")
    if (inputTable["NatTable.Enable"] ~= nat) then
        valid = fwView.priorityMapUpdate()
    end
    DBTable = "NatTable"
    if (valid) then
        valid = fw.config(nwModeTable, rowid, operation)
    end

    if (valid) then
    	local wanModeTable = db.getRow("WanMode", "_ROWID_", "1")
    	wanModeTable["WanMode.Wanmode"] = inputTable["WanMode.Wanmode"] or ''
    	if (inputTable["WanMode.LoadBalanceAlgo"] ~= nil) then
    		wanModeTable["WanMode.LoadBalanceAlgo"] = inputTable["WanMode.LoadBalanceAlgo"] or ''
    	end
    	if (inputTable["WanMode.FailoverKey"] ~= nil) then
    		wanModeTable["WanMode.FailoverKey"] = inputTable["WanMode.FailoverKey"] or ''
    	end
    	if (inputTable["WanMode.FailoverSecLogicalIfName"] ~= nil) then
    		wanModeTable["WanMode.FailoverSecLogicalIfName"] = inputTable["WanMode.FailoverSecLogicalIfName"] or ''
    	end
    	if(wanModeTable["WanMode.FailoverKey"] ~= nil or wanModeTable["WanMode.FailoverKey"] ~= '') then
	    	if (wanModeTable["WanMode.FailoverKey"] == "1") then
	    		wanModeTable["WanMode.Wanmode"] = "2"
                if (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil) then
                    wanModeTable["WanMode.FailoverPriLogicalIfName"] = inputTable["WanMode.FailoverPriLogicalIfName"] or ''
                end
            else 
                if (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil) then
                    wanModeTable["WanMode.DedicatedLogicalIfName"] = inputTable["WanMode.FailoverPriLogicalIfName"] or ''
                end
	    	end
	    end

		if (wanModeTable["WanMode.Wanmode"] ~= nil and wanModeTable["WanMode.Wanmode"] == "1") then
        	local type1 = db.getRowWhere("NimfConf", "LogicalIfName = 'WAN1' AND  AddressFamily = 2")
            local connection1 = type1["NimfConf.ConnectionType"]
        	if((connection1 == "l2tp") or (connection1 == "pptp")) then
            	idle1 = db.getAttribute(connection1,"LogicalIfName","WAN1","IdleTimeOutFlag")
	        elseif(connection1 == "pppoe") then
	            profile1 = db.getAttribute("pppoe","LogicalIfName","WAN1","ProfileName")
	            idle1 = db.getAttribute("PppoeProfile","ProfileName",profile1,"IdleTimeOutFlag")
	        else
	            idle1 = "0"
	        end

	        local type2 = db.getRowWhere("NimfConf","LogicalIfName = 'WAN2' AND  AddressFamily = 2")
	        local connection2 = type2["NimfConf.ConnectionType"]
			if((connection2 == "l2tp") or (connection2 == "pptp")) then
	            idle2=db.getAttribute(connection2,"LogicalIfName","WAN2","IdleTimeOutFlag")
	        elseif(connection2 == "pppoe") then
	            profile2 = db.getAttribute("pppoe","LogicalIfName","WAN2","ProfileName")
	            idle2 = db.getAttribute("PppoeProfile","ProfileName",profile2,"IdleTimeOutFlag")
	        else
	            idle2 = "0"
	        end

	        --[[local type3 = db.getRowWhere("NimfConf","LogicalIfName = 'WAN3' AND  AddressFamily = 2")
	        local connection3 = type3["NimfConf.ConnectionType"]
	   		if((connection3 == "l2tp") or (connection3 == "pptp")) then
	            idle3=db.getAttribute(connection3,"LogicalIfName","WAN3","IdleTimeOutFlag")
	        elseif(connection3 == "pppoe") then
	            profile3 = db.getAttribute("pppoe","LogicalIfName","WAN3","ProfileName")
	            idle3 = db.getAttribute("PppoeProfile","ProfileName",profile3,"IdleTimeOutFlag")
	        else
	            idle3 = "0"
	        end

	        local type4 = db.getRowWhere("NimfConf","LogicalIfName = 'WAN4' AND  AddressFamily = 2")
	        local connection4 = type4["NimfConf.ConnectionType"]
	   		if((connection4 == "l2tp") or (connection4 == "pptp")) then
	            idle4=db.getAttribute(connection4,"LogicalIfName","WAN4","IdleTimeOutFlag")
	        elseif(connection4 == "pppoe") then
	            profile4 = db.getAttribute("pppoe","LogicalIfName","WAN4","ProfileName")
	            idle4 = db.getAttribute("PppoeProfile","ProfileName",profile4,"IdleTimeOutFlag")
	        else
	            idle4 = "0"
	        end]]

	        if (idle1 == "1" or idle2 == "1") then
                db.rollback()
	            return "ERROR", "IDLE_TIMEOUT_ERROR"
	        end
    	end

    	-- If WAN Mode changed to "Dedicated WAN"
	    -- Check for enabled VPN Policies with same traffic selectors
	    if (db.existsTable and db.existsTable ("WanMode")) then
	        local oldWanMode = db.getAttribute ("WanMode", "_ROWID_", "1", "Wanmode") or ''
	        if (wanModeTable["WanMode.Wanmode"] ~= nil and wanModeTable["WanMode.Wanmode"] == "0" and
	            (oldWanMode == "1" or oldWanMode == "2")) then
	            if (db.existsTable ("IpsecVPNPolicy")) then
	                require "teamf1lualib/vipSecure"
	                local vpnPolicies = db.getTable ("IpsecVPNPolicy")
	                if (vpnPolicies ~= nil and #vpnPolicies > 1) then
	                    for k,v in pairs (vpnPolicies) do
	                        local dupPolicies = vipSecure.getDulicatePolicies (v)
	                        if (dupPolicies ~= nil and #dupPolicies > 1) then
                                db.rollback()
	                            return "ERROR", "DUPLICATE_VPN_POLICIES_ENABLED_ERROR"
	                        end
	                    end
	                end
	            end
	        end
	    end

	    if (db.existsRow("WanMode", "_ROWID_", "1")) then
	    	valid = db.update("WanMode", wanModeTable, "1")
	    else
	    	valid = db.insert("WanMode", wanModeTable)
	    end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "WM_CONFIG_FAILED"
    end
end

-- WAN Mode Configuration for SRX55308 (Routing Mode (NAT/Classical Routing) Configuration + Wan Mode)
function fwView.wanMode6Set (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    local valid = true

    local wanModeTable = db.getRow("WanMode", "_ROWID_", "2")
    wanModeTable["WanMode.Wanmode"] = inputTable["WanMode.Wanmode"] or ''
    if (inputTable["WanMode.LoadBalanceAlgo"] ~= nil) then
        wanModeTable["WanMode.LoadBalanceAlgo"] = inputTable["WanMode.LoadBalanceAlgo"] or ''
    end
    if (inputTable["WanMode.FailoverKey"] ~= nil) then
        wanModeTable["WanMode.FailoverKey"] = inputTable["WanMode.FailoverKey"] or ''
    end
    if (inputTable["WanMode.FailoverSecLogicalIfName"] ~= nil) then
        wanModeTable["WanMode.FailoverSecLogicalIfName"] = inputTable["WanMode.FailoverSecLogicalIfName"] or ''
    end
    if(wanModeTable["WanMode.FailoverKey"] ~= nil or wanModeTable["WanMode.FailoverKey"] ~= '') then
        if (wanModeTable["WanMode.FailoverKey"] == "1") then
            wanModeTable["WanMode.Wanmode"] = "2"
            if (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil) then
                wanModeTable["WanMode.FailoverPriLogicalIfName"] = inputTable["WanMode.FailoverPriLogicalIfName"] or ''
            end
        else 
            if (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil) then
                wanModeTable["WanMode.DedicatedLogicalIfName"] = inputTable["WanMode.FailoverPriLogicalIfName"] or ''
            end
        end
    end
    
    if (db.existsRow("WanMode", "_ROWID_", "2")) then
        valid = db.update("WanMode", wanModeTable, "2")
    else
        valid = db.insert("WanMode", wanModeTable)
    end

    db.beginTransaction() --begin transactions
 
    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "WM_CONFIG_FAILED"
    end
end

-- Firewall Sessions Configurations
function fwView.fwSessionconfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true

    -- Check for edit operation. If it is edit operation then get the row from the back end
    -- and update that row and then save.

    local globalSettingsRow = {}
    DBTable = "GlobalSettings"

    if(operation == "edit") then

        -- Get the row from GlobalSettings table which needs to be updated

        globalSettingsRow = db.getRow (DBTable, "_ROWID_", rowid)
        if(globalSettingsRow ~= nil) then
            globalSettingsRow["GlobalSettings.TcpSessionTimoutDuration"] = inputTable["GlobalSettings.TcpSessionTimoutDuration"]
            globalSettingsRow["GlobalSettings.UdpSessionTimoutDuration"] = inputTable["GlobalSettings.UdpSessionTimoutDuration"]
            globalSettingsRow["GlobalSettings.OtherSessionTimeoutDuration"] = inputTable["GlobalSettings.OtherSessionTimeoutDuration"]
        end
    else
        globalSettingsRow["GlobalSettings.TcpSessionTimoutDuration"] = inputTable["GlobalSettings.TcpSessionTimoutDuration"]
        globalSettingsRow["GlobalSettings.UdpSessionTimoutDuration"] = inputTable["GlobalSettings.UdpSessionTimoutDuration"]
        globalSettingsRow["GlobalSettings.OtherSessionTimeoutDuration"] = inputTable["GlobalSettings.OtherSessionTimeoutDuration"]

        -- Assigning default values to the fields in the table which are not being passed from the GUI and
        -- they are not numm fields

        globalSettingsRow["GlobalSettings.MaxUnidentifiedSessions"] = "32"
        globalSettingsRow["GlobalSettings.MaxHalfOpenSessions"] = "1024"
        globalSettingsRow["GlobalSettings.TcpSessionCleanupLatency"] = "10"

    end

    valid = fw.config(globalSettingsRow, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "FW_SESSIONS_CONFIG_FAILED"
    end
end

-- Firewall Traffic Meter Configuration
function fwView.trafficMeterConfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true

    local trafficMetrConfig = {}
    trafficMetrConfig["FwTrafficMeter.TrafficMeterStatus"] = inputTable["FwTrafficMeter.TrafficMeterStatus"]
    trafficMetrConfig["FwTrafficMeter.TrafficLimitType"] = inputTable["FwTrafficMeter.TrafficLimitType"]
    trafficMetrConfig["FwTrafficMeter.TrafficMonthlyLimit"] = inputTable["FwTrafficMeter.TrafficMonthlyLimit"]
    trafficMetrConfig["FwTrafficMeter.IncreaseLimitStatus"] = inputTable["FwTrafficMeter.IncreaseLimitStatus"]
    trafficMetrConfig["FwTrafficMeter.IncreaseLimitBy"] = inputTable["FwTrafficMeter.IncreaseLimitBy"]
    trafficMetrConfig["FwTrafficMeter.RestartCounter"] = inputTable["FwTrafficMeter.RestartCounter"]

--Converting the time into 24 hours format.
    if(trafficMetrConfig["FwTrafficMeter.RestartCounter"] == "1") then
        if (inputTable["FwTrafficMeter.startMeridian"] == "1") then
            if (tonumber(inputTable["FwTrafficMeter.spHour"]) == 12) then
                trafficMetrConfig["FwTrafficMeter.TimeHH"] = "12"
            else
                trafficMetrConfig["FwTrafficMeter.TimeHH"] = inputTable["FwTrafficMeter.spHour"] + 12
            end
            trafficMetrConfig["FwTrafficMeter.TimeMM"] = inputTable["FwTrafficMeter.spMinute"]
        else
            if (tonumber(inputTable["FwTrafficMeter.spHour"]) == 12) then
                trafficMetrConfig["FwTrafficMeter.TimeHH"] = "00"
            else
                trafficMetrConfig["FwTrafficMeter.TimeHH"] = inputTable["FwTrafficMeter.spHour"]
            end
            trafficMetrConfig["FwTrafficMeter.TimeMM"] = inputTable["FwTrafficMeter.spMinute"]
        end
    end

    trafficMetrConfig["FwTrafficMeter.DayOfMonth"] = inputTable["FwTrafficMeter.DayOfMonth"]
    trafficMetrConfig["FwTrafficMeter.SendEmailReport"] = inputTable["FwTrafficMeter.SendEmailReport"]
    trafficMetrConfig["FwTrafficMeter.BlockTraffic"] = inputTable["FwTrafficMeter.BlockTraffic"]
    trafficMetrConfig["FwTrafficMeter.SendEmailAlert"] = inputTable["FwTrafficMeter.SendEmailAlert"]

    valid = fw.config(trafficMetrConfig, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "FW_TM_CONFIG_FAILED"
    end
end

--Traffic Meter specific Time getting
function fwView.trafficMtr_ConfigGet (wanId)
    local specificTimeConfig = db.getRow("FwTrafficMeter","WanId", wanId)
    local fwTrafficConfig = {}

    if (specificTimeConfig == nil) then
        fwTrafficConfig["FwTrafficMeter.TimeHH"] = "1"
        fwTrafficConfig["FwTrafficMeter.TimeMM"] = "1"
        fwTrafficConfig["FwTrafficMeter.startMeridian"] = "0"
    else
        if (tonumber (specificTimeConfig["FwTrafficMeter.TimeHH"]) == 0) then
            fwTrafficConfig["FwTrafficMeter.startMeridian"] = "0"
            fwTrafficConfig["FwTrafficMeter.spHour"] = "12"
	elseif (tonumber(specificTimeConfig["FwTrafficMeter.TimeHH"]) == 12) then
            fwTrafficConfig["FwTrafficMeter.startMeridian"] = "1"
            fwTrafficConfig["FwTrafficMeter.spHour"] = specificTimeConfig["FwTrafficMeter.TimeHH"]
        elseif (tonumber (specificTimeConfig["FwTrafficMeter.TimeHH"]) > 12 and tonumber (specificTimeConfig["FwTrafficMeter.TimeHH"]) <= 23) then
            fwTrafficConfig["FwTrafficMeter.startMeridian"] = "1"
            fwTrafficConfig["FwTrafficMeter.spHour"] = tonumber (specificTimeConfig["FwTrafficMeter.TimeHH"]) - 12
        elseif(tonumber (specificTimeConfig["FwTrafficMeter.TimeHH"]) > 0 and tonumber (specificTimeConfig["FwTrafficMeter.TimeHH"]) <= 11) then
            fwTrafficConfig["FwTrafficMeter.spHour"] = specificTimeConfig["FwTrafficMeter.TimeHH"]
            fwTrafficConfig["FwTrafficMeter.startMeridian"] = "0"
        end
        fwTrafficConfig["FwTrafficMeter.spMinute"] = specificTimeConfig["FwTrafficMeter.TimeMM"]
    end
    return fwTrafficConfig
end

-- NATServices Delete
function fwView.deleteNatRules (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true

        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["OneToOneNat._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "CONFIG_FAILED"
	end

end

function fwView.natSeviceConfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true

    valid = fw.config(inputTable, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "CONFIG_FAILED"
    end
end

function fwView.deleteNatServices (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true

        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["OneToOneNatService._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "CONFIG_FAILED"
	end

end

-- Groups Configuration
function fwView.disableBlockGroups (inputTable, rowid, operation)
        -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                return "ACCESS_DENIED", "ADMIN_REQD"
        end

        db.beginTransaction() --begin transaction
        local valid = false
        local inTable = {}
        local group = 0
	group = tonumber(db.getAttribute("BlockSites","_ROWID_","1","KeywordBlockingTo"))

	local blockRow = db.getRow("BlockSites","_ROWID_","1")
	local oldConfig = fwView.groupConfigGet(blockRow)
	if (inputTable["BlockSites.group1"] == "1" and oldConfig["BlockSites.group1"] == "1") then
            group = group - 1
        end
        if (inputTable["BlockSites.group2"] == "1" and oldConfig["BlockSites.group2"] == "1") then
            group = group - 2
        end
        if (inputTable["BlockSites.group3"] == "1" and oldConfig["BlockSites.group3"] == "1") then
            group = group - 4
        end
        if (inputTable["BlockSites.group4"] == "1" and oldConfig["BlockSites.group4"] == "1") then
            group = group - 8
        end
        if (inputTable["BlockSites.group5"] == "1" and oldConfig["BlockSites.group5"] == "1") then
            group = group - 16
        end
        if (inputTable["BlockSites.group6"] == "1" and oldConfig["BlockSites.group6"] == "1") then
            group = group - 32
        end
        if (inputTable["BlockSites.group7"] == "1" and oldConfig["BlockSites.group7"] == "1") then
            group = group - 64
        end
        if (inputTable["BlockSites.group8"] == "1" and oldConfig["BlockSites.group8"] == "1") then
            group = group - 128
        end

        inTable["BlockSites.KeywordBlockingTo"] = group

        valid = fw.config (inTable,  rowid, operation)

        -- return
        if (valid) then
                db.commitTransaction()
                return "OK", "STATUS_OK"
        else
                db.rollback()
                return "ERROR", "DISABLE_GROUPS_FAILED"
        end
end


-- Groups Configuration
function fwView.enableBlockGroups (inputTable, rowid, operation)
        -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                return "ACCESS_DENIED", "ADMIN_REQD"
        end

        db.beginTransaction() --begin transaction
        local valid = false
        local inTable = {}
        local group = 0
	local blockRow = db.getRow("BlockSites","_ROWID_","1")
	local oldConfig = fwView.groupConfigGet(blockRow)
--Â´	group = tonumber(db.getAttribute("BlockSites","_ROWID_","1","KeywordBlockingTo"))
        if (inputTable["BlockSites.group1"] == "1" or oldConfig["BlockSites.group1"] == "1") then
            group = group + 1
        end
        if (inputTable["BlockSites.group2"] == "1" or oldConfig["BlockSites.group2"] == "1") then
            group = group + 2
        end
        if (inputTable["BlockSites.group3"] == "1" or oldConfig["BlockSites.group3"] == "1") then
            group = group + 4
        end
        if (inputTable["BlockSites.group4"] == "1" or oldConfig["BlockSites.group4"] == "1") then
            group = group + 8
        end
        if (inputTable["BlockSites.group5"] == "1" or oldConfig["BlockSites.group5"] == "1") then
            group = group + 16
        end
        if (inputTable["BlockSites.group6"] == "1" or oldConfig["BlockSites.group6"] == "1") then
            group = group + 32
        end
        if (inputTable["BlockSites.group7"] == "1" or oldConfig["BlockSites.group7"] == "1") then
            group = group + 64
        end
        if (inputTable["BlockSites.group8"] == "1" or oldConfig["BlockSites.group8"] == "1") then
            group = group + 128
        end

        inTable["BlockSites.KeywordBlockingTo"] = group

        valid = fw.config (inTable,  rowid, operation)

	-- return
        if (valid) then
                db.commitTransaction()
                return "OK", "STATUS_OK"
        else
                db.rollback()
                return "ERROR", "ENABLE_GROUPS_FAILED"
        end
end


-- Schedules Configuration Get
function fwView.groupConfigGet (inputTable)

    local inTable = {}
    local group = tonumber(inputTable["BlockSites.KeywordBlockingTo"])

    if (group ~= nil) then
        if (group >= 128) then
           inTable["BlockSites.group8"] = "1"
           group = group - 128
        else
           inTable["BlockSites.group8"] = "0"
        end

        if (group >= 64) then
            inTable["BlockSites.group7"] = "1"
            group = group - 64
        else
            inTable["BlockSites.group7"] = "0"
        end

        if (group >= 32) then
            inTable["BlockSites.group6"] = "1"
            group = group - 32
        else
            inTable["BlockSites.group6"] = "0"
        end

        if (group >= 16) then
            inTable["BlockSites.group5"] = "1"
            group = group - 16
        else
            inTable["BlockSites.group5"] = "0"
        end

        if (group >= 8) then
            inTable["BlockSites.group4"] = "1"
            group = group - 8
        else
            inTable["BlockSites.group4"] = "0"
        end

        if (group >= 4) then
            inTable["BlockSites.group3"] = "1"
            group = group - 4
        else
            inTable["BlockSites.group3"] = "0"
        end

        if (group >= 2) then
            inTable["BlockSites.group2"] = "1"
            group = group - 2
        else
            inTable["BlockSites.group2"] = "0"
        end

        if (group >= 1) then
            inTable["BlockSites.group1"] = "1"
            group = group - 1
        else
            inTable["BlockSites.group1"] = "0"
        end
    end

    return inTable
end


--[[
/******************************************************************************
*
* fwView.saveBinding - Wrapper to store save binding values.
*
* RETURNS: "OK" or 'ERROR'
*/
]]--

function fwView.saveBinding (rows,tableName)

    -- Declaring flag

    local valid = false

    -- iterate over all the rowIds received from htm page
    db.beginTransaction ()

    for k,v in pairs (rows) do

        -- Get the rows from DhcpfixedIpAddress table based on the rowids received from htm page
        -- and then store the values in the IPMACBinding table

        DBTable = tableName
        local dhcpFixedIpRow = db.getRow (DBTable, "_ROWID_", v)
        local macAddr = {}
        if(DBTable == "DhcpfixedIpAddress") then
            macAddr = db.getRowWhere("IPMACBinding","SourceMACAddress='" .. dhcpFixedIpRow["DhcpfixedIpAddress.MacAddr"] .. "'")
        elseif(DBTable == "AvailableLanHost") then
            macAddr = db.getRowWhere("IPMACBinding","SourceMACAddress='" .. dhcpFixedIpRow["AvailableLanHost.MacAddr"] .. "'")
        end
        if(macAddr == nil) then
             valid = fwView.storeIpMacBinding (dhcpFixedIpRow,DBTable)
        end
        --if(valid == false) then
            --break
        --end
    end
    -- Commit Transaction and return "OK"
    if(valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback ()
        return "ERROR", "FW_IPMACBINDING_CONFIG_FAILED"
    end
end

--[[
/******************************************************************************
*
* fwView.storeIpMacBinding - Stores Save Binding values.
*
* RETURNS: "OK" or 'ERROR'
*/
]]--

function fwView.storeIpMacBinding (inputTable,table)

    local macBindingTable = {}
    DBTable = "IPMACBinding"
    if(table == "DhcpfixedIpAddress") then
	    macBindingTable["IPMACBinding.Name"] = inputTable ["DhcpfixedIpAddress.IpMacName"]
	    macBindingTable["IPMACBinding.IPAddress"] = inputTable ["DhcpfixedIpAddress.IpAddr"]
	    macBindingTable["IPMACBinding.SourceMACAddress"] = inputTable ["DhcpfixedIpAddress.MacAddr"]
	    macBindingTable["IPMACBinding.LogStatus"] = "0"
    end
    if(table == "AvailableLanHost") then
	    macBindingTable["IPMACBinding.Name"] = inputTable ["AvailableLanHost.Name"]
	    macBindingTable["IPMACBinding.IPAddress"] = inputTable ["AvailableLanHost.IPAddr"]
	    macBindingTable["IPMACBinding.SourceMACAddress"] = inputTable ["AvailableLanHost.MacAddr"]
	    macBindingTable["IPMACBinding.LogStatus"] = "0"
    end

    -- Begin Transaction
    return fw.config (macBindingTable,"-1", "add" )
end

--[[
/******************************************************************************
*
* fwView.sesLimitConfig - Stores session limit values
*
* RETURNS: "OK" or 'ERROR'
*/
]]--

function fwView.sesLimitConfig (inputTable, rowid, operation)

    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true

    -- Check for edit operation. If it is edit operation then get the row from the back end
    -- and update that row and then save.

    local sessionLimitRow = {}
    DBTable = "SessionLimit"

    if(operation == "edit") then

        -- Get the row from SessionLimittable which needs to be updated

        sessionLimitRow = db.getRow (DBTable, "_ROWID_", rowid)
        if(sessionLimitRow ~= nil) then
            sessionLimitRow["SessionLimit.SessLimitEnable"] = inputTable["SessionLimit.SessLimitEnable"]
            if(inputTable["SessionLimit.SessLimitEnable"] ~= nil and inputTable["SessionLimit.SessLimitEnable"] == "1") then
                if(inputTable["SessionLimit.ConnLimitType"] ~= nil) then
                    sessionLimitRow["SessionLimit.ConnLimitType"] = inputTable["SessionLimit.ConnLimitType"]
                end
                if(inputTable["SessionLimit.UserConnLimit"] ~= nil) then
                    sessionLimitRow["SessionLimit.UserConnLimit"] = inputTable["SessionLimit.UserConnLimit"]
                end
		if(inputTable["SessionLimit.ConnectionTimeType"] ~= nil) then
	            sessionLimitRow["SessionLimit.ConnectionTimeType"] = inputTable["SessionLimit.ConnectionTimeType"]
	            if(inputTable["SessionLimit.ConnectionTimeType"] == "0") then
	                sessionLimitRow["SessionLimit.BlocKNewSession"] = inputTable["SessionLimit.BlocKNewSession"]
	                if(inputTable["SessionLimit.BlocKNewSession"] == "1") then
	                    sessionLimitRow["SessionLimit.TimePeriod"] = inputTable["SessionLimit.TimePeriod1"]
	                else
	                    sessionLimitRow["SessionLimit.TimePeriod"] = inputTable["SessionLimit.TimePeriod2"]
	                end
	            end
	        end	    
            end

        end
    else
        sessionLimitRow["SessionLimit.SessLimitEnable"] = inputTable["SessionLimit.SessLimitEnable"]
        if(inputTable["SessionLimit.SessLimitEnable"] ~= nil and inputTable["SessionLimit.SessLimitEnable"] == "1") then
            if(inputTable["SessionLimit.ConnLimitType"] ~= nil) then
                sessionLimitRow["SessionLimit.ConnLimitType"] = inputTable["SessionLimit.ConnLimitType"]
            else
                sessionLimitRow["SessionLimit.ConnLimitType"] = "1"
            end
            if(inputTable["SessionLimit.UserConnLimit"] ~= nil) then
                sessionLimitRow["SessionLimit.UserConnLimit"] = inputTable["SessionLimit.UserConnLimit"]
            else
                sessionLimitRow["SessionLimit.UserConnLimit"] = "1"
            end
           if(inputTable["SessionLimit.ConnectionTimeType"] ~= nil) then
	    sessionLimitRow["SessionLimit.ConnectionTimeType"] = inputTable["SessionLimit.ConnectionTimeType"]
	    if(inputTable["SessionLimit.ConnectionTimeType"] == "0") then
	        sessionLimitRow["SessionLimit.BlocKNewSession"] = inputTable["SessionLimit.BlocKNewSession"]
	        if(inputTable["SessionLimit.BlocKNewSession"] == "1") then
	            sessionLimitRow["SessionLimit.TimePeriod"] = inputTable["SessionLimit.TimePeriod1"]
	        else
	            sessionLimitRow["SessionLimit.TimePeriod"] = inputTable["SessionLimit.TimePeriod2"]
	        end
	    end
	end

        end
    end

    valid = fw.config(sessionLimitRow, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "FW_SESSIONS_LIMIT_CONFIG_FAILED"
    end
end

-- Move Firewall rules
function fwView.moveRules_NG (rowId, operation, ruleType, position)
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = false
    local i = 0
    local j = 0
    local n = 0
    local rulesPriority = nil
    local swapRow = nil
    local currentRow = nil

    -- grab the all firewall rules based on ruleType
    local where = "RuleType = 'SECURE_INSECURE'"
    where = where .. " or RuleType = 'INSECURE_SECURE'"
    where = where .. " or RuleType = 'PUBLIC_SECURE'"
    where = where .. " or RuleType = 'SECURE_PUBLIC'"
    where = where .. " or RuleType = 'INSECURE_PUBLIC'"
    where = where .. " or RuleType = 'PUBLIC_INSECURE'"
    local fwRules = db.getRowsWhere ("FirewallRules", where)
    local firewallRules = {}
    local rulesPriority = db.getAttribute ("fwPriorityMap", "RuleType", "ALL", "PriorityStr") or ''
    if (rulesPriority ~= nil and rulesPriority ~= "") then
	    local ruleIds = util.split (rulesPriority, ",")
		if (ruleIds ~= false and #ruleIds ~= 0) then
		    local idx = 0
			for k,v in pairs(ruleIds) do
			    for m,n in pairs (fwRules) do
				   if (n["FirewallRules.PriorityId"] == v) then
				       idx = idx + 1
				       firewallRules[idx] = n
				       break
				   end
			    end
			end
		end
	end

    for k,v in pairs(firewallRules) do
        i = i + 1
        local row = firewallRules[i]
        -- checking table rowId with given rowId
        if(row["FirewallRules._ROWID_"] == rowId) then
            break
        end
    end

    for k,v in pairs(firewallRules) do
        j = j + 1
        local row = firewallRules[j]
        -- checking table rowId with given rowId
        if(row["FirewallRules.RuleType"] == ruleType) then
            n = n + 1
            if (n == tonumber(position)) then
                break
            end
        end
    end
    -- taking before,after and current rows

    if (operation == "move") then
        if (i > j) then
            tmpRow = firewallRules[i]
            for n=i,j+1,-1 do
                firewallRules[n]=firewallRules[n-1]
            end
            firewallRules [j] = tmpRow 
        elseif (i < j) then
            tmpRow = firewallRules[i]
            for n=i,j-1,1 do
                firewallRules[n]=firewallRules[n+1]
            end
            firewallRules [j] = tmpRow 
        end
    end

    local rulesPriConfig = db.getRow ("fwPriorityMap", "RuleType", "ALL")
    if (rulesPriConfig ~= nil) then
        rulesPriority = rulesPriConfig["fwPriorityMap.PriorityStr"] or ''
    end
    j = 0
    if(rulesPriority ~= nil or rulesPriority ~= "") then
        local ruleIds = util.split(rulesPriority, ",")
        if(ruleIds ~= false) then
            for kk,vv in pairs(ruleIds) do
                j = j + 1
                if (firewallRules[j] ~= nil) then
                    ruleIds[kk] = firewallRules [j]["FirewallRules.PriorityId"]
                end
            end
            rulesPriority = util.join (ruleIds, ",")
        end
    end

    rulesPriConfig["fwPriorityMap.PriorityStr"] = rulesPriority
    DBTable = "fwPriorityMap"
    valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap._ROWID_"], "edit")

	-- return
	if (valid) then
		db.commitTransaction(true)
        os.execute ("/pfrm2.0/bin/snmpTrapConfig 42 Major IPv4 Custom Firewall Rules sequence Modified")
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "MOVE_OPERATION_FAILED"
    end
end

-- IP Groups Configuration
function fwView.ipGroupsConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local START_IP_GROUP_ID = "51"
    local ipGroupTBL = {}

    if (operation == "add") then
        if(db.existsRow("LanGroup", "GroupName", inputTable["ipGroup.ipGroupName"])) then
            db.rollback()
            return "ERROR", "Default IP Group with same name Exists"
        end
        local ipGroupTbl = db.getTable("ipGroup")
        if(#ipGroupTbl ~= nil and #ipGroupTbl ~= 0) then
            if(db.existsRowWhere ("ipGroup", "ipGroupName='" .. inputTable["ipGroup.ipGroupName"] .. "'")) then
                db.rollback()
                return "ERROR", "IP_GROUP_NAME_EXISTED"
            end
            local maxIPGroupId = db.columnMaxValue("ipGroup", "ipGroupId")    
            inputTable["ipGroup.ipGroupId"] = tonumber(maxIPGroupId) + 1
        else
            inputTable["ipGroup.ipGroupId"] = START_IP_GROUP_ID
        end
        valid = fw.config(inputTable, "-1", "add")
        
     elseif (operation == "edit") then
        if(db.existsRow("LanGroup", "GroupName", inputTable["ipGroup.ipGroupName"])) then
            db.rollback()
            return "ERROR", "Default IP Group with same name exists"
        end
        local ipGroupid = db.getAttribute("ipGroup", "_ROWID_", rowid, "ipGroupId")
        local oldIPgroupName = db.getAttribute("ipGroup", "_ROWID_", rowid, "ipGroupName")
        local whereCond = "DestinationAddressType='" .. ipGroupid .. "' OR SourceAddressType='" .. ipGroupid .. "'"
        if(db.existsRowWhere("FirewallRules", whereCond)) then
            db.rollback()
            return "ERROR", "IP_GROUP_IN_USE"
        end
        local ipGrouptbl = {}
        local ipGroupIPtbl = {}
        ipGrouptbl["ipGroup.ipGroupId"] = ipGroupid
        ipGrouptbl["ipGroup.ipGroupName"] = inputTable["ipGroup.ipGroupName"]
        ipGrouptbl["ipGroup.ipGroupType"] = inputTable["ipGroup.ipGroupType"]
        valid = fw.config(ipGrouptbl, rowid, "edit")
        if(valid) then
            local tmpipGroupIpRows = db.getRowsWhere("ipGroupIp", "ipGroup='" .. oldIPgroupName .. "'")
            if(#tmpipGroupIpRows ~= 0 and #tmpipGroupIpRows ~= nil) then
                local m = 0
                for k,v in pairs(tmpipGroupIpRows) do
                    m = m + 1
                    local row = tmpipGroupIpRows[m]
                    ipGroupIPtbl["ipGroupIp.ipGroup"] = inputTable["ipGroup.ipGroupName"]
                    valid = db.update("ipGroupIp", ipGroupIPtbl, row["ipGroupIp._ROWID_"])
                end
            end 
	end
     end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
	return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IP_GROUP_CONFIG_FAILED"
    end
end

function fwView.deleteIpGroups(rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    --local rowsTemp = {}
    -- delete routes
    for k,v in pairs(rowIds) do
        local ipGroupID = db.getAttribute("ipGroup", "_ROWID_", v, "ipGroupId")
        local ipGroupName = db.getAttribute("ipGroup", "_ROWID_", v, "ipGroupName")
        local whereCond = "DestinationAddressType='" .. ipGroupID .. "' OR SourceAddressType='" .. ipGroupID .. "'"
        if(db.existsRowWhere("FirewallRules", whereCond)) then
            db.rollback()
            return "ERROR", "IP_GROUP_IN_USE"
        end
        valid = db.deleteRow("ipGroup", "_ROWID_", v)
        
        if(valid) then
            if(db.existsRowWhere("ipGroupIp", "ipGroup='" .. ipGroupName .. "'")) then
            valid = db.deleteRowWhere("ipGroupIp", "ipGroup='" .. ipGroupName .. "'")
            end
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IP_GROUP_DELETE_FAILED"
    end
end

-- IP Group IP Add
function fwView.ipGroupIP_Add (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    if (operation == "add") then
        local ipGroupIPTbl = db.getTable("ipGroupIp")
        if(#ipGroupIPTbl ~= nil) then
            if(db.existsRowWhere ("ipGroupIp", "ipAddress='" .. inputTable["ipGroupIp.ipAddress"] .. "'")) then
                db.rollback()
                return "ERROR", "IP_GROUP_IP_EXISTED"
            end
        end
        valid = fw.config(inputTable, "-1", "add")
     elseif (operation == "edit") then
        return nil
     elseif (operation == "delete") then
        return nil
     end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
	return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IP_GROUP_IP_ADD_FAILED"
    end
end

--Delete IP Group IPs
function fwView.deleteIpGroupIP(rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    --local rowsTemp = {}
    -- delete groupIP
    for k,v in pairs(rowIds) do
        valid = db.deleteRow("ipGroupIp", "_ROWID_", v)
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IP_GROUP_IP_DELETE_FAILED"
    end
end

-- Firewall LAN Traffic Meter Configuration
--[[function fwView.trafficMeterLanConfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = false

    if(operation == "add") then
        if(db.existsRowWhere("FwTrafficMeterLan", "LanIP='" .. inputTable["FwTrafficMeterLan.LanIP"] .. "'")) then
            return "ERROR", "LAN_IP_ALREADY_EXISTS"
        end
    end

    local trafficMetrLanConfig = {}
    trafficMetrLanConfig["FwTrafficMeterLan.LanIP"] = inputTable["FwTrafficMeterLan.LanIP"]
    trafficMetrLanConfig["FwTrafficMeterLan.TrafficLimitType"] = inputTable["FwTrafficMeterLan.TrafficLimitType"]
    trafficMetrLanConfig["FwTrafficMeterLan.TrafficMonthlyLimit"] = inputTable["FwTrafficMeterLan.TrafficMonthlyLimit"]
    if(inputTable["FwTrafficMeterLan.RestartCounter"] ~= nil) then
        trafficMetrLanConfig["FwTrafficMeterLan.RestartCounter"] = inputTable["FwTrafficMeterLan.RestartCounter"]
    else
        trafficMetrLanConfig["FwTrafficMeterLan.RestartCounter"] = "0"
    end

--Converting the time into 24 hours format.
    if(trafficMetrLanConfig["FwTrafficMeterLan.RestartCounter"] == "1") then
        if (inputTable["FwTrafficMeterLan.startMeridian"] == "1") then
            if (tonumber(inputTable["FwTrafficMeterLan.spHour"]) == 12) then
                trafficMetrLanConfig["FwTrafficMeterLan.TimeHH"] = "12"
            else
                trafficMetrLanConfig["FwTrafficMeterLan.TimeHH"] = inputTable["FwTrafficMeterLan.spHour"] + 12
            end
            trafficMetrLanConfig["FwTrafficMeterLan.TimeMM"] = inputTable["FwTrafficMeterLan.spMinute"]
        else
            if (tonumber(inputTable["FwTrafficMeterLan.spHour"]) == 12) then
                trafficMetrLanConfig["FwTrafficMeterLan.TimeHH"] = "00"
            else
                trafficMetrLanConfig["FwTrafficMeterLan.TimeHH"] = inputTable["FwTrafficMeterLan.spHour"]
            end
            trafficMetrLanConfig["FwTrafficMeterLan.TimeMM"] = inputTable["FwTrafficMeterLan.spMinute"]
        end
    else
        trafficMetrLanConfig["FwTrafficMeterLan.TimeHH"] = "00"
        trafficMetrLanConfig["FwTrafficMeterLan.TimeMM"] = "00"
        trafficMetrLanConfig["FwTrafficMeterLan.DayOfMonth"] = "1"
    end

    trafficMetrLanConfig["FwTrafficMeterLan.DayOfMonth"] = inputTable["FwTrafficMeterLan.DayOfMonth"]
    trafficMetrLanConfig["FwTrafficMeterLan.SendEmailReport"] = inputTable["FwTrafficMeterLan.SendEmailReport"]
    if(inputTable["FwTrafficMeterLan.BlockTraffic"] ~= nil) then
        trafficMetrLanConfig["FwTrafficMeterLan.BlockTraffic"] = inputTable["FwTrafficMeterLan.BlockTraffic"]
    else
        trafficMetrLanConfig["FwTrafficMeterLan.BlockTraffic"] = "0"
    end
    trafficMetrLanConfig["FwTrafficMeterLan.SendEmailAlert"] = "0" --inputTable["FwTrafficMeterLan.SendEmailAlert"]

    util.appendDebugOut("Traffic Meter Config"..util.tableToString (trafficMetrLanConfig))

    -- valid = fw.config(trafficMetrLanConfig, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "FW_TM_CONFIG_FAILED"
    end
end

--Lan Traffic Meter specific Time getting
function fwView.trafficMtrLan_ConfigGet (rowId)
    local lanSpecificTimeConfig = db.getRow("FwTrafficMeterLan","_ROWID_", rowId)
    local fwTrafficLanConfig = {}

    if (lanSpecificTimeConfig == nil) then
        fwTrafficLanConfig["FwTrafficMeterLan.TimeHH"] = "1"
        fwTrafficLanConfig["FwTrafficMeterLan.TimeMM"] = "1"
        fwTrafficLanConfig["FwTrafficMeterLan.startMeridian"] = "0"
    else
        if (tonumber (lanSpecificTimeConfig["FwTrafficMeterLan.TimeHH"]) == 0) then
            fwTrafficLanConfig["FwTrafficMeterLan.startMeridian"] = "0"
            fwTrafficLanConfig["FwTrafficMeterLan.spHour"] = "12"
        elseif (tonumber (lanSpecificTimeConfig["FwTrafficMeterLan.TimeHH"]) >= 12 and tonumber (lanSpecificTimeConfig["FwTrafficMeterLan.TimeHH"]) <= 23) then
            fwTrafficLanConfig["FwTrafficMeterLan.startMeridian"] = "1"
            fwTrafficLanConfig["FwTrafficMeterLan.spHour"] = tonumber (lanSpecificTimeConfig["FwTrafficMeterLan.TimeHH"]) - 12
        elseif(tonumber (lanSpecificTimeConfig["FwTrafficMeterLan.TimeHH"]) > 0 and tonumber (lanSpecificTimeConfig["FwTrafficMeterLan.TimeHH"]) <= 11) then
            fwTrafficLanConfig["FwTrafficMeterLan.spHour"] = lanSpecificTimeConfig["FwTrafficMeterLan.TimeHH"]
            fwTrafficLanConfig["FwTrafficMeterLan.startMeridian"] = "0"
        end
        fwTrafficLanConfig["FwTrafficMeterLan.spMinute"] = lanSpecificTimeConfig["FwTrafficMeterLan.TimeMM"]
    end
    return fwTrafficLanConfig
end]]--

-- Delete LAN TrafficMeter rules
function fwView.deleteLanTrafficMeter (rowIds)
    if (ACCESS_LEVEL ~= 0) then
       return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"
    
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["FwTrafficMeterLan._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "LAN_TRAFFIC_METER_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "LAN_TRAFFIC_METER_DELETED"
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", errStr
    else
        db.rollback()
        return "ERROR", errStr
    end
end

-- QOS Profiles Configuration
function fwView.qosProfileConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    if(operation == "add") then
        local rows = db.getTable("qosProfile") or {}
        if (#rows >= 124) then
            db.rollback()
            return "ERROR", "MAX_LMT_RCHD"
        end
        local profName = inputTable["qosProfile.ProfileName"]
        if(db.existsRowWhere("qosProfile", "ProfileName = '" .. profName .. "'")) then
            db.rollback()
            return "ERROR", "QOS_PROFILE_NAME_ALREADY_EXISTS"
        end
        maxId = db.columnMaxValueWhere("qosProfile", "xmlId", "ProfileName IS NOT NULL")
        if (maxId == "" or maxId == nil) then
            maxId = -1
        end
        inputTable["qosProfile.xmlId"] = tonumber(maxId) + 1
    end
    if(operation == "edit") then
        local profName = inputTable["qosProfile.ProfileName"]
        if(db.existsRowWhere("FirewallRules", "QosProfileName = '" .. profName .. "' AND Status=1")) then
            db.rollback()
            return "ERROR", "QOS_PROFILE_IN_USE"
        end
    end

    -- config QOS Profile
    valid = fw.config(inputTable, rowid, operation)

     -- return
     if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
     else
        db.rollback()
        return "ERROR", "QOS_PROFILE_CONFIG_FAILED"
     end
end

-- QOS Profiles Delete
function fwView.deleteQosProfiles (rowIds)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    if (valid) then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["Services._ROWID_"] = v;
            local profName = db.getAttribute (DBTable, "_ROWID_", v, "ProfileName")
            if (profName ~= nil) then
                if(db.existsRowWhere("FirewallRules", "QosProfileName = '" .. profName .. "'")) then
                    db.rollback()
                    return "ERROR", "QOS_PROFILE(".. profName ..")_IN_USE"
                end
            end
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "NETQOS_PROFILE_DELETE_FAILED"
                break
            end
        end
    end
   
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", errStr
    end
end

-- This function will set status of Network Qos

function fwView.qosProfileStatusConfig(inputTable,rowid,operation)
-- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    valid = fw.config(inputTable, rowid, operation)
    
    -- return
    if (valid) then
    	db.commitTransaction(true)
        -- hardware offload /sppe update
        fwView.hwOffloadUpdate()
    	return "OK", "STATUS_OK"
    else
    	db.rollback()        
    end
end

function fwView.modifyQosProfile (rowIds, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    if (operation == "enable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["NetworkQosProfile.QosProfileStatus"] = "1";
            valid = db.update("NetworkQosProfile", inTable, v)
            if (valid == false) then
                errStr = "NETQOS_PROFILE_ENABLED_FAILED"
                break
            end
        end

        if (valid) then
            errStr = "NETQOS_PROFILE_ENABLED"       
        end
    elseif (operation == "disable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["NetworkQosProfile.QosProfileStatus"] = "0";
            valid = db.update("NetworkQosProfile", inTable, v)
            if (valid == false) then
                errStr = "NETQOS_PROFILE_DISABLED_FAILED"
                break
            end
        end
        if (valid) then
            errStr = "NETQOS_PROFILE_DISABLED"
        end
    else
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["NetworkQosProfile._ROWID_"] = v;
            valid = db.delete("NetworkQosProfile", inTable)
            if (valid == false) then
                errStr = "NETQOS_PROFILE_DELETE_FAILED"
                break
            end
        end
        if (valid) then
            errStr = "NETQOS_PROFILE_DELETED"
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", errStr
    else
        db.rollback()
        return "ERROR", errStr
    end
end
function fwView.profileConfig(inputTable,rowid,operation)
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    if(operation == "add") then
        local rows = db.getTable("NetworkQosProfile") or {}
        if (#rows >= 16) then
            return "ERROR", "MAX_LMT_RCHD"
        end
    end

    if (inputTable["NetworkQosProfile.Hosts"] == "1" or inputTable["NetworkQosProfile.Hosts"] == "3") then
        local value = ''
        local field = ''
        if (inputTable["NetworkQosProfile.Hosts"] == "1") then
            value = inputTable["NetworkQosProfile.StartIp"]
            field = "StartIp"
        elseif (inputTable["NetworkQosProfile.Hosts"] == "3") then
            value = inputTable["NetworkQosProfile.GroupName"]
            field = "GroupName"
        end
        if(operation == "add") then
            local serName = inputTable["NetworkQosProfile.ServiceName"]
	        if(db.existsRowWhere("NetworkQosProfile", "QosType = '" .. inputTable["NetworkQosProfile.QosType"] .. "' AND InterfaceName = '" .. inputTable["NetworkQosProfile.InterfaceName"] .. "' AND ServiceName = '" .. serName .. "' AND "..field.." = '"..value.."'")) then
                return "ERROR", "NET_QOS_ALREADY_EXISTS"
            end
        else
            local serName = inputTable["NetworkQosProfile.ServiceName"]
            if(db.existsRowWhere("NetworkQosProfile", "QosType = '" .. inputTable["NetworkQosProfile.QosType"] .. "' AND InterfaceName = '" .. inputTable["NetworkQosProfile.InterfaceName"] .. "' AND ServiceName = '" .. serName .. "' AND _ROWID_ != '" .. rowid .. "' AND "..field.." = '"..value.."'")) then
                return "ERROR", "NET_QOS_ALREADY_EXISTS"
            end
        end
        if (inputTable["NetworkQosProfile.Hosts"] == "1") then
            local qosRows = db.getRowsWhere("NetworkQosProfile","Hosts = 2 AND _ROWID_ != " .. rowid)
            local i = 0
            for k,v in pairs(qosRows) do
                local status = ''
                i = i + 1
                local row = qosRows[i]
                if(row["NetworkQosProfile.StartIp"] ~= nil and row["NetworkQosProfile.StartIp"] ~= "" and
                   row["NetworkQosProfile.EndIp"] ~= nil and row["NetworkQosProfile.EndIp"] ~= "") then
                    status = ifDevLib.ipCheck (row["NetworkQosProfile.StartIp"],row["NetworkQosProfile.EndIp"],inputTable["NetworkQosProfile.StartIp"])
                end
                if(operation == "add") then
                    local serName = inputTable["NetworkQosProfile.ServiceName"]
	                if(db.existsRowWhere("NetworkQosProfile", "QosType = '" .. inputTable["NetworkQosProfile.QosType"] .. "' AND InterfaceName = '" .. inputTable["NetworkQosProfile.InterfaceName"] .. "' AND ServiceName = '" .. serName .. "'") and (status == "OK")) then
                        return "ERROR", "NET_QOS_ALREADY_EXISTS"
                    end
                else
                    local serName = inputTable["NetworkQosProfile.ServiceName"]
                    if(db.existsRowWhere("NetworkQosProfile", "QosType = '" .. inputTable["NetworkQosProfile.QosType"] .. "' AND InterfaceName = '" .. inputTable["NetworkQosProfile.InterfaceName"] .. "' AND ServiceName = '" .. serName .. "' AND _ROWID_ != '" .. rowid .. "'") and (status == "OK")) then
                        return "ERROR", "NET_QOS_ALREADY_EXISTS"
                    end
                end
            end
        end
    elseif (inputTable["NetworkQosProfile.Hosts"] == "2") then
        local qosRows = db.getRowsWhere("NetworkQosProfile","(Hosts = 1 OR Hosts = 2) AND _ROWID_ != " .. rowid)
        local i = 0
        for k,v in pairs(qosRows) do
            local status1 = ''
            local status2 = ''
            i = i + 1
            local row = qosRows[i]
            if(row["NetworkQosProfile.StartIp"] ~= nil and row["NetworkQosProfile.StartIp"] ~= "") then
                status1 = ifDevLib.ipCheck (inputTable["NetworkQosProfile.StartIp"],inputTable["NetworkQosProfile.EndIp"],row["NetworkQosProfile.StartIp"])
            end
            if(row["NetworkQosProfile.EndIp"] ~= nil and row["NetworkQosProfile.EndIp"] ~= "") then
                status2 = ifDevLib.ipCheck (inputTable["NetworkQosProfile.StartIp"],inputTable["NetworkQosProfile.EndIp"],row["NetworkQosProfile.EndIp"])
            end
            if(operation == "add") then
                local serName = inputTable["NetworkQosProfile.ServiceName"]
	            if(db.existsRowWhere("NetworkQosProfile", "QosType = '" .. inputTable["NetworkQosProfile.QosType"] .. "' AND InterfaceName = '" .. inputTable["NetworkQosProfile.InterfaceName"] .. "' AND ServiceName = '" .. serName .. "'") and (status1 == "OK" or status2 == "OK")) then
                    return "ERROR", "NET_QOS_ALREADY_EXISTS"
                end
            else
                local serName = inputTable["NetworkQosProfile.ServiceName"]
                if(db.existsRowWhere("NetworkQosProfile", "QosType = '" .. inputTable["NetworkQosProfile.QosType"] .. "' AND InterfaceName = '" .. inputTable["NetworkQosProfile.InterfaceName"] .. "' AND ServiceName = '" .. serName .. "' AND _ROWID_ != '" .. rowid .. "'") and (status1 == "OK" or status2 == "OK")) then
                    return "ERROR", "NET_QOS_ALREADY_EXISTS"
                end
            end
        end
    end


	db.beginTransaction() --begin transaction
	local valid = false

    qosRow = db.getRow("NetworkQos", "_ROWID_", "1")

    if(qosRow["NetworkQos.Enable"] == "1") then
      if(inputTable["NetworkQosProfile.QosType"] == qosRow["NetworkQos.qosType"]) then
          inputTable["NetworkQosProfile.QosProfileStatus"] = "1"           
      else
          inputTable["NetworkQosProfile.QosProfileStatus"] = "0"
      end
    else
         inputTable["NetworkQosProfile.QosProfileStatus"] = "0"
    end



	-- config Service
	valid = fw.config(inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NET_QOS_CONFIG_FAIL"
	end
end


-- Firewall Traffic Meter Configuration
function fwView.trafficMeterLanConfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true

    if(operation == "add") then
        if(db.existsRowWhere("FwTrafficMeterLan", "LanIP='" .. inputTable["FwTrafficMeterLan.LanIP"] .. "'")) then
            db.rollback()
            return "ERROR", "LAN_IP_ALREADY_EXISTS"
        end
    end

    local lanStatConfig = {}

    local trafficMetrConfig = {}
    trafficMetrConfig["FwTrafficMeterLan.LanIP"] = inputTable["FwTrafficMeterLan.LanIP"]
    trafficMetrConfig["FwTrafficMeterLan.TrafficMeterStatus"] = inputTable["FwTrafficMeterLan.TrafficMeterStatus"]
    trafficMetrConfig["FwTrafficMeterLan.TrafficLimitType"] = inputTable["FwTrafficMeterLan.TrafficLimitType"]
    trafficMetrConfig["FwTrafficMeterLan.TrafficMonthlyLimit"] = inputTable["FwTrafficMeterLan.TrafficMonthlyLimit"]
    trafficMetrConfig["FwTrafficMeterLan.IncreaseLimitStatus"] = inputTable["FwTrafficMeterLan.IncreaseLimitStatus"]
    trafficMetrConfig["FwTrafficMeterLan.IncreaseLimitBy"] = inputTable["FwTrafficMeterLan.IncreaseLimitBy"]
    trafficMetrConfig["FwTrafficMeterLan.RestartCounter"] = inputTable["FwTrafficMeterLan.RestartCounter"]

--Converting the time into 24 hours format.
    if(trafficMetrConfig["FwTrafficMeterLan.RestartCounter"] == "1") then
        if (inputTable["FwTrafficMeterLan.startMeridian"] == "1") then
            if (tonumber(inputTable["FwTrafficMeterLan.spHour"]) == 12) then
                trafficMetrConfig["FwTrafficMeterLan.TimeHH"] = "12"
            else
                trafficMetrConfig["FwTrafficMeterLan.TimeHH"] = inputTable["FwTrafficMeterLan.spHour"] + 12
            end
            trafficMetrConfig["FwTrafficMeterLan.TimeMM"] = inputTable["FwTrafficMeterLan.spMinute"]
        else
            if (tonumber(inputTable["FwTrafficMeterLan.spHour"]) == 12) then
                trafficMetrConfig["FwTrafficMeterLan.TimeHH"] = "00"
            else
                trafficMetrConfig["FwTrafficMeterLan.TimeHH"] = inputTable["FwTrafficMeterLan.spHour"]
            end
            trafficMetrConfig["FwTrafficMeterLan.TimeMM"] = inputTable["FwTrafficMeterLan.spMinute"]
        end
    end

    trafficMetrConfig["FwTrafficMeterLan.DayOfMonth"] = inputTable["FwTrafficMeterLan.DayOfMonth"]
    trafficMetrConfig["FwTrafficMeterLan.SendEmailReport"] = inputTable["FwTrafficMeterLan.SendEmailReport"]
    trafficMetrConfig["FwTrafficMeterLan.BlockTraffic"] = inputTable["FwTrafficMeterLan.BlockTraffic"]
    trafficMetrConfig["FwTrafficMeterLan.SendEmailAlert"] = inputTable["FwTrafficMeterLan.SendEmailAlert"]
    

    valid = fw.config(trafficMetrConfig, rowid, operation)

    if(operation == "add" and valid) then
        DBTable = "LanTrafficStatistics"
        lanStatConfig["LanTrafficStatistics.StartTime"] = os.date("%a %b %d %X %Y")
        lanStatConfig["LanTrafficStatistics.LanIP"] = inputTable["FwTrafficMeterLan.LanIP"]
        valid = fw.config(lanStatConfig, rowid, operation)
    end

    if(operation == "edit" and valid) then
        -- dummy update on the table when the configure restart counters now
        if (trafficMetrConfig["FwTrafficMeterLan.RestartCounter"] == "0") then
            db.setAttribute (DBTable, "_ROWID_", rowid, "RestartCounter", 0)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "FW_TM_CONFIG_FAILED"
    end
end

--Traffic Meter specific Time getting
function fwView.trafficMtrLan_ConfigGet (rowId)
    local specificTimeConfig = db.getRow("FwTrafficMeterLan","_ROWID_", rowId)
    local fwTrafficConfig = {}

    if (rowId == "-1") then
        fwTrafficConfig["FwTrafficMeterLan.spHour"] = "0"
        fwTrafficConfig["FwTrafficMeterLan.spMinute"] = "0"
        fwTrafficConfig["FwTrafficMeterLan.startMeridian"] = "0"
    else
        if (tonumber (specificTimeConfig["FwTrafficMeterLan.TimeHH"]) == 0) then
            fwTrafficConfig["FwTrafficMeterLan.startMeridian"] = "0"
            fwTrafficConfig["FwTrafficMeterLan.spHour"] = "12"
        elseif (tonumber (specificTimeConfig["FwTrafficMeterLan.TimeHH"]) >= 12 and tonumber (specificTimeConfig["FwTrafficMeterLan.TimeHH"]) <= 23) then
            fwTrafficConfig["FwTrafficMeterLan.startMeridian"] = "1"
            fwTrafficConfig["FwTrafficMeterLan.spHour"] = tonumber (specificTimeConfig["FwTrafficMeterLan.TimeHH"]) - 12
        elseif(tonumber (specificTimeConfig["FwTrafficMeterLan.TimeHH"]) > 0 and tonumber (specificTimeConfig["FwTrafficMeterLan.TimeHH"]) <= 11) then
            fwTrafficConfig["FwTrafficMeterLan.spHour"] = specificTimeConfig["FwTrafficMeterLan.TimeHH"]
            fwTrafficConfig["FwTrafficMeterLan.startMeridian"] = "0"
        end
        fwTrafficConfig["FwTrafficMeterLan.spMinute"] = specificTimeConfig["FwTrafficMeterLan.TimeMM"]
    end
    return fwTrafficConfig
end

-- Lan Traffic Delete
function fwView.trafficLanMeterDelete (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"
	

    if (valid) then
        for k,v in pairs(rowIds) do
            local inTable = {}
            local inStatTable = {}
            DBTable = "FwTrafficMeterLan"
            inTable["FwTrafficMeterLan._ROWID_"] = v;            
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "FW_TRAFFIC_DELETE_FAILED"
                break                
            end
	    end
    end

    -- LAN Statstics Delete
    if (valid) then
        for k,v in pairs(rowIds) do
            local inTable = {}
            local inStatTable = {}
            DBTable = "LanTrafficStatistics"
            inTable["LanTrafficStatistics._ROWID_"] = v;            
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "FW_TRAFFIC_DELETE_FAILED"
                break            
            end
	    end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", errStr
	end
end
-- Modify Firewall_IPv6 Rules (Enable/Disable/Delete)
function fwView.modifyRules6 (rowIds, operation)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    if (operation == "enable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules6.Status"] = "1";
            valid = fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = "FW__IPV6_RULES_ENABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_IPV6_RULES_ENABLED"        
        end
    elseif (operation == "disable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules6.Status"] = "0";
            valid = fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = "FW_IPV6_RULES_DISABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_IPV6_RULES_DISABLED"
        end
    else
        local priIds = {}
        local idx = 0
        for k,v in pairs(rowIds) do
            idx = idx + 1
            priIds[idx] = db.getAttribute ("FirewallRules6", "_ROWID_", v, "PriorityId") or ''
            local inTable = {}
            inTable["FirewallRules6._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "FW_IPV6_RULES_DELETE_FAILED"
                break
            end
        end
        if (valid ~= false) then
            errStr = "FW_IPV6_RULES_DELETED"
        end

        if (valid ~= false) then
            -- Re-arrange Rules priority
            local rulesPriority = ""
            local rulesPriConfig = db.getRow ("fwPriorityMap6", "RuleType", "ALL")
            if (rulesPriConfig ~= nil) then
                rulesPriority = rulesPriConfig["fwPriorityMap6.PriorityStr"] or ''
                if (rulesPriority ~= nil and rulesPriority ~= "") then
                    local ruleIds = util.split (rulesPriority, ",")
                    if (ruleIds ~= false) then
                        rulesPriority = ""
                        for k,v in pairs (ruleIds) do
                            local notInDeleteList = true
                            for m,n in pairs (priIds) do
                                if (v == n) then
                                    notInDeleteList = false
                                    break
                                end
                            end

                            if (notInDeleteList) then
                                rulesPriority = rulesPriority .. v .. ","
                            end
                        end

                       	rulesPriority = string.sub(rulesPriority, 1, string.len(rulesPriority) - 1) --shave last comma

                        rulesPriConfig["fwPriorityMap6.PriorityStr"] = rulesPriority
                        DBTable = "fwPriorityMap6"
                        valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap6._ROWID_"], "edit")

                        if (valid) then
                        else
                            errStr = "FW_IPV6_RULES_PRIORITY_CONFIG_FAILED"
                        end
                    else
                        valid = false
                        errStr = "FW_IPV6_RULES_PRIORITY_CONFIG_FAILED"
                    end
                else
                    valid = false
                    errStr = "FW_IPV6_RULES_PRIORITY_CONFIG_FAILED"
                end
            else
                valid = false
                errStr = "FW_IPV6_RULES_PRIORITY_CONFIG_FAILED"
            end
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Move Firewall IPv6 rules
function fwView.moveRules6_NG (rowId, operation, ruleType, position)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = false
    local i = 0
    local j = 0
    local n = 0    
    local rulesPriority = nil
    local swapRow = nil
    local currentRow = nil
    
    -- grab the all firewall rules based on ruleType
    local where = "RuleType = 'SECURE_INSECURE'"
    where = where .. " or RuleType = 'INSECURE_SECURE'"
    where = where .. " or RuleType = 'PUBLIC_SECURE'"
    where = where .. " or RuleType = 'SECURE_PUBLIC'"
    where = where .. " or RuleType = 'INSECURE_PUBLIC'"
    where = where .. " or RuleType = 'PUBLIC_INSECURE'"
    local fwRules = db.getRowsWhere ("FirewallRules6", where)
    local firewallRules6 = {}
    local rulesPriority = db.getAttribute ("fwPriorityMap6", "RuleType", "ALL", "PriorityStr") or ''
    if (rulesPriority ~= nil and rulesPriority ~= "") then
	    local ruleIds = util.split (rulesPriority, ",")
		if (ruleIds ~= false and #ruleIds ~= 0) then
		    local idx = 0
			for k,v in pairs(ruleIds) do
			    for m,n in pairs (fwRules) do
				   if (n["FirewallRules6.PriorityId"] == v) then
				       idx = idx + 1
				       firewallRules6[idx] = n
				       break
				   end
			    end
			end
		end
	end
    
    for k,v in pairs(firewallRules6) do
        i = i + 1
        local row = firewallRules6[i]
        -- checking table rowId with given rowId
        if(row["FirewallRules6._ROWID_"] == rowId) then
            break
        end       
    end
    for k,v in pairs(firewallRules6) do
        j = j + 1
        local row = firewallRules6[j]
        -- checking table rowId with given rowId
        if(row["FirewallRules6.RuleType"] == ruleType) then
            n = n + 1
            if (n == tonumber(position)) then
                break
            end
        end
    end    
    -- taking before,after and current rows
    
    if (operation == "move") then
        if (i > j) then
            tmpRow = firewallRules6[i]
            for n=i,j+1,-1 do
                firewallRules6[n]=firewallRules6[n-1]
            end
            firewallRules6 [j] = tmpRow 
        elseif (i < j) then
            tmpRow = firewallRules6[i]
            for n=i,j-1,1 do
                firewallRules6[n]=firewallRules6[n+1]
            end
            firewallRules6 [j] = tmpRow 
        end
    end
	
    local rulesPriConfig = db.getRow ("fwPriorityMap6", "RuleType", "ALL")
    if (rulesPriConfig ~= nil) then
        rulesPriority = rulesPriConfig["fwPriorityMap6.PriorityStr"] or ''
    end

    j = 0
    
    if(rulesPriority ~= nil or rulesPriority ~= "") then
        local ruleIds = util.split(rulesPriority, ",")
        if(ruleIds ~= false) then
            for kk,vv in pairs(ruleIds) do
                j = j + 1                
                if (firewallRules6[j] ~= nil) then
                    ruleIds[kk] = firewallRules6 [j]["FirewallRules6.PriorityId"]
                end                
            end
            rulesPriority = util.join (ruleIds, ",")
        end        
    end   
    
    rulesPriConfig["fwPriorityMap6.PriorityStr"] = rulesPriority
    DBTable = "fwPriorityMap6"
    valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap6._ROWID_"], "edit")

	-- return
	if (valid) then
		db.commitTransaction(true)
        os.execute ("/pfrm2.0/bin/snmpTrapConfig 42 Major IPv6 Custom Firewall Rules sequence Modified")
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_IPV6_MOVE_OPERATION_FAILED"
    end
end

function fwView.getMaxPriority_v6 ()
    -- Get All Firewall Rules to find the newly added rule.
    local ruleType = "RuleType = 'SECURE_INSECURE'"
    ruleType = ruleType .. " or RuleType = 'INSECURE_SECURE'"
    ruleType = ruleType .. " or RuleType = 'PUBLIC_SECURE'"
    ruleType = ruleType .. " or RuleType = 'SECURE_PUBLIC'"
    ruleType = ruleType .. " or RuleType = 'INSECURE_PUBLIC'"
    ruleType = ruleType .. " or RuleType = 'PUBLIC_INSECURE'"

    local cur = db.execute(string.format([[SELECT MAX(PriorityId) PriorityId FROM FirewallRules6 WHERE %s]], ruleType))

     -- check for rows
    local result = nil
    if (cur) then
        local row = cur:fetch ({}, "a")
        cur:close()
        if (row) then result = row["PriorityId"] or '' end
    end
    return result
end

-- Firewall IPv6 Rules of Configuration
function fwView.firewallRules6Config(inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    if (inputTable["FirewallRules6.RuleType"] == nil) then
        inputTable["FirewallRules6.RuleType"] = inputTable["FirewallRules6.FromZoneType"] .. "_" .. inputTable["FirewallRules6.ToZoneType"]
    else
        local pos = string.find(inputTable["FirewallRules6.RuleType"], "_")
        inputTable["FirewallRules6.FromZoneType"] = string.sub(inputTable["FirewallRules6.RuleType"], 0, pos-1)
        inputTable["FirewallRules6.ToZoneType"] = string.sub(inputTable["FirewallRules6.RuleType"], pos+1)
    end
    
    local newPriority = 0
    if (valid and operation == "add") then
        local maxPriorityId = fwView.getMaxPriority_v6 ()
        if (maxPriorityId == nil or
            maxPriorityId == "" or
            maxPriorityId == "0") then
            newPriority = 1
        else
            newPriority = tonumber(maxPriorityId) + 1
        end

    if (newPriority == 4294967295) then
        newPriority = 1    
            while (true) do 
                local priorityRow = db.getRow ("FirewallRules6", "PriorityId", newPriority)
                if (priorityRow == nil or priorityRow == "") then
                    break
                else
                    newPriority = newPriority + 1
                end
            end
        end
        inputTable["FirewallRules6.PriorityId"] = newPriority
    end    

    valid = fw.config(inputTable, rowid, operation)
    -- Add the added rule into priority list
    if (valid and operation == "add") then
        -- Get All Firewall Rules to find the newly added rule.
        local ruleType = "RuleType = 'SECURE_INSECURE'"
        ruleType = ruleType .. " or RuleType = 'INSECURE_SECURE'"
        ruleType = ruleType .. " or RuleType = 'PUBLIC_SECURE'"
        ruleType = ruleType .. " or RuleType = 'SECURE_PUBLIC'"
        ruleType = ruleType .. " or RuleType = 'INSECURE_PUBLIC'"
        ruleType = ruleType .. " or RuleType = 'PUBLIC_INSECURE'"
        local fwRules = db.getRowsWhere ("FirewallRules6", ruleType)
        local rulesPriority = db.getAttribute ("fwPriorityMap6", "RuleType", "ALL", "PriorityStr") or ''
        if (rulesPriority == "") then
            -- First Firewall Rule Added
            rulesPriority = newPriority
        else
            rulesPriority = rulesPriority .. "," .. newPriority
        end

        local rulesPriConfig = db.getRow ("fwPriorityMap6", "RuleType", "ALL")
        if (rulesPriConfig ~= nil) then
            rulesPriConfig["fwPriorityMap6.PriorityStr"] = rulesPriority
            DBTable = "fwPriorityMap6"
            valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap6._ROWID_"], "edit")
        else
            valid = false
        end
    end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK" , newPriority
    else
        db.rollback()
        return "ERROR", "FW_IPV6_CONFIG_FAILED"
    end
end

function fwView.priorityMapUpdate(args)
    db.beginTransaction()
    local valid = true
    local priorityStr = db.getAttribute ("fwPriorityMap", "RuleType", "ALL", "PriorityStr")
    if (priorityStr ~= "" or priorityStr ~= nil) then
        local priorityId = util.split (priorityStr,",")
        local i = 1
        local newStr = ""
        while (priorityId[i] ~= nil) do
            if (db.existsRowWhere("FirewallRules", "PriorityId="..priorityId[i].." AND RuleType!='INSECURE_SECURE' AND RuleType!='INSECURE_PUBLIC'")) then
                if (newStr == "") then
                    newStr = priorityId[i]
                else
                    newStr = newStr..","..priorityId[i]
                end
            end
            i = i + 1
        end
        if (priorityStr ~= newStr) then
            local PriConfig = db.getRow ("fwPriorityMap", "RuleType", "ALL")
            if (PriConfig ~= nil) then
                PriConfig["fwPriorityMap.PriorityStr"] = newStr
                DBTable = "fwPriorityMap"
                valid = fw.config(PriConfig, PriConfig["fwPriorityMap._ROWID_"], "edit")
            else
                valid = false
            end
        end
    end
    if (valid) then
        db.commitTransaction(true)
        return true
    else
        db.rollback()
        return false
    end
end

function fwView.hwOffloadUpdate()
    local prog = db.getAttribute("environment", "name", "HW_OFFLOAD_SCRIPT", "value")
    if (prog ~= nil) then
        util.appendDebugOut("Exec = " .. os.execute("/pfrm2.0/bin/lua" .. " " .. prog .. " " .. DB_FILE_NAME ))
    end
end

-- Service Groups get
function fwView.getServiceGroups ()
    local serviceTable = db.getTable ("serviceGroup")
    for k,v in pairs (serviceTable) do
        local serviceListTbl = {}
        local table = {}
        serviceListTbl = db.getRowsWhere ("serviceGroupList", "serviceGroupName = '" .. v["serviceGroup.serviceGroupName"] .. "'")
        local i = 0
        if (serviceListTbl ~= nil and #serviceListTbl ~= 0) then
            for kk, vv in pairs (serviceListTbl) do
                i = i + 1
                table [i] = vv["serviceGroupList.ServiceName"]
            end
            v["serviceGroup.serviceList"] = util.join(table, ',')
        end
    end
    return serviceTable
end

-- service Groups Configuration
function fwView.serviceGroupConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local services = {}
    services = util.split (inputTable["serviceGroup.serviceList"], ",")

    if (operation == "add") then
        local rows = db.getTable("serviceGroup") or {}
        if (#rows >= 32) then
            db.rollback()
            return "ERROR", "MAX_LMT_RCHD"
        end
        local whereQuery = "ServiceName IN ('" .. util.join(services, "','") .. "')"
        local protoTbl = db.getDistinctValuesWhere ("Services", "Protocol", whereQuery)
        local i = 0
        for k,v in pairs (protoTbl) do i = i + 1 end
        if ( i ~= 1 ) then
            return "ERROR", "Selected services are not of same type"
        end
        inputTable["serviceGroup.serviceGroupType"] = protoTbl[1]["Services.Protocol"]
        if(db.existsRow("Services", "ServiceName", inputTable["serviceGroup.serviceGroupName"])) then
            db.rollback()
            return "ERROR", "Service exists with name " .. inputTable["serviceGroup.serviceGroupName"]
        end
        if(db.existsRow ("serviceGroup", "serviceGroupName", inputTable["serviceGroup.serviceGroupName"])) then             
            db.rollback()
            return "ERROR", "Service Group exists with name " .. inputTable["serviceGroup.serviceGroupName"]
        end
        valid = db.insert ("serviceGroup", inputTable)
    elseif (operation == "edit") then
        local serviceGroupName = db.getAttribute("serviceGroup", "_ROWID_", rowid, "serviceGroupName")
        local whereCond = "ServiceName='" .. serviceGroupName .. "'"
        if(db.existsRowWhere("FirewallRules", whereCond)) then
            db.rollback()
            return "ERROR", "Service Group '"..serviceGroupName.."' is in use"
        end
        local whereQuery = "ServiceName IN ('" .. util.join(services, "','") .. "')"
        local protoTbl = db.getDistinctValuesWhere ("Services", "Protocol", whereQuery)
        local i = 0
        for k,v in pairs (protoTbl) do i = i + 1 end
        if ( i ~= 1 ) then
            return "ERROR", "Selected services are not of same type"
        end
        valid = db.deleteRowWhere ("serviceGroupList", "ServiceGroupName='" .. inputTable["serviceGroup.serviceGroupName"].. "'")
        db.setAttribute("serviceGroup", "_ROWID_", rowid, "serviceGroupName", serviceGroupName) --dummy update for sake of snmp trap for edit.
    end
    for k,v in pairs (services) do
        local tmpTbl = {}
        tmpTbl ["serviceGroupList.ServiceGroupName"] = inputTable ["serviceGroup.serviceGroupName"]
        tmpTbl ["serviceGroupList.ServiceName"] = v
        if (valid) then
            valid = db.insert ("serviceGroupList", tmpTbl)
        end
    end
   
    -- return
    if (valid) then
        db.commitTransaction(true)
	return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SERVICE_GROUP_CONFIG_FAILED"
    end
end

function fwView.deleteServiceGroups(rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    --local rowsTemp = {}
    -- delete routes
    for k,v in pairs(rowIds) do
        local serviceGroupName = db.getAttribute("serviceGroup", "_ROWID_", v, "serviceGroupName")
        local whereCond = "ServiceName='" .. serviceGroupName .. "'"
        if(db.existsRowWhere("FirewallRules", whereCond)) then
            db.rollback()
            return "ERROR", "Service Group '"..serviceGroupName.."' is in use"
        end
        valid = db.deleteRow("serviceGroup", "_ROWID_", v)
        
        if(valid) then
            if(db.existsRowWhere("serviceGroupList", "ServiceGroupName='" .. serviceGroupName .. "'")) then
            valid = db.deleteRowWhere("serviceGroupList", "ServiceGroupName='" .. serviceGroupName .. "'")
            end
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SERVICE_GROUP_DELETE_FAILED"
    end
end

