--[[
#### Hussain Vali
#### TeamF1
#### www.TeamF1.com
#### Mar 28, 2008

#### File: lan.lua
#### Description: lan functions

#### Revisions:
None
01a, 16Dec13, sig Changes made for FVS336GV3 dmz led.
]]--


--************* Requires *************
require "teamf1lualib/dhcp"
require "teamf1lualib/ifDev"

--************* Initial Code *************

lan = {}

--************* Functions *************

-- lan config
function lan.config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

	-- validate
	if (lan.inputvalidate(inputTable, operation)) then
		if (operation == "add") then
			return nil
		elseif (operation == "edit") then
			valid = db.update("Lan", inputTable, rowid)
		elseif (operation == "delete") then
			return nil
		end
	end

    -- configure DHCP Server,
    -- if lan ipaddress not the present dhcp address subnet
    if (inputTable["dhcp.set"] ~= nil) then
        if (inputTable["dhcp.set"] == "1") then
            local inTableDhcp = {}
            inTableDhcp["dhcp.startAddress"] = inputTable["dhcp.startAddress"]
            inTableDhcp["dhcp.endAddress"] = inputTable["dhcp.endAddress"]
            if (valid) then
                DBTable = "dhcp"
                errorFlag, statusCode = dhcp.config(inTableDhcp, "1","edit")
                if (errorFlag == "OK") then
                    valid = true
                else
                    db.rollback()
                    return "ERROR", statusCode
                end
            end
        end
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LAN_CONFIG_FAILED"
    end
end

-- lan inputvalidate
function lan.inputvalidate (inputTable, operation)
	if (true) then
		return db.typeAndRangeValidate(inputTable)
	end
	return false
end

-- IPV4 Lan Configuration
function lan.ipv4LanConfig(inputTable, rowid, operation)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local configPortTbl = {}
    local hostName = db.getAttribute("system","_ROWID_","1","name")
    local configPortOldVal = db.getAttribute("ConfigPort","_ROWID_","1","LogicalIfName")
    configPortTbl["ConfigPort.LogicalIfName"] = inputTable["ConfigPort.LogicalIfName"]
    valid = db.update("ConfigPort", configPortTbl, "1")
    local configPortIpv6 = db.getAttribute("ConfigPort", "_ROWID_", "2", "LogicalIfName")
    if(configPortIpv6 == "LAN") then
    if(configPortOldVal ~= "DMZ" and inputTable["ConfigPort.LogicalIfName"] == "DMZ") then
        if(valid) then
            local port4Rows = db.getRows ("vlan","portName","port4")
            if (#port4Rows ~= 0) then
            valid = db.setAttribute("vlan","portName","port4","vlanProfState","0")
            end
            local dmzMacOffset = db.getAttribute("vlanMac", "_ROWID_", "1", "vlanDmzMacOffset")
            if(valid) then
                local vlanTmpTable = {}
				vlanTmpTable["vlan.OID"] = "NULL"
				vlanTmpTable["vlan.portName"] = "dummy"
				vlanTmpTable["vlan.vlanId"] = "4094"
				vlanTmpTable["vlan.vlanName"] = 'DMZ'
				vlanTmpTable["vlan.vlanProfState"] = "1"
				vlanTmpTable["vlan.vlanMacOffset"] = "1"
				valid = db.insert("vlan",vlanTmpTable)
				if(valid) then
				    vlanTmpTable["vlan.OID"] = "NULL"
					vlanTmpTable["vlan.portName"] = "port10"
					vlanTmpTable["vlan.vlanId"] = "4094"
					vlanTmpTable["vlan.vlanName"] = 'DMZ'
					vlanTmpTable["vlan.vlanProfState"] = "1"
					vlanTmpTable["vlan.vlanMacOffset"] = dmzMacOffset
					valid = db.insert("vlan",vlanTmpTable)
					if(valid) then
						vlanTmpTable["vlan.OID"] = "NULL"
						vlanTmpTable["vlan.portName"] = "port4"
						vlanTmpTable["vlan.vlanId"] = "4094"
						vlanTmpTable["vlan.vlanName"] = 'DMZ'
						vlanTmpTable["vlan.vlanProfState"] = "1"
						vlanTmpTable["vlan.vlanMacOffset"] = "1"
						valid = db.insert("vlan",vlanTmpTable)
						if(valid) then
						    local tmpvlanPort = db.getAttribute("VlanPortMgmt","PortNumber","3","DefaultPVID")
						    valid = db.setAttribute("VlanPortMgmt","PortNumber","3","oldVlanId", tmpvlanPort)
						    if(valid) then
						        valid = db.setAttribute("VlanPortMgmt","PortNumber","3","PhyInterfaceName","DMZ")
						        if(valid) then
						           valid = db.setAttribute("VlanPortMgmt","PortNumber","3","DefaultPVID","4094")
						        end
						    end
						end    
					end			
				end
            end
        end
    else
        if(configPortOldVal == "DMZ" and inputTable["ConfigPort.LogicalIfName"] == "LAN") then
           if(valid) then
               local vlanTmptbl = {}
               vlanTmptbl = db.getRowsWhere("vlan", "vlanId=4094")
                   if(vlanTmptbl ~= nil) then
                       valid = db.deleteRowWhere("vlan","vlanId=4094")
                   end    
               if(valid) then
                   local port4Rows = db.getRows ("vlan","portName","port4")
                   if (#port4Rows ~= 0) then
                       valid = db.setAttribute("vlan","portName","port4","vlanProfState","1")
                   end
                   if(valid) then
                       local tmpOldVlanId = db.getAttribute("VlanPortMgmt","PortNumber","3","oldVlanId")
                       if (db.existsRowWhere ("vlan", "portName='dummy' and vlanProfState='1' and vlanId="..tmpOldVlanId)) then
                           valid = db.setAttribute("VlanPortMgmt","PortNumber","3","DefaultPVID",tmpOldVlanId)
                       else
                           valid = db.setAttribute("VlanPortMgmt","PortNumber","3","DefaultPVID","1")
                       end
                   end
               end
           end 
        end            
    end
    end
    -- validate & ifStatic Config
    --if (lan.inputvalidate(inputTable, operation)) then
    if (valid) then

	if (operation == "add") then
            return nil
	elseif (operation == "edit") then
--            local addFamily = db.getAttribute("ifStatic", "_ROWID_", rowid, "AddressFamily")
            local qry = nil
            if(inputTable["ConfigPort.LogicalIfName"] == "DMZ") then
--               if(tonumber(addFamily) == 2) then
               qry = "UPDATE ifStatic SET StaticIp = \'"..inputTable["ifStatic.StaticIp"].."\' , NetMask = \'"..inputTable["ifStatic.NetMask"].."\' WHERE _ROWID_ = \'"..rowid.."\'"
--           elseif(tonumber(addFamily) == 10) then
--               qry = "UPDATE ifStatic SET StaticIp = \'"..inputTable["ifStatic.StaticIp"].."\' , PrefixLength = \'"..inputTable["ifStatic.PrefixLength"].."\' WHERE _ROWID_ = \'"..rowid.."\'"
--            end

               cur,errStr = db.execute(qry)
               if(errStr == "") then valid = true
               else valid = false
               end
               -- enable DMZ LED
               if(configPortIpv6 == "LAN") then
                   if (hostName == "FVS336GV3") then
                       os.execute("/pfrm2.0/bin/ledctrl dmz on")
                   else
                       os.execute("/pfrm2.0/bin/ledTog 1 1")
                   end
                   db.setAttribute("dmzLED","_ROWID_","1","ledStatus","1")
               end
           elseif ((inputTable["ConfigPort.LogicalIfName"] == "LAN") and (configPortIpv6 == "DMZ")) then
               qry = "UPDATE ifStatic SET StaticIp = \'".. "0.0.0.0" .."\' , NetMask = \'".. "0.0.0.0".."\' WHERE _ROWID_ = \'"..rowid.."\'"
               cur,errStr = db.execute(qry)
               if(errStr == "") then
                   valid = true
               else 
                   valid = false
               end
           elseif ((inputTable["ConfigPort.LogicalIfName"] == "LAN") and (configPortIpv6 == "LAN")) then
               -- disable DMZ LED
               if (hostName == "FVS336GV3") then
                   os.execute("/pfrm2.0/bin/ledctrl dmz off")
               else
                   os.execute("/pfrm2.0/bin/ledTog 0 1")
               end
               db.setAttribute("dmzLED","_ROWID_","1","ledStatus","0")
                --dummy update on ifStatic Table to update the status in
                --networkInterface 
                valid = db.setAttribute("ifStatic", "_ROWID_", rowid, "OID", "0")
           else
               valid = db.update("ifStatic", inputTable, rowid)
           end
       elseif (operation == "delete") then
	    return nil
	end
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV4_LAN_CONFIG_FAILED"
    end
end


-- IPV6 Lan Configuration
function lan.ipv6LanConfig(inputTable, rowid, operation)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end
    --if( ifDev.IPv6SubnetAlreadyInUse(inputTable["ConfigPort.LogicalIfName"],inputTable["ifStatic.StaticIp"] , inputTable["ifStatic.PrefixLength"] )== "ERROR")then
	--print("IPv6 address already in use, try other ip.")
        --return "ERROR", "ERR_IPV6_SUBNET_IN_USE"
    --end
    db.beginTransaction() --begin transaction
    local valid = false
    local configPortTbl = {}
    local hostName = db.getAttribute("system","_ROWID_","1","name")
    local configPortOldVal = db.getAttribute("ConfigPort","_ROWID_","2","LogicalIfName")
    configPortTbl["ConfigPort.LogicalIfName"] = inputTable["ConfigPort.LogicalIfName"]
    valid = db.update("ConfigPort", configPortTbl, "2")
    local configPortIpv4 = db.getAttribute("ConfigPort", "_ROWID_", "1", "LogicalIfName")
    if(configPortIpv4 == "LAN") then
    if(configPortOldVal ~= "DMZ" and inputTable["ConfigPort.LogicalIfName"] == "DMZ") then
        if( ifDev.IPv6SubnetAlreadyInUse(inputTable["ConfigPort.LogicalIfName"],inputTable["ifStatic.StaticIp"] , inputTable["ifStatic.PrefixLength"] )== "ERROR")then
	print("IPv6 address already in use, try other ip.")
        return "ERROR", "ERR_IPV6_SUBNET_IN_USE"
    end

        if(valid) then
            local port4Rows = db.getRows ("vlan","vlanid != 4094 AND portName","port4")
            if (#port4Rows ~= 0) then
                valid = db.setAttribute("vlan","portName","port4","vlanProfState","0")
            end
            local dmzMacOffset = db.getAttribute("vlanMac", "_ROWID_", "1", "vlanDmzMacOffset")
            if(valid) then
                local vlanTmpTable = {}
				vlanTmpTable["vlan.OID"] = "NULL"
				vlanTmpTable["vlan.portName"] = "dummy"
				vlanTmpTable["vlan.vlanId"] = "4094"
				vlanTmpTable["vlan.vlanName"] = 'DMZ'
				vlanTmpTable["vlan.vlanProfState"] = "1"
				vlanTmpTable["vlan.vlanMacOffset"] = "1"
				valid = db.insert("vlan",vlanTmpTable)
				if(valid) then
				    vlanTmpTable["vlan.OID"] = "NULL"
					vlanTmpTable["vlan.portName"] = "port10"
					vlanTmpTable["vlan.vlanId"] = "4094"
					vlanTmpTable["vlan.vlanName"] = 'DMZ'
					vlanTmpTable["vlan.vlanProfState"] = "1"
					vlanTmpTable["vlan.vlanMacOffset"] = dmzMacOffset
					valid = db.insert("vlan",vlanTmpTable)
					if(valid) then
						vlanTmpTable["vlan.OID"] = "NULL"
						vlanTmpTable["vlan.portName"] = "port4"
						vlanTmpTable["vlan.vlanId"] = "4094"
						vlanTmpTable["vlan.vlanName"] = 'DMZ'
						vlanTmpTable["vlan.vlanProfState"] = "1"
						vlanTmpTable["vlan.vlanMacOffset"] = "1"
						valid = db.insert("vlan",vlanTmpTable)
						if(valid) then
						    local tmpvlanPort = db.getAttribute("VlanPortMgmt","PortNumber","3","DefaultPVID")
						    valid = db.setAttribute("VlanPortMgmt","PortNumber","3","oldVlanId", tmpvlanPort)
						    if(valid) then
						        valid = db.setAttribute("VlanPortMgmt","PortNumber","3","PhyInterfaceName","DMZ")
						        if(valid) then
						           valid = db.setAttribute("VlanPortMgmt","PortNumber","3","DefaultPVID","4094")
						        end
						    end
						end    
					end			
				end
            end
        end
    else
        if(configPortOldVal == "DMZ" and inputTable["ConfigPort.LogicalIfName"] == "LAN") then
           if(valid) then
               local vlanTmptbl = {}
               vlanTmptbl = db.getRowsWhere("vlan", "vlanId=4094")
                   if(vlanTmptbl ~= nil) then
                       valid = db.deleteRowWhere("vlan","vlanId=4094")
                   end    
               if(valid) then
                   local port4Rows = db.getRows ("vlan","portName","port4")
                   if (#port4Rows ~= 0) then
                       valid = db.setAttribute("vlan","portName","port4","vlanProfState","1")
                   end
                   if(valid) then
                       local tmpOldVlanId = db.getAttribute("VlanPortMgmt","PortNumber","3","oldVlanId")
                       if (db.existsRowWhere ("vlan", "portName='dummy' and vlanProfState='1' and vlanId="..tmpOldVlanId)) then
                           valid = db.setAttribute("VlanPortMgmt","PortNumber","3","DefaultPVID",tmpOldVlanId)
                       else
                           valid = db.setAttribute("VlanPortMgmt","PortNumber","3","DefaultPVID","1")
                       end
                   end
               end
           end 
        end            
    end
    end
    -- validate & ifStatic Config
    --if (lan.inputvalidate(inputTable, operation)) then
    if (valid) then

    	if (operation == "add") then
            return nil
	    elseif (operation == "edit") then
--            local addFamily = db.getAttribute("ifStatic", "_ROWID_", rowid, "AddressFamily")
--            local qry = nil
            if(inputTable["ConfigPort.LogicalIfName"] == "DMZ") then
--               if(tonumber(addFamily) == 2) then
--               qry = "UPDATE ifStatic SET StaticIp = \'"..inputTable["ifStatic.StaticIp"].."\' , NetMask = \'"..inputTable["ifStatic.NetMask"].."\' WHERE _ROWID_ = \'"..rowid.."\'"
--           elseif(tonumber(addFamily) == 10) then
               local qry = "UPDATE ifStatic SET StaticIp = \'"..inputTable["ifStatic.StaticIp"].."\' , PrefixLength = \'"..inputTable["ifStatic.PrefixLength"].."\' WHERE _ROWID_ = \'"..rowid.."\'"
--            end

               cur,errStr = db.execute(qry)
               if(errStr == "") then
                   valid = true
               else 
                   valid = false
               end
               -- enable DMZ LED
               if(configPortIpv4 == "LAN") then
                   if (hostName == "FVS336GV3") then
                       os.execute("/pfrm2.0/bin/ledctrl dmz on")
                   else
                       os.execute("/pfrm2.0/bin/ledTog 1 1")
                   end
               db.setAttribute("dmzLED","_ROWID_","1","ledStatus","1")
               end
            elseif ((inputTable["ConfigPort.LogicalIfName"] == "LAN") and (configPortIpv4 == "DMZ")) then
                qry = "UPDATE ifStatic SET StaticIp = \'".. "" .."\' , PrefixLength = \'".. "".."\' WHERE _ROWID_ = \'"..rowid.."\'"
                cur,errStr = db.execute(qry)
                if(errStr == "") then
                    valid = true
                else 
                    valid = false
                end
            elseif ((inputTable["ConfigPort.LogicalIfName"] == "LAN") and (configPortIpv4 == "LAN")) then
                -- disable DMZ LED
                if (hostName == "FVS336GV3") then
                    os.execute("/pfrm2.0/bin/ledctrl dmz off")
                else
                    os.execute("/pfrm2.0/bin/ledTog 0 1")
                end
                db.setAttribute("dmzLED","_ROWID_","1","ledStatus","0")
            else
	            valid = db.update("ifStatic", inputTable, rowid)
            end
	    elseif (operation == "delete") then
	        return nil
	    end
    end
    if (valid) then
        if (inputTable["ConfigPort.LogicalIfName"] == "LAN") then
            valid = db.setAttribute("radvd","LogicalIfName","DMZ","isEnabled","0")
        end
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV6_LAN_CONFIG_FAILED"
    end
end




-- IPV6 Lan Configuration
function lan.obsoleteipv6LanConfig(inputTable, rowid, operation)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end
    if( ifDev.IPv6SubnetAlreadyInUse("LAN",inputTable["ifStatic.StaticIp"] , inputTable["ifStatic.PrefixLength"] )== "ERROR")then
	print("IPv6 address already in use, try other ip.")
        return "ERROR", "ERR_IPV6_SUBNET_IN_USE"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    -- validate & ifStatic Config
    if (lan.inputvalidate(inputTable, operation)) then
	if (operation == "add") then
            return nil
	elseif (operation == "edit") then
	    valid = db.update("ifStatic", inputTable, rowid)
	elseif (operation == "delete") then
	    return nil
	end
    end

    -- Dhcpv6s Configuration
    if(valid) then
        local isExists = db.existsRow ("dhcpv6s", "_ROWID_", "1")
        if (isExists) then
            valid = db.update("dhcpv6s", inputTable, "1")
        else
            valid = db.insert("dhcpv6s", inputTable)        
        end
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV6_LAN_CONFIG_FAILED"
    end
end

-- IPV6 Lan Address Pool Deletion
function lan.lanPoolsDelete (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local errStr = ''
    local errStrFail = ''
    if(DBTable == "dhcpv6sDMZAddrPool") then
        errStr = "IPV6_DMZ_POOL_DELETE_SUCCESS"
        errStrFail = "IPV6_DMZ_POOL_DELETE_FAILED"
    else
        errStr = "IPV6_LAN_POOL_DELETE_SUCCESS"
        errStrFail = "IPV6_LAN_POOL_DELETE_FAILED"
    end
    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable[DBTable .. "._ROWID_"] = v
        valid = db.delete(DBTable, inTable) 
        if(valid==false) then
            break
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", errStr
    else
        db.rollback()
        return "ERROR", errStrFail
    end
end

-- IPV6 Lan Pool Configuration
function lan.poolConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    if( ifDev.twoIPv6AddressInSameSubnet(inputTable["dhcpv6sDMZAddrPool.startAddress"], inputTable["dhcpv6sDMZAddrPool.endAddress"], inputTable["dhcpv6sDMZAddrPool.prefixLength"])== "ERROR")then
	print("Please check subnet of address pool.")
        return "ERROR", "ERR_IPV6_ADDR_POOL_DIFF"
    end
 
    if( ifDev.twoIPv6AddressInSameSubnet(inputTable["dhcpv6sLANAddrPool.startAddress"], inputTable["dhcpv6sLANAddrPool.endAddress"], inputTable["dhcpv6sLANAddrPool.prefixLength"])== "ERROR")then
	print("Please check subnet of address pool.")
        return "ERROR", "ERR_IPV6_ADDR_POOL_DIFF"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local errStr = ''
    if(DBTable == "dhcpv6sDMZAddrPool") then
        errStr = "IPV6_DMZ_POOL_CONFIG_FAILED"
    else
        errStr = "IPV6_LAN_POOL_CONFIG_FAILED"
    end

    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
	    valid = db.delete(DBTable, inputTable)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", errStr
    end
end

-- IPV6 Lan Prefix Configuration
function lan.prefixDelegationConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
 
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert("dhcpv6sLANPrefixPool", inputTable)
        elseif (operation == "edit") then
            valid = db.update("dhcpv6sLANPrefixPool", inputTable, rowid)
        elseif (operation == "delete") then
	    valid = db.delete("dhcpv6sLANPrefixPool", inputTable)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV6_LAN_PREFIX_CONFIG_FAILED"
    end
end

-- IPV6 Lan Prefix Pool Deletion
function lan.lanPrefixPoolsDelete (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local errStr = ''
    local errStrFail = ''
    errStr = "IPV6_LAN_PREFIX_DELETE_SUCCESS"
    errStrFail = "IPV6_LAN_PREFIX_DELETE_FAILED"
    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["dhcpv6sLANPrefixPool" .. "._ROWID_"] = v
        valid = db.delete("dhcpv6sLANPrefixPool", inTable) 
        if(valid==false) then
            break
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", errStr
    else
        db.rollback()
        return "ERROR", errStrFail
    end
end


