--[[
#### Adarsh Uppula
#### TeamF1
#### www.TeamF1.com
#### June 29, 2007

#### File: logging.lua
#### Description: logging functions

--Modification History
--01b, 25feb13, ksr fixed SPR 32231
#### Revisions: Venu Babu.M
#### OCT 25, 2010
####
#### Modified the function logging.emailLogs_config()
#### 
#### Added new function logging.RoutLogs_getInfo()
None
]]--


--************* Requires *************
require "loggingLib"
require "teamf1lualib/crontab"
--************* Initial Code *************

--package logging
logging = {}

--table packages
logConfig = {}
sysLogInfo = {}
emailLogs = {}
logEventConfig = {}
--************* Functions *************

-- logging config
function logging.logConfig_config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- config logConfig
    valid = logConfig.config(inputTable, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LOGGING_CONFIG_FAILED"
    end
end

-- logConfig config
function logConfig.config (inputTable, rowid, operation)
    -- validate
    if (logConfig.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("logConfig", inputTable)
        elseif (operation == "edit") then
            return db.update("logConfig", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("logConfig", inputTable)
        end
    end
    return false
end

-- logConfig inputvalidate
function logConfig.inputvalidate (inputTable, operation)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

--emailLogs config

function logging.emailLogs_config(inputTable)
    local FirewallLogs = {}
    local Firewall6logs = {}
    local OtherSysLogs = {}
    local entryName = db.getAttribute ("emailLogs","_ROWID_",1,"entryName")
    emailLogsBinary = db.getAttribute ("environment","name","EMAILLOGS_PROGRAM","value") .. " " .. DB_FILE_NAME

    if(inputTable["emailLogs.mailLogs"] == "0") then
        valid = db.update ("emailLogs", inputTable, 1)
        if (valid == false) then 
            return "ERROR" ,"LOGGING_EMAILLOGS_CONFIG_FAILED"
        end
        
        valid = db.deleteRow("crontab","command",emailLogsBinary)
        if (valid == false) then 
            return "ERROR" ,"CRONTAB_CONFIG_FAILED"
        end
    else
        if((tonumber(inputTable["crontab.unit"]) == 2) or (tonumber(inputTable["crontab.unit"]) == 3)) then
            if(tonumber(inputTable["crontab.meridiem"]) == 1) then
                inputTable["crontab.hour"] = 12 + tonumber(inputTable["crontab.hour"])
            end
        end
        valid = db.update("emailLogs",inputTable,1)
        if(valid == false) then
            return "ERROR","LOGGING_EMAILLOGS_CONFIG_FAILED"
        end
        inputTable["smtpServer.logFile"] = db.getAttribute("environment","name","LOG_FILE","value")
        if (db.existsRow("smtpServer","entryName",entryName)) then
            local RowId = db.getAttribute ("smtpServer","entryName",entryName,"_ROWID_")
            valid = db.update ("smtpServer",inputTable, RowId)
        else
            inputTable["smtpServer.entryName"]=entryName
            valid = db.insert("smtpServer",inputTable)
        end
        if (valid == false) then
            return "ERROR","LOGGING_SMTP_CONFIG_FAILED"
        end
       
        local errorString = "OK"
        
        if (inputTable["crontab.unit"] == "0") then
            valid = db.deleteRow("crontab","command",emailLogsBinary)
        else
            valid = db.deleteRow("crontab","command",emailLogsBinary)
            errorString = crontab.config(inputTable,-1,"insert",emailLogsBinary)
        end

        if (errorString ~= "OK" or valid == false) then
                return "ERROR","LOGGING_CRONTAB_CONFIG_FAILED"
        end
    end
    if (valid) then     	
        FirewallLogs["FirewallLogs.LANToWANAccept"] = inputTable["FirewallLogs.LANToWANAccept"]
        FirewallLogs["FirewallLogs.LANToWANDrop"] = inputTable["FirewallLogs.LANToWANDrop"]
        FirewallLogs["FirewallLogs.LANToDMZAccept"] = inputTable["FirewallLogs.LANToDMZAccept"]
        FirewallLogs["FirewallLogs.LANToDMZDROP"] = inputTable["FirewallLogs.LANToDMZDROP"]
        FirewallLogs["FirewallLogs.DMZToWANAccept"] = inputTable["FirewallLogs.DMZToWANAccept"]
        FirewallLogs["FirewallLogs.DMZToWANDrop"] = inputTable["FirewallLogs.DMZToWANDrop"]
        FirewallLogs["FirewallLogs.WANToLANAccept"] = inputTable["FirewallLogs.WANToLANAccept"]
        FirewallLogs["FirewallLogs.WANToLANDrop"] = inputTable["FirewallLogs.WANToLANDrop"]
        FirewallLogs["FirewallLogs.DMZToLANAccept"] = inputTable["FirewallLogs.DMZToLANAccept"]
        FirewallLogs["FirewallLogs.DMZToLANDrop"] = inputTable["FirewallLogs.DMZToLANDrop"]
        FirewallLogs["FirewallLogs.WANToDMZAccept"] = inputTable["FirewallLogs.WANToDMZAccept"]
        FirewallLogs["FirewallLogs.WANToDMZDrop"] = inputTable["FirewallLogs.WANToDMZDrop"]
        FirewallLogs["FirewallLogs.UnicastTraffic"] = inputTable["FirewallLogs.UnicastTraffic"]
        FirewallLogs["FirewallLogs.BroadCastORMulticastTraffic"] = inputTable["FirewallLogs.BroadCastORMulticastTraffic"]
        FirewallLogs["FirewallLogs.SourceMACFilter"] = inputTable["FirewallLogs.SourceMACFilter"]
        FirewallLogs["FirewallLogs.SessionLimitLogs"] = inputTable["FirewallLogs.SessionLimitLogs"]
        FirewallLogs["FirewallLogs.BandwidthLimitLogs"] = inputTable["FirewallLogs.BandwidthLimitLogs"]
        valid = db.update("FirewallLogs",FirewallLogs,1)
        Firewall6logs["FirewallLogs6.LANToWANAccept"] = inputTable["FirewallLogs.LANToWANAccept"]
        Firewall6logs["FirewallLogs6.LANToWANDrop"] = inputTable["FirewallLogs.LANToWANDrop"]
        Firewall6logs["FirewallLogs6.LANToDMZAccept"] = inputTable["FirewallLogs.LANToDMZAccept"]
        Firewall6logs["FirewallLogs6.LANToDMZDROP"] = inputTable["FirewallLogs.LANToDMZDROP"]
        Firewall6logs["FirewallLogs6.DMZToWANAccept"] = inputTable["FirewallLogs.DMZToWANAccept"]
        Firewall6logs["FirewallLogs6.DMZToWANDrop"] = inputTable["FirewallLogs.DMZToWANDrop"]
        Firewall6logs["FirewallLogs6.WANToLANAccept"] = inputTable["FirewallLogs.WANToLANAccept"]
        Firewall6logs["FirewallLogs6.WANToLANDrop"] = inputTable["FirewallLogs.WANToLANDrop"]
        Firewall6logs["FirewallLogs6.DMZToLANAccept"] = inputTable["FirewallLogs.DMZToLANAccept"]
        Firewall6logs["FirewallLogs6.DMZToLANDrop"] = inputTable["FirewallLogs.DMZToLANDrop"]
        Firewall6logs["FirewallLogs6.WANToDMZAccept"] = inputTable["FirewallLogs.WANToDMZAccept"]
        Firewall6logs["FirewallLogs6.WANToDMZDrop"] = inputTable["FirewallLogs.WANToDMZDrop"]
        if(valid) then
            valid = db.update("FirewallLogs6",Firewall6logs,1)
        end
    end
    if(valid) then
        OtherSysLogs["OtherSystemLogs.ChangeOfTimeByNTPLogs"] = inputTable["OtherSystemLogs.ChangeOfTimeByNTPLogs"]
        OtherSysLogs["OtherSystemLogs.LoginAttemptsLogs"] = inputTable["OtherSystemLogs.LoginAttemptsLogs"]
        OtherSysLogs["OtherSystemLogs.SecureLoginAttempts"] = inputTable["OtherSystemLogs.SecureLoginAttempts"]
        OtherSysLogs["OtherSystemLogs.RebootLogs"] = inputTable["OtherSystemLogs.RebootLogs"]
        OtherSysLogs["OtherSystemLogs.WANStatusLogs"] = inputTable["OtherSystemLogs.WANStatusLogs"]
        OtherSysLogs["OtherSystemLogs.ResolvedDNSNames"] = inputTable["OtherSystemLogs.ResolvedDNSNames"]
        OtherSysLogs["OtherSystemLogs.VPNLogs"] = inputTable["OtherSystemLogs.VPNLogs"]
        OtherSysLogs["OtherSystemLogs.DhcpServerLogs"] = inputTable["OtherSystemLogs.DhcpServerLogs"]
        valid = db.update("OtherSystemLogs", OtherSysLogs, 1)
    end

    if(valid) then
        local logConfig = {}
        if(inputTable["logConfig.wirelessLogs"] == "1") then
            logConfig["logConfig.emergency"] = 5
            logConfig["logConfig.alert"] = 5
            logConfig["logConfig.critical"] = 5
            logConfig["logConfig.error"] = 5
            logConfig["logConfig.warning"] = 5
            logConfig["logConfig.notice"] = 5
            logConfig["logConfig.information"] = 5
            logConfig["logConfig.debug"] = 5
        else
            logConfig["logConfig.emergency"] = 0
            logConfig["logConfig.alert"] = 0
            logConfig["logConfig.critical"] = 0
            logConfig["logConfig.error"] = 0
            logConfig["logConfig.warning"] = 0
            logConfig["logConfig.notice"] = 0
            logConfig["logConfig.information"] = 0
            logConfig["logConfig.debug"] = 0
        end
        local rowId = db.existsRow("logConfig","facility","Local0-wireless")
        if (rowId) then
            valid = db.update ("logConfig",logConfig, rowId)
        else
            logConfig["logConfig.facility"] = "Local0-wireless"
            logConfig["logConfig.facilityId"] = 16
            valid = db.insert("logConfig", logConfig)
        end

    end

    -- Set hostname
    --if(inputTable["emailLogs.logId"] ~= nil) then
      --  os.execute("/bin/hostname "..inputTable["emailLogs.logId"])
    --end

    return "OK","STATUS_OK"
end

--emailLogs getn info() for the logs_config_emails.htm page

function logging.emailLogs_getInfo()
    configRow = {}
    local entryName=db.getAttribute("emailLogs","_ROWID_",1,"entryName")
    
    if (not db.existsRow("smtpServer","entryName",entryName)) then
        configRow['smtpServer.emailServer'] = ''
        configRow['smtpServer.fromAddr'] = ''
        configRow['smtpServer.toAddr'] = ''
        configRow['smtpServer.userName'] = ''
        configRow['smtpServer.passWord'] = ''
        configRow["crontab.unit"] = "0"
        
    else
        
        local currentInfo = db.getRow("smtpServer","entryName",entryName)
        for k,v in pairs(currentInfo) do
            configRow[k] = v
        end
        if (tonumber(configRow['smtpServer.auth']) == -1) then
            configRow['smtpServer.userName'] = ''
            configRow['smtpServer.passWord'] = ''
        end
    end
    
    emailLogsBinary = db.getAttribute("environment","name","EMAILLOGS_PROGRAM","value") .. " " .. DB_FILE_NAME
    configRow["emailLogs.mailLogs"] = db.getAttribute("emailLogs","_ROWID_",1,"mailLogs")
    configRow["emailLogs.logId"] = db.getAttribute("emailLogs","_ROWID_",1,"logId")
    
    if(not db.existsRow("crontab","command",emailLogsBinary)) then
        configRow["crontab.unit"] = "0"
    else
        local RowId = db.getAttribute("crontab", "command", emailLogsBinary,"_ROWID_")
        configRow=crontab.getSchedule(RowId,configRow)
        if(tonumber(configRow["crontab.hour"]) > 11) then
            configRow["crontab.hour"] = tonumber(configRow["crontab.hour"]) - 12
            configRow["crontab.meridiem"] = "1"
            configRow["crontab.hour"] = tostring(configRow["crontab.hour"]) 
        else
            configRow["crontab.meridiem"] = "0"
        end
    end
	return configRow
end

-- logging.RoutLogs_getInfo() , for displaying values

function logging.RoutLogs_getInfo()
    local outTable = {}
    local routLogInfo = {}
    local OtherSysInfo = {}
    local logConfigInfo = {}
    local routLogInfo = db.getRowWhere ("FirewallLogs","_ROWID_=1")
    local OtherSysInfo = db.getRowWhere ("OtherSystemLogs","_ROWID_=1")
    local logConfig = db.getRowWhere ("logConfig","facility='Local0-wireless'")

    if ((logConfig["logConfig.emergency"] == "5") and
        (logConfig["logConfig.alert"] == "5") and
        (logConfig["logConfig.critical"] == "5") and
        (logConfig["logConfig.error"] == "5") and
        (logConfig["logConfig.warning"  ] == "5") and
        (logConfig["logConfig.notice"] == "5") and
        (logConfig["logConfig.information"] == "5") and
        (logConfig["logConfig.debug"] == "5") 
        )then
        logConfigInfo ["logConfig.wirelessLogs"] = "1"
    else
        logConfigInfo ["logConfig.wirelessLogs"] = "0"
    end


    if(routLogInfo ~= nil and OtherSysInfo ~= nil) then    
    outTable = util.tableAppend (routLogInfo, OtherSysInfo)
    end

    if(outTable ~= nil and logConfigInfo ~= nil) then    
    outTable = util.tableAppend (outTable, logConfigInfo)
    end
    
    return outTable
end


-- sysLogInfo_config(), for updating the sysLogInfo table.

function logging.sysLogInfo_config(inputTable, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    --db.beginTransaction()
    local valid = false
    
    if (operation == "add") then
        valid =  db.insert("sysLogInfo", inputTable)
    elseif (operation == "edit") then
        valid = db.update("sysLogInfo", inputTable, 1)
    end
    
    if (valid) then
        --db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        --db.rollback()
        return "ERROR", "LOGGING_SYSLOG_CONFIG_FAILED"
    end
end

--logging events config
function logging.logEventsConfig_config(inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- config logEventEmailConfig
    valid = logEventConfig.config(inputTable, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LOGGING_CONFIG_FAILED"
    end
end

-- logEventConfig config
function logEventConfig.config (inputTable, rowid, operation)
    -- validate
    if (logConfig.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("logEventEmailConfig", inputTable)
        elseif (operation == "edit") then
            return db.update("logEventEmailConfig", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("logEventEmailConfig", inputTable)
        end
    end
    return false
end

-- email Log Config
function logging.emailLogsConfig(inputTable)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
	
    local valid = false
    db.beginTransaction()

    errorFlag, statusCode = logging.emailLogs_config(inputTable)
    if(errorFlag == "OK") then
	local syslogInfo = db.getTable ("sysLogInfo")
        for k,v in pairs (syslogInfo) do
            if(inputTable["sysLogInfo.Enable".. v["sysLogInfo.serverId"]] ~= nil) then
                v["sysLogInfo.Enable"] = inputTable["sysLogInfo.Enable" .. v["sysLogInfo.serverId"]]
		if(v["sysLogInfo.Enable"] == "1") then
	                v["sysLogInfo.serverName"] = inputTable["sysLogInfo.serverName" .. v["sysLogInfo.serverId"]]
        	        v["sysLogInfo.facilityId"] = inputTable["sysLogInfo.facilityId" .. v["sysLogInfo.serverId"]]
                	v["sysLogInfo.severity"] = inputTable["sysLogInfo.severity" .. v["sysLogInfo.serverId"]]
		end
                valid = db.update("sysLogInfo", v, v["sysLogInfo._ROWID_"])
                if (valid == false) then
                    break
                end
            end
	end	
    end
	
    if (valid and errorFlag == "OK") then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LOGGING_SYSLOG_CONFIG_FAILED"
    end
end

function logging.emailLogsConfigGet()
    local outTable = logging.emailLogs_getInfo()
    if (outTable == nil) then
        outTable = {}
    end

    local syslogInfo = db.getTable ("sysLogInfo")
    for k,v in pairs(syslogInfo) do
        outTable["sysLogInfo.Enable" .. v["sysLogInfo.serverId"]] = v["sysLogInfo.Enable"]
        outTable["sysLogInfo.serverName" .. v["sysLogInfo.serverId"]] = v["sysLogInfo.serverName"]
        outTable["sysLogInfo.facilityId" .. v["sysLogInfo.serverId"]] = v["sysLogInfo.facilityId"]
        outTable["sysLogInfo.severity" .. v["sysLogInfo.serverId"]] = v["sysLogInfo.severity"]
	end
    return outTable
end

function logging.logsOperation(logType,operation)
    local logType = logType
    local operation = operation
    local status = loggingLib.logOperation (logType,operation)
    
    if (string.find(status, "0")) then
        return "OK",""
    else
        return "ERROR",""
    end

end
