-- nimf6CLI.lua
-- Samer Sayeed 
-- TeamF1
-- www.TeamF1.com
--
-- Modification History
-- 25sep13,ksr fixed SPR #37754
-- 25feb13,pbr fixed #32896 - not showing pppoe6 ip address
-- 04mar10,db  added Dual Wan Support
-- 31jan09,ss  added nimf6CfgInputVal, ipModeCfgInputVal routine
-- 23jan09,ss  written
--
-- Description
-- CLISH nimf LUA routines

require "teamf1lualib/nimfView"

function wanV6SetupGet(wanType)
    local resultTab = {}
    local where = "AddressFamily = 10 and LogicalIfName = '"..wanType.."'"
    local nimfTabel = db.getRowWhere ("NimfConf", where)
    if (ipModeCfgInputVal()) then
    if (nimfTabel ["NimfConf.ConnectionType6"] == "ifStatic6") then
        resTab.insertField (resultTab, "Static IPv6 Configuration","")
        setupRow = db.getRowWhere ("ifStatic", "LogicalIfName='"..wanType.."' and AddressFamily=10")
        resTab.insertField (resultTab, "IPv6 Address", setupRow["ifStatic.StaticIp"]or"")
        resTab.insertField (resultTab, "IPv6 Prefix Length", setupRow["ifStatic.PrefixLength"]or"")
        resTab.insertField (resultTab, "Default IPv6 Gateway", setupRow["ifStatic.Gateway"]or"")
        resTab.insertField (resultTab, "Primary DNS Server", setupRow["ifStatic.PrimaryDns"]or"")
        resTab.insertField (resultTab, "Secondary DNS Server", setupRow["ifStatic.SecondaryDns"]or"")
    end
       
    if (nimfTabel ["NimfConf.ConnectionType6"] == "dhcp6c") then
        resTab.insertField (resultTab, "Dynamic IPv6 (DHCP) Configuration","")
        setupRow = db.getRowWhere ("dhcpv6c", "LogicalIfName='"..wanType.."'")
        if(setupRow["dhcpv6c.statelessMode"] == "1") then
            resTab.insertField (resultTab, "Stateless Address Auto Configuration", "Enabled")
            if(setupRow["dhcpv6c.prefixDelegation"] == "1") then
                resTab.insertField (resultTab, "Prefix Delegation", "Enabled")
            elseif(setupRow["dhcpv6c.prefixDelegation"] == "0") then
                resTab.insertField (resultTab, "Prefix Delegation", "Disabled")
            end
        elseif(setupRow["dhcpv6c.statelessMode"] == "0") then
            resTab.insertField (resultTab, "Stateful Address Auto Configuration", "Enabled")
        end
    end

    if (nimfTabel ["NimfConf.ConnectionType6"] == "pppoe6") then
        resTab.insertField (resultTab, "PPPoE IPv6 Configuration","")
        setupRow = db.getRowWhere ("Pppoe6", "LogicalIfName='"..wanType.."'")
        resTab.insertField (resultTab, "User Name", setupRow["Pppoe6.UserName"]or"")
        resTab.insertField (resultTab, "Password", setupRow["Pppoe6.Password"]or"")
        local dhcpOpt = ''
        if (setupRow["Pppoe6.Dhcpv6Opt"] == "0") then
            dhcpOpt = "DHCPv6 Disabled"
        elseif (setupRow["Pppoe6.Dhcpv6Opt"] == "1") then
            dhcpOpt = "DHCPv6 StatelessMode"
        elseif (setupRow["Pppoe6.Dhcpv6Opt"] == "2") then
            dhcpOpt = "DHCPv6 StatefulMode"
        elseif (setupRow["Pppoe6.Dhcpv6Opt"] == "3") then
            dhcpOpt = "DHCPv6 Prefix Delegation"
        end
        resTab.insertField (resultTab, "DHCPv6 Option", dhcpOpt)
        if (setupRow["Pppoe6.Dhcpv6Opt"] == "0") then
            resTab.insertField (resultTab, "Primary DNS Server", setupRow["Pppoe6.PrimaryDns"]or"")
            resTab.insertField (resultTab, "Secondary DNS Server", setupRow["Pppoe6.SecondaryDns"]or"")
        end
    end
    printLabel ("IPv6 "..wanType.." Setup")
    resTab.print (resultTab, 0)
    end

end

--
-- get the field values from the dbase table
function wanV6StatusGet(wanType)
    local resultTab = {}   
    local ConnectionType = ""
    local where = "AddressFamily = 10 and LogicalIfName = '"..wanType.."'"
    local nimfTabel = db.getRowWhere ("NimfConf", where)

    if (ipModeCfgInputVal()) then
    if (nimfTabel ["NimfConf.ConnectionType6"] == "ifStatic6") then
        ConnectionType = "Static IPv6"
    elseif (nimfTabel ["NimfConf.ConnectionType6"] == "dhcp6c") then
        ConnectionType = "Dynamic IPv6 (DHCP)"
    elseif (nimfTabel ["NimfConf.ConnectionType6"] == "pppoe6") then
        ConnectionType = "PPPoE IPv6"
    end
   
    --if (wanType ~= "WAN2") then 
        stat6Row = db.getRowWhere ("NimfStatus", "LogicalIfName = '"..wanType.."' and AddressFamily=10")
        resTab.insertField (resultTab, "IPv6 Connection Type", ConnectionType)
        resTab.insertField (resultTab, "IPv6 Connection State", stat6Row["NimfStatus.Nimfstatus"]or"")
    --end
    local wanType1 = wanType
    if(ConnectionType == "PPPoE IPv6") then
        wanType1 = wanType .. "PPPOE6"
    end
    stat6Rows = db.getRowsWhere ("ipAddressTable", "LogicalIfName='"..wanType1.."' and addressFamily=10")
    stat6Rows1 = db.getRowsWhere ("ipAddressTable", "LogicalIfName='"..wanType1.."' and addressFamily=10 AND ipAddress NOT LIKE 'fe80%%'")
    if(stat6Rows1 ~= nil) then
        stat6Rows = stat6Rows1
    end

    local i = 0
    local ip6Addr=""
    if(stat6Rows ~= nil) then
        for k,v in pairs(stat6Rows) do
            i = i + 1
            local row = stat6Rows[i]
            ip6Addr = ip6Addr ..row["ipAddressTable.ipAddress"] .."/"..row["ipAddressTable.ipv6PrefixLen"]
            if(i ~= #stat6Rows) then 
                ip6Addr = ip6Addr .. ", "
            end
        end
    end

    if(stat6Row ~= nil) then
    resTab.insertField (resultTab, "IPv6 Address", ip6Addr or"")
    stat6Row = db.getRowWhere ("defaultRouters", "LogicalIfName='"..wanType.."' and addressFamily=10")
    if(stat6Row ~= nil) then
    resTab.insertField (resultTab, "Default IPv6 Gateway", stat6Row["defaultRouters.nextHopGateway"]or"")
    end
    stat6Row = db.getRowWhere ("resolverConfig", "LogicalIfName='"..wanType.."' and addressFamily=10")
    if(stat6Row ~= nil) then
    resTab.insertField (resultTab, "Primary DNS Server", stat6Row["resolverConfig.nameserver1"]or"")
    resTab.insertField (resultTab, "Secondary DNS Server", stat6Row["resolverConfig.nameserver2"]or"")
    end
    end
    printLabel ("IPv6 "..wanType.." Status")
    resTab.print (resultTab, 0)
    end
end

--
-- This routine save WAN configuration
function nimf6CfgSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
   
    configRow["NimfConf.LogicalIfName"] = configRow["wanName"]
    configRow["wanName"] = nil
    errorFlag, statusCode = nimfView.ipv6WanConfig(configRow, configRow["NimfConf._ROWID_"], "edit")
	
	-- save db if no error
    if (errorFlag == "OK") then db.save() end
      statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    
    return errorFlag, statusMessage 
end

-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function nimf6CfgInit (args)
    local configRow = {}
    local rowId = -1 
    configRow = nimfView.ipv6WanConfigGet(args[1])
    configRow["wanName"] = args[1]
--    print (configRow ["NimfConf._ROWID_"], configRow["NimfConf.AddressFamily"], configRow["NimfConf.ConnectionType6"])
    return configRow["NimfConf._ROWID_"], configRow
end

function ipModeCfgInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 WAN Configurations.")
        return false
    end
    return true
end

function nimf6CfgInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 WAN Configurations.")
        return false
    end
    if (configRow["NimfConf.ConnectionType6"] == "ifStatic6") then
        if (configRow["ifStatic.StaticIp"] == "" or configRow["ifStatic.StaticIp"] == nil) then
            printCLIError("Enter valid static ipv6 address")
            return false
        end
        if (configRow["ifStatic.PrefixLength"] == "" or configRow["ifStatic.PrefixLength"] == nil) then
            printCLIError("Enter valid prefix length")
            return false
        end
        if (configRow["ifStatic.Gateway"] == "" or configRow["ifStatic.Gateway"] == nil) then
            printCLIError("Enter valid gateway ipv6 address")
            return false
        end
        if (configRow["ifStatic.PrimaryDns"] == "" or configRow["ifStatic.PrimaryDns"] == nil) then
            printCLIError("Enter valid primary ipv6 dns\n")
            return false
        end
        --if (configRow["ifStatic.SecondaryDns"] == "" or configRow["ifStatic.SecondaryDns"] == nil) then
            --printCLIError("Enter valid secondary ipv6 dns")
            --return false
        --end
    elseif (configRow["NimfConf.ConnectionType6"] == "dhcp6c") then
        if (configRow["dhcpv6c.statelessMode"] == "" or configRow["dhcpv6c.statelessMode"] == nil) then
            printCLIError("Enter valid Configuration Mode for ipv6 Dhcp")
            return false
        end
    elseif (configRow["NimfConf.ConnectionType6"] == "pppoe6") then
        if (configRow["Pppoe6.UserName"] == "" or configRow["Pppoe6.UserName"] == nil) then
            printCLIError("Enter valid pppoe user name")
            return false
        end
        if (configRow["Pppoe6.Password"] == "" or configRow["Pppoe6.Password"] == nil) then
            printCLIError("Enter valid pppoe password")
            return false
        end
        if (configRow["Pppoe6.Dhcpv6Opt"] == "" or configRow["Pppoe6.Dhcpv6Opt"] == nil) then
            printCLIError("Enter valid pppoe dhcpv6 option")
            return false
        end
        if (configRow["Pppoe6.Dhcpv6Opt"] == "0") then
            if (configRow["Pppoe6.PrimaryDns"] == "" or configRow["Pppoe6.PrimaryDns"] == nil) then
                printCLIError("Enter valid pppoe primary dns")
                return false
            end
        end
    end
    return true
end
--
-- This routine save WAN2 configuration
function nimf6Wan2CfgSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
   
    configRow["NimfConf.LogicalIfName"] = "WAN2"
    errorFlag, statusCode = nimfView.ipv6WanConfig(configRow, "1", "edit")
	
	-- save db if no error
    if (errorFlag == "OK") then db.save() end
      statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    
    return errorFlag, statusMessage 
end

-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function nimf6Wan2CfgInit (args)
    local configRow = {}
    local rowId = -1 
    configRow = nimfView.ipv6WanConfigGet("WAN2")
    print (configRow ["NimfConf._ROWID_"], configRow["NimfConf.AddressFamily"], 
           configRow["NimfConf.ConnectionType6"]) 
    return configRow["NimfConf._ROWID_"], configRow
end

function nimf6Wan2CfgInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 WAN Configurations.")
        return false
    end
    if (configRow["NimfConf.ConnectionType6"] == "ifStatic6") then
        if (configRow["ifStatic.StaticIp"] == "" or configRow["ifStatic.StaticIp"] == nil) then
            printCLIError("Enter valid static ipv6 address")
            return false
        end
        if (configRow["ifStatic.PrefixLength"] == "" or configRow["ifStatic.PrefixLength"] == nil) then
            printCLIError("Enter valid prefix length")
            return false
        end
        if (configRow["ifStatic.Gateway"] == "" or configRow["ifStatic.Gateway"] == nil) then
            printCLIError("Enter valid gateway ipv6 address")
            return false
        end
        if (configRow["ifStatic.PrimaryDns"] == "" or configRow["ifStatic.PrimaryDns"] == nil) then
            printCLIError("Enter valid primary ipv6 dns\n")
            return false
        end
        if (configRow["ifStatic.SecondaryDns"] == "" or configRow["ifStatic.SecondaryDns"] == nil) then
            printCLIError("Enter valid secondary ipv6 dns")
            return false
        end
    elseif (configRow["NimfConf.ConnectionType6"] == "dhcp6c") then
        if (configRow["dhcpv6c.statelessMode"] == "" or configRow["dhcpv6c.statelessMode"] == nil) then
            printCLIError("Enter valid Configuration Mode for ipv6 Dhcp")
            return false
        end
    end
    return true
end

function wanPort6CfgInit (args)
    local configRow = {}
	configRow = db.getRowWhere ("WanFailure", "LogicalIfName = '" .. args[1] .. "' AND AddressFamily = 10")
    configRow ["WanFailure.wanName"] = args[1]
    return 1, configRow
end

function wanPort6CfgSave (configRow)
    
    errorFlag = ""
    statusCode = ""
    statusMessage = ""
    
    errorFlag, statusCode = wan.wan6AdvConfigSet(configRow)
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    print(statusMessage)
    return errorFlag, statusMessage
end
--
--wanPortCfgInputVal
--
function wanPort6CfgInputVal (configRow)
   local opFlag = true

    if(configRow == {}) then
	return false
    end
    
    if (tonumber(configRow ["WanFailure.RetryTime"]) < 30 or tonumber(configRow ["WanFailure.RetryTime"]) > 999) then
        printCLIError("Enter retry_interval between (30 - 999) \n")
        return false
    end
    if (tonumber(configRow ["WanFailure.RetryAttempts"]) < 2 or tonumber(configRow ["WanFailure.RetryAttempts"]) > 999) then
        printCLIError("Enter retry_attempts between (2 - 999) \n")
        return false
    end

    return opFlag
end

function wanPort6CfgGet (args)
    local resultTab = {}
    printLabel (args[1] .. " Port Setup")
    configRow2 = db.getRowWhere("WanFailure", "LogicalIfName='" .. args[1] .. "' AND AddressFamily=10")
    
    resTab.insertField (resultTab, "PING IP",configRow2 ["WanFailure.PingIp"])
    resTab.insertField (resultTab, "Retry Interval", configRow2 ["WanFailure.RetryTime"] or"")
    resTab.insertField (resultTab, "Failover After", configRow2 ["WanFailure.RetryAttempts"] or"")
    resTab.print (resultTab, 0)
end

