--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### Mar 11, 2008
#### File: nimfView.lua
#### Description: NIMF Setup functions
#### Revisions:
01j,09oct13, ksr fixed SPR 39954
01i 26dec12, sig fixed SPR 36211
01h 11jul12, nik fixed SPR 33473
01g 09Jun12, nik fixed spr 32505
01f 09May12,sig fixed for spr#31488,spr#31438
01e 05Aug11, Smita P. Added nimfView.secAddrDelete, nimfView.secAddrConfig functions for SRX5308
01d 05Aug11, Smita P. Modified nimfView.config, nimfView.wanConfigGet, nimfView.wanStatusGet, nimfView.wanConfigSetWrap functions to support 4 WANS in SRX5308
01c 08jun10, bng Fix for spr#18447
01b 23apr10, bng Fix for spr#17757
01a 19Jun08, sdv Added ipv6 configuration functions
]]--

--************* Requires *************

require "teamf1lualib/ifStatic"
require "teamf1lualib/dhcpc"
require "teamf1lualib/pppoe"
require "teamf1lualib/pptp"
require "teamf1lualib/l2tp"
require "teamf1lualib/nat"
require "teamf1lualib/igmp"
require "teamf1lualib/nimf"
require "teamf1lualib/ifDev"

--************* Initial Code *************

--package NIMF Configuration
nimfView = {}

--************* Global Variables *************
logIfName = "WAN1"
addressFamily = "2"

--************* Functions *************
-- Wan Configuration
function nimfView.config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	logIfName = inputTable["NimfConf.LogicalIfName"]
    -- get wan Configuration in NimfConf
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
	
	-- Retrying for configuration in NimfConf.
    local retryIdx = 1
    while (NimfRow == nil and retryIdx <= 3) do
        NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        retryIdx = retryIdx + 1
    end
		
	-- return error if wan Configuration not found
    if (NimfRow == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end

    rowid = NimfRow["NimfConf._ROWID_"]

	db.beginTransaction() --begin transaction
	local valid = false
    local NimfConfTable = {}
    local wanConfTable = {}

	-- config NIMF
    NimfConfTable["NimfConf.LogicalIfName"] = logIfName
    NimfConfTable["NimfConf.RequireLogin"] = inputTable["Wan.IspLoginRequired"]
    NimfConfTable["NimfConf.IfStatus"] = "1"
    if (inputTable["Wan.IspLoginRequired"] == "0") then    -- dhcpc or ifStatic configuration
        if (inputTable["Wan.IsGetIpDyn"] == "1") then -- dhcpc
            NimfConfTable["NimfConf.ConnectionType"] = "dhcpc"
            wanConfTable["Dhcpc.LogicalIfName"] = logIfName
            wanConfTable["Dhcpc.GetDnsFromIsp"] = inputTable["Wan.GetDnsFromIsp"]
            if (inputTable["Wan.GetDnsFromIsp"] == "0") then
            	if (inputTable["Wan.PriDnsIpAddr"] ~= nil) then
                	wanConfTable["Dhcpc.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
                end
                if (inputTable["Wan.SecDnsIpAddr"] ~= nil) then
                	wanConfTable["Dhcpc.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
                end
            end
            wanConfTable["Dhcpc.vendorClassId"] = inputTable["Wan.vendorClassId"] or ''
			wanConfTable["Dhcpc.clientId"] =  inputTable["Wan.clientId"] or ''
			wanConfTable["Dhcpc.clientIdName"] = inputTable["Wan.clientIdName"] or ''
            if (inputTable["Wan.AccountName"] ~= nil and inputTable["Wan.AccountName"] ~= "") then
                wanConfTable["Dhcpc.HostName"] = inputTable["Wan.AccountName"]
            end
            if (db.existsRowWhere("Dhcpc", "LogicalIfName = '".. logIfName .. "'")) then
                valid = dhcpc.config (wanConfTable, db.getAttribute ("Dhcpc", "LogicalIfName", logIfName, "_ROWID_"), "edit")
            else
                valid = dhcpc.config (wanConfTable, "-1", "add")
            end
        else -- ifStatic
            NimfConfTable["NimfConf.ConnectionType"] = "ifStatic"
            wanConfTable["ifStatic.LogicalIfName"] = logIfName

            if (inputTable["Wan.IpAddr"] ~= nil and inputTable["Wan.SnetMask"] ~= nil) then
                local status = ifDev.ipSubnet(inputTable["Wan.IpAddr"],inputTable["Wan.SnetMask"])
                if (status ~= "OK") then
                    db.rollback()
                    return "ERROR", status
                end
            end
            if (inputTable["Wan.IpAddr"] ~= nil) then           
            	wanConfTable["ifStatic.StaticIp"] = inputTable["Wan.IpAddr"]
            end
            if (inputTable["Wan.SnetMask"] ~= nil) then
            	wanConfTable["ifStatic.NetMask"] = inputTable["Wan.SnetMask"]
            end
            if (inputTable["Wan.GwIpAddr"] ~= nil) then
            	wanConfTable["ifStatic.Gateway"] = inputTable["Wan.GwIpAddr"]
            end
            if (inputTable["Wan.PriDnsIpAddr"] ~= nil) then
            	wanConfTable["ifStatic.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
            end
            if (inputTable["Wan.SecDnsIpAddr"] ~= nil) then
            	wanConfTable["ifStatic.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
            end
            --- added for IPv6 support
            wanConfTable["ifStatic.AddressFamily"] = "2"
            if (db.existsRowWhere("ifStatic", "LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)) then
                valid = ifStatic.config (wanConfTable, db.getAttribute ("ifStatic", "LogicalIfName", logIfName, "_ROWID_"), "edit")
            else
                valid = ifStatic.config (wanConfTable, "-1", "add")
            end            
        end
    
    else -- pppoe or pptp or l2tp configuration
        if (inputTable["Wan.IspType"] == "1") then	-- pppoe connection
            NimfConfTable["NimfConf.ConnectionType"] = "pppoe"
            
            -- WAN Configuration having PPPoE Profiles support
            wanConfTable = db.getRowWhere ("Pppoe", "LogicalIfName = '".. logIfName .. "'")
            if (wanConfTable ~= nil) then
                -- Update old profile status
                if (wanConfTable["Pppoe.ProfileName"] ~= nil) then
                    local profileConfig = db.getRow ("PppoeProfile", "ProfileName", wanConfTable["Pppoe.ProfileName"])
                    if (profileConfig ~= nil) then
                        profileConfig["PppoeProfile.Status"] = "0"
                        valid = db.update ("PppoeProfile", profileConfig, profileConfig["PppoeProfile._ROWID_"])
                    else
                        valid = true
                    end
                else
                    valid = true
                end

                -- Update new/selected profile status
                if (valid) then
                    local profileConfig = db.getRow ("PppoeProfile", "ProfileName", inputTable["Wan.PppoeProfileName"])
                    -- Check if profile is already in use
                    if (profileConfig ~= nil and profileConfig["PppoeProfile.Status"] ~= "1") then
                        profileConfig["PppoeProfile.Status"] = "1"
                        profileConfig["PppoeProfile.UserName"] = inputTable["Wan.LoginName"]
                        if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then
                            profileConfig["PppoeProfile.Password"] = inputTable["Wan.LoginPassword"]
                        end
                        profileConfig["PppoeProfile.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]                   
                        if (inputTable["Wan.IdleTimeOutFlag"]  == "1") then
                            profileConfig["PppoeProfile.IdleTimeOutValue"] = inputTable["Wan.IdleTime"] 
                        end
                        profileConfig["PppoeProfile.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"] 
                        profileConfig["PppoeProfile.GetDnsFromIsp"] = inputTable["Wan.GetDnsFromIsp"]
                        if (inputTable["Wan.IsGetIpDyn"]  == "0") then
                            if (inputTable["Wan.StaticIp"] ~= nil) then
                                profileConfig["PppoeProfile.StaticIp"] = inputTable["Wan.StaticIp"]
                            else
                                profileConfig["PppoeProfile.StaticIp"] = util.joinFields(inputTable,"Wan","StaticIp", ".",4)
                            end
                            if (inputTable["Wan.NetMask"] ~= nil) then
                                profileConfig["PppoeProfile.NetMask"] = inputTable["Wan.NetMask"]
                            else
                                profileConfig["PppoeProfile.NetMask"] = util.joinFields(inputTable,"Wan","NetMask", ".",4)
                            end
                            if (profileConfig["PppoeProfile.StaticIp"] ~= nil and profileConfig["PppoeProfile.NetMask"] ~= nil) then
                                local status = ifDev.ipSubnet(profileConfig["PppoeProfile.StaticIp"],profileConfig["PppoeProfile.NetMask"])
                                if (status ~= "OK") then
                                    db.rollback()
                                    return "ERROR", status
                                end
                            end
                        end
                        if (inputTable["Wan.GetDnsFromIsp"]  == "0") then
                            if (inputTable["Wan.PrimaryDns"] ~= nil) then
                                profileConfig["PppoeProfile.PrimaryDns"] = inputTable["Wan.PrimaryDns"]
                            else
                                profileConfig["PppoeProfile.PrimaryDns"] = util.joinFields(inputTable,"Wan","PrimaryDns", ".",4)
                            end
                            if (inputTable["Wan.SecondaryDns"] ~= nil) then
                                profileConfig["PppoeProfile.SecondaryDns"] = inputTable["Wan.SecondaryDns"]
                            else
                                profileConfig["PppoeProfile.SecondaryDns"] = util.joinFields(inputTable,"Wan","SecondaryDns", ".",4)
                            end
                        end
                        profileConfig["PppoeProfile.ConResetStatus"] = inputTable["Wan.ConResetStatus"]	
                        if (inputTable["Wan.ConResetStatus"]  == "1") then
                            if(inputTable["Wan.ConResetHour"] ~= nil) then
                                profileConfig["PppoeProfile.ConResetHour"] = inputTable["Wan.ConResetHour"]
                            end
                            if(inputTable["Wan.ConResetMin"] ~= nil) then
                                profileConfig["PppoeProfile.ConResetMin"] = inputTable["Wan.ConResetMin"]
                            end
                            if(inputTable["Wan.ConResetDelay"] ~= nil) then
                                profileConfig["PppoeProfile.ConResetDelay"] = inputTable["Wan.ConResetDelay"]
                            end                   
                        end        	
                        valid = db.update ("PppoeProfile", profileConfig, profileConfig["PppoeProfile._ROWID_"])
                    else
                         valid = false
                    end
                end
			end
            
            if (valid) then
                local pppoeConfig = {}
                pppoeConfig["Pppoe.ProfileName"] = inputTable["Wan.PppoeProfileName"]
                pppoeConfig["Pppoe.LogicalIfName"] = logIfName            
        		pppoeConfig["Pppoe.UserName"] = inputTable["Wan.LoginName"]
                if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then
                	pppoeConfig["Pppoe.Password"] = inputTable["Wan.LoginPassword"]
            	end
	            if (inputTable["Wan.AccountName"] ~= nil) then            
	                pppoeConfig["Pppoe.AccountName"] = inputTable["Wan.AccountName"] 
	            end
		        if (inputTable["Wan.DomainName"] ~= nil) then            
		            pppoeConfig["Pppoe.DomainName"] = inputTable["Wan.DomainName"] 
		        end
	            pppoeConfig["Pppoe.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"] 
	            pppoeConfig["Pppoe.GetDnsFromIsp"] = inputTable["Wan.GetDnsFromIsp"] 
	            if (inputTable["Wan.IsGetIpDyn"]  == "0") then
                    if (inputTable["Wan.IpAddr"] ~= nil and inputTable["Wan.SnetMask"] ~= nil) then
                        local status = ifDev.ipSubnet(inputTable["Wan.IpAddr"],inputTable["Wan.SnetMask"])
                        if (status ~= "OK") then
                            db.rollback()
                            return "ERROR", status
                        end
                    end
	            	if (inputTable["Wan.IpAddr"] ~= nil) then
	               		pppoeConfig["Pppoe.StaticIp"] = inputTable["Wan.IpAddr"]
	               	end
	               	if (inputTable["Wan.SnetMask"] ~= nil) then
	               		pppoeConfig["Pppoe.NetMask"] = inputTable["Wan.SnetMask"]
	               	end
	            end
	            if (inputTable["Wan.GetDnsFromIsp"]  == "0") then
	            	if (inputTable["Wan.PriDnsIpAddr"] ~= nil) then
	               		pppoeConfig["Pppoe.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
	               	end
	               	if (inputTable["Wan.SecDnsIpAddr"] ~= nil) then
	               		pppoeConfig["Pppoe.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
	               	end
	            end
	            pppoeConfig["Pppoe.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"] 
	            if (inputTable["Wan.IdleTimeOutFlag"]  == "1") then
	                pppoeConfig["Pppoe.IdleTimeOutValue"] = inputTable["Wan.IdleTime"] 
	            end
	            pppoeConfig["Pppoe.ConResetStatus"] = inputTable["Wan.ConResetStatus"] 
	            if (inputTable["Wan.ConResetStatus"]  == "1") then
	            	pppoeConfig["Pppoe.ConResetHour"] = inputTable["Wan.ConResetHour"] 
	            	pppoeConfig["Pppoe.ConResetMin"] = inputTable["Wan.ConResetMin"] 
	            	pppoeConfig["Pppoe.ConResetDelay"] = inputTable["Wan.ConResetDelay"] 
	            end
	            if (db.existsRowWhere("Pppoe", "LogicalIfName = '".. logIfName .. "'")) then
	                valid = pppoe.config (pppoeConfig, db.getAttribute ("Pppoe","LogicalIfName",logIfName,"_ROWID_"), "edit")
	            else
	                valid = pppoe.config (pppoeConfig, "-1", "add")
	            end
            end
            
       elseif (inputTable["Wan.IspType"] == "2") then	--pptp connection
            NimfConfTable["NimfConf.ConnectionType"] = "pptp"
            wanConfTable["Pptp.LogicalIfName"] = logIfName            
            wanConfTable["Pptp.UserName"] = inputTable["Wan.LoginName"]
            if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then
                wanConfTable["Pptp.Password"] = inputTable["Wan.LoginPassword"]
            end
            wanConfTable["Pptp.MppeEncryptSupport"] = inputTable["Wan.MppeEncryptSupport"]
            if (inputTable["Wan.MyIpAddr"] ~= nil) then
            	wanConfTable["Pptp.MyIp"] = inputTable["Wan.MyIpAddr"]
            end
            if (inputTable["Wan.SvrIpAddr"] ~= nil) then            
                wanConfTable["Pptp.ServerIp"] = inputTable["Wan.SvrIpAddr"]
            end
            wanConfTable["Pptp.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"]
            wanConfTable["Pptp.GetDnsFromIsp"] = inputTable["Wan.GetDnsFromIsp"]
            if (inputTable["Wan.IsGetIpDyn"] == "0") then
                if (inputTable["Wan.IpAddr"] ~= nil and inputTable["Wan.SnetMask"] ~= nil) then
                    local status = ifDev.ipSubnet(inputTable["Wan.IpAddr"],inputTable["Wan.SnetMask"])
                    if (status ~= "OK") then
                        db.rollback()
                        return "ERROR", status
                    end
                end
            	if (inputTable["Wan.IpAddr"] ~= nil) then
                	wanConfTable["Pptp.StaticIp"] = inputTable["Wan.IpAddr"]
                end
                if (inputTable["Wan.SnetMask"] ~= nil) then
                	wanConfTable["Pptp.NetMask"] = inputTable["Wan.SnetMask"]
                end
            end
            if (inputTable["Wan.GetDnsFromIsp"] == "0") then
            	if (inputTable["Wan.PriDnsIpAddr"] ~= nil) then
                	wanConfTable["Pptp.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
                end
                if (inputTable["Wan.SecDnsIpAddr"] ~= nil) then
                	wanConfTable["Pptp.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
                end
            end
            wanConfTable["Pptp.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
            if (inputTable["Wan.IdleTimeOutFlag"] == "1") then
                wanConfTable["Pptp.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
            end
            if (db.existsRowWhere("Pptp", "LogicalIfName = '".. logIfName .. "'")) then
                valid = pptp.config (wanConfTable, db.getAttribute ("Pptp","LogicalIfName",logIfName,"_ROWID_"), "edit")
            else
                valid = pptp.config (wanConfTable, "-1", "add")
            end
        end
    end

    if (valid) then
        -- Check the previous WAN ISP Type, If it is PPPoE then
        -- set the Status to '0' for the PPPoE Profile if WAN Supports
        -- PPPoE Prifile selection
        local oldConnType = db.getAttribute ("NimfConf", "_ROWID_", rowid, "ConnectionType")
        if (oldConnType == "pppoe" and NimfConfTable["NimfConf.ConnectionType"] ~= "pppoe") then
            local pppoeConfig = db.getRowWhere ("Pppoe", "LogicalIfName = '".. logIfName .. "'")
            if (pppoeConfig ~= nil and pppoeConfig["Pppoe.ProfileName"] ~= nil) then
                -- Update old profile status
                local profileConfig = db.getRow ("PppoeProfile", "ProfileName", pppoeConfig["Pppoe.ProfileName"])
                if (profileConfig ~= nil) then
                    profileConfig["PppoeProfile.Status"] = "0"
                    valid = db.update ("PppoeProfile", profileConfig, profileConfig["PppoeProfile._ROWID_"])
                else
                    valid = true
                end
            else
                valid = true
            end
        else
            valid = true
        end
    end

    -- Set the wan advanced configuration
    NimfConfTable["NimfConf.AccountName"] = inputTable["Wan.AccountName"] or ''
    NimfConfTable["NimfConf.DomainName"] = inputTable["Wan.DomainName"] or ''
    NimfConfTable["NimfConf.ConResetStatus"] = inputTable["Wan.ConResetStatus"]
    if (inputTable["Wan.ConResetStatus"] == "1") then
        NimfConfTable["NimfConf.ConResetHour"] = inputTable["Wan.ConResetHour"] or ''
        NimfConfTable["NimfConf.ConResetMin"] = inputTable["Wan.ConResetMin"] or ''
        NimfConfTable["NimfConf.ConResetDelay"] = inputTable["Wan.ConResetDelay"] or ''
    end
    
	if (valid) then
        valid = nimf.config (NimfConfTable, rowid, "edit")
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end
end

-- Wan Configuration Get
function nimfView.wanConfigGet (wanLogicalIfName)
    local outTable = {}
    logIfName = wanLogicalIfName
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)    
    if (NimfRow ~= nil) then
        outTable["Wan.IspLoginRequired"] =  NimfRow["NimfConf.RequireLogin"]
        outTable["Wan.AccountName"] = NimfRow["NimfConf.AccountName"] or ''
        outTable["Wan.DomainName"] = NimfRow["NimfConf.DomainName"] or ''
        if (outTable["Wan.IspLoginRequired"] == "1") then
        	if (NimfRow["NimfConf.ConnectionType"] == "pppoe") then
        		outTable["Wan.IspType"] = "1"
        	elseif (NimfRow["NimfConf.ConnectionType"] == "pptp") then
        		outTable["Wan.IspType"] = "2"
        	else
        	    outTable["Wan.IspType"] = "2"
        	end
        --else
        	--outTable["Wan.IspType"] = "2"
        end
        outTable["Wan.ConResetStatus"] = NimfRow["NimfConf.ConResetStatus"]
        outTable["Wan.ConResetHour"] = NimfRow["NimfConf.ConResetHour"]
        outTable["Wan.ConResetMin"] = NimfRow["NimfConf.ConResetMin"]
        outTable["Wan.ConResetDelay"] = NimfRow["NimfConf.ConResetDelay"]
        outTable["Wan.vendorClassId"] = '0'
		outTable["Wan.clientId"] = '0'
		outTable["Wan.clientIdName"] = ''
        if (NimfRow["NimfConf.ConnectionType"] == "dhcpc") then
            outTable["Wan.IsGetIpDyn"] = "1"
            local ispConfigRow = db.getRowWhere ("Dhcpc","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then  
            	outTable["Wan.vendorClassId"] = ispConfigRow["Dhcpc.vendorClassId"] or ''
        		outTable["Wan.clientId"] = ispConfigRow["Dhcpc.clientId"] or ''
        		outTable["Wan.clientIdName"] = ispConfigRow["Dhcpc.clientIdName"] or ''              
                outTable["Wan.GetDnsFromIsp"] = ispConfigRow["Dhcpc.GetDnsFromIsp"]
                if (ispConfigRow["Dhcpc.GetDnsFromIsp"] == "0") then
                    outTable["Wan.PriDnsIpAddr"] = util.split(ispConfigRow["Dhcpc.PrimaryDns"], ".")
                    outTable["Wan.SecDnsIpAddr"] = util.split(ispConfigRow["Dhcpc.SecondaryDns"], ".")
                end
            end
        elseif (NimfRow["NimfConf.ConnectionType"] == "ifStatic") then
            outTable["Wan.IsGetIpDyn"] = "0"
            outTable["Wan.GetDnsFromIsp"] = "0"            
            local ispConfigRow = db.getRowWhere ("ifStatic","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               outTable["Wan.IpAddr"] = util.split(ispConfigRow["ifStatic.StaticIp"], ".")    
               outTable["Wan.SnetMask"] = util.split(ispConfigRow["ifStatic.NetMask"], ".")                  
               outTable["Wan.GwIpAddr"] = util.split(ispConfigRow["ifStatic.Gateway"], ".")    
               --begin: split gateway address
               outTable["Wan.Gateway1"] = outTable["Wan.GwIpAddr"][1]
               outTable["Wan.Gateway2"] = outTable["Wan.GwIpAddr"][2]
               outTable["Wan.Gateway3"] = outTable["Wan.GwIpAddr"][3]
               outTable["Wan.Gateway4"] = outTable["Wan.GwIpAddr"][4]
               -- end: split gateway address
               outTable["Wan.PriDnsIpAddr"] = util.split(ispConfigRow["ifStatic.PrimaryDns"], ".")     
               outTable["Wan.SecDnsIpAddr"] = util.split(ispConfigRow["ifStatic.SecondaryDns"], ".")
            end
        elseif (NimfRow["NimfConf.ConnectionType"] == "pppoe") then
            local ispConfigRow = db.getRowWhere ("Pppoe","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               outTable["Wan.PppoeProfileName"] = ispConfigRow["Pppoe.ProfileName"] or ''
               outTable["Wan.LoginName"] = ispConfigRow["Pppoe.UserName"]
               outTable["Wan.LoginPassword"] = util.mask(ispConfigRow["Pppoe.Password"])               
               
               --Begin: Setting Connection Reset
               outTable["Wan.ConResetStatus"] = ispConfigRow["Pppoe.ConResetStatus"]
               if(outTable["Wan.ConResetStatus"] == "1") then
                   outTable["Wan.ConResetHour"] = ispConfigRow["Pppoe.ConResetHour"]
                   outTable["Wan.ConResetMin"] = ispConfigRow["Pppoe.ConResetMin"]
                   outTable["Wan.ConResetDelay"] = ispConfigRow["Pppoe.ConResetDelay"]
               end
               --End: Setting Connection Reset               
               outTable["Wan.IsGetIpDyn"] = ispConfigRow["Pppoe.GetIpFromIsp"]               
               outTable["Wan.GetDnsFromIsp"] = ispConfigRow["Pppoe.GetDnsFromIsp"]
               if (ispConfigRow["Pppoe.GetIpFromIsp"] == "0") then
                   outTable["Wan.IpAddr"] = util.split(ispConfigRow["Pppoe.StaticIp"], ".")
                   outTable["Wan.SnetMask"] = util.split(ispConfigRow["Pppoe.NetMask"], ".")
               end
               if (ispConfigRow["Pppoe.GetDnsFromIsp"] == "0") then                   
                   outTable["Wan.PriDnsIpAddr"] = util.split(ispConfigRow["Pppoe.PrimaryDns"], ".")
                   outTable["Wan.SecDnsIpAddr"] = util.split(ispConfigRow["Pppoe.SecondaryDns"], ".")
               end
               outTable["Wan.IdleTimeOutFlag"] = ispConfigRow["Pppoe.IdleTimeOutFlag"]
               if (ispConfigRow["Pppoe.IdleTimeOutFlag"] == "1") then
                   outTable["Wan.IdleTime"] = ispConfigRow["Pppoe.IdleTimeOutValue"]                   
               end
            end
        elseif (NimfRow["NimfConf.ConnectionType"] == "pptp") then
            local ispConfigRow = db.getRowWhere ("Pptp","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               outTable["Wan.LoginName"] = ispConfigRow["Pptp.UserName"]
               outTable["Wan.LoginPassword"] = util.mask(ispConfigRow["Pptp.Password"])     
               
               outTable["Wan.MyIpAddr"] = util.split(ispConfigRow["Pptp.MyIp"], ".")
               --begin: split MyIp address
               outTable["Wan.MyIp1"] = outTable["Wan.MyIpAddr"][1]
               outTable["Wan.MyIp2"] = outTable["Wan.MyIpAddr"][2]
               outTable["Wan.MyIp3"] = outTable["Wan.MyIpAddr"][3]
               outTable["Wan.MyIp4"] = outTable["Wan.MyIpAddr"][4]
               -- end: split MY Ip address              
               
               outTable["Wan.SvrIpAddr"] = util.split(ispConfigRow["Pptp.ServerIp"], ".")
               --begin: split MyIp address
               outTable["Wan.ServerIp1"] = outTable["Wan.SvrIpAddr"][1]
               outTable["Wan.ServerIp2"] = outTable["Wan.SvrIpAddr"][2]
               outTable["Wan.ServerIp3"] = outTable["Wan.SvrIpAddr"][3]
               outTable["Wan.ServerIp4"] = outTable["Wan.SvrIpAddr"][4]
               -- end: split MY Ip address
               
               outTable["Wan.IsGetIpDyn"] = ispConfigRow["Pptp.GetIpFromIsp"]
               outTable["Wan.GetDnsFromIsp"] = ispConfigRow["Pptp.GetDnsFromIsp"]     
               
			   if (ispConfigRow["Pptp.GetIpFromIsp"] == "0") then
                   outTable["Wan.IpAddr"] = util.split(ispConfigRow["Pptp.StaticIp"], ".")
                   outTable["Wan.SnetMask"] = util.split(ispConfigRow["Pptp.NetMask"], ".")
               end
               if (ispConfigRow["Pptp.GetDnsFromIsp"] == "0") then                  
                   outTable["Wan.PriDnsIpAddr"] = util.split(ispConfigRow["Pptp.PrimaryDns"], ".")
                   outTable["Wan.SecDnsIpAddr"] = util.split(ispConfigRow["Pptp.SecondaryDns"], ".")
               end
               outTable["Wan.IdleTimeOutFlag"] = ispConfigRow["Pptp.IdleTimeOutFlag"]
               if (ispConfigRow["Pptp.IdleTimeOutFlag"] == "1") then
                   outTable["Wan.IdleTime"] = ispConfigRow["Pptp.IdleTimeOutValue"]                   
               end
            end
        end
        if (outTable["Wan.GetDnsFromIsp"] == "0") then
        	--begin: split primary dns ip
            outTable["Wan.PrimaryDns1"] = outTable["Wan.PriDnsIpAddr"][1]
            outTable["Wan.PrimaryDns2"] = outTable["Wan.PriDnsIpAddr"][2]
            outTable["Wan.PrimaryDns3"] = outTable["Wan.PriDnsIpAddr"][3]
            outTable["Wan.PrimaryDns4"] = outTable["Wan.PriDnsIpAddr"][4]
            -- end: split primary dns ip
            
            --begin: split secondary dns ip
            outTable["Wan.SecondaryDns1"] = outTable["Wan.SecDnsIpAddr"][1]
            outTable["Wan.SecondaryDns2"] = outTable["Wan.SecDnsIpAddr"][2]
            outTable["Wan.SecondaryDns3"] = outTable["Wan.SecDnsIpAddr"][3]
            outTable["Wan.SecondaryDns4"] = outTable["Wan.SecDnsIpAddr"][4]
            -- end: split secondary dns ip
        end
        if (outTable["Wan.IsGetIpDyn"] == "0") then
        	--begin: split ip address           
           outTable["Wan.StaticIp1"] = outTable["Wan.IpAddr"][1]
           outTable["Wan.StaticIp2"] = outTable["Wan.IpAddr"][2]
           outTable["Wan.StaticIp3"] = outTable["Wan.IpAddr"][3]
           outTable["Wan.StaticIp4"] = outTable["Wan.IpAddr"][4]
           -- end: split split ip address
           
           --begin: split subnet mask address
           outTable["Wan.NetMask1"] = outTable["Wan.SnetMask"][1]
           outTable["Wan.NetMask2"] = outTable["Wan.SnetMask"][2]
           outTable["Wan.NetMask3"] = outTable["Wan.SnetMask"][3]
           outTable["Wan.NetMask4"] = outTable["Wan.SnetMask"][4]
           -- end: split subnet mask address
        end
    end

    return outTable
end

-- Wan Advanced Configuration
function nimfView.advConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    -- return error if wan Configuration not found
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)

    -- Retrying for configuration in NimfConf.
    local retryIdx = 1
    while (NimfRow == nil and retryIdx <= 3) do
        NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        retryIdx = retryIdx + 1
    end    

    if (NimfRow == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end

    rowid = NimfRow["NimfConf._ROWID_"]

	db.beginTransaction() --begin transaction
	local valid = false
    local wanConfTable = {}

	-- config Advanced NIMF Configurations
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "'")
    if (NimfRow ~= nil) then
        NimfRow["NimfConf.UseDefaultMtu"] = inputTable["Wan.DefMtu"]
        NimfRow["NimfConf.SetMacAddress"] = inputTable["Wan.UseMac"]
        NimfRow["NimfConf.PortSpeed"] = inputTable["Wan.PortSpeed"]        
        if (inputTable["Wan.DefMtu"] == "0") then
            NimfRow["NimfConf.MtuSize"] = inputTable["Wan.MtuSize"]
        else
            if (NimfRow["NimfConf.RequireLogin"] == "0") then
                NimfRow["NimfConf.MtuSize"] = "1500"
            else
                NimfRow["NimfConf.MtuSize"] = "1492"
            end
        end
        if (inputTable["Wan.UseMac"] == "2") then
            NimfRow["NimfConf.MacAddress"] = inputTable["Wan.MacAddr"]
        end
        NimfRow["NimfConf.AddressFamily"] = "2"
        valid = nimf.config (NimfRow, rowid, "edit")
        if (valid == false) then
    		db.rollback()
	    	return "ERROR", "NIMF_CONFIG_FAILED"            
        end
    end
	
    -- setting NIMF Mode / NAT
    if (inputTable["NatTable.Enable"] ~= nil) then
        if (valid) then
            local natTable = {}
            natTable["NatTable.Enable"] = inputTable["NatTable.Enable"]
            valid = nat.config (natTable, "1", "edit")
        end
    end

    -- setting IGMP Proxy Settings.
    if (inputTable["Igmp.IgmpEnable"] ~= nil) then
        if (valid) then
            local igmpTable = {}
            igmpTable ["Igmp.IgmpEnable"] = inputTable["Igmp.IgmpEnable"]
            valid = igmp.config (igmpTable, "1", "edit")
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end
end

-- Advanced Wan Configuration Get
function nimfView.wanAdvConfigGet ()
    local outTable = {}
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
    if (NimfRow ~= nil) then
    	outTable["Wan.DefMtu"] = NimfRow["NimfConf.UseDefaultMtu"] or ''
        outTable["Wan.MtuSize"] = NimfRow["NimfConf.MtuSize"] or ''
        outTable["Wan.UseMac"] = NimfRow["NimfConf.SetMacAddress"] or ''
        outTable["Wan.PortSpeed"] = NimfRow["NimfConf.PortSpeed"] or '' 
        outTable["Wan.MacAddr"] = NimfRow["NimfConf.MacAddress"] or ''
        if (NimfRow["NimfConf.RequireLogin"] == "0") then
            outTable["Wan.MtuSizeMin"] = "68"
            outTable["Wan.MtuSizeMax"] = "1500"
        else
            outTable["Wan.MtuSizeMin"] = "128"
            outTable["Wan.MtuSizeMax"] = "1492"
        end        
    end

    return outTable
end

function nimfView.connResetConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"                
    end    

    db.beginTransaction() --begin transaction
    local valid = false
    valid = db.update("scheduleReboot", inputTable, rowid)

    -- return    
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "Scheduled reboot configuration failed"
    end
end

-- Wan Status Get
function nimfView.wanStatusGet ()
    local outTable = {}
    local wanStatusRow = nil
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
    if (NimfRow ~= nil) then
        local connectionType = NimfRow["NimfConf.ConnectionType"]
        if (addressFamily == "10") then --ipv6 address
            connectionType = NimfRow["NimfConf.ConnectionType6"]
        end
        if (connectionType == "dhcpc") then
            outTable["Wan.ConnectionType"] = "Dynamic IP (DHCP)"
            local dhcpStatus = db.getRowWhere ("DhcpStatus","LogicalIfName = '" .. logIfName .. "'")
            if (dhcpStatus ~= nil) then
                outTable["Dhcpc.DhcpServerIp"] = dhcpStatus["DhcpStatus.DhcpServerIp"]
                outTable["Dhcpc.LeaseTime"] = dhcpStatus["DhcpStatus.LeaseTime"]
                outTable["Dhcpc.LeaseObtained"] = dhcpStatus["DhcpStatus.LeaseObtained"]                
            end
        elseif (connectionType == "dhcp6c") then
            outTable["Wan.ConnectionType"] = "Dynamic IP (DHCP)"
            outTable["Wan.IPVersion"] = "ipv6"
        elseif (connectionType == "ifStatic" or connectionType == "ifStatic6") then
            outTable["Wan.ConnectionType"] = "Static IP"
        elseif (connectionType == "pppoe") then
            outTable["Wan.ConnectionType"] = "PPPoE (Username/Password)"
        elseif (connectionType == "pptp") then
            outTable["Wan.ConnectionType"] = "PPTP (Username/Password)"
        elseif (connectionType == "pppoe6") then
            outTable["Wan.ConnectionType"] = "PPPoE over IPv6"
        end

        if (NimfRow["NimfConf.IfStatus"] == "0") then
            outTable["Wan.operation"] = "Connect"
        elseif (NimfRow["NimfConf.IfStatus"] == "1") then
            outTable["Wan.operation"] = "Disconnect"            
        end
    end

    wanStatusRow = db.getRowWhere ("NimfStatus","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
    if (wanStatusRow ~= nil) then
        outTable["Wan.ConnectionState"] = wanStatusRow["NimfStatus.Nimfstatus"] or ""
        outTable["Wan.ConnectionTime"] =  wanStatusRow["NimfStatus.WanUpTime"]  or ""
        outTable["Wan.LinkState"] = wanStatusRow["NimfStatus.Linkstatus"] or ""
    end

    if (addressFamily == "10") then
        local wan6Rows = {}
        if ( outTable["Wan.ConnectionType"] == "PPPoE over IPv6") then
            wan6Rows = db.getRowsWhere ("ipAddressTable","LogicalIfName = '" .. logIfName .. "PPPOE6' and AddressFamily = " .. addressFamily)
        else
            wan6Rows = db.getRowsWhere ("ipAddressTable","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        end
        local gwRows = db.getRowsWhere ("defaultRouters","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        local nsRows = db.getRowsWhere ("resolverConfig","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)

        outTable["Wan.IpAddr"] = ""
        outTable["Wan.Gateway"] = ""
        outTable["Wan.DNS"] = ""
        outTable["Wan.SecDNS"] = ""
        local i = 0
        for k,v in pairs(wan6Rows) do
            i = i + 1
            local row = wan6Rows[i]
            outTable["Wan.IpAddr"] = outTable["Wan.IpAddr"] .. row["ipAddressTable.ipAddress"] .. "/" .. row["ipAddressTable.ipv6PrefixLen"]
            if(i ~= #wan6Rows) then
                outTable["Wan.IpAddr"] = outTable["Wan.IpAddr"] .. ", "
			end
        end

        i = 0
        for k,v in pairs(gwRows) do
            i = i + 1
            local row = gwRows[i]
            outTable["Wan.Gateway"] = outTable["Wan.Gateway"] .. row["defaultRouters.nextHopGateway"] or ''
            if(i ~= #gwRows) then
                outTable["Wan.Gateway"] = outTable["Wan.Gateway"] .. ", "
			end
        end

        i = 0
        for k,v in pairs(nsRows) do
            i = i + 1
            local row = nsRows[i]
            outTable["Wan.DNS"] = outTable["Wan.DNS"] .. row["resolverConfig.nameserver1"] or ''
            if(i ~= #nsRows) then
                outTable["Wan.DNS"] = outTable["Wan.DNS"] .. ", "
			end
        end

        i = 0
        for k,v in pairs(nsRows) do
            i = i + 1
            local row = nsRows[i]
            outTable["Wan.SecDNS"] = outTable["Wan.SecDNS"] .. row["resolverConfig.nameserver2"] or ''
            if(i ~= #nsRows) then
                outTable["Wan.SecDNS"] = outTable["Wan.SecDNS"] .. ", "
			end
        end

    else
        wanStatusRow = db.getRowWhere ("ipAddressTable","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        local gwStatusRow = db.getRowWhere ("defaultRouters","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        local dnsStatusRow = db.getRowWhere ("resolverConfig","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        if (wanStatusRow ~= nil) then
            outTable["Wan.IpAddr"] = wanStatusRow["ipAddressTable.ipAddress"]
            outTable["Wan.SnetMask"] = wanStatusRow["ipAddressTable.subnetMask"]
        end        
        if (gwStatusRow ~= nil) then
            outTable["Wan.Gateway"] = gwStatusRow["defaultRouters.nextHopGateway"]      
        end
        if (dnsStatusRow ~= nil) then        
            outTable["Wan.DNS"] = dnsStatusRow["resolverConfig.nameserver1"]
            outTable["Wan.SecDNS"] = dnsStatusRow["resolverConfig.nameserver2"]
        end
    end

    local wanState = db.getAttribute ("networkInterface", "LogicalIfName", logIfName, "IfStatus") or ''
    if (wanState ~= '') then
        if (wanState == "1" or wanState == "3" or wanState == "2") then
            if (logIfName == 'WAN1' and wanState == "1") then
                if (addressFamily == "10") then
                    outTable["Wan.WanState"] = "DOWN"
                else
                    outTable["Wan.WanState"] = "UP"
                end
            elseif (logIfName == 'WAN1' and wanState == "2") then
                if (addressFamily == "2") then
                    outTable["Wan.WanState"] = "DOWN"
                else
                    outTable["Wan.WanState"] = "UP"
                end
            elseif (logIfName == 'WAN1' and wanState == "3") then
                outTable["Wan.WanState"] = "UP"
            else
                outTable["Wan.WanState"] = "UP"
            end
        elseif (wanState == "0") then
            outTable["Wan.WanState"] = "DOWN"
        end
    end

	return outTable
end

-- Wan Disable/Enable
function nimfView.wanStatusChange (operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    local valid = false
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)

    -- Retrying for configuration in NimfConf.
    local retryIdx = 1
    while (NimfRow == nil and retryIdx <= 3) do
        NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        retryIdx = retryIdx + 1
    end

    if (NimfRow == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end

    local rowid = NimfRow["NimfConf._ROWID_"]
    if (NimfRow ~= nil) then
        local nimfInTable = {}
        if (operation == "Enable") then
            nimfInTable["NimfConf.IfStatus"] = "1"
        elseif (operation == "Disable") then
            nimfInTable["NimfConf.IfStatus"] = "0"            
        end
        valid = nimf.config (nimfInTable, rowid, "edit")
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end    
end

-- DHCP Release/Renew
function nimfView.dhcpStatusChange (operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    local valid = false
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)

    -- Retrying for configuration in NimfConf.
    local retryIdx = 1
    while (NimfRow == nil and retryIdx <= 3) do
        NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        retryIdx = retryIdx + 1
    end

    if (NimfRow == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end

    local rowid = NimfRow["NimfConf._ROWID_"]
    if (addressFamily == "10") then
        valid = true;
    else
        if (db.existsRowWhere("Dhcpc", "LogicalIfName = '".. logIfName .. "'")) then
            local dhcpcInTable = {}
            dhcpcInTable["Dhcpc.ReleaseLease"] = "1"
            valid = dhcpc.config (dhcpcInTable, db.getAttribute ("Dhcpc","LogicalIfName",logIfName,"_ROWID_"), "edit")
        end
    end

    -- Dummy update the NimfConf Table to trigger the event handler.
    if (valid) then
        if (NimfRow ~= nil) then
            if (operation == "release") then
                local nimfInTable = {}
                nimfInTable["NimfConf.IfStatus"] = "0"
                valid = nimf.config (nimfInTable, rowid, "edit")
            elseif (operation == "renew") then
                local nimfInTable = {}
                nimfInTable["NimfConf.IfStatus"] = "1"
                valid = nimf.config (nimfInTable, rowid, "edit")
            end
        else
            valid = false
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end    
end

-- WAN Configuration Get Wrapper
function nimfView.wanConfigGetWrap (wanLogicalIfName)
    local wanConfig = {}
    wanConfig = nimfView.wanConfigGet (wanLogicalIfName)
    return wanConfig
end

-- WAN Configuration Set Wrapper
function nimfView.wanConfigSetWrap (inputTable, rowid, operation)
    if (inputTable["Wan.IspType"] == "1") then
    	local where = "AddressFamily = 2"
        local idleTimeOutFlag = db.getAttribute("PppoeProfile","ProfileName",inputTable["Wan.PppoeProfileName"],"IdleTimeOutFlag")
        local rows = db.getRowWhere("WanMode", where)
        local wanMode = rows["WanMode.Wanmode"] or ''
        if (idleTimeOutFlag == "1" and wanMode == "1" ) then 
            return "ERROR", "LOAD_BALANCE_ERROR"
        end
    end
    return nimfView.config (inputTable, rowid, operation)
end

function nimfView.wanStatusChangeWrap (operation, wanLogicalIfName, wanAddressFamily)
    logIfName =  wanLogicalIfName or 'WAN1'
    addressFamily = wanAddressFamily or '2'
    return nimfView.wanStatusChange (operation)
end

function nimfView.dhcpStatusChangeWrap (operation, wanLogicalIfName, wanAddressFamily)
    logIfName =  wanLogicalIfName or 'WAN1'
    addressFamily = wanAddressFamily or '2'
    return nimfView.dhcpStatusChange (operation)
end

function nimfView.wanStatusGetWrap (wanLogicalIfName, wanAddressFamily)
    logIfName =  wanLogicalIfName or 'WAN1'
    addressFamily = wanAddressFamily or '2'    
    return nimfView.wanStatusGet ()
end

function nimfView.portModeConfig (inputTable, rowid, operation)
	-- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false

    local boardName = db.getAttribute ("unitName","_ROWID_","1","unitName")
    if (boardName ~= nil and boardName ~= "WRVS5110") then
        boardName = true
    else
        boardName = false
    end

    if (boardName and (inputTable["ConfigPort.LogicalIfName"] ~= nil and inputTable["ConfigPort.LogicalIfName"] ~= "LAN")) then
        -- check that vlanId:4094(Reserved for WAN/DMZ) is there in vlan or not
        local isExistsVlan = db.existsRow ("vlan", "vlanId", "4094")
        if (isExistsVlan) then
		    db.rollback()
    		return "ERROR", "VLAN_4094_EXISTS"        
        end
        -- check that PVID:4094(Reserved for WAN/DMZ) is there in VlanPortMgmt or not
        local isRows = db.getRowsWhere ("VlanPortMgmt", "DefaultPVID = 4094 and PortName != 'OptionalPort'")
        if (#isRows ~= 0) then
		    db.rollback()
    		return "ERROR", "PORT_VLAN_PVID_4094_EXISTS"        
        end

        -- Enable vlan Status if config Port set to WAN/DMZ
        --local vlanState = {}
        --vlanState["VlanStatus.VlanState"] = "1"
        --valid = db.update("VlanStatus", vlanState, "1")
        valid = true
    else
        valid = true
    end

    --default config for optional port is wan.
    local optPortRowid = 3
    local IfgroupId = 2
    if (inputTable["ConfigPort.LogicalIfName"] == "DMZ")  then
       IfgroupId = 4
    elseif (inputTable["ConfigPort.LogicalIfName"] == "LAN" ) then
        IfgroupId = 1
    end
    if (valid) then
    	local isExists = db.existsRow ("ConfigPort", "_ROWID_", rowid)
        if (isExists) then
            local ifGrpInfo = {}
            ifGrpInfo["DefaultIfGrpInfo.LogicalIfName"] = inputTable["ConfigPort.LogicalIfName"]
            ifGrpInfo["DefaultIfGrpInfo.IfGroupId"] = IfgroupId
            valid = db.update("DefaultIfGrpInfo", ifGrpInfo, optPortRowid)
            if(valid) then
                valid = db.update("ConfigPort", inputTable, rowid)
            end
        else
            valid = db.insert("ConfigPort", inputTable)        
        
        end
    end

    -- Disable dhcp server on DMZ interface
    -- if configurable port not equal to DMZ
    local dmzDhcpConfig = db.getRow ("dhcp", "logicalIfName", "DMZ")
    if (valid and dmzDhcpConfig ~= nil) then
        if (inputTable["ConfigPort.LogicalIfName"] ~= "DMZ" and dmzDhcpConfig["dhcp.dhcpEnabled"] == "1") then
            dmzDhcpConfig["dhcp.oldState"] = "1"
            dmzDhcpConfig["dhcp.dhcpEnabled"] = "0"
        elseif (inputTable["ConfigPort.LogicalIfName"] == "DMZ" and dmzDhcpConfig["dhcp.oldState"] == "1") then
            dmzDhcpConfig["dhcp.oldState"] = "0"
            dmzDhcpConfig["dhcp.dhcpEnabled"] = "1"
        end
        valid = db.update("dhcp", dmzDhcpConfig, dmzDhcpConfig["dhcp._ROWID_"])
    end

	-- return    
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_MODE_SET_FAILED"
	end    
end

function nimfView.wanModeConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end

    if (inputTable["WanMode.Wanmode"] ~= nil and inputTable["WanMode.Wanmode"] == "1") then
        local type1=db.getRowWhere("NimfConf","LogicalIfName = 'WAN1' AND  AddressFamily = 2")
        local connection1 = type1["NimfConf.ConnectionType"]
        if((connection1 == "l2tp") or (connection1 == "pptp")) then 
            idle1=db.getAttribute(connection1,"LogicalIfName","WAN1","IdleTimeOutFlag")
        elseif((connection1 == "pppoe")) then
            profile1=db.getAttribute("pppoe","LogicalIfName","WAN1","ProfileName")
            idle1=db.getAttribute("PppoeProfile","ProfileName",profile1,"IdleTimeOutFlag")
        else
            idle1="0"
        end

        local type2=db.getRowWhere("NimfConf","LogicalIfName = 'WAN2' AND  AddressFamily = 2")
        local connection2 = type2["NimfConf.ConnectionType"]
   
        if((connection2 == "l2tp") or (connection2 == "pptp")) then
            idle2=db.getAttribute(connection2,"LogicalIfName","WAN2","IdleTimeOutFlag")
        elseif((connection2 == "pppoe")) then
            profile2=db.getAttribute("pppoe","LogicalIfName","WAN2","ProfileName")
            idle2=db.getAttribute("PppoeProfile","ProfileName",profile2,"IdleTimeOutFlag")
        else
            idle2="0"
        end
        
        if (idle1 == "1" or idle2 == "1") then
            return "ERROR", "IDLE_TIMEOUT_ERROR"
        end
    end

    -- If WAN Mode changed to "Dedicated WAN"
    -- Check for enabled VPN Policies with same traffic selectors
    if (db.existsTable and db.existsTable ("WanMode")) then
        local oldWanMode = db.getAttribute ("WanMode", "_ROWID_", "1", "Wanmode") or ''
        if (inputTable["WanMode.Wanmode"] ~= nil and
            inputTable["WanMode.Wanmode"] == "0" and
            (oldWanMode == "1" or oldWanMode == "2")) then
            if (db.existsTable ("IpsecVPNPolicy")) then
                require "teamf1lualib/vipSecure"
                local vpnPolicies = db.getTable ("IpsecVPNPolicy")
                if (vpnPolicies ~= nil and #vpnPolicies > 1) then
                    for k,v in pairs (vpnPolicies) do
                        local dupPolicies = vipSecure.getDulicatePolicies (v)
                        if (dupPolicies ~= nil and #dupPolicies > 1) then
                            return "ERROR", "DUPLICATE_VPN_POLICIES_ENABLED_ERROR"
                        end
                    end
                end
            end
        end
    end

    db.beginTransaction() --begin transaction
    local valid = false

    if (inputTable["WanMode.Wanmode"] ~= nil and inputTable["WanMode.Wanmode"] == "2") then
        if (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil and inputTable["WanMode.FailoverPriLogicalIfName"] == "WAN1") then
            inputTable["WanMode.FailoverSecLogicalIfName"] = "WAN2"
        elseif (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil and inputTable["WanMode.FailoverPriLogicalIfName"] == "WAN2") then
            inputTable["WanMode.FailoverSecLogicalIfName"] = "WAN1"
        end
    end

    if(db.existsRow("WanMode", "_ROWID_", "1")) then
    	valid=db.update("WanMode",inputTable,rowid)
    else
    	valid=db.insert("WanMode",inputTable)
    end

    local failureTable = {}
    failureTable["WanFailure.LogicalIfName"]="WAN1"
    failureTable["WanFailure.AddressFamily"]="2"    
    failureTable["WanFailure.PingIp"]=inputTable["WanFailure.PingIp"]
    failureTable["WanFailure.DnsIp"]=inputTable["WanFailure.DnsIp"]

    if(valid) then
	    if(db.existsRow("WanFailure","LogicalIfName","WAN1")) then
            local rowId = db.getAttribute("WanFailure","LogicalIfName","WAN1","_ROWID_")
	        valid=db.update("WanFailure",failureTable,rowId)
    	else
	        valid=db.insert("WanFailure",failureTable)
    	end
    end

    failureTable = {}
    failureTable["WanFailure.LogicalIfName"]="WAN2"
    failureTable["WanFailure.AddressFamily"]="2"
    failureTable["WanFailure.PingIp"]=inputTable["WanFailure.PingIp1"]
    failureTable["WanFailure.DnsIp"]=inputTable["WanFailure.DnsIp1"]

    if(valid) then
    	if(db.existsRow("WanFailure","LogicalIfName","WAN2")) then
            local rowId = db.getAttribute("WanFailure","LogicalIfName","WAN2","_ROWID_")            
	        valid=db.update("WanFailure",failureTable,rowId)
    	else
	        valid=db.insert("WanFailure",failureTable)
    	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "WM_CONFIG_FAILED"
    end
end

function nimfView.wanMode6Config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
   
    if (inputTable["WanMode.Wanmode"] ~= nil and inputTable["WanMode.Wanmode"] == "2") then
        if (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil and inputTable["WanMode.FailoverPriLogicalIfName"] == "WAN1") then
            inputTable["WanMode.FailoverSecLogicalIfName"] = "WAN2"
        elseif (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil and inputTable["WanMode.FailoverPriLogicalIfName"] == "WAN2") then
            inputTable["WanMode.FailoverSecLogicalIfName"] = "WAN1"
        end
    end

    if(db.existsRow("WanMode", "_ROWID_", "2")) then
    	valid=db.update("WanMode",inputTable,rowid)
    else
    	valid=db.insert("WanMode",inputTable)
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "WM_CONFIG_FAILED"
    end
end

function nimfView.protocolBindingConfig (inputTable, rowid, operation)
end

-- Function for IPV6 WAN Config
function nimfView.ipv6WanConfig(inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    --TODO inputTable["ifStatic.StaticIp"] and inputTable["ifStatic.PrefixLength"] may be nil. so take care of it.
    if( nimfView.IPv6SubnetAlreadyInUseExceptWanIP(inputTable["ifStatic.StaticIp"] , inputTable["ifStatic.PrefixLength"] )== "ERROR")then
	print("IPv6 address already in use, try other ip.")
        return "ERROR", "ERR_IPV6_SUBNET_IN_USE"
    end
    local wanName = inputTable["NimfConf.LogicalIfName"]    
    -- checking for Wan Configuration
    local where = "LogicalIfName = '" .. wanName .. "' AND AddressFamily = 10"
    local nimfConfig = db.getRowWhere ("NimfConf", where)
    if (nimfConfig == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end
	rowid = nimfConfig["NimfConf._ROWID_"]
    db.beginTransaction() --begin transaction
    local valid = false
    
    local NimfConfTable = {}
    
    NimfConfTable["NimfConf.LogicalIfName"] = wanName
    NimfConfTable["NimfConf.AddressFamily"] = "10"

    if(inputTable["NimfConf.ConnectionType6"] == "dhcp6c") then
        NimfConfTable["NimfConf.ConnectionType6"] = "dhcp6c"
	    inputTable["dhcpv6c.LogicalIfName"] = wanName
	    if (db.existsRowWhere("dhcpv6c", "LogicalIfName='" .. wanName .. "'")) then
            valid = nimfView.dhcp6Config (inputTable, db.existsRowWhere ("dhcpv6c", "LogicalIfName='" .. wanName .. "'"), "edit")
        else
            valid = nimfView.dhcp6Config (inputTable, "-1", "add")
	    end
    elseif (inputTable["NimfConf.ConnectionType6"] == "ifStatic6") then
        NimfConfTable["NimfConf.ConnectionType6"] = "ifStatic6"
	    inputTable["ifStatic.LogicalIfName"] = wanName
	    inputTable["ifStatic.AddressFamily"] = "10"
        if (db.existsRowWhere("ifStatic", where)) then
            valid = ifStatic.config (inputTable, db.existsRowWhere("ifStatic", where), "edit")
        else
            valid = ifStatic.config (inputTable, "-1", "add")
        end
    else
        NimfConfTable["NimfConf.ConnectionType6"] = "pppoe6"
        inputTable["Pppoe6.LogicalIfName"] = wanName
        if (db.existsRowWhere("Pppoe6", "LogicalIfName='" .. wanName .. "'")) then
            valid = nimfView.pppoe6Config (inputTable, db.existsRowWhere ("Pppoe6", "LogicalIfName='" .. wanName .. "'"), "edit")
        else
            valid = nimfView.pppoe6Config (inputTable, "-1", "add")
        end
        if (db.existsRowWhere("dhcpv6c", "LogicalIfName='" .. wanName .. "PPPOE6'")) then
            local dhcpv6Table = {}
            if (inputTable["Pppoe6.Dhcpv6Opt"] == "1") then
                dhcpv6Table["dhcpv6c.statelessMode"] = "1"
                dhcpv6Table["dhcpv6c.prefixDelegation"] = "0"
            elseif (inputTable["Pppoe6.Dhcpv6Opt"] == "2") then
                dhcpv6Table["dhcpv6c.statelessMode"] = "0"
                dhcpv6Table["dhcpv6c.prefixDelegation"] = "0"
            elseif (inputTable["Pppoe6.Dhcpv6Opt"] == "3") then
                dhcpv6Table["dhcpv6c.statelessMode"] = "1"    
                dhcpv6Table["dhcpv6c.prefixDelegation"] = "1"
            end
            if (dhcpv6Table ~= nil) then
                valid = nimfView.dhcp6Config (dhcpv6Table, db.existsRowWhere ("dhcpv6c", "LogicalIfName='" .. wanName .. "PPPOE6'"), "edit")
            end
        end
    end
    
    if (valid) then
        valid = nimf.config (NimfConfTable, rowid, "edit")            
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV6WAN_CONFIG_FAILED"
    end
end


-- Getting IPv6 Wan Configuration 
function nimfView.ipv6WanConfigGet(wanName)
    local outTable = {}
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. wanName .. "' AND AddressFamily=10")
    if (NimfRow ~= nil) then
    	if(NimfRow["NimfConf.ConnectionType6"] == "dhcp6c") then
	        outTable["NimfConf.ConnectionType6"] = "dhcp6c"
	        local DhcpRow = db.getRowWhere ("dhcpv6c","LogicalIfName = '" .. wanName .. "'")
    	    if(DhcpRow ~= nil) then
	            outTable["dhcpv6c.statelessMode"] = DhcpRow["dhcpv6c.statelessMode"]
    	    	outTable["dhcpv6c.requestPreferredAddress"] = DhcpRow["dhcpv6c.requestPreferredAddress"]
	    	    outTable["dhcpv6c.preferredAddress"] = DhcpRow["dhcpv6c.preferredAddress"]
    		    outTable["dhcpv6c.preferredAddressPrefixLength"] = DhcpRow["dhcpv6c.preferredAddressPrefixLength"]
                outTable["dhcpv6c.prefixDelegation"] = DhcpRow["dhcpv6c.prefixDelegation"]
	        end
    	elseif(NimfRow["NimfConf.ConnectionType6"] == "ifStatic6") then
	        outTable = db.getRowWhere ("ifStatic","LogicalIfName = '" .. wanName .. "' AND AddressFamily=10")
	        outTable["NimfConf.ConnectionType6"] = "ifStatic6"
        elseif(NimfRow["NimfConf.ConnectionType6"] == "pppoe6") then
            outTable = db.getRowWhere ("Pppoe6","LogicalIfName = '" .. wanName .. "'")
            outTable["NimfConf.ConnectionType6"] = "pppoe6"
        end
    end
    return outTable
end

--dhcp6c Configuration
function nimfView.dhcp6Config(inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert("dhcpv6c", inputTable)
        elseif (operation == "edit") then
            return db.update("dhcpv6c", inputTable, rowid)
        elseif (operation == "delete") then
            return nil
        end
    end
    return false
end

--Pppoe6 Configuration
function nimfView.pppoe6Config(inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert("Pppoe6", inputTable)
        elseif (operation == "edit") then
            return db.update("Pppoe6", inputTable, rowid)
        elseif (operation == "delete") then
            return nil
        end
    end
    return false
end

-- IP Mode Configuration
function nimfView.ipModeConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local isExists = db.existsRow ("networkInfo", "_ROWID_", rowid)
    if (isExists) then
        valid = db.update("networkInfo", inputTable, rowid)
    else
        valid = db.insert("networkInfo", inputTable)        
    end

    -- return    
    if (valid) then
	db.commitTransaction(true)
	return "OK", "STATUS_OK"
    else
	db.rollback()
	return "ERROR", "IP_MODE_SET_FAILED"
    end    
end

-- 6to4 Tunnel Configuration
function nimfView.sixtoFourTunnelConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local isExists = db.existsRow ("sixToFourTunnel", "_ROWID_", rowid)
    if (isExists) then
        valid = db.update("sixToFourTunnel", inputTable, rowid)
    else
        valid = db.insert("sixToFourTunnel", inputTable)        
    end

    -- return    
    if (valid) then
	db.commitTransaction(true)
	return "OK", "STATUS_OK"
    else
	db.rollback()
	return "ERROR", "6TO4_TUNNEL_CONFIG_FAILED"
    end    
end


-- ISATAP Tunnel Deletion
function nimfView.isatapTunnelDelete (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["isatapTunnel._ROWID_"] = v
	valid = db.delete("isatapTunnel", inTable) 
	if(valid==false) then
	    break
	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "ISATAP_TUNNEL_DELETE_SUCCESS"
    else
        db.rollback()
        return "ERROR", "ISATAP_TUNNEL_DELETE_FAILED"
    end
end


 -- ISATAP Tunnel Configuration
function nimfView.isatapTunnelConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    local old_record = nil 
    if(inputTable["isatapTunnel.useLanAddress"] == '1')then
	old_record = db.getRowWhere("isatapTunnel","useLanAddress=1 AND _ROWID_!="..rowid)
	if(old_record ~= nil)then
		return "ERROR","ISATAP_REDUNDANT_IP"
	end
    elseif(inputTable["isatapTunnel.useLanAddress"] == '0')then
	if(inputTable["isatapTunnel.localIPv4Address"] ~= nil)then
		old_record = db.getRowWhere("isatapTunnel","localIPv4Address='"..inputTable["isatapTunnel.localIPv4Address"].."' AND _ROWID_!="..rowid)
		if(old_record ~= nil)then
			return "ERROR","ISATAP_REDUNDANT_IP"
		end
	end
    end
    db.beginTransaction() --begin transaction
    local valid = false
    
    inputTable["isatapTunnel.transportIfName"] = "LAN"

    if(inputTable["isatapTunnel.useLanAddress"] == "1") then
        inputTable["isatapTunnel.localIPv4Address"] = ""
    end

    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
	    valid = db.delete(DBTable, inputTable)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "ISATAP_TUNNEL_CONFIG_FAILED"
    end
end

-- IPV6 MLD Configuration
function nimfView.ipv6MldConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local isExists = db.existsRow ("mldproxy", "_ROWID_", rowid)
    if (isExists) then
        valid = db.update("mldproxy", inputTable, rowid)
    else
        valid = db.insert("mldproxy", inputTable)        
    end

    -- return    
    if (valid) then
	db.commitTransaction(true)
	return "OK", "STATUS_OK"
    else
	db.rollback()
	return "ERROR", "IPV6_MLD_CONFIG_FAILED"
    end    
end

function nimfView.configurablePortUsageCheck (inputTable)
    if (inputTable["ConfigPort.LogicalIfName"] ~= "LAN") then
        local usingRows = db.getRowsWhere ("TrafficSelector","PortName = 'OptionalPort'")
        if (usingRows ~= nil and #usingRows ~= 0) then
            return "ERROR", "CONFIG_LAN_IN_USE_BY_BW_TRAFFIC_SELECTOR"
        end
        usingRows = db.getRowsWhere ("PortMirroring", "MirroringEnable = 1 and CapturePort = 'OptionalPort'")
        if (usingRows ~= nil and #usingRows ~= 0) then
            return "ERROR", "CONFIG_LAN_IN_USE_BY_PORT_MIRRORING"
        end
    end
    return "OK", "STATUS_OK"
end

function nimfView.checkOptionWanTables (inputTable)
	local oldPort = db.getAttribute ("ConfigPort", "_ROWID_", "1","LogicalIfName")
	local profileTable = db.getRowsWhere ("qosClassQueue","ProfileKey=2 and ConfigDefault=0")
	local fwTable = db.getRowsWhere ("FirewallRules",
	                                 "((DestinationPublicInterface = 'WAN2' AND RuleType = 'INSECURE_SECURE') OR " ..
					  "(DestinationPublicInterface = 'WAN2' AND RuleType = 'INSECURE_PUBLIC') OR " ..
					  "(SNATInterface = 'WAN2' AND RuleType ='SECURE_INSECURE') OR " ..
					  "(SNATInterface = 'WAN2' AND RuleType ='PUBLIC_INSECURE'))")
	local protocalTable = db.getRowsWhere ("ProtocolBinding","LocalGatewayType = 2")
	local routeTable = db.getRowsWhere ("route","interfaceName = 'WAN2'")
	local vpnTable = db.getRowsWhere ("IpsecVPNPolicy","LocalGateway = 1")			

	local alertFlag = 0
	if(((profileTable ~= nil) and (#profileTable ~= 0)) 
		or ((fwTable ~= nil) and (#fwTable ~= 0)) 
		or ((protocalTable ~= nil) and (#protocalTable ~= 0)) 
		or ((routeTable ~= nil) and (#routeTable ~= 0)) 
		or ((vpnTable ~= nil) and (#vpnTable ~= 0)))then
		alertFlag = 1
	end
	local len = 0
	local tblArr = {}
	if(#profileTable ~= 0) then
		len = len+1
		tblArr[len] = "BandwidthProfiles"
	end
	if(#fwTable ~= 0) then
		len = len+1
		tblArr[len] = "Firewall"
	end
	if(#protocalTable ~= 0) then
		len = len+1
		tblArr[len] = "ProtocalBindings"
	end
	if(#routeTable ~= 0) then
		len = len+1
		tblArr[len] = "StaticRoutes"
	end
	if(#vpnTable ~= 0) then
		len = len+1
		tblArr[len] = "VPNProfiles"
	end
	
	local tables = tblArr[1] or ''
	if(len ~= 1) then
		for i=2,len do
			if(i ~= len) then
				tables = tables .. ", " .. tblArr[i]
			else
				tables = tables .. " and " .. tblArr[i]
			end
		end
	end	

	
	if((oldPort == 'WAN2') and (inputTable["ConfigPort.LogicalIfName"] ~= 'WAN2') and (alertFlag == 1)) then
		return "ERROR", "Please remove Optional WAN Configurations from " .. tables
	else
		return "OK", "STATUS_OK"
	end
end

function getConnectionTypes (conLogIfName)
	local ispType = "0"
    local addressFamily = "2"
	local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. conLogIfName .. "' and AddressFamily = " .. addressFamily)
	if(NimfRow ~= nil) then
		if(NimfRow["NimfConf.ConnectionType"] == "pptp") then
			-- PPTP
			local ispConfigRow = db.getRowWhere ("Pptp","LogicalIfName = '" .. conLogIfName .. "'")
			if (ispConfigRow ~= nil) then
				if(ispConfigRow["Pptp.MyIp"] == nil or ispConfigRow["Pptp.MyIp"] == "") then
					ispType = "5"
				else
					ispType = "3"
				end
			end
		elseif(NimfRow["NimfConf.ConnectionType"] == "pppoe") then
		-- PPPoE
			local pppoeRows = db.getRowsWhere ("PppoeProfile","LogicalIfName = '" .. conLogIfName .. "' and Status=1")	
			if (pppoeRows ~= nil) then
				if (#pppoeRows == 2) then			
					ispType = "7"
				else
					ispType = "2"			
				end
			end
		elseif (NimfRow["NimfConf.ConnectionType"] == "l2tp") then
			-- L2TP
			local ispConfigRow = db.getRowWhere ("L2tp","LogicalIfName = '" .. conLogIfName .. "'")
			if (ispConfigRow ~= nil) then
				if(ispConfigRow["L2tp.MyIp"] == nil or ispConfigRow["L2tp.MyIp"] == "") then
					ispType = "6"
				else
					ispType = "4"
				end
			end
		elseif(NimfRow["NimfConf.ConnectionType"] == "threeg") then
			-- ThreeG
			local ispConfigRow = db.getRowWhere ("ThreeG","LogicalIfName = '" .. conLogIfName .. "'")
			if (ispConfigRow ~= nil) then
				ispType = "8"
			end
		elseif (NimfRow["NimfConf.ConnectionType"] == "ifStatic") then
			ispType = "1"
		elseif (NimfRow["NimfConf.ConnectionType"] == "dhcpc") then
			ispType = "0"
		end
	end
	return ispType
end

-- WAN Secondary Addresses configuration
function nimfView.secAddrConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    local virtualIfName
    virtualIfName = db.getAttribute ("networkInterface", "LogicalIfName",  inputTable["ifStaticIPAlias.LogicalIfName"], "VirtualIfName")
    
    inputTable["ifStaticIPAlias.PhyIfName"] = virtualIfName
    db.beginTransaction() --begin transaction
    local valid = false
    -- config Secondary Addresses
    valid = nimf.secConfig(inputTable, rowid, operation, "ifStaticIPAlias")
    
    -- return
    if (valid) then
    	db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
    	db.rollback()
        return "ERROR", "NIMF_SEC_ADDR_CONFIG_FAILED"
    end
end

-- Delete Secondary Addresses
function nimfView.secAddrDelete(rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    db.beginTransaction() --begin transaction
    local valid = false
    local rowsTemp = {}
    
    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["ifStaticIPAlias._ROWID_"] = v
        tempSecAddr = db.getAttribute ("ifStaticIPAlias", "_ROWID_", v, "IPAddress")
        isExistsRule = db.existsRow("FirewallRules","DestinationPublicAddress", tempSecAddr)
        if(isExistsRule) then
                db.rollback()
                return "ERROR", "ADDRESS_USED_IN_FIREWALL_RULE"
        end
        isExistsRule = db.existsRow("FirewallRules","SNATAddress", tempSecAddr) 
        if(isExistsRule) then
                db.rollback()
                return "ERROR", "ADDRESS_USED_IN_FIREWALL_RULE"
        end
        valid = db.delete("ifStaticIPAlias", inTable)
    end
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "NIMF_SEC_ADDR_DELETE_FAILED"
    end
end

-- Display values of IPv4 WAN
function nimfView.dispWANConfigGet (wanLogicalIfName)
    local outTable = {}
    local logIfName = wanLogicalIfName
    local addressFamily = 2

    local where = "LogicalIfName = '" .. logIfName .. "' and addressFamily = 2"
    local wanv4Config = db.getRowWhere ("NimfConf", where)
    local wanRows = db.getRowsWhere ("ipAddressTable", where)
    local nsRows = db.getRowsWhere("resolverConfig", where)
    local wanIntName = db.getAttribute ("networkInterface", "LogicalIfName", logIfName, "interfaceName") or ''
    local wanState = db.getAttribute ("networkInterface", "LogicalIfName", logIfName, "IfStatus") or ''
    local wanMacAddr = ""
    local natEnabled = ""
    local displayConnectionType = ""
    local connetionType4 = ""
    local connetionStatus4 = ""
    local pppoeRows = {}
    local pppoeNumRows = {}
    local ispType = ""

    -- Getting Connection Type of the WAN Interface
    if (wanv4Config["NimfConf.ConnectionType"] ~= nil) then
        -- get connection type
        ispType = getConnectionTypes (logIfName)
        if(ispType == "0") then
            connetionType4 = "Dynamic IP (DHCP)"
            displayConnectionType = "Dynamic IP (DHCP)"
        elseif(ispType == "1")  then
            connetionType4 = "STATIC"
            displayConnectionType = "Static IP"
        elseif(ispType == "2") then
            connetionType4 = "PPPOE"
            displayConnectionType = "PPPoE (Username/Password)"
            pppoeRows = db.getRowsWhere("PppoeProfile","LogicalIfName = '" .. logIfName .. "' and Status = 1") or {}
            pppoeNumRows = db.getRows("PppoeProfile","LogicalIfName", logIfName) or  {}
        elseif(ispType == "3") then
            connetionType4 = "PPTP"
            displayConnectionType = "PPTP (Username/Password)"
        elseif(ispType == "4") then
            connetionType4 = "L2TP"
            displayConnectionType = "L2TP (Username/Password)"
        elseif(ispType == "5") then
            connetionType4 = "RPPTP"
            displayConnectionType = "Russian dual access PPTP"
        elseif(ispType == "6") then
            connetionType4 = "RL2TP"
            displayConnectionType = "Russian dual access L2TP"
        elseif(ispType == "7") then
            connetionType4 = "JPPPOE"
            displayConnectionType = "Japanese multiple PPPoE"
            pppoeRows = db.getRowsWhere("PppoeProfile","LogicalIfName = '" .. logIfName .. "' and Status = 1") or {}
            pppoeNumRows = db.getRows("PppoeProfile","LogicalIfName", logIfName) or  {}
        end
    end
    
    outTable["Wan.WanConnType"] = displayConnectionType

    -- Getting MAC Address of the WAN Interface
    if (wanIntName ~= nil) then
        if(ispType == "2" or ispType == "3" or ispType == "4") then
            wanMacAddr = "00:00:00:00:00:00"
        else
	    if (wanState ~= "0") then
        	wanMacAddr = db.getAttribute ("ethernet", "interfaceName", wanIntName, "macAddress") or ''
	    else
		wanMacAddr = ''
	    end
        end
    end    
    outTable["Wan.wanMacAddr"] = wanMacAddr

    -- Getting NAT Mode Status.
    local natEnabled = ""
    if (db.getAttribute("NatTable", "_ROWID_", "1", "Enable") == "1") then
        natEnabled = "Enabled"
    else
        natEnabled = "Disabled"
    end
    outTable["Wan.natEnabled"] = natEnabled

    --Display WAN Mode
    local wmodeConfig = db.getRow("WanMode", "_ROWID_", "1")
    if (wmodeConfig ~= nil) then
        local wan = ""
        if (wmodeConfig["WanMode.Wanmode"] == "2") then
            wan = wmodeConfig["WanMode.FailoverPriLogicalIfName"]
        else
            wan = wmodeConfig["WanMode.DedicatedLogicalIfName"]
        end
        if (wmodeConfig["WanMode.Wanmode"] == "2") then
            wanMode = "Auto-Rollover using WAN port" .. wan
        elseif (wmodeConfig["WanMode.Wanmode"] == "1") then
            wanMode = "Load Balancing"
        elseif (wmodeConfig["WanMode.Wanmode"] == "0") then
            -- wanMode = "Use only single WAN port" .. wan
            wanMode = "Single Port" 
        end
    end
    outTable["Wan.Wanmode"] = wanMode
    
    --Display WAN State
    if (wanState == "1") then
        wanState = "UP"
    elseif (wanState == "2") then
        wanState = "UP(IPv6)"
    elseif (wanState == "3") then
        wanState = "UP(IPv4 and IPv6)"
    elseif (wanState == "0") then
        wanState = "Down"
    end
    if(connetionType4 == "JPPPOE") then
        --  Japan PPPoE
        local i = 0
        local state = ""
        state = state .. wanState .. " (" .. "Primary" .. ")"
        if(#pppoeRows == 2) then
            local f = io.open("/var/wan101Ip", "r")
            if(f) then
                state =  state .. "," .."UP" .. " (" .. "Secondary" ..")"
                f:close()
            else
                state = state .. ",".. "DOWN" .. " (" .. "Secondary" .. ")"
            end
        end
        wanState = state
    end
    outTable["Wan.wanState"] = wanState
    
    -- Getting Connection State of the WAN(ipv4) Interface
    local nimfStatusConfig = db.getRowWhere("NimfStatus", where)
    if (nimfStatusConfig ~= nil) then
        connetionStatus4 = nimfStatusConfig["NimfStatus.Nimfstatus"] or ""
    end
    outTable["Wan.wanConnState"] = connetionStatus4

    -- WAN IP Address get
    local ipAddrStr = ""
    local subnetMask = ""
    local i = 0
    if(connetionType4 ~= "JPPPOE") then
        if(wanRows) then
            for k,v in pairs(wanRows) do
                i = i + 1
                local row = wanRows[i]
                ipAddrStr = ipAddrStr .. row["ipAddressTable.ipAddress"] 
                subnetMask = subnetMask .. row["ipAddressTable.subnetMask"]
                if(i ~= #wanRows) then
                    ipAddrStr = ipAddrStr .. ", "
                    subnetMask = subnetMask .. ", "
                end
            end
        end
    elseif(connetionType4 == "JPPPOE") then
        if(wanRows) then
            i = 0
            for k,v in pairs(wanRows) do
                i = i + 1
                local row = wanRows[i]
                ipAddrStr = ipAddrStr .. row["ipAddressTable.ipAddress"] .. " (" .. "Primary" ..")"
                subnetMask = subnetMask .. row["ipAddressTable.subnetMask"]
                if(i ~= #wanRows) then
                    ipAddrStr = ipAddrStr .. ", "
                    subnetMask = subnetMask .. ", "
                end
            end
        end
        if(#pppoeNumRows == 2) then
            if(#pppoeRows == 2) then
                -- Checking for second pppoe profile
                local profile2Ip = ""
                local f = io.open("/var/wan101Ip", "r")
                if(f) then
                    profile2Ip = f:read("*all") or ''
                    ipAddrStr = ipAddrStr .. ", " .. profile2Ip .. " (" .. "Secondary" ..")"
                    f:close()
                end
            else
                ipAddrStr = ipAddrStr
            end
        end
    end
    outTable["Wan.IPv4Address"] = ipAddrStr
    outTable["Wan.subnetMask"] = subnetMask
    
    -- Get Gateway IPs
    local gwRows = db.getRowsWhere("defaultRouters", "LogicalIfName = '".. logIfName .. "' AND addressFamily = '2'")
    local gwIps = ""
    if(gwRows) then
        local i = 0
        for k,v in pairs(gwRows) do
            i = i + 1        
            local row = gwRows[i]
            if (row["defaultRouters.nextHopGateway"] ~= nil and row["defaultRouters.nextHopGateway"] ~= "") then
                if (gwIps == "") then
                   gwIps = gwIps .. row["defaultRouters.nextHopGateway"]
                else
                   gwIps = gwIps .. ", " .. row["defaultRouters.nextHopGateway"]
                end
            end
        end
    end
    if(connetionType4 == "JPPPOE") then
        gwIps = gwIps .. " (" .. "Primary" .. ")"
        if(#pppoeRows == 2) then
            local f = io.open("/var/wan101Gw", "r")
            if(f) then
                local profile2Gw = f:read("*all") or ''
                gwIps = gwIps .. ", " .. profile2Gw .. " (" .. "Secondary" .. ")"
                f:close()
            end
        end
    end
    outTable["Wan.gateWayIP"] = gwIps
    
    --Get Primary DNS
    local priDNSIps = ""
    local secDNSIps = ""
    if(nsRows) then
        local i = 0
        for k,v in pairs(nsRows) do
            i = i + 1
            local row = nsRows[i]
            if (row["resolverConfig.nameserver1"] ~= nil and row["resolverConfig.nameserver1"] ~= "") then
                if (priDNSIps == "") then
                    priDNSIps = priDNSIps .. row["resolverConfig.nameserver1"]
                else
                    priDNSIps = priDNSIps
                end
            end
            if (row["resolverConfig.nameserver2"] ~= nil and row["resolverConfig.nameserver2"] ~= "") then
                if (secDNSIps == "") then
                    secDNSIps = secDNSIps .. row["resolverConfig.nameserver2"]
                else
                    secDNSIps = secDNSIps
                end
            end
        end
    end
    if(connetionType4 == "JPPPOE") then
        priDNSIps = priDNSIps .. " (" .. "Primary" .. ")"
        if(#pppoeRows == 2) then
            local f = io.open("/var/wan101Dns", "r")
            if(f) then
                local profile2Dns = f:read("*line") or ''
                priDNSIps = priDNSIps .. ", " .. profile2Dns .. " (" .. "Secondary" .. ")"
                f:close()
            end
        end
    end
    outTable["Wan.primaryDNSIPs"] = priDNSIps
    
    -- Secondary Dns
    if(connetionType4 == "JPPPOE") then
        secDNSIps = secDNSIps .. " (" .. "Primary" .. ")"
        if(#pppoeRows == 2) then
            local f = io.open("/var/wan101Dns", "r")
            if(f) then
                local profile2Dns
                i = 1 
                while true do
                    profile2Dns = f:read("*line")
                    if(i == 2) then break end
                    i = i+1
                end
                secDNSIps = secDNSIps .. ", " .. profile2Dns .. " (" .. "Secondary" .. ")"
                f:close()
            end
        end
    end
    outTable["Wan.secondaryDNSIPs"] = secDNSIps

    local wanUpSpeed = db.getAttribute ("WanSpeed", "LogicalIfName", logIfName, "UploadSpeed") or ''
    local wanDownSpeed = db.getAttribute ("WanSpeed", "LogicalIfName", logIfName, "DownloadSpeed") or ''
    outTable["Wan.UploadSpeed"] = wanUpSpeed
    outTable["Wan.DownloadSpeed"] = wanDownSpeed

    return outTable
end

-- get WAN IPv6 values
function nimfView.dispWAN6ConfigGet (wanLogicalIfName)
    local outTable = {}
    logIfName = wanLogicalIfName
    addressFamily = 10
    local where6 = "LogicalIfName = '" .. logIfName .. "' and addressFamily = 10"
    local where6PPPoe = "LogicalIfName = '" .. logIfName .. "' and addressFamily = 10"
    local wanv6Config = db.getRowWhere ("NimfConf", where6)
    local netWorkMode = db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") or ''
    
    if (netWorkMode == "3") then
        if (wanv6Config["NimfConf.ConnectionType6"] ~= nil and wanv6Config["NimfConf.ConnectionType6"] == "pppoe6") then
            where6PPPoe = "LogicalIfName = '" .. logIfName .. "PPPOE6' and addressFamily = 10" 
        end
    end
    local wan6Rows = db.getRowsWhere ("ipAddressTable", where6PPPoe)
    local nsRows = db.getRowsWhere("resolverConfig", where6)
    local gwRows = db.getRowsWhere("defaultRouters", "LogicalIfName='" .. logIfName .. "' and addressFamily=10")
    local connectionType6 =""
    local connetionStatus6 = ""
    
    -- Getting Connection Type of the WAN(ipv6) Interface
    if (netWorkMode == "3") then
        if (wanv6Config["NimfConf.ConnectionType6"] ~= nil) then
            if (wanv6Config["NimfConf.ConnectionType6"] == "dhcp6c") then
                connectionType6 = "Dynamic IP (DHCPv6)"
            elseif (wanv6Config["NimfConf.ConnectionType6"] == "ifStatic6") then
                connectionType6 = "Static IPv6"
            elseif (wanv6Config["NimfConf.ConnectionType6"] == "pppoe6") then
                connectionType6 = "PPPoE Over IPv6"
            elseif (wanv6Config["NimfConf.ConnectionType6"] == "") then
                connectionType6 = "Disabled"
            end
        end
    else
        connectionType6 = "IPv6 is disabled" 
    end
    outTable["WAN.connectionType"] = connectionType6
    
    -- Getting Connection State of the WAN(ipv6) Interface
    if (netWorkMode == "3") then
        nimfStatusConfig = db.getRowWhere("NimfStatus", where6)
        if (nimfStatusConfig ~= nil) then
            connetionStatus6 = nimfStatusConfig["NimfStatus.Nimfstatus"] or ""
        end
    else
        connetionStatus6 = "IPv6 is disabled" 
    end
    outTable["WAN.connectionStatus"] = connetionStatus6

    -- Disp wan IPv6 address
    local ipv6AddrStr = ""
    if (netWorkMode == "3") then
        i = 0
        if(wan6Rows) then
            for k,v in pairs(wan6Rows) do
                i = i + 1
                local row = wan6Rows[i]
                ipv6AddrStr = ipv6AddrStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.ipv6PrefixLen"]
                if(i ~= #wan6Rows) then
                    ipv6AddrStr = ipv6AddrStr .. ", "
                end
            end
        end
    end
    outTable["WAN.IPAddress"] = ipv6AddrStr
    
    i = 0
    local gatewayIPv6 = ""
    for k,v in pairs(gwRows) do
        i = i + 1
        local row = gwRows[i]
        gatewayIPv6 = gatewayIPv6 .. row["defaultRouters.nextHopGateway"] or ''
        if(i ~= #gwRows) then
            gatewayIPv6 = gatewayIPv6 .. ", "
        end
    end
    outTable["Wan.Gateway"] = gatewayIPv6
    
    local priDNSIps = ""
    local secDNSIps = ""
    if(nsRows) then
        local i = 0
        for k,v in pairs(nsRows) do
            i = i + 1
            local row = nsRows[i]
            if (connetionStatus6 ~= "Not Yet Connected") then
                if (row["resolverConfig.nameserver1"] ~= nil and row["resolverConfig.nameserver1"] ~= "") then
                    if (priDNSIps == "") then
                        priDNSIps = priDNSIps .. row["resolverConfig.nameserver1"]
                    else
                        priDNSIps = priDNSIps
                    end
                end
                if (row["resolverConfig.nameserver2"] ~= nil and row["resolverConfig.nameserver2"] ~= "") then
                    if (secDNSIps == "") then
                        secDNSIps = secDNSIps .. row["resolverConfig.nameserver2"]
                    else
                        secDNSIps = secDNSIps
                    end
                end
            else
                priDNSIps =  ""
                secDNSIps = ""
            end
        end
    end
    outTable["Wan.primaryDNS"] = priDNSIps
    outTable["Wan.secondaryDNS"] = secDNSIps
    
    return outTable
end

--checking the same subnet ip is available in ipaddress table EXCEPT WAN. 
function nimfView.IPv6SubnetAlreadyInUseExceptWanIP( ipv6Addr, subnetLength)
	local count = 0
	if( ipv6Addr ~= nil and subnetLength ~= nil) then
		local qryRowIpAddressTable = db.getRowsWhere("ipAddressTable","addressFamily = 10 AND LogicalIfName not like 'WAN%'" ) or {}
		if(#qryRowIpAddressTable ~= 0)then
			for k,v in pairs (qryRowIpAddressTable) do
				count = count + 1
				local row = qryRowIpAddressTable[count]
				if(row ~= nil) then
					if(row["ipAddressTable.ipAddress"] ~= nil and row["ipAddressTable.ipv6PrefixLen"]  ~= nil) then
						if(ifDevLib.ifDevIPv6SubnetAddress(ipv6Addr,subnetLength) == ifDevLib.ifDevIPv6SubnetAddress(row["ipAddressTable.ipAddress"],row["ipAddressTable.ipv6PrefixLen"]))then
							return "ERROR" 
						end
					end
				end
			end
		end
	end
	return "OK"
end


function nimfView.cmsWanConfigGet (wanLogicalIfName)

    local outTable = {}
    logIfName = wanLogicalIfName
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)    
    if (NimfRow ~= nil) then
        outTable["Wan.IspLoginRequired"] =  NimfRow["NimfConf.RequireLogin"]
        outTable["Wan.AccountName"] = NimfRow["NimfConf.AccountName"] or ''
        outTable["Wan.DomainName"] = NimfRow["NimfConf.DomainName"] or ''
        if (outTable["Wan.IspLoginRequired"] == "1") then
        	if (NimfRow["NimfConf.ConnectionType"] == "pppoe") then
        		outTable["Wan.IspType"] = "1"
        	elseif (NimfRow["NimfConf.ConnectionType"] == "pptp") then
        		outTable["Wan.IspType"] = "2"
        	else
        	    outTable["Wan.IspType"] = "2"
        	end
        --else
        	--outTable["Wan.IspType"] = "2"
        end

        local dhcpConfigRow = db.getRowWhere ("Dhcpc","LogicalIfName = '" .. logIfName .. "'")
        outTable["Wan.vendorClassId"] = dhcpConfigRow["Dhcpc.vendorClassId"]
		outTable["Wan.clientId"] = dhcpConfigRow["Dhcpc.clientId"]
        outTable["Wan.clientIdName"] = dhcpConfigRow["Dhcpc.clientIdName"]
        if (NimfRow["NimfConf.ConnectionType"] == "dhcpc") then
            outTable["Wan.IsGetIpDyn"] = "1"
            local ispConfigRow = db.getRowWhere ("Dhcpc","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
                outTable["Wan.GetDnsFromIsp"] = ispConfigRow["Dhcpc.GetDnsFromIsp"]
                if (ispConfigRow["Dhcpc.PrimaryDns"] ~= nil) then
                    outTable["Wan.PriDnsIpAddr"] = util.split(ispConfigRow["Dhcpc.PrimaryDns"], ".")
                end
                if (ispConfigRow["Dhcpc.SecondaryDns"] ~= nil) then
                    outTable["Wan.SecDnsIpAddr"] = util.split(ispConfigRow["Dhcpc.SecondaryDns"], ".")
                end
            end
        end
        --ifStatic
        local staticConfigRow = db.getRowWhere ("ifStatic","LogicalIfName = '" .. logIfName .. "'")
        outTable["Wan.IpAddr"] = util.split(staticConfigRow["ifStatic.StaticIp"], ".") --ifstatic.static ip is given preference over pptp.staticip.
        outTable["Wan.SnetMask"] = util.split(staticConfigRow["ifStatic.NetMask"], ".")
        if (staticConfigRow["ifStatic.Gateway"] ~= nil) then
            outTable["Wan.GwIpAddr"] = util.split(staticConfigRow["ifStatic.Gateway"], ".") --default priority
            outTable["Wan.Gateway1"] = outTable["Wan.GwIpAddr"][1]
            outTable["Wan.Gateway2"] = outTable["Wan.GwIpAddr"][2]
            outTable["Wan.Gateway3"] = outTable["Wan.GwIpAddr"][3]
            outTable["Wan.Gateway4"] = outTable["Wan.GwIpAddr"][4]
        end
        --TODO : prioritize any DNS ips
        if (NimfRow["NimfConf.ConnectionType"] == "ifStatic") then
            outTable["Wan.IsGetIpDyn"] = "0"
            outTable["Wan.GetDnsFromIsp"] = "0"            
            local ispConfigRow = db.getRowWhere ("ifStatic","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               --outTable["Wan.SnetMask"] = util.split(ispConfigRow["ifStatic.NetMask"], ".")                  
               
               if (ispConfigRow["ifStatic.PrimaryDns"] ~= nil) then
                   outTable["Wan.PriDnsIpAddr"] = util.split(ispConfigRow["ifStatic.PrimaryDns"], ".")
               end
               if (ispConfigRow["ifStatic.SecondaryDns"] ~= nil) then
                   outTable["Wan.SecDnsIpAddr"] = util.split(ispConfigRow["ifStatic.SecondaryDns"], ".")
               end
            end
        end
        --pppoe
        local pppoeConfigRow = db.getRowWhere ("Pppoe","LogicalIfName = '" .. logIfName .. "'")
        outTable["Wan.PppoeProfileName"] = pppoeConfigRow["Pppoe.ProfileName"] or ''
        outTable["Wan.LoginName"] = pppoeConfigRow["Pppoe.UserName"]
        outTable["Wan.LoginPassword"] = util.mask(pppoeConfigRow["Pppoe.Password"])               
        outTable["Wan.ConResetStatus"] = pppoeConfigRow["Pppoe.ConResetStatus"]
        if (pppoeConfigRow["Pppoe.ConResetHour"] ~= nil) then
            outTable["Wan.ConResetHour"] = pppoeConfigRow["Pppoe.ConResetHour"]
        end
        if (pppoeConfigRow["Pppoe.ConResetMin"] ~= nil) then
            outTable["Wan.ConResetMin"] = pppoeConfigRow["Pppoe.ConResetMin"]
        end
        if (pppoeConfigRow["Pppoe.ConResetDelay"] ~= nil) then
            outTable["Wan.ConResetDelay"] = pppoeConfigRow["Pppoe.ConResetDelay"]
        end
        outTable["Wan.IdleTimeOutFlag"] = pppoeConfigRow["Pppoe.IdleTimeOutFlag"] -- prefered over pptp
        outTable["Wan.IdleTime"] = pppoeConfigRow["Pppoe.IdleTimeOutValue"]       -- prefered over pptp
        if (NimfRow["NimfConf.ConnectionType"] == "pppoe") then
            local ispConfigRow = db.getRowWhere ("Pppoe","LogicalIfName = '" .. logIfName .. "'")
            outTable["Wan.IsGetIpDyn"] = ispConfigRow["Pppoe.GetIpFromIsp"]
            outTable["Wan.GetDnsFromIsp"] = ispConfigRow["Pppoe.GetDnsFromIsp"]
            if (ispConfigRow["Pppoe.StaticIp"] ~= nil) then
                outTable["Wan.IpAddr"] = util.split(ispConfigRow["Pppoe.StaticIp"], ".")
            end
            if (ispConfigRow["Pppoe.NetMask"] ~= nil ) then
                outTable["Wan.SnetMask"] = util.split(ispConfigRow["Pppoe.NetMask"], ".")
            end
            if (ispConfigRow["Pppoe.PrimaryDns"] ~= nil) then
                outTable["Wan.PriDnsIpAddr"] = util.split(ispConfigRow["Pppoe.PrimaryDns"], ".")
            end
            if (ispConfigRow["Pppoe.SecondaryDns"] ~= nil) then
                outTable["Wan.SecDnsIpAddr"] = util.split(ispConfigRow["Pppoe.SecondaryDns"], ".")
            end
        end
        --pptp
        local pptpConfigRow = db.getRowWhere ("Pptp","LogicalIfName = '" .. logIfName .. "'")
        outTable["Wan.MyIpAddr"] = util.split(pptpConfigRow["Pptp.MyIp"], ".")
               --begin: split MyIp address
               outTable["Wan.MyIp1"] = outTable["Wan.MyIpAddr"][1]
               outTable["Wan.MyIp2"] = outTable["Wan.MyIpAddr"][2]
               outTable["Wan.MyIp3"] = outTable["Wan.MyIpAddr"][3]
               outTable["Wan.MyIp4"] = outTable["Wan.MyIpAddr"][4]
               -- end: split MY Ip address              
               
               outTable["Wan.SvrIpAddr"] = util.split(pptpConfigRow["Pptp.ServerIp"], ".")
               --begin: split MyIp address
               outTable["Wan.ServerIp1"] = outTable["Wan.SvrIpAddr"][1]
               outTable["Wan.ServerIp2"] = outTable["Wan.SvrIpAddr"][2]
               outTable["Wan.ServerIp3"] = outTable["Wan.SvrIpAddr"][3]
               outTable["Wan.ServerIp4"] = outTable["Wan.SvrIpAddr"][4]
         if (NimfRow["NimfConf.ConnectionType"] == "pptp") then
            local ispConfigRow = db.getRowWhere ("Pptp","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               outTable["Wan.LoginName"] = ispConfigRow["Pptp.UserName"]
               outTable["Wan.LoginPassword"] = util.mask(ispConfigRow["Pptp.Password"])
               outTable["Wan.IsGetIpDyn"] = ispConfigRow["Pptp.GetIpFromIsp"]
               outTable["Wan.GetDnsFromIsp"] = ispConfigRow["Pptp.GetDnsFromIsp"]     
               
			   if (ispConfigRow["Pptp.GetIpFromIsp"] == "0") then
                   outTable["Wan.IpAddr"] = util.split(ispConfigRow["Pptp.StaticIp"], ".")
                   outTable["Wan.SnetMask"] = util.split(ispConfigRow["Pptp.NetMask"], ".")
               end
               if (ispConfigRow["Pptp.PrimaryDns"] ~= nil) then                  
                   outTable["Wan.PriDnsIpAddr"] = util.split(ispConfigRow["Pptp.PrimaryDns"], ".")
               end
               if (ispConfigRow["Pptp.SecondaryDns"] ~= nil) then
                   outTable["Wan.SecDnsIpAddr"] = util.split(ispConfigRow["Pptp.SecondaryDns"], ".")
               end
               outTable["Wan.IdleTimeOutFlag"] = ispConfigRow["Pptp.IdleTimeOutFlag"]
               if (ispConfigRow["Pptp.IdleTimeOutValue"] ~= nil) then
                   outTable["Wan.IdleTime"] = ispConfigRow["Pptp.IdleTimeOutValue"]                   
               end
            end
         end
         --partitioning ip addresses
         if (outTable["Wan.PriDnsIpAddr"] ~= nil) then
        	--begin: split primary dns ip
            outTable["Wan.PrimaryDns1"] = outTable["Wan.PriDnsIpAddr"][1]
            outTable["Wan.PrimaryDns2"] = outTable["Wan.PriDnsIpAddr"][2]
            outTable["Wan.PrimaryDns3"] = outTable["Wan.PriDnsIpAddr"][3]
            outTable["Wan.PrimaryDns4"] = outTable["Wan.PriDnsIpAddr"][4]
            -- end: split primary dns ip
         end
         if (outTable["Wan.SecDnsIpAddr"] ~= nil) then
            --begin: split secondary dns ip
            outTable["Wan.SecondaryDns1"] = outTable["Wan.SecDnsIpAddr"][1]
            outTable["Wan.SecondaryDns2"] = outTable["Wan.SecDnsIpAddr"][2]
            outTable["Wan.SecondaryDns3"] = outTable["Wan.SecDnsIpAddr"][3]
            outTable["Wan.SecondaryDns4"] = outTable["Wan.SecDnsIpAddr"][4]
            -- end: split secondary dns ip
        end
        if (outTable["Wan.IpAddr"] ~= nil) then
        	--begin: split ip address           
           outTable["Wan.StaticIp1"] = outTable["Wan.IpAddr"][1]
           outTable["Wan.StaticIp2"] = outTable["Wan.IpAddr"][2]
           outTable["Wan.StaticIp3"] = outTable["Wan.IpAddr"][3]
           outTable["Wan.StaticIp4"] = outTable["Wan.IpAddr"][4]
           -- end: split split ip address
        end
        if (outTable["Wan.SnetMask"] ~= nil) then
           --begin: split subnet mask address
           outTable["Wan.NetMask1"] = outTable["Wan.SnetMask"][1]
           outTable["Wan.NetMask2"] = outTable["Wan.SnetMask"][2]
           outTable["Wan.NetMask3"] = outTable["Wan.SnetMask"][3]
           outTable["Wan.NetMask4"] = outTable["Wan.SnetMask"][4]
           -- end: split subnet mask address
        end
    end
    return outTable
end


function nimfView.cmsWanConfigSetWrap (inputTable, rowid, operation)
    if (inputTable["Wan.IspType"] == "1") then
    	local where = "AddressFamily = 2"
        local idleTimeOutFlag = db.getAttribute("PppoeProfile","ProfileName",inputTable["Wan.PppoeProfileName"],"IdleTimeOutFlag")
        local rows = db.getRowWhere("WanMode", where)
        local wanMode = rows["WanMode.Wanmode"] or ''
        if (idleTimeOutFlag == "1" and wanMode == "1" ) then 
            return "ERROR", "LOAD_BALANCE_ERROR"
        end
    end
    return nimfView.cmsConfig (inputTable, rowid, operation)
end


function nimfView.cmsConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	logIfName = inputTable["NimfConf.LogicalIfName"]
    -- get wan Configuration in NimfConf
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
	
	-- Retrying for configuration in NimfConf.
    local retryIdx = 1
    while (NimfRow == nil and retryIdx <= 3) do
        NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        retryIdx = retryIdx + 1
    end
		
	-- return error if wan Configuration not found
    if (NimfRow == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end

    rowid = NimfRow["NimfConf._ROWID_"]

	db.beginTransaction() --begin transaction
	local valid = false
    local NimfConfTable = {}
    local dhcpConfTable = {}
    local staticConfTable = {}
    local pptpConfTable = {}

	-- config NIMF
    NimfConfTable["NimfConf.LogicalIfName"] = logIfName
    NimfConfTable["NimfConf.RequireLogin"] = inputTable["Wan.IspLoginRequired"]
    NimfConfTable["NimfConf.IfStatus"] = "1"
    --isp type determination
    if (inputTable["Wan.IspLoginRequired"] == "0") then    -- dhcpc or ifStatic configuration
        if (inputTable["Wan.IsGetIpDyn"] == "1") then -- dhcpc
            NimfConfTable["NimfConf.ConnectionType"] = "dhcpc"
        else
            NimfConfTable["NimfConf.ConnectionType"] = "ifStatic"
        end
    else
        if(inputTable["Wan.IspType"] == "1") then	-- pppoe connection
            NimfConfTable["NimfConf.ConnectionType"] = "pppoe"
        elseif(inputTable["Wan.IspType"] == "2") then
            NimfConfTable["NimfConf.ConnectionType"] = "pptp"
        end
    end
    -- isp type determination over
    -- Dhcpc table update
    --
    --
    dhcpConfTable["Dhcpc.LogicalIfName"] = logIfName
    dhcpConfTable["Dhcpc.GetDnsFromIsp"] = inputTable["Wan.GetDnsFromIsp"]
    if (inputTable["Wan.PriDnsIpAddr"] ~= nil) then
        dhcpConfTable["Dhcpc.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
    end
    if (inputTable["Wan.SecDnsIpAddr"] ~= nil) then
        dhcpConfTable["Dhcpc.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
    end
    dhcpConfTable["Dhcpc.vendorClassId"] = inputTable["Wan.vendorClassId"] or ''
    dhcpConfTable["Dhcpc.clientId"] =  inputTable["Wan.clientId"] or ''
    dhcpConfTable["Dhcpc.clientIdName"] = inputTable["Wan.clientIdName"] or ''
    if (inputTable["Wan.AccountName"] ~= nil and inputTable["Wan.AccountName"] ~= "") then
        dhcpConfTable["Dhcpc.HostName"] = inputTable["Wan.AccountName"]
    end
    if (db.existsRowWhere("Dhcpc", "LogicalIfName = '".. logIfName .. "'")) then
        valid = dhcpc.config (dhcpConfTable, db.getAttribute ("Dhcpc", "LogicalIfName", logIfName, "_ROWID_"), "edit")
    else
        valid = dhcpc.config (dhcpConfTable, "-1", "add")
    end

    --ifStatic
    staticConfTable["ifStatic.LogicalIfName"] = logIfName
    if (inputTable["Wan.IpAddr"] ~= nil and inputTable["Wan.SnetMask"] ~= nil and inputTable["Wan.IpAddr"] ~= "" and inputTable["Wan.SnetMask"] ~= "") then
        local status = ifDev.ipSubnet(inputTable["Wan.IpAddr"],inputTable["Wan.SnetMask"])
        if (status ~= "OK") then
            db.rollback()
            return "ERROR", status
        end
    end
    if (inputTable["Wan.IpAddr"] ~= nil ) then           
    	staticConfTable["ifStatic.StaticIp"] = inputTable["Wan.IpAddr"]
    end
    if (inputTable["Wan.SnetMask"] ~= nil ) then
    	staticConfTable["ifStatic.NetMask"] = inputTable["Wan.SnetMask"]
    end
    if (inputTable["Wan.GwIpAddr"] ~= nil ) then
    	staticConfTable["ifStatic.Gateway"] = inputTable["Wan.GwIpAddr"]
    end
    if (inputTable["Wan.PriDnsIpAddr"] ~= nil ) then
    	staticConfTable["ifStatic.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
    end
    if (inputTable["Wan.SecDnsIpAddr"] ~= nil ) then
    	staticConfTable["ifStatic.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
    end
    --- added for IPv6 support
    staticConfTable["ifStatic.AddressFamily"] = "2"
    if (db.existsRowWhere("ifStatic", "LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)) then
        valid = ifStatic.config (staticConfTable, db.getAttribute ("ifStatic", "LogicalIfName", logIfName, "_ROWID_"), "edit")
    else
        valid = ifStatic.config (staticConfTable, "-1", "add")
    end
    -- ifStatic Done
    --
    --
    -- pppoe Starts here
    --
     -- WAN Configuration having PPPoE Profiles support
     wanConfTable = db.getRowWhere ("Pppoe", "LogicalIfName = '".. logIfName .. "'")
     if (wanConfTable ~= nil) then
         -- Update old profile status
         if (wanConfTable["Pppoe.ProfileName"] ~= nil) then
             local profileConfig = db.getRow ("PppoeProfile", "ProfileName", wanConfTable["Pppoe.ProfileName"])
             if (profileConfig ~= nil) then
                 profileConfig["PppoeProfile.Status"] = "0"
                 valid = db.update ("PppoeProfile", profileConfig, profileConfig["PppoeProfile._ROWID_"])
             else
                 valid = true
             end
         else
             valid = true
         end
         -- Update new/selected profile status
         if (valid) then
             local profileConfig = db.getRow ("PppoeProfile", "ProfileName", inputTable["Wan.PppoeProfileName"])
                    -- Check if profile is already in use
              if (profileConfig ~= nil and profileConfig["PppoeProfile.Status"] ~= "1") then
                  profileConfig["PppoeProfile.Status"] = "1"
                  profileConfig["PppoeProfile.UserName"] = inputTable["Wan.LoginName"]
                  if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then
                      profileConfig["PppoeProfile.Password"] = inputTable["Wan.LoginPassword"]
                  end
                  if (inputTable["Wan.IdleTimeOutFlag"] ~= nil) then
                      profileConfig["PppoeProfile.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
                  end
                  if (inputTable["Wan.IdleTime"] ~= nil) then
                      profileConfig["PppoeProfile.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
                  end
                  profileConfig["PppoeProfile.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"]
                  profileConfig["PppoeProfile.GetDnsFromIsp"] = inputTable["Wan.GetDnsFromIsp"]
                  if (inputTable["Wan.IpAddr"] ~= nil) then
                      profileConfig["PppoeProfile.StaticIp"] = inputTable["Wan.IpAddr"]
                  else
                      profileConfig["PppoeProfile.StaticIp"] = util.joinFields(inputTable,"Wan","StaticIp", ".",4)
                  end
                  if (inputTable["Wan.SnetMask"] ~= nil) then
                      profileConfig["PppoeProfile.NetMask"] = inputTable["Wan.SnetMask"]
                  else
                      profileConfig["PppoeProfile.NetMask"] = util.joinFields(inputTable,"Wan","NetMask", ".",4)
                  end
                  if (profileConfig["PppoeProfile.StaticIp"] ~= nil and profileConfig["PppoeProfile.NetMask"] ~= nil and profileConfig["PppoeProfile.StaticIp"] ~= "" and profileConfig["PppoeProfile.NetMask"] ~= "") then
                      local status = ifDev.ipSubnet(profileConfig["PppoeProfile.StaticIp"],profileConfig["PppoeProfile.NetMask"])
                      if (status ~= "OK") then
                          db.rollback()
                          return "ERROR", status
                      end
                  end
                  if (inputTable["Wan.PriDnsIpAddr"] ~= nil) then
                      profileConfig["PppoeProfile.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
                  else
                      profileConfig["PppoeProfile.PrimaryDns"] = util.joinFields(inputTable,"Wan","PrimaryDns", ".",4)
                  end
                  if (inputTable["Wan.SecDnsIpAddr"] ~= nil) then
                      profileConfig["PppoeProfile.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
                  else
                      profileConfig["PppoeProfile.SecondaryDns"] = util.joinFields(inputTable,"Wan","SecondaryDns", ".",4)
                  end
                  profileConfig["PppoeProfile.ConResetStatus"] = inputTable["Wan.ConResetStatus"]
                  if(inputTable["Wan.ConResetHour"] ~= nil) then
                      profileConfig["PppoeProfile.ConResetHour"] = inputTable["Wan.ConResetHour"]
                  end
                  if(inputTable["Wan.ConResetMin"] ~= nil) then
                      profileConfig["PppoeProfile.ConResetMin"] = inputTable["Wan.ConResetMin"]
                  end
                  if(inputTable["Wan.ConResetDelay"] ~= nil) then
                      profileConfig["PppoeProfile.ConResetDelay"] = inputTable["Wan.ConResetDelay"]
                  end
                  valid = db.update ("PppoeProfile", profileConfig, profileConfig["PppoeProfile._ROWID_"])
              else
                  valid = false
              end
          end
      end
      if (valid) then
          local pppoeConfig = {}
          pppoeConfig["Pppoe.ProfileName"] = inputTable["Wan.PppoeProfileName"]
         pppoeConfig["Pppoe.LogicalIfName"] = logIfName            
         pppoeConfig["Pppoe.UserName"] = inputTable["Wan.LoginName"]
         if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then
         	pppoeConfig["Pppoe.Password"] = inputTable["Wan.LoginPassword"]
         end
	     if (inputTable["Wan.AccountName"] ~= nil) then            
	         pppoeConfig["Pppoe.AccountName"] = inputTable["Wan.AccountName"] 
	     end
		 if (inputTable["Wan.DomainName"] ~= nil) then            
		     pppoeConfig["Pppoe.DomainName"] = inputTable["Wan.DomainName"] 
		 end
	     pppoeConfig["Pppoe.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"] 
	     pppoeConfig["Pppoe.GetDnsFromIsp"] = inputTable["Wan.GetDnsFromIsp"]
         if (inputTable["Wan.IpAddr"] ~= nil and inputTable["Wan.SnetMask"] ~= nil and inputTable["Wan.IpAddr"] ~= "" and inputTable["Wan.SnetMask"] ~= "") then
             local status = ifDev.ipSubnet(inputTable["Wan.IpAddr"],inputTable["Wan.SnetMask"])
             if (status ~= "OK") then
                 db.rollback()
                 return "ERROR", status
             end
         end
	    if (inputTable["Wan.IpAddr"] ~= nil) then
	    	pppoeConfig["Pppoe.StaticIp"] = inputTable["Wan.IpAddr"]
	    end
	    if (inputTable["Wan.SnetMask"] ~= nil) then
	    	pppoeConfig["Pppoe.NetMask"] = inputTable["Wan.SnetMask"]
	    end
	    if (inputTable["Wan.PriDnsIpAddr"] ~= nil) then
	    	pppoeConfig["Pppoe.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
	    end
	    if (inputTable["Wan.SecDnsIpAddr"] ~= nil) then
	    	pppoeConfig["Pppoe.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
	    end
	     pppoeConfig["Pppoe.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"] 
	     pppoeConfig["Pppoe.IdleTimeOutValue"] = inputTable["Wan.IdleTime"] 
	     pppoeConfig["Pppoe.ConResetStatus"] = inputTable["Wan.ConResetStatus"] 
	     pppoeConfig["Pppoe.ConResetHour"] = inputTable["Wan.ConResetHour"] 
	     pppoeConfig["Pppoe.ConResetMin"] = inputTable["Wan.ConResetMin"] 
	     pppoeConfig["Pppoe.ConResetDelay"] = inputTable["Wan.ConResetDelay"] 
	     if (db.existsRowWhere("Pppoe", "LogicalIfName = '".. logIfName .. "'")) then
	         valid = pppoe.config (pppoeConfig, db.getAttribute ("Pppoe","LogicalIfName",logIfName,"_ROWID_"), "edit")
	     else
	         valid = pppoe.config (pppoeConfig, "-1", "add")
	     end
     end
     -- pppoe config end
     --
     --
     -- pptp starts here
      pptpConfTable["Pptp.LogicalIfName"] = logIfName            
      pptpConfTable["Pptp.UserName"] = inputTable["Wan.LoginName"]
      if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then
          pptpConfTable["Pptp.Password"] = inputTable["Wan.LoginPassword"]
      end
      pptpConfTable["Pptp.MppeEncryptSupport"] = inputTable["Wan.MppeEncryptSupport"]
      if (inputTable["Wan.MyIpAddr"] ~= nil) then
      	pptpConfTable["Pptp.MyIp"] = inputTable["Wan.MyIpAddr"]
      end
      if (inputTable["Wan.SvrIpAddr"] ~= nil) then            
          pptpConfTable["Pptp.ServerIp"] = inputTable["Wan.SvrIpAddr"]
      end
      pptpConfTable["Pptp.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"]
      pptpConfTable["Pptp.GetDnsFromIsp"] = inputTable["Wan.GetDnsFromIsp"]
      if (inputTable["Wan.IpAddr"] ~= nil and inputTable["Wan.SnetMask"] ~= nil and inputTable["Wan.IpAddr"] ~= "" and inputTable["Wan.SnetMask"] ~= "") then
          local status = ifDev.ipSubnet(inputTable["Wan.IpAddr"],inputTable["Wan.SnetMask"])
          if (status ~= "OK") then
              db.rollback()
              return "ERROR", status
          end
      end
      if (inputTable["Wan.IpAddr"] ~= nil) then
          pptpConfTable["Pptp.StaticIp"] = inputTable["Wan.IpAddr"]
      end
      if (inputTable["Wan.SnetMask"] ~= nil) then
          pptpConfTable["Pptp.NetMask"] = inputTable["Wan.SnetMask"]
      end
      if (inputTable["Wan.PriDnsIpAddr"] ~= nil) then
        	pptpConfTable["Pptp.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
      end
      if (inputTable["Wan.SecDnsIpAddr"] ~= nil) then
        	pptpConfTable["Pptp.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
      end
      pptpConfTable["Pptp.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
      if (inputTable["Wan.IdleTime"] ~= nil) then
          pptpConfTable["Pptp.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
      end
      if (db.existsRowWhere("Pptp", "LogicalIfName = '".. logIfName .. "'")) then
          valid = pptp.config (pptpConfTable, db.getAttribute ("Pptp","LogicalIfName",logIfName,"_ROWID_"), "edit")
      else
          valid = pptp.config (pptpConfTable, "-1", "add")
      end
      -- pptp completed here
      --
      -- pppoe status update
      if (valid) then
          -- Check the previous WAN ISP Type, If it is PPPoE then
          -- set the Status to '0' for the PPPoE Profile if WAN Supports
          -- PPPoE Prifile selection
          local oldConnType = db.getAttribute ("NimfConf", "_ROWID_", rowid, "ConnectionType")
          if (oldConnType == "pppoe" and NimfConfTable["NimfConf.ConnectionType"] ~= "pppoe") then
              local pppoeConfig1 = db.getRowWhere ("Pppoe", "LogicalIfName = '".. logIfName .. "'")
              if (pppoeConfig1 ~= nil and pppoeConfig1["Pppoe.ProfileName"] ~= nil) then
                  -- Update old profile status
                  local profileConfig1 = db.getRow ("PppoeProfile", "ProfileName", pppoeConfig1["Pppoe.ProfileName"])
                  if (profileConfig1 ~= nil) then
                      profileConfig1["PppoeProfile.Status"] = "0"
                      valid = db.update ("PppoeProfile", profileConfig1, profileConfig1["PppoeProfile._ROWID_"])
                  else
                      valid = true
                  end
              else
                  valid = true
              end
          else
              valid = true
          end
      end
      --pppoe profile status configuration done


      -- Set the wan advanced configuration
    NimfConfTable["NimfConf.AccountName"] = inputTable["Wan.AccountName"] or ''
    NimfConfTable["NimfConf.DomainName"] = inputTable["Wan.DomainName"] or ''
    
	if (valid) then
        valid = nimf.config (NimfConfTable, rowid, "edit")
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end
end

function nimfView.cmsIpv6WanConfigGet(wanName)
    local outTable = {}
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. wanName .. "' AND AddressFamily=10")
    if (NimfRow ~= nil) then
    	if(NimfRow["NimfConf.ConnectionType6"] == "dhcp6c") then
	        outTable["NimfConf.ConnectionType6"] = "dhcp6c"
        elseif(NimfRow["NimfConf.ConnectionType6"] == "ifStatic6") then
            outTable["NimfConf.ConnectionType6"] = "ifStatic6"
        elseif(NimfRow["NimfConf.ConnectionType6"] == "pppoe6") then
            outTable["NimfConf.ConnectionType6"] = "pppoe6"
        end

        --adding v6 dhcp row
        local DhcpRow = db.getRowWhere ("dhcpv6c","LogicalIfName = '" .. wanName .. "'")
        if(DhcpRow ~= nil) then
            outTable["dhcpv6c.statelessMode"] = DhcpRow["dhcpv6c.statelessMode"]
            outTable["dhcpv6c.requestPreferredAddress"] = DhcpRow["dhcpv6c.requestPreferredAddress"]
            outTable["dhcpv6c.preferredAddress"] = DhcpRow["dhcpv6c.preferredAddress"]
            outTable["dhcpv6c.preferredAddressPrefixLength"] = DhcpRow["dhcpv6c.preferredAddressPrefixLength"]
            outTable["dhcpv6c.prefixDelegation"] = DhcpRow["dhcpv6c.prefixDelegation"]
        end
        local staticTable = db.getRowWhere ("ifStatic","LogicalIfName = '" .. wanName .. "' AND AddressFamily=10")
        outTable = util.tableAppend (outTable, staticTable)
        local pppoeTable = db.getRowWhere ("Pppoe6","LogicalIfName = '" .. wanName .. "'")
        outTable = util.tableAppend (outTable, pppoeTable)
    end
    return outTable
end

function nimfView.cmsIpv6WanConfig(inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    --TODO inputTable["ifStatic.StaticIp"] and inputTable["ifStatic.PrefixLength"] may be nil. so take care of it.
    if( nimfView.IPv6SubnetAlreadyInUseExceptWanIP(inputTable["ifStatic.StaticIp"] , inputTable["ifStatic.PrefixLength"] )== "ERROR")then
	print("IPv6 address already in use, try other ip.")
        return "ERROR", "ERR_IPV6_SUBNET_IN_USE"
    end
    local wanName = inputTable["NimfConf.LogicalIfName"]    
    -- checking for Wan Configuration
    local where = "LogicalIfName = '" .. wanName .. "' AND AddressFamily = 10"
    local nimfConfig = db.getRowWhere ("NimfConf", where)
    if (nimfConfig == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end
	rowid = nimfConfig["NimfConf._ROWID_"]
    db.beginTransaction() --begin transaction
    local valid = false
    
    local NimfConfTable = {}
    
    NimfConfTable["NimfConf.LogicalIfName"] = wanName
    NimfConfTable["NimfConf.AddressFamily"] = "10"

    --isp determination
    if(inputTable["NimfConf.ConnectionType6"] == "dhcp6c") then
        NimfConfTable["NimfConf.ConnectionType6"] = "dhcp6c"
    elseif (inputTable["NimfConf.ConnectionType6"] == "ifStatic6") then
        NimfConfTable["NimfConf.ConnectionType6"] = "ifStatic6"
    else
        NimfConfTable["NimfConf.ConnectionType6"] = "pppoe6"
    end

    --dhcpv6c
    inputTable["dhcpv6c.LogicalIfName"] = wanName
    if (db.existsRowWhere("dhcpv6c", "LogicalIfName='" .. wanName .. "'")) then
        valid = nimfView.dhcp6Config (inputTable, db.existsRowWhere ("dhcpv6c", "LogicalIfName='" .. wanName .. "'"), "edit")
    else
        valid = nimfView.dhcp6Config (inputTable, "-1", "add")
    end

    --ifStatic of v6
    inputTable["ifStatic.LogicalIfName"] = wanName
	inputTable["ifStatic.AddressFamily"] = "10"
    if (db.existsRowWhere("ifStatic", where)) then
        valid = ifStatic.config (inputTable, db.existsRowWhere("ifStatic", where), "edit")
    else
        valid = ifStatic.config (inputTable, "-1", "add")
    end

    --Pppoe6
    inputTable["Pppoe6.LogicalIfName"] = wanName
    if (db.existsRowWhere("Pppoe6", "LogicalIfName='" .. wanName .. "'")) then
        valid = nimfView.pppoe6Config (inputTable, db.existsRowWhere ("Pppoe6", "LogicalIfName='" .. wanName .. "'"), "edit")
    else
        valid = nimfView.pppoe6Config (inputTable, "-1", "add")
    end
    if (db.existsRowWhere("dhcpv6c", "LogicalIfName='" .. wanName .. "PPPOE6'")) then
        local dhcpv6Table = {}
        if (inputTable["Pppoe6.Dhcpv6Opt"] == "1") then
            dhcpv6Table["dhcpv6c.statelessMode"] = "1"
            dhcpv6Table["dhcpv6c.prefixDelegation"] = "0"
        elseif (inputTable["Pppoe6.Dhcpv6Opt"] == "2") then
            dhcpv6Table["dhcpv6c.statelessMode"] = "0"
            dhcpv6Table["dhcpv6c.prefixDelegation"] = "0"
        elseif (inputTable["Pppoe6.Dhcpv6Opt"] == "3") then
            dhcpv6Table["dhcpv6c.statelessMode"] = "1"    
            dhcpv6Table["dhcpv6c.prefixDelegation"] = "1"
        end
        if (dhcpv6Table ~= nil) then
            valid = nimfView.dhcp6Config (dhcpv6Table, db.existsRowWhere ("dhcpv6c", "LogicalIfName='" .. wanName .. "PPPOE6'"), "edit")
        end
    end

    --nimfConf  Saving
    if (valid) then
        valid = nimf.config (NimfConfTable, rowid, "edit")            
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV6WAN_CONFIG_FAILED"
    end
end


