-- remoteMgmtCLI.lua
-- Bhanu R
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 19mar13,nik  fixed SPR #36595
-- 27Jun12,sig Fixed for SPR 32985
-- 23nov11,nik changes to fix SPR 27996
-- 27dec08,ss  did modifications for rmi
-- 27sep08,rbh written.
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/firewall"
require "teamf1lualib/ifDev"
--require "teamf1lualib/sslvpn"

--
-- This routine validates the accessMgmt configuration information.
function accessMgmtInputVal(configRow)
    if (configRow["accessMgmt.accessMgmtEnable"] == "0" and configRow["accessMgmt.accessMgmt6Enable"] == "0" ) then
        return true
    end
    if (configRow["accessMgmt.accessMgmt6Enable"] == "1" and db.getAttribute("networkInfo","_ROWID_","1","netWorkMode") ~= "3") then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure enableIPv6 option.")
        return false
    end
    if (configRow["accessMgmt.IPVersion"] == nil or configRow["accessMgmt.IPVersion"] == '') then
        printCLIError("Please enter ip_version.")
        return false
    end
    if (configRow["accessMgmt.IPVersion"] == "10" and db.getAttribute("networkInfo","_ROWID_","1","netWorkMode") ~= "3") then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure ip_version option as IPv6.")
        return false
    end
    if (configRow["accessMgmt.IPVersion"] == "2" and configRow["accessMgmt.accessMgmtEnable"] == "1") then
        if (configRow["accessMgmt.accessType"] == nil or configRow["accessMgmt.accessType"] == '') then
            printCLIError("Please enter access_type.")
            return false
        end
    end
    if (configRow["accessMgmt.IPVersion"] == "10" and configRow["accessMgmt.accessMgmt6Enable"] == "1") then
        if (configRow["accessMgmt.accessType6"] == nil or configRow["accessMgmt.accessType6"] == '') then
            printCLIError("Please enter access_type6.")
            return false
        end
    end
    if(configRow["accessMgmt.accessMgmtEnable"] == "1") then
        if (configRow["accessMgmt.accessType"] == "2" ) then
            if (configRow["accessMgmt.onlyThisIP"] == nil or configRow["accessMgmt.onlyThisIP"] == '') then
                printCLIError ("Invalid Only_This_PC IP ")
                return false
            end
        end
        if (configRow["accessMgmt.accessType"] == "1") then
            if (configRow["accessMgmt.accessMgmtIP2"] == nil or configRow["accessMgmt.accessMgmtIP2"] == '') then
                printCLIError ("Invalid end_address")
                return false
            end
            if (configRow["accessMgmt.accessMgmtIP1"] == nil or configRow["accessMgmt.accessMgmtIP1"] == '') then
                printCLIError ("Invalid from_address ")
                return false
            end
            local range = ifDev.ipAddrRange(configRow["accessMgmt.accessMgmtIP1"],configRow["accessMgmt.accessMgmtIP2"])
            if (range == -1) then
                printCLIError ("from_address cannot be greater than end_address ")
                return false
            end
        end
    end
    if(configRow["accessMgmt.accessMgmt6Enable"] == "1") then
        if (configRow["accessMgmt.accessType6"] == "2" ) then
            if (configRow["accessMgmt.onlyThisIP6"] == nil or configRow["accessMgmt.onlyThisIP6"] == '') then
                printCLIError ("Invalid Only_This_PC IPv6 IP")
                return false
            end
        end
        if (configRow["accessMgmt.accessType6"] == "1" ) then
            if (configRow["accessMgmt.accessMgmtIP4"] == nil or configRow["accessMgmt.accessMgmtIP4"] == '') then
                printCLIError ("Invalid end_address6")
                return false
            end
            if (configRow["accessMgmt.accessMgmtIP3"] == nil or configRow["accessMgmt.accessMgmtIP3"] == '') then
                printCLIError ("Invalid from_address6")
                return false
            end
        end
    end
    if (configRow["accessMgmt.serviceType"] == "0" ) then
        if ((tonumber(configRow["accessMgmt.port"]) <= 1024 ) and 
            (configRow["accessMgmt.port"] ~= "443") ) then
            printCLIError ("Port already in use")
            return false
        end
    end
    return true
end

--
-- This routine reads in the current remote https configuration information
-- into a configuration object.
function accessMgmtHttpsInit(args)
    local rowId = -1

    local configRow = db.getRow("accessMgmt", "servicetype", "0")
    if (configRow == nil) then
       return rowId, {}
    end
    if (db.getAttribute("networkInfo","_ROWID_","1","netWorkMode") ~= "3") then
        configRow["accessMgmt.accessMgmt6Enable"] = "0"
    end
    if(configRow["accessMgmt.accessType"] == "2" ) then
        configRow["accessMgmt.onlyThisIP"] = configRow["accessMgmt.accessMgmtIP1"]
    end
    if(configRow["accessMgmt.accessType6"] == "2" ) then
        configRow["accessMgmt.onlyThisIP6"] = configRow["accessMgmt.accessMgmtIP3"]
    end
    rowId = configRow["accessMgmt._ROWID_"]
    configRow["accessMgmt.accessType"] = nil
    configRow["accessMgmt.accessType6"] = nil
    return rowId, configRow
end

--
-- This routine reads in the current telnet https configuration information
-- into a configuration object.
function accessMgmtTelnetInit(args)
    local rowId = -1

    local configRow = db.getRow("accessMgmt", "serviceType", "1")
    if (configRow == nil) then
       return rowId, {}
    end
    if (db.getAttribute("networkInfo","_ROWID_","1","netWorkMode") ~= "3") then
        configRow["accessMgmt.accessMgmt6Enable"] = "0"
    end
    if(configRow["accessMgmt.accessType"] == "2" ) then
        configRow["accessMgmt.onlyThisIP"] = configRow["accessMgmt.accessMgmtIP1"]
    end
    if(configRow["accessMgmt.accessType6"] == "2" ) then
        configRow["accessMgmt.onlyThisIP6"] = configRow["accessMgmt.accessMgmtIP3"]
    end
    rowId = configRow["accessMgmt._ROWID_"]   
    configRow["accessMgmt.accessType"] = nil
    configRow["accessMgmt.accessType6"] = nil
    return rowId, configRow
end


--
-- This routine saves the modified remote management configuration information.
function accessMgmtCfgSave(configRow)
   
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = ""

    --[[errorFlag, statusCode = remoteMgmt.config (configRow)

    -- save db if no error
    if (errorFlag == "OK") then 
        db.save()
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage]]--

    if(configRow["accessMgmt.accessType"] == "2" ) then
        configRow["accessMgmt.accessMgmtIP1"] = configRow["accessMgmt.onlyThisIP"]
    end
    if(configRow["accessMgmt.accessType6"] == "2" ) then
        configRow["accessMgmt.accessMgmtIP3"] = configRow["accessMgmt.onlyThisIP6"]
    end
    configRow["accessMgmt.onlyThisIP"] = nil
    configRow["accessMgmt.onlyThisIP6"] = nil

    if(configRow["accessMgmt.serviceType"] == "0") then
        local httpTable = {}
        httpTable["httpsMgmt.httpsMgmtPort"] = configRow["accessMgmt.port"]
        errorFlag, statusCode = httpsMgmt.config(httpTable, "1", "edit")
        if (errorFlag == "OK") then
            errorFlag, statusCode = fwAccessMgmt.config(configRow, "0", "edit")
        end
    elseif (configRow["accessMgmt.serviceType"] == "1") then
        errorFlag, statusCode = fwAccessMgmt.config(configRow, "1", "edit")        
    end

    --[[if (errorFlag == "OK") then
       local rmtSnmpConfig = db.getRow ("RemoteSnmp", "_ROWID_", "1")
       if (rmtSnmpConfig == nil) then
           errorFlag, statusMessage = fwAccessMgmt.remoteSNMPConfig (configRow, "-1", "add")
       else
           errorFlag, statusMessage = fwAccessMgmt.remoteSNMPConfig (configRow, "1", "edit")
       end
    end]]--

    if (errorFlag == "OK") then db.save() end             
    statusMessage= db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    return errorFlag, statusMessage
end

--
-- This routine implements the CLI command to show remote mgmt
-- configuration
function remoteMgmtGet()
    local accessMgmtTbl = nil
    local serviceType = {}
    serviceType ["https"] = 0
    serviceType ["telnet"] = 1

    for k,v in pairs(serviceType) do

        printLabel ("Remote Mgmt Configuration for " .. k )    
        accessMgmtTbl = db.getRow ("accessMgmt","serviceType", serviceType[k])
        if (accessMgmtTbl ~= nil) then
            if (accessMgmtTbl["accessMgmt.accessMgmtEnable"] == "0" and accessMgmtTbl["accessMgmt.accessMgmt6Enable"] == "0") then
                print ("Remote access through " .. k .. " is currently disabled for both IPv4 and IPv6")
            else
                if (accessMgmtTbl["accessMgmt.accessMgmtEnable"] == "1") then
                    if (accessMgmtTbl["accessMgmt.accessType"] == "0" ) then
                        print ("IPv4 access granted to everyone")
                    elseif (accessMgmtTbl["accessMgmt.accessType"] == "1" ) then
                        print ("IPv4 access granted to a range of IPs from :" .. accessMgmtTbl["accessMgmt.accessMgmtIP1"] .. " to ".. accessMgmtTbl["accessMgmt.accessMgmtIP2"])
                    elseif (accessMgmtTbl["accessMgmt.accessType"] == "2" ) then
                        print ("IPv4 access granted to the IP " .. accessMgmtTbl["accessMgmt.accessMgmtIP1"])
                    end
                else
                    print ("IPv4 Remote access through " .. k .. " is currently disabled")
                end
                if (accessMgmtTbl["accessMgmt.accessMgmt6Enable"] == "1") then
                    if (accessMgmtTbl["accessMgmt.accessType6"] == "0" ) then
                        print ("IPv6 access granted to everyone")
                    elseif (accessMgmtTbl["accessMgmt.accessType6"] == "1" ) then
                        print ("IPv6 access granted to a range of IPs from :" .. accessMgmtTbl["accessMgmt.accessMgmtIP3"] .. " to ".. accessMgmtTbl["accessMgmt.accessMgmtIP4"])
                    elseif (accessMgmtTbl["accessMgmt.accessType6"] == "2" ) then
                        print ("IPv6 access granted to the IP " .. accessMgmtTbl["accessMgmt.accessMgmtIP3"])
                    end
                else
                     print ("IPv6 Remote access through " .. k .. " is currently disabled")
                end
                print ("port being used : " .. accessMgmtTbl["accessMgmt.port"])
            end
        end
    end
end



