--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### June 10, 2008
#### File: sslvpn.lua
#### Description: SSL VPN functions
#### Revisions:
01d, 28oct13, ksr , changes to return error if portal/domain being configured is already present
01c, 23oct13, ksr , support for sslvpn wizard
01b, 26aug13, ksr , fixed SPR 39121
01a, 17May12, sig ,fixed for spr 31796
]]--


--************* Requires *************
require "teamf1lualib/db"
require "sslvpnLuaLib"
--************* Initial Code *************

--package Admin Configurations
sslvpn = {}
httpsMgmt = {}

-- Subnet Mask Length array
subnetLength = {}
subnetLength[0] = 0
subnetLength[128] = 1
subnetLength[192] = 2
subnetLength[224] = 3
subnetLength[240] = 4
subnetLength[248] = 5
subnetLength[252] = 6
subnetLength[254] = 7
subnetLength[255] = 8

--************* Functions *************
----
--function for restricting some restricted characters as thesse special character create problem and user is not able to login
--
function sslvpn.restrictedChar(inputString)
	if (inputString == nil or inputString == '' ) then return 'N' end
	local flag= 'N'
	local mask = "[$~`!#&*()%-+\\|;:><]"
	for i in string.gmatch(inputString, mask) do 
		flag = 'Y' 
		statusMessage = i .. " is not allowed in username and password."
	end
	return flag or 'N'
end

-- Portal Layout Configuration
function sslvpn.portalLayoutConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- if add , prepare Portal Url
    if (operation == "add") then
        local wanIp = db.getAttribute("networkInterface", "LogicalIfName", "WAN1", "ipaddr")
        inputTable["SSLVPNPortalLayout.PortalURL"] = "https://" .. wanIp .. "/portal/" .. inputTable["SSLVPNPortalLayout.PortalLayoutName"]
    end
    
    -- validate
   -- if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            if (inputTable["SSLVPNPortalLayout.UseCount"] > "0") then
                valid = false       
            else
                valid = db.delete(DBTable, inputTable)
            end
        end
   -- end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_PORTAL_LAYOUT_CONFIG_FAILED"
    end
end

-- Function for Deleting Portal Layout(s)  
function sslvpn.deleteLayouts (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["SSLVPNPortalLayout._ROWID_"] = v;
        local row = db.getRow("SSLVPNPortalLayout", "_ROWID_", v)
        if (row["SSLVPNPortalLayout.UseCount"] > "0") then
            valid = false
        else
            valid = db.delete("SSLVPNPortalLayout", inTable) 
        end
        if(valid==false) then
            break;
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "SSLVPN_PORTAL_LAYOUT_DELETE_SUCCESS"
    else
        db.rollback()
        return "ERROR", "SSLVPN_PORTAL_LAYOUT_HAS_DOMAIN_DEL_FAILED"
    end
end

-- Function for setting default Layout
function sslvpn.setDefault (rowId)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    -- checking for default row exists
    if( db.existsRow ("SSLVPNPortalLayout", "DefaultPortal", "2")) then
        local defRow = db.getRow("SSLVPNPortalLayout", "DefaultPortal", "2")
        defRow["SSLVPNPortalLayout.DefaultPortal"] = "0"
        valid = db.update("SSLVPNPortalLayout", defRow, defRow["SSLVPNPortalLayout._ROWID_"])
    end
    
    -- setting default Layout
    local row = db.getRow("SSLVPNPortalLayout", "_ROWID_", rowId)
    if(row["SSLVPNPortalLayout.UseCount"] == "0") then
        valid = false
    else
        if (row["SSLVPNPortalLayout.DefaultPortal"] ~= "1") then
            row["SSLVPNPortalLayout.DefaultPortal"] = "2"
        end
        valid = db.update("SSLVPNPortalLayout", row, rowId)
    end
    

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_PORTAL_LAYOUT_NO_DOMAIN_SET_AS_DEFAULT_FAILED"
    end
end

-- Port Forwarding Config
function sslvpn.portForwardConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    if (operation == "add") then
        local rows = db.getTable(DBTable) or {}
        if (#rows >= 32) then
            db.rollback()
            return "ERROR", "MAX_LMT_RCHD"
        end
    end
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = db.delete(DBTable, inputTable)
        end
    end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
            return "OK", "STATUS_OK"            
    else
        db.rollback()
        return "ERROR", "SSLVPN_PORTFORWARDING_CONFIG_FAILED"
    end
end

-- Resources Config
function sslvpn.resourceDeleteConfig (inputTable, rowid, operation)
    -- validate
    if (operation == "delete") then
            return db.delete(DBTable, inputTable)
    end    
    return false
end

function sslvpn.resourceConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = db.delete(DBTable, inputTable)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"            
    else
        db.rollback()
        return "ERROR", "SSLVPN_RESOURCE_CONFIG_FAILED"
    end
end

-- resources Delete
function sslvpn.deleteResources (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    -- Check for Using Services
    for k,v in pairs(rowIds) do
        local l = 0
        local name = db.getAttribute("SSLVPNResource", "_ROWID_", v, "ResourceName")
        local count = db.getAttribute("SSLVPNResource", "_ROWID_", v, "UseCount")
        local rows = db.getRows ("SSLVPNResourceObject", "ResourceName",name)
        if(tonumber(count) > 0) then
            rows = nil
            valid = false
            errStr = "SSLVPN_RESOURCE_IN_USE_DELETE_FAILED"
        end               
        if (rows) then
            for i,j in pairs(rows) do
                local inTable = {}  
                inTable["SSLVPNResourceObject._ROWID_"] = j["SSLVPNResourceObject._ROWID_"]
                DBTable = "SSLVPNResourceObject"
                valid = sslvpn.resourceDeleteConfig(inTable, "-1", "delete")
                if (valid == false) then
                    errStr = "RESOURCE_OBJECT_DELETE_FAILED"
                    break
                end
            end
        end
    end

    for k,v in pairs(rowIds) do
        if (valid) then
            local inTable1 = {}
            inTable1["SSLVPNResource._ROWID_"] = v;
            DBTable = "SSLVPNResource"
            valid = sslvpn.resourceDeleteConfig(inTable1, "-1", "delete")
            if (valid == false) then
                errStr = "RESOURCE_DELETE_FAILED"
                break
            end
        end
    end 

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "RESOURCE_DELETE_OK"
    else
        db.rollback()
        return "ERROR", errStr
    end
end

-- SSL VPN Client Configuration
function sslvpn.sslVpnClientConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        end
    end 
    
    -- return
    if (valid) then
        db.commitTransaction(true)
            return "OK", "STATUS_OK"            
    else
        db.rollback()
        return "ERROR", "VPN_SSLVPNCLIENT_CONFIG_FAILED"
    end
end

-- Function to configure sslvpn client routes
function sslvpn.sslClientRouteConfig (inputTable, rowid, operation)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    
    if (operation == "add") then
        valid = db.insert(DBTable, inputTable)
    elseif (operation == "edit") then
        valid = false
    elseif (operation == "delete") then
        valid = db.delete(DBTable, inputTable)
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSL_VPN_CLIENT_ROUTE_CONFIG_FAILED"
    end
end

-- Function for SSL VPN Policy Configuration
function sslvpn.policyConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    -- Setting Policy Owner Name
    if(inputTable["SSLVPNPolicy.PolicyType"] == "global") then
        inputTable["SSLVPNPolicy.PolicyOwnerName"] = "global"
    end

    -- Setting value for Destination Object 
    if(inputTable["SSLVPNPolicy.DestinationObjectType"] == "1") then
        inputTable["SSLVPNPolicy.DestinationObject"] =  inputTable["SSLVPNPolicy.ResourceName"]
    elseif(inputTable["SSLVPNPolicy.DestinationObjectType"] == "2") then
        inputTable["SSLVPNPolicy.DestinationObject"] =  inputTable["SSLVPNPolicy.Address"] .. ":" .. inputTable["SSLVPNPolicy.StartPort"] .. "-" .. inputTable["SSLVPNPolicy.EndPort"]
    elseif(inputTable["SSLVPNPolicy.DestinationObjectType"] == "3") then
        inputTable["SSLVPNPolicy.DestinationObject"] =  inputTable["SSLVPNPolicy.Address"] .. "/" .. inputTable["SSLVPNPolicy.MaskLength"] .. ":" .. inputTable["SSLVPNPolicy.StartPort"] .. "-" .. inputTable["SSLVPNPolicy.EndPort"]
    elseif(inputTable["SSLVPNPolicy.DestinationObjectType"] == "4") then
     if(inputTable ["SSLVPNPolicy.IPVersion"] == "2") then  
	inputTable["SSLVPNPolicy.DestinationObject"] =  "::/0:" .. inputTable["SSLVPNPolicy.StartPort"] .. "-" .. inputTable["SSLVPNPolicy.EndPort"]
     else
	inputTable["SSLVPNPolicy.DestinationObject"] =  "0.0.0.0:" .. inputTable["SSLVPNPolicy.StartPort"] .. "-" .. inputTable["SSLVPNPolicy.EndPort"]
     end
    end
    

    if(operation == "edit") then
        if (inputTable["SSLVPNPolicy.DestinationObjectType"] == "1") then
            inputTable["SSLVPNPolicy.ServiceType"] = ""
        end
    end

    -- Checking the resource Objects count
    if ((operation == "add" or operation == "edit") and inputTable["SSLVPNPolicy.ResourceName"] ~= nil) then
        local resObjTable = db.getRowsWhere ("SSLVPNResourceObject", "ResourceName = '" .. inputTable["SSLVPNPolicy.ResourceName"] .. "'")
        if (resObjTable ~= nil and #resObjTable == 0) then
            db.rollback()
            return "ERROR", "NO_RESOURCE_OBJECTS_CONFIGURED"
        end
    end

    -- geting Old Resource Name
    local Rname = ""
    local dstObjType = ""
    if (operation == "edit") then
        dstObjType = db.getAttribute("SSLVPNPolicy", "_ROWID_", rowid, "DestinationObjectType")
        if(dstObjType == "1") then
            Rname = db.getAttribute("SSLVPNPolicy", "_ROWID_", rowid, "ResourceName")
        end
    end

    -- config Policy
    valid = sslvpn.config(inputTable, rowid, operation)

    -- Updating SSLVPNResource Usecount 
    if (valid and inputTable["SSLVPNPolicy.DestinationObjectType"] == "1") then
        if (operation == "add" ) then
            sslvpn.updateResource (inputTable["SSLVPNPolicy.ResourceName"], operation)
        end
    end

    -- decrementing Old Resource UseCount according to Policy
    if (valid and operation  == "edit") then
        if (dstObjType == "1" and inputTable["SSLVPNPolicy.DestinationObjectType"] == "1" ) then
            if (Rname ~= inputTable["SSLVPNPolicy.ResourceName"]) then
                valid = sslvpn.updateResource (Rname, "delete")
                if(valid) then
                    valid = sslvpn.updateResource (inputTable["SSLVPNPolicy.ResourceName"] , "edit")
                end
            end
        elseif(dstObjType == "1" and inputTable["SSLVPNPolicy.DestinationObjectType"] ~= "1" ) then
            valid = sslvpn.updateResource (Rname, "delete")
        elseif(dstObjType ~= "1" and inputTable["SSLVPNPolicy.DestinationObjectType"] == "1") then
            valid = sslvpn.updateResource (inputTable["SSLVPNPolicy.ResourceName"] , "edit")
        end
    end
   
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_POLICY_CONFIG_FAILED"
    end
end


-- SSL VPN Policy Config
function sslvpn.config (inputTable, rowid, operation)
 -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end

-- Function for Deleting SSL VPN Policies
function sslvpn.deletePolicy (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["SSLVPNPolicy._ROWID_"] = v;
    
        -- Updating Resource , if Destination Type is Network Resource
        local dstType = db.getAttribute("SSLVPNPolicy", "_ROWID_", inTable["SSLVPNPolicy._ROWID_"], "DestinationObjectType")
        if(dstType == "1") then
            local ResourceName = db.getAttribute("SSLVPNPolicy", "_ROWID_", inTable["SSLVPNPolicy._ROWID_"], "ResourceName")
            valid = sslvpn.updateResource(ResourceName, "delete");
        else
            valid = true
        end

        if(valid) then
            valid = db.delete("SSLVPNPolicy", inTable) 
        end
    
        if(valid==false) then
            break;
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_POLICY_DELETE_FAILED"
    end
end

-- Function for Updating Resource UseCount
function sslvpn.updateResource (ResourceName, operation)
    local valid = false
    local tmpTable = {}
    local useCount = db.getAttribute("SSLVPNResource", "ResourceName", ResourceName, "UseCount")
    local rowID = db.getAttribute("SSLVPNResource", "ResourceName", ResourceName, "_ROWID_")

    if (operation == "add") then
        tmpTable["SSLVPNResource.UseCount"] = tonumber(useCount) + 1
        valid = db.update("SSLVPNResource", tmpTable, rowID)
    end
    if (operation == "edit") then
        tmpTable["SSLVPNResource.UseCount"] = tonumber(useCount) + 1
        valid = db.update("SSLVPNResource", tmpTable, rowID)
    end

    if (operation == "delete") then
        tmpTable["SSLVPNResource.UseCount"] = tonumber(useCount) - 1
        valid = db.update("SSLVPNResource", tmpTable, rowID)
    end

     -- return
    if (valid) then
        return true
    else
        return false
    end
end

-- Function for setting SSL VPN Active User State
function sslvpn.setUserState (rowid)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local inputTable = {}
    local userName = ""
    
    userName = db.getAttribute("SSLVPNActiveUsers", "_ROWID_", rowid, "UserName")
    local id = db.getAttribute("SSLVPNActiveUsers", "UserName", userName, "_ROWID_")
    inputTable["SSLVPNUserState.ActiveUser"] = "0"
    
    valid = db.update("SSLVPNUserState", inputTable, id)

      -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "SSL_VPN_USER_DISCONNECT_SUCCESS"
    else
        db.rollback()
        return "ERROR", "SSL_VPN_USER_DISCONNECT_FAILED"
    end
end

-- 
-- Funtion to update activeUsers Tbl with associated Tunnel's
-- Statistics
-- 
function sslvpn.sslvpnTunnelStatsGet()
    local status = sslvpnLuaLib.sslTunnelStatsGet()
    if (status == 0) then
        return 0
    else
        return -1
    end
end


-- httpsMgmt config
function httpsMgmt.config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "Administrator Privileges Are Required"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local count = 0
    local portMapTbl = db.getRowsWhere("upnpPortMap","active == 'Yes'" ) or {}
    
    -- validate
    if (httpsMgmt.inputvalidate(inputTable, operation)) then        
        if (operation == "add") then
            return nil
        elseif (operation == "edit") then
            if (inputTable["httpsMgmt.httpsMgmtPort"] == "0") then
                return "ERROR", "Invalid Port Configured."
            elseif (inputTable["httpsMgmt.httpsMgmtPort"] == "80") then
                return "ERROR", "This Port Already Reserved."
            elseif(#portMapTbl ~= 0) then
                for k,v in pairs (portMapTbl) do
                    count = count + 1
                    local row = portMapTbl[count]
                    if(row ~= nil) then
                        if((inputTable["httpsMgmt.httpsMgmtPort"] == row["upnpPortMap.intPort"]) or 
                            (inputTable["httpsMgmt.httpsMgmtPort"] == row["upnpPortMap.extPort"])) then
                            return "ERROR","This Port Already Reserved for UPnP."
                        end
                    end
                end
            end
            valid = db.update("httpsMgmt", inputTable, rowid)
        elseif (operation == "delete") then
            return nil
        end
    end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "Operation Succedded"
    else
        db.rollback()
        return "ERROR", "Configuration Update Failed"
    end
end

-- httpsMgmt inputvalidate
function httpsMgmt.inputvalidate (inputTable, operation)
    if (true) then
            return db.typeAndRangeValidate(inputTable)
        end
    return false
end

function  sslvpn.deleteDefinedResource (rowIds, rName)
	local resObjTable = db.getRowsWhere ("SSLVPNResourceObject", "ResourceName = '" .. rName .. "'")
	local policiesTable = db.getRowsWhere ("SSLVPNPolicy", "ResourceName= '" .. rName .. "'")
	if (policiesTable ~= nil and resObjTable ~= nil and #policiesTable ~= 0 and (#resObjTable == 1 or #resObjTable == #rowIds)) then
		return "ERROR", "CANNOT_DELETE_ONLY_RESOURCE"
	end
	for k,v in pairs(rowIds) do
		configRowId = v
		local rows = {configRowId}
		errorFlag, statusCode = sslvpn.resourceConfig (rows,"-1","delete")
		if(errorFlag ~= "OK") then
			return errorFlag, statusCode
		end	
	end
	return errorFlag, statusCode
end

--[[
**************************************************************
* function sslvpn.calculateMskLength: calculates the mask length
  from a given subnet mask
* parameters: Subnet mask

--]]

function sslvpn.calculateMskLength (subnetMask)
    
    local masklength = 8
   -- split subnetmak  
   local splitSubnetMask = util.split (subnetMask, ".")
   if(splitSubnetMask[2] ~= "255") then
       if(subnetLength[tonumber (splitSubnetMask[2])]) then
           masklength = masklength + subnetLength[tonumber (splitSubnetMask[2])]
       end
   elseif(splitSubnetMask[3] ~= "255") then
       masklength = 16
       if(subnetLength[tonumber (splitSubnetMask[3])]) then
           masklength = masklength + subnetLength[tonumber (splitSubnetMask[3])]
       end
   elseif(splitSubnetMask[4] ~= "255") then
       masklength = 24
       if(subnetLength[tonumber (splitSubnetMask[4])]) then
           masklength = masklength + subnetLength[tonumber (splitSubnetMask[4])]
       end
   else
       masklength = 32
   end
   return masklength
end

--[[
**************************************************************
* function getSubnetMask: calculates subnet mask from mask length  
* parameters: rowid

--]]

function sslvpn.getSubnetMask (subnetMaskLen)
    local maskTbl = {}
    local num = nil    
    local subnetLengthNum = tonumber (subnetMaskLen)
    if( subnetLengthNum~= nil and subnetLengthNum ~= '')then
        if(subnetLengthNum == 32) then
            maskTbl[1] = "255"
            maskTbl[2] = "255"
            maskTbl[3] = "255"
            maskTbl[4] = "255"
        elseif (subnetLengthNum == 24) then
            maskTbl[1] = "255"
            maskTbl[2] = "255"
            maskTbl[3] = "255"
            maskTbl[4] = "0"
        elseif (subnetLengthNum == 16) then
            maskTbl[1] = "255"
            maskTbl[2] = "255"
            maskTbl[3] = "0"
            maskTbl[4] = "0"
        elseif (subnetLengthNum == 8) then
            maskTbl[1] = "255"
            maskTbl[2] = "0"
            maskTbl[3] = "0"
            maskTbl[4] = "0"
        elseif (subnetLengthNum > 24) then        
            maskTbl[1] = "255"
            maskTbl[2] = "255"
            maskTbl[3] = "255"
            num = subnetLengthNum - 24
            for k,v in pairs (subnetLength) do
                if(num == v) then
                    maskTbl[4] = k
                    break
                end
            end
        elseif (subnetLengthNum > 16 and subnetLengthNum < 24) then
            maskTbl[1] = "255"
            maskTbl[2] = "255"
            maskTbl[4] = "0"
            num = subnetLengthNum - 16
            for k,v in pairs (subnetLength) do
                if(num == v) then
                    maskTbl[3] = k
                    break
                end
            end
        elseif (subnetLengthNum > 8 and subnetLengthNum < 16) then
            maskTbl[1] = "255"        
            maskTbl[3] = "0"
            maskTbl[4] = "0"
            num = subnetLengthNum - 8
            for k,v in pairs (subnetLength) do
                if(num == v) then
                    maskTbl[2] = k
                    break
                end
            end
        end
    end
    return maskTbl
end


function sslvpn.wizardPortalLayoutConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    local valid = false   
    -- if add , prepare Portal Url
    if (operation == "add") then
        local wanIp = db.getAttribute("networkInterface", "LogicalIfName", "WAN1", "ipaddr")
        inputTable["SSLVPNPortalLayout.PortalURL"] = "https://" .. wanIp .. "/portal/" .. inputTable["SSLVPNPortalLayout.PortalLayoutName"]
    end
    local  DBTable = "SSLVPNPortalLayout"
    -- validate     
    if (operation == "add") then
        valid = db.insert(DBTable, inputTable)
    elseif (operation == "edit") then
        valid = db.update(DBTable, inputTable, rowid)
    elseif (operation == "delete") then
        if (inputTable["SSLVPNPortalLayout.UseCount"] > "0") then
            valid = false       
        else
            valid = db.delete(DBTable, inputTable)
        end
    end

    -- return
    if (valid) then
        return "OK", "STATUS_OK"
    else
        return "ERROR", "SSLVPN_PORTAL_LAYOUT_CONFIG_FAILED"
    end
end

function sslvpn.wizardSslVpnClientConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    local valid = false
    local DBTable = "SSLVPNClientConf"
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        end
    end 
    
    -- return
    if (valid) then
            return "OK", "STATUS_OK"            
    else
        return "ERROR", "VPN_SSLVPNCLIENT_CONFIG_FAILED"
    end
end

function sslvpn.wizardPortForwardConfig (inputTable, rowid, operation, DBTable)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    local valid = false

    if (operation == "add") then
        valid = db.insert(DBTable, inputTable)
    elseif (operation == "edit") then
        valid = db.update(DBTable, inputTable, rowid)
    elseif (operation == "delete") then
        valid = db.delete(DBTable, inputTable)
    end
    
    -- return
    if (valid) then
        return "OK", "STATUS_OK"            
    else
        return "ERROR", "SSLVPN_PORTFORWARDING_CONFIG_FAILED"
    end
end

function sslvpn.wizardSet (wizardTbl)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    local layoutTbl = {}
    db.beginTransaction() --begin transaction
    local valid = false
    layoutTbl["SSLVPNPortalLayout.PortalLayoutName"] = wizardTbl["SSLVPNPortalLayout.PortalLayoutName"]
    layoutTbl["SSLVPNPortalLayout.PortalSiteTitle"] = wizardTbl["SSLVPNPortalLayout.PortalSiteTitle"]
    layoutTbl["SSLVPNPortalLayout.BannerTitle"] = wizardTbl["SSLVPNPortalLayout.BannerTitle"]
    layoutTbl["SSLVPNPortalLayout.BannerMessage"] = wizardTbl["SSLVPNPortalLayout.BannerMessage"]
    layoutTbl["SSLVPNPortalLayout.DisplayBanner"] = wizardTbl["SSLVPNPortalLayout.DisplayBanner"]
    layoutTbl["SSLVPNPortalLayout.EnableHTTPMetaTags"] = wizardTbl["SSLVPNPortalLayout.EnableHTTPMetaTags"]
    layoutTbl["SSLVPNPortalLayout.EnableActiveXWebCacheCleaner"] = wizardTbl["SSLVPNPortalLayout.EnableActiveXWebCacheCleaner"]
    layoutTbl["SSLVPNPortalLayout.EnableVPNTunnel"] = wizardTbl["SSLVPNPortalLayout.EnableVPNTunnel"]
    layoutTbl["SSLVPNPortalLayout.EnablePortForwarding"] = wizardTbl["SSLVPNPortalLayout.EnablePortForwarding"]
    if (layoutTbl["SSLVPNPortalLayout.PortalLayoutName"] ~= nil and layoutTbl["SSLVPNPortalLayout.PortalLayoutName"] ~= "") then
        DBTable = "SSLVPNPortalLayout"
        local wanIp = db.getAttribute("networkInterface", "LogicalIfName", "WAN1", "ipaddr")
        layoutTbl["SSLVPNPortalLayout.PortalURL"] = "https://" .. wanIp .. "/portal/" .. layoutTbl["SSLVPNPortalLayout.PortalLayoutName"]
        valid = db.insert(DBTable, layoutTbl)
        if (not valid) then
            return "ERROR", "SSLVPN_PORTAL_LAYOUT_CONFIG_FAILED"
        end
    end
    local domainTbl = {}
    domainTbl["USERDBDomains.DomainName"] = wizardTbl["USERDBDomains.DomainName"]
    domainTbl["USERDBDomains.AuthenticationType"] = wizardTbl["USERDBDomains.AuthenticationType"]
    domainTbl["USERDBDomains.PortalLayoutName"] = wizardTbl["SSLVPNPortalLayout.PortalLayoutName"]
    domainTbl["USERDBDomains.AuthenticationServer"] = wizardTbl["USERDBDomains.AuthenticationServer"]
    domainTbl["USERDBDomains.AuthenticationRadiusSecret"] = wizardTbl["USERDBDomains.AuthenticationRadiusSecret"]
    domainTbl["USERDBDomains.NTDomainWorkGroup"] = wizardTbl["USERDBDomains.NTDomainWorkGroup"]
    domainTbl["USERDBDomains.LDAPBaseDN"] = wizardTbl["USERDBDomains.LDAPBaseDN"]
    domainTbl["USERDBDomains.ActiveDirectoryDomain"] = wizardTbl["USERDBDomains.ActiveDirectoryDomain"]
    domainTbl["USERDBDomains.TimeOut"] = wizardTbl["USERDBDomains.TimeOut"]
    domainTbl["USERDBDomains.Retries"] = wizardTbl["USERDBDomains.Retries"]
    if (valid) then
        if (domainTbl["USERDBDomains.DomainName"] ~= nil and domainTbl["USERDBDomains.DomainName"] ~= "") then
            if (domainTbl["USERDBDomains.PortalLayoutName"] == nil or domainTbl["USERDBDomains.PortalLayoutName"] == "") then
                domainTbl["USERDBDomains.PortalLayoutName"] = db.getAttribute ("SSLVPNPortalLayout", "_ROWID_", "1", "PortalLayoutName") or ''
            end
            DBTable = "USERDBDomains"

            if (db.existsRow ("SSLVPNPortalLayout", "PortalLayoutName", domainTbl["USERDBDomains.PortalLayoutName"])) then
                local portalRowId = db.getAttribute ("SSLVPNPortalLayout", "PortalLayoutName", domainTbl["USERDBDomains.PortalLayoutName"], "_ROWID_")
                local portalTable = {}
                if(domainTbl["USERDBDomains.AuthenticationType"] == "active-directory" and domainTbl["USERDBDomains.ActiveDirectoryDomain"] ~= nil) then
                    if(db.existsRow("USERDBDomains", "ActiveDirectoryDomain", domainTbl["USERDBDomains.ActiveDirectoryDomain"])) then
                        valid = false
                        return "ERROR", "SSLVPN_DUPLICATE_DOMAIN_CONFIG_FAILED"
                    end
                end
                local useCount =  db.getAttribute ("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount")
                if (valid) then
                    valid = db.setAttribute("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount", tonumber(useCount) + 1)
                end
                if(valid and db.typeAndRangeValidate(domainTbl)) then
                    valid = db.insert(DBTable, domainTbl)
                    local inTable = {}                
                    if (valid) then
                        inTable["USERDBGroups.GroupName"] = domainTbl["USERDBDomains.DomainName"]
                        inTable["USERDBGroups.DomainName"] = domainTbl["USERDBDomains.DomainName"]
                        inTable["USERDBGroups.GroupTimeOut"] = domainTbl["USERDBDomains.TimeOut"]
                        inTable["USERDBGroups.DefaultGroup"] = 1
                        if (valid) then
                            valid = db.insert("USERDBGroups", inTable)
                        end                    
                    end
                end
                if (not valid) then
                    return "ERROR", "SSLVPN_DOMAIN_CONFIG_FAILED"
                end
            --errorFlag, statusCode = sslAdmin.domainConfig (domainTbl, "-1", "add")
            end
        end
    end
    local userTbl = {}
    userTbl["USERDBUsers.UserName"] = wizardTbl["USERDBUsers.UserName"]
    userTbl["USERDBUsers.UserType"] = wizardTbl["USERDBUsers.UserType"]
    userTbl["USERDBUsers.GroupName"] = wizardTbl["USERDBDomains.DomainName"]
    userTbl["USERDBUsers.UserTimeOut"] = wizardTbl["USERDBUsers.UserTimeOut"]
    userTbl["USERDBUsers.Password"] = wizardTbl["USERDBUsers.Password"]
    if (valid) then
        if (userTbl["USERDBUsers.GroupName"] == nil or userTbl["USERDBUsers.GroupName"] == "") then
            userTbl["USERDBUsers.GroupName"] = db.getAttribute ("USERDBDomains", "_ROWID_", "1", "DomainName") or ''
        end

        local oldUname = ""
        local newUname = userTbl["USERDBUsers.UserName"]

        -- Changing user type to 1 : external user
        -- -- if user type selected as sslvpn and authentication
        -- -- type is not local.
        local domainName = ''
        if (userTbl["USERDBUsers.GroupName"] ~= nil and userTbl["USERDBUsers.GroupName"] ~= "") then
            domainName = db.getAttribute("USERDBGroups","GroupName",userTbl["USERDBUsers.GroupName"],"DomainName") or ''
        end
        if (userTbl["USERDBUsers.UserType"] == "3" or userTbl["USERDBUsers.UserType"] == "4") then
            userTbl["USERDBUsers.DenyLoginFromWan"] = 1
        else
            userTbl["USERDBUsers.DenyLoginFromWan"] = 0
        end

        if (domainName ~= '' and domainName ~= nil and (db.getAttribute("USERDBDomains","DomainName",domainName,"AuthenticationType") ~= "local")) then
            local domainTbl = db.getRowsWhere ("USERDBDomains","AuthenticationType!='local'")
            local i = 0
            local count = 0
            for k,v in pairs(domainTbl) do
                i = i + 1
                local row = domainTbl[i]
                if (row ~= nil) then
                    local groups = db.getRows ("USERDBGroups","DomainName",row["USERDBDomains.DomainName"])
                    local j = 0
                    for k,v in pairs(groups) do
                        j = j + 1
                        local row1 = groups[j]
                        if (row1 ~= nil) then
                            local users = db.getRows ("USERDBUsers","GroupName",row1["USERDBGroups.GroupName"])
                            count = count + (#users)
                        end
                    end
                end
            end
            if (count >= 128) then
                return "ERROR", "EXT_USER_COUNT_EXCEEDS_MAX_LIMIT"
            end
        end
    end
    DBTable = "USERDBUsers"
	if (sslvpn.restrictedChar(userTbl["USERDBUsers.UserName"]) == 'Y' or sslvpn.restrictedChar(userTbl["USERDBUsers.Password"]) == 'Y')then
		return "ERROR", statusMessage
	end
    if (valid and db.typeAndRangeValidate(userTbl)) then
        valid = db.insert(DBTable, userTbl)
    end
    --errorFlag, statusCode = sslAdmin.userConfig (userTbl, "-1", "add")
    local clientTbl = {}
    clientTbl["SSLVPNClientConf.EnableFullTunnel"] = wizardTbl["SSLVPNClientConf.EnableFullTunnel"]
    clientTbl["SSLVPNClientConf.DNSSuffix"] = wizardTbl["SSLVPNClientConf.DNSSuffix"]
    clientTbl["SSLVPNClientConf.PrimaryDNSServer"] = wizardTbl["SSLVPNClientConf.PrimaryDNSServer"]
    clientTbl["SSLVPNClientConf.SecondaryDNSServer"] = wizardTbl["SSLVPNClientConf.SecondaryDNSServer"]
    clientTbl["SSLVPNClientConf.BeginClientAddress"] = wizardTbl["SSLVPNClientConf.BeginClientAddress"]
    clientTbl["SSLVPNClientConf.EndClientAddress"] = wizardTbl["SSLVPNClientConf.EndClientAddress"]
    clientTbl["SSLVPNClientConf.lcpTimeout"] = wizardTbl["SSLVPNClientConf.lcpTimeout"]
    if (valid) then
        configRow = db.getRow ("SSLVPNClientConf", "_ROWID_", "1")
        DBTable = "SSLVPNClientConf"
        if (configRow == nil) then
            valid = db.insert(DBTable, clientTbl)
            --errorFlag, statusCode = sslvpn.sslVpnClientConfig (clientTbl, "-1", "add")
        else
            valid = db.update(DBTable, clientTbl, "1")
            --errorFlag, statusCode = sslvpn.sslVpnClientConfig (clientTbl, "1", "edit")
        end
        if (not valid) then
            return "ERROR", "VPN_SSLVPNCLIENT_CONFIG_FAILED"
        end
    end
    local routeTbl = {}
    routeTbl["SSLVPNClientRoute.DestinationNetwork"] = wizardTbl["SSLVPNClientRoute.DestinationNetwork"]
    routeTbl["SSLVPNClientRoute.SubnetMask"] = wizardTbl["SSLVPNClientRoute.SubnetMask"]
    DBTable = "SSLVPNClientRoute"
    if (valid) then
        routeTbl["SSLVPNClientRoute.IPVersion"] = "1"
        if(routeTbl["SSLVPNClientRoute.DestinationNetwork"] ~= nil and routeTbl["SSLVPNClientRoute.DestinationNetwork"] ~= "" and routeTbl["SSLVPNClientRoute.SubnetMask"] ~= nil and routeTbl["SSLVPNClientRoute.SubnetMask"] ~= "") then  
            valid = db.insert(DBTable, routeTbl)
        end
        --errorFlag, statusCode = sslvpn.sslClientRouteConfig (routeTbl, "-1", "add")
        if (not valid) then
            return "ERROR", "SSL_VPN_CLIENT_ROUTE_CONFIG_FAILED"
        end
    end
    local portForwAppTbl = {}
    portForwAppTbl["SSLVPNPortForwardingAppsMap.ServerIP"] = wizardTbl["SSLVPNPortForwardingAppsMap.ServerIP"]
    portForwAppTbl["SSLVPNPortForwardingAppsMap.Port"] = wizardTbl["SSLVPNPortForwardingAppsMap.Port"]
    DBTable = "SSLVPNPortForwardingAppsMap"

    if (valid) then
        if(portForwAppTbl["SSLVPNPortForwardingAppsMap.ServerIP"] ~= nil and portForwAppTbl["SSLVPNPortForwardingAppsMap.ServerIP"] ~= "" and portForwAppTbl["SSLVPNPortForwardingAppsMap.Port"] ~= nil and portForwAppTbl["SSLVPNPortForwardingAppsMap.Port"] ~= "") then
            valid = db.insert(DBTable, portForwAppTbl)
        end
        --errorFlag, statusCode = sslvpn.portForwardConfig (portForwAppTbl, "-1", "add")
        if (not valid) then
            return "ERROR", "SSLVPN_PORTFORWARDING_CONFIG_FAILED"
        end
    end
    local portForwHostTbl = {}
    portForwHostTbl["SSLVPNPortForwardingFQDNMap.ServerIP"] = wizardTbl["SSLVPNPortForwardingFQDNMap.ServerIP"]
    portForwHostTbl["SSLVPNPortForwardingFQDNMap.FQDNName"] = wizardTbl["SSLVPNPortForwardingFQDNMap.FQDNName"]
    DBTable = "SSLVPNPortForwardingFQDNMap"
    if (valid) then
        if (portForwHostTbl["SSLVPNPortForwardingFQDNMap.ServerIP"] ~= nil and portForwHostTbl["SSLVPNPortForwardingFQDNMap.ServerIP"] ~= "" and portForwHostTbl["SSLVPNPortForwardingFQDNMap.FQDNName"] ~= nil and portForwHostTbl["SSLVPNPortForwardingFQDNMap.FQDNName"] ~= "") then
            valid = db.insert(DBTable, portForwHostTbl)
        end
        --errorFlag, statusCode = sslvpn.portForwardConfig (portForwHostTbl, "-1", "add")
        if (not valid) then
            return "ERROR", "SSLVPN_PORTFORWARDING_CONFIG_FAILED"
        end
    end
    if(valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_WIZARD_CONFIG_FAILED"
    end    
end
