-- systemCLI.lua 
-- Gaurav Mathur
-- TeamF1
-- 10/23/07
--
-- Modification History
-- 12feb14,nik Added CLI support to show ntp logs.
-- 12sep13,sig cli support for schedule reboot
-- 03Apr13,akr fixed 36711
-- 15Mar13.akr fixed spr 36714
-- 13Mar13,akr commented out secondary image information as this is not applicable for fvs336gv2.
-- 28nov12,ksr fixed SPR 35247
-- 11Oct12,ksr fixed SPR 33250
-- 24May12,sig fixed spr 31975
-- 10may12,nik fixed SPR 31514
-- 06jan12,akr fixed 28995
-- 02mar10,db  Added status related routines
-- 05jan10,db  Added SNMP Access control specific routines and modified the time
--         zoneTab by adding a zone for GMT-04:30
-- 25dec07,gnm Added check for nil firmwareVer and system name fields
--	       in sysInfoGet; Added alias interface name argument
--	       value to aliasipGet() call in sysStatusGet; Added
--	       ethCLISingleGet() calls to display eth1 and eth0
--	       information for show system status command
-- 24dec07,gnm modified sysLogFacilityInfoGet() to add check for
--	       invalid facility values
-- 28nov07 gnm written
--
-- Description
-- CLISH system LUA routines
--
--
require "teamf1lualib/firmware"

local dayTab = {
"Sunday", 
"Monday",
"Tuesday",
"Wednesday",
"Thursday", 
"Friday", 
"Saturday",
}

local timezoneTab= {
    "(GMT) Greenwich Mean Time : Edinburgh, London",
    "(GMT-12:00) Eniwetok",
    "(GMT-12:00) Kwajalein",
    "(GMT-11:00) Midway Island",
    "(GMT-11:00) Samoa",
    "(GMT-10:00) Hawaii",
    "(GMT-09:30) Marquesas Is.",
    "(GMT-09:00) Alaska",
    "(GMT-08:30) Pitcairn Is.",
    "(GMT-08:00) Pacific Time(Canada)",
    "(GMT-08:00) Pacific Time(US)",
    "(GMT-08:00) Tijuana",
    "(GMT-07:00) Mountain Time(Canada)",
    "(GMT-07:00) Mountain Time(US)",
    "(GMT-06:00) Central Time(Canada)",
    "(GMT-06:00) Central Time(US)",
    "(GMT-05:00) Eastern Time(Canada)",
    "(GMT-05:00) Eastern Time(Lima)",
    "(GMT-05:00) Eastern Time(US)",
    "(GMT-04:30) Caracas",
    "(GMT-04:00) Atlantic Time(Canada)",
    "(GMT-03:30) Newfoundland",
    "(GMT-03:00) Brasilia, Buenos Aires",
    "(GMT-02:00) Mid-Atlantic",
    "(GMT-01:00) Azores",
    "(GMT-01:00) Cape Verde Is.",
    "(GMT+01:00) Europe",
    "(GMT+02:00) Athens",
    "(GMT+02:00) Istanbul",
    "(GMT+02:00) Minsk",
    "(GMT+02:00) Cairo",
    "(GMT+03:00) Baghdad",
    "(GMT+03:00) Kuwait",
    "(GMT+03:00) Moscow",
    "(GMT+03:30) Tehran",
    "(GMT+04:00) Abu Dhabi",
    "(GMT+04:00) Muscat",
    "(GMT+04:00) Baku",
    "(GMT+04:30) Kabul",
    "(GMT+05:00) Ekaterinburg",
    "(GMT+05:00) Islamabad",
    "(GMT+05:00) Karachi",
    "(GMT+05:30) Bombay, Calcutta, Madras, Delhi",
    "(GMT+05:30) Colombo",
    "(GMT+06:00) Almaty",
    "(GMT+06:00) Dhaka",
    "(GMT+06:30) Burma",
    "(GMT+07:00) Bangkok",
    "(GMT+07:00) Hanoi",
    "(GMT+07:00) Jakarta",
    "(GMT+08:00) Beijing, Chongqing, Hong Kong",
    "(GMT+08:00) (AWST)Perth",
    "(GMT+09:00) Osaka, Sapporo, Tokyo",
    "(GMT+09:00) Seoul",
    "(GMT+09:30) (ACST)Adelaide",
    "(GMT+09:30) (ACST)Darwin",
    "(GMT+09:30) (ACST)Broken Hill, NSW",
    "(GMT+10:00) (AEST)Brisbane",
    "(GMT+10:00) Guam",
    "(GMT+10:00) Port Moresby",
    "(GMT+10:00) (AEST)Canberra",
    "(GMT+10:00) (AEST)Melbourne",
    "(GMT+10:00) (AEST)Sydney",
    "(GMT+10:00) (AEST)Hobart",
    "(GMT+10:30) Lord Howe Is.",
    "(GMT+11:00) Magadan",
    "(GMT+11:00) Solomon Is.",
    "(GMT+11:00) New Caledonia",
    "(GMT+11:30) Norfolk I.",
    "(GMT+12:00) Auckland",
    "(GMT+12:00) Wellington",
    "(GMT+12:00) New Zealand",
    "(GMT+12:00) Fiji",
    "(GMT+13:00) Tonga",
    "(GMT+14:00) Kiribati",
}

function tzInfoGet ()
    local resultTab = {}
    local idx = "1"
    row = db.getRow("ntp", "ntp._ROWID_", 1) -- the ntp table should
					--only have one entry
    resTab.insertField (resultTab, "Current Time", util.time())
    idx = tonumber(row["ntp.timezone"])
    idx = idx + 1
    resTab.insertField (resultTab, "Timezone", timezoneTab[idx])
    if (row["ntp.autoDaylight"] == "1") then 
       resTab.insertField (resultTab, "Automatically Adjust for Daylight " ..
       				      "Savings Time", "Yes")
    else
       resTab.insertField (resultTab, "Automatically Adjust for Daylight " ..
				      "Savings Time", "No")
    end				     
    if (row["ntp.useDefServers"] == "1") then
       resTab.insertField (resultTab, "Default NTP servers used ", "Yes")
    else
       resTab.insertField (resultTab, "Default NTP servers used ", "No")
       resTab.insertField (resultTab, "Server 1 Name / IP Address",
			  row["ntp.server1"])
       resTab.insertField (resultTab, "Server 2 Name / IP Address",
			  row["ntp.server2"])
     end
--resTab.insertField (resultTab, "ReSynch NTP value",row["ntp.reSyncNtpVal"])
if(tonumber(row["ntp.networkMode"]) == 4) then
resTab.insertField (resultTab, "Resolving ipV6 address","Disabled")
elseif (tonumber(row["ntp.networkMode"]) == 6) then
resTab.insertField (resultTab, "Resolving ipV6 address","Enabled") 
end
if(tonumber(row["ntp.ntpMode"]) == 0) then
resTab.insertField (resultTab, "NTP Mode","Authoritative Mode")
resTab.insertField (resultTab, "Stratum",row["ntp.Stratum"])
elseif(tonumber(row["ntp.ntpMode"]) == 1) then
resTab.insertField (resultTab, "NTP Mode", "Internet Mode")
elseif(tonumber(row["ntp.ntpMode"]) == 2) then
resTab.insertField (resultTab, "NTP Mode", "VPN NTP")
resTab.insertField (resultTab, "VPN Policy",row["ntp.VPNPolicy"])
end
resTab.insertField (resultTab, "ReSynch NTP value",row["ntp.reSyncNtpVal"])


    return resultTab 	    
end

--
-- This routine returns the Identifier that's prepended to all log messages
-- in the system
--
function logIdentifierGet ()
    local ident = db.getAttribute("emailLogs","_ROWID_",1,"logId") or "EAP3000"
    return ident
end

--
-- This routine returns the Syslog server set on the system
--
function syslogServerGet ()
    return db.getAttribute("sysLogInfo","_ROWID_", 1, "serverName")
end

function sysLogFacilityInfoGet (facility)

    if (facility == nil) then --sanity check
       local resultTab = {}
       resTab.insertField (resultTab, "", "No such facility")
       return resultTab
    end

    local logTab = db.getRow ("logConfig", "logConfig.facility", facility)
    if (logTab == nil) then --sanity check
       local resultTab = {}
       resTab.insertField (resultTab, "", "No such facility")
       return resultTab
    end

    local resultTab = resTab.createRowLbl ("Emergency", "Alert", "Critical", 
	    "Error", "Warning",	"Notification", "Information", "Debugging")
    
    local emergencyChoices = util.integerToBits(logTab["logConfig.emergency"])
    local alertChoices = util.integerToBits(logTab["logConfig.alert"])
    local criticalChoices = util.integerToBits(logTab["logConfig.critical"])
    local errorChoices = util.integerToBits(logTab["logConfig.error"])
    local warningChoices = util.integerToBits(logTab["logConfig.warning"])
    local noticeChoices = util.integerToBits(logTab["logConfig.notice"])
    local informationChoices = util.integerToBits(logTab["logConfig.information"])
    local dbgChoices = util.integerToBits(logTab["logConfig.debug"])
    
    local idxTab = {1,3,4}
    for i, idx in ipairs (idxTab) do
	if (idx == 1) then colLbl = "Display in event log"
	elseif (idx == 3) then colLbl = "Send to syslog"
	elseif (idx == 4) then colLbl = "Display on telnet/SSH monitor"
	end
	
	resTab.insertYNField (resultTab, emergencyChoices[idx] == 1, colLbl)
        resTab.insertYNField (resultTab, alertChoices[idx] == 1, colLbl)
	resTab.insertYNField (resultTab, criticalChoices[idx] == 1, colLbl)
        resTab.insertYNField (resultTab, errorChoices[idx] == 1, colLbl)
	resTab.insertYNField (resultTab, warningChoices[idx] == 1, colLbl)
	resTab.insertYNField (resultTab, noticeChoices[idx] == 1, colLbl)
	resTab.insertYNField (resultTab, informationChoices[idx] == 1,colLbl)
	resTab.insertYNField (resultTab, dbgChoices[idx] == 1, colLbl)
    end

    return resultTab 	    
end

--
--
--
local function eth2Get ()
    local resultTab = {}
    local row = db.getRowWithJoin(
	  	  {"ethernet:networkInterface:interfaceName"}, 
		  "ethernet.interfaceName", "eth1")
    if (row["ethernet.vlanEnabled"] == "0") then
        row["ethernet.vlanId"]= "0";
    end
    resTab.insertField (resultTab, "MAC Address",
		       row["ethernet.macAddress"] or "")
    resTab.insertField (resultTab, "IP Address", 
		       row["networkInterface.ipaddr"])
    resTab.insertField (resultTab, "IP Subnet Mask", 
		       row["networkInterface.subnetmask"])
    resTab.insertField (resultTab, "Gateway IP Address", 
		       row["networkInterface.gateway"] or "")
    resTab.insertField (resultTab, "DNS Server", 
		       row["networkInterface.dns1"] or "")
    resTab.insertField (resultTab, "VLAN", vlanId or "")
    return resultTab    
end


--
--
--
local function eth1Get ()
    local resultTab = {}

    local rowBdg = db.getRowWithJoin(
		 {"bridgeTable:networkInterface:interfaceName"},
		 "bridgeTable.interfaceName", "bdg")
    local rowEth = db.getRowWithJoin({"ethernet:networkInterface:interfaceName"},
		 "ethernet.interfaceName", "eth0")
    local vlanId = "0";

    if (rowEth["ethernet.vlanEnabled"] == "1") then 
       vlanId = db.getAttribute("ethernet", "interfaceName", "eth0", "vlanId")
    end

    resTab.insertField (resultTab, "MAC Address",
		       rowBdg["bridgeTable.macAddress"] or "")
    resTab.insertField (resultTab, "IP Address", 
		       rowBdg["networkInterface.ipaddr"])
    resTab.insertField (resultTab, "IP Subnet Mask", 
		       rowBdg["networkInterface.subnetmask"])
    resTab.insertField (resultTab, "Gateway IP Address", 
		       rowBdg["networkInterface.gateway"] or "")
    resTab.insertField (resultTab, "DNS Server", 
		       rowBdg["networkInterface.dns1"] or "")
    resTab.insertField (resultTab, "VLAN", vlanId or "")
    return resultTab
end

--
-- build SNMP result table
-- 
local function buildSNMP (resultTab, row)
    if (row == nil) then return end --sanity check

    resTab.insertField (resultTab, "IP Address", row["snmpTrap.ipAddr"])
    resTab.insertField (resultTab, "Snmp Version", row["snmpTrap.snmpVersion"])
    resTab.insertField (resultTab, "Port", row["snmpTrap.port"])
    resTab.insertField (resultTab, "Community", row["snmpTrap.commName"])
end

--
-- build SNMP Access Control table
-- 
local function buildSNMPACL (resultTabACL, row)
    if (row == nil) then return end --sanity check

    resTab.insertField (resultTabACL, "IP Address", row["snmpAccessControl.ipAddr"])
    resTab.insertField (resultTabACL, "Subnet Mask", row["snmpAccessControl.subnetMask"])
    resTab.insertField (resultTabACL, "Access Type", row["snmpAccessControl.accessType"])
    resTab.insertField (resultTabACL, "Community", row["snmpAccessControl.commName"])
end

--
-- This routine gets the system SNMP configuration
-- 
function sysSNMPConfigGet ()
    local resultTab = {}

    row = db.getRow ("system","system._ROWID_", 1)
    if (row == nil) then
       return nil
    end
    resTab.insertField (resultTab, "SysContact", row["system.sysContact"] or "")
    resTab.insertField (resultTab, "SysLocation", row["system.sysLoc"] or "")
    resTab.insertField (resultTab, "SysName", row["system.name"])
    return resultTab
end

-- This routine returns firmware version
function _firmVer()
    local firmVersion = io.open("/var/firmVersion", "r")
    if(firmVersion) then
    	firmwareVer = firmVersion:read("*all") or ''
        firmVersion:close()
    end
    return firmwareVer or ''
end
--
-- get general system information
-- 
local function sysInfoGet (args)
    local resultTab = {}
    local system = db.getRow("system", "_ROWID_", "1")

--  resTab.insertField (resultTab, "System Up Time", util.uptime())
    resTab.insertField (resultTab, "System Name", 
		       system["system.name"] or "")
    status, firmVer1 = firmware.getVersion ("image1")
    status, firmVer2 = firmware.getVersion ("image2")
    resTab.insertField (resultTab, "Firmware Version", firmVer1 or "")
--    resTab.insertField (resultTab, "Secondary Firmware Version", firmVer2 or "")
--    local serNum = db.getAttribute("unitInfo", "_ROWID_", "1", "serialNumber")
--    resTab.insertField (resultTab, "Serial Number", 
--		       serNum or "")
    return resultTab
end

--
--
--
local function hourStringMake (field)
    local hour = "-1"

    if (field == nil) then return hour end --sanity check
    hour = tonumber (field)
    if (hour >= 0 and hour < 12) then         
	hour = hour .. ":00" .. " A.M."
    elseif (hour >= 12 and hour <= 23) then
	hour = hour - 12
	hour = hour .. ":00" .. " P.M."
    end
    return hour     
end


--
--
--
local function schedGet (resultTab, cmd)
    row = db.getRow ("cronTab", "command", cmd)
    if (row == nil) then
        resTab.insertField (resultTab, "Unit", "Never")
	resTab.insertField (resultTab, "Day", "-")
	resTab.insertField (resultTab, "Time", "-")
	return resultTab
    end
    if (row["cronTab.unit"] == "1") then
        resTab.insertField (resultTab, "Unit", "Hourly")
        resTab.insertField (resultTab, "Day", "-")
        resTab.insertField (resultTab, "Time", "-")
    elseif (row["cronTab.unit"] == "2") then                    
        resTab.insertField (resultTab, "Unit", "Daily")
        resTab.insertField (resultTab, "Day", "-")
        resTab.insertField (resultTab, "Time", hourStringMake(row["cronTab.hour"]))
    elseif (row["cronTab.unit"] == "3") then
	resTab.insertField (resultTab, "Unit", "Weekly")
	local day = row["cronTab.dayOfWeek"]
	day = tonumber(day) + 1
	resTab.insertField (resultTab, "Day", dayTab[day])
	resTab.insertField (resultTab, "Time", hourStringMake(row["cronTab.hour"]))
    end
end

--
--
--
function sysSchedGet ()
    local resultTab = resTab.create ()
        
    emailLogs = db.getAttribute ("emailLogs", "_ROWID_", 1, "mailLogs")
    if (emailLogs ~= "1") then
       return nil
    end

    schdCmd = db.getAttribute("environment","name","EMAILLOGS_PROGRAM","value") .. 
	    " " .. DB_FILE_NAME
    schedGet (resultTab, schdCmd)	
    return resultTab
end

--
--
--
function sysEmailLogGet ()
    local resultTab = resTab.create ()

    emailLogs = db.getAttribute ("emailLogs", "_ROWID_", 1, "mailLogs")
    if (emailLogs ~= "1") then
       return nil
    end

    local entryName = db.getAttribute("emailLogs","_ROWID_",1,"entryName")
    row = db.getRow ("smtpServer", "entryName", entryName)
    if (row == nil ) then
    else
        resTab.insertField (resultTab, "E-Mail Server Address", 
			   row['smtpServer.emailServer'])
        resTab.insertField (resultTab, "Return E-Mail Address", 
			   row['smtpServer.fromAddr'])
        resTab.insertField (resultTab, "Send to E-Mail Address", 
			   row['smtpServer.toAddr'])
        resTab.insertField (resultTab, "Send to E-Mail Address", 
			   row['smtpServer.toAddr'])

        local auth = row['smtpServer.auth']
	auth = tonumber (auth)
	local tmpStr = ""
	if (auth == -1) then tmpStr = "No Authentication" 
	elseif (auth == 0) then tmpStr = "Login Plain"
	elseif (auth == 1) then tmpStr = "CRAM-MD5"
	end
        resTab.insertField (resultTab, "Authentication", tmpStr)
	
	if (auth == 0 or auth == 1) then 
	   resTab.insertField (resultTab, "User Name", row['smtpServer.userName'])
	   resTab.insertField (resultTab, "Password", row['smtpServer.passWord'])
        end
	resTab.insertYNField (resultTab, row['smtpServer.respondIdentd'] == "1", 
			     "Respond to Identd from SMTP Server")
    end
    return resultTab
end

--
--
--
function sysEmailSchedGet (args)

    print ("\n" .. "Log Identifier: " .. logIdentifierGet())	 	 
    resultTab = sysEmailLogGet ()
    if (resultTab == nil) then
        print ("\n" .. "E-Mail logs are disabled" .. "\n")
    else	
        printLabel ("Enable E-Mail Logs")
        resTab.print (resultTab, 0)
    end
    resultTab = sysSchedGet()
    if (resultTab ~= nil) then
       printLabel ("Send E-mail logs by Schedule")
       resTab.print (resultTab, 0)
    end                      
    local sysLogInfoRow = db.getRow("sysLogInfo","rowid","1")
    if(sysLogInfo ~= nil) then
	    printLabel("Syslog Configuration")
    	    local sysLogTab = resTab.create ()
	    if(sysLogInfoRow["sysLogInfo.Enable"] == "0") then
		    resTab.insertField(sysLogTab,"Syslog Server", "Disabled" )
	    elseif(sysLogInfoRow["sysLogInfo.Enable"] == "1") then
		    resTab.insertField(sysLogTab,"Server", "Enabled" )
		    resTab.insertField(sysLogTab,"Server Name", sysLogInfoRow["sysLogInfo.serverName"] or '')
		    if (sysLogInfoRow["sysLogInfo.severity"] == "128") then
			    severity = "LOG DEBUG"
		    elseif (sysLogInfoRow["sysLogInfo.severity"] == "64") then
			    severity = "LOG INFO"
		    elseif (sysLogInfoRow["sysLogInfo.severity"] == "32") then
			    severity = "LOG NOTICE"
		    elseif (sysLogInfoRow["sysLogInfo.severity"] == "16") then
			    severity = "LOG WARNING"
		    elseif (sysLogInfoRow["sysLogInfo.severity"] == "8") then
			    severity = "LOG ERROR"
		    elseif (sysLogInfoRow["sysLogInfo.severity"] == "4") then
			    severity = "LOG CRITICAL"
		    elseif (sysLogInfoRow["sysLogInfo.severity"] == "2") then
			    severity = "LOG ALERT"
		    elseif (sysLogInfoRow["sysLogInfo.severity"] == "1") then
			    severity = "LOG EMERG"
		    end
		    resTab.insertField(sysLogTab,"Severity", severity or '')
	    end
	    resTab.print(sysLogTab,0)
    end
end

--
-- main function for system get
-- 
function sysLoggingGet (args)
    local facility = args[1]
    local ident = logIdentifierGet() or "" 	  
    local server = syslogServerGet() or ""

    print ("\n" .. "Log Identifier: " .. ident)
    print ("Syslog Server: " .. server)

    printLabel ("Logs Facility and Level Configuration")
    if (facility == "all") then
       local logTab = db.getTable ("logConfig")
       for k, v in pairs (logTab) do
	   printLabel (v["logConfig.facility"])
	   resultTab = sysLogFacilityInfoGet(v["logConfig.facility"])
           resTab.print (resultTab)
       end
    else
       resultTab = sysLogFacilityInfoGet (facility)
       printLabel (facility)
       resTab.print (resultTab)
    end
end
--
-- main function for system get
-- 
function sysTimezoneGet (args)
    printLabel ("Time Zone & NTP Servers Configuration")    
    resultTab = tzInfoGet()
    resTab.print (resultTab, 0)    
end   

--
-- main function for system get
-- 
function sysUsersGet (args)
    printLabel ("User Selection")    
    resultTab = sysUserGet ()
    resTab.print (resultTab, 0)
    printLabel ("Available Access Points")
    resultTab = dot11APAllGet ()
    resTab.print (resultTab, 0)
    printLabel ("Ethernet Port 1")    
    resultTab = eth1Get()
    resTab.print (resultTab, 0)
    printLabel ("Ethernet Port 2")    
    resultTab = eth2Get()
    resTab.print (resultTab, 0)
end   



function statusGet (wanType)
    local resultTab = {}
    if (wanType == "WAN1" or wanType == "WAN2" or wanType == "WAN3" or wanType == "WAN4") then
    
    local where = "LogicalIfName = '"..wanType.."' and addressFamily = 2"
						local where6 = "LogicalIfName = '"..wanType.."' and addressFamily = 10"
					    local wanv4Config = db.getRowWhere ("NimfConf", where)
					    local wanv6Config = db.getRowWhere ("NimfConf", where6)
						local wanRows = db.getRowsWhere ("ipAddressTable", where)
						local wan6Rows = db.getRowsWhere ("ipAddressTable", where6)	
					    local nsRows = db.getRows("resolverConfig", "LogicalIfName", wanType)
					    local gwRows = db.getRows("defaultRouters", "LogicalIfName", wanType)
					    local wanIntName = db.getAttribute ("networkInterface", "LogicalIfName", wanType, "interfaceName")
					    local wanState = db.getAttribute ("networkInterface", "LogicalIfName", wanType, "IfStatus") or ''
					    local wanMacAddr = ""
					    local natEnabled = ""    
					    local connetionType4 = ""
					    local connectionType6 =""
					    local connetionStatus4 = ""	
			            local connetionStatus6 = ""
			            local linkStatus4 = ""
					
						-- Getting MAC Address of the WAN Interface
						if (wanIntName ~= nil) then	
							wanMacAddr = db.getAttribute ("ethernet", "interfaceName", wanIntName, "macAddress") or ''
						end
					
						-- Getting Connection Type of the WAN(ipv4) Interface
					    if (wanv4Config["NimfConf.ConnectionType"] ~= nil) then
					        if (wanv4Config["NimfConf.ConnectionType"] == "dhcpc") then
					            connetionType4 = "Dynamic IP (DHCP)"
					        elseif (wanv4Config["NimfConf.ConnectionType"] == "ifStatic") then
					            connetionType4 = "STATIC"
					        elseif (wanv4Config["NimfConf.ConnectionType"] == "pppoe") then
					            connetionType4 = "PPPOE"
					        elseif (wanv4Config["NimfConf.ConnectionType"] == "pptp") then
					            connetionType4 = "PPTP"
					        elseif (wanv4Config["NimfConf.ConnectionType"] == "l2tp") then
					            connetionType4 = "L2TP"
                            elseif (wanv4Config["NimfConf.ConnectionType"] == "threeg") then
					            connetionType4 = "ThreeG"
					        end
					    end
					    
						-- Getting Connection Type of the WAN(ipv6) Interface
					    if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then
							if (wanv6Config["NimfConf.ConnectionType6"] ~= nil) then
								if (wanv6Config["NimfConf.ConnectionType6"] == "dhcp6c") then
					                connectionType6 = "Dynamic IP (DHCPv6)"
					            elseif (wanv6Config["NimfConf.ConnectionType6"] == "ifStatic6") then
						            connectionType6 = "Static IPv6"
								elseif (wanv6Config["NimfConf.ConnectionType6"] == "") then
						            connectionType6 = "Disabled"
								end
						    end
						else
							connectionType6 = "IPv6 is disabled" 
						end                                            
					
						-- Getting Connection State of the WAN(ipv4) Interface
					    local nimfStatusConfig = db.getRowWhere("NimfStatus", where)
					    if (nimfStatusConfig ~= nil) then
					    	connetionStatus4 = nimfStatusConfig["NimfStatus.Nimfstatus"] or ""
					    end
					
						-- Getting Connection State of the WAN(ipv6) Interface
						if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then
							nimfStatusConfig = db.getRowWhere("NimfStatus", where6)
							if (nimfStatusConfig ~= nil) then
								connetionStatus6 = nimfStatusConfig["NimfStatus.Nimfstatus"] or ""
							end
						else
							connetionStatus6 = "IPv6 is disabled" 
			            end  
			
			            local nimfStatusConfig = db.getRowWhere("NimfStatus", where)
			            if (nimfStatusConfig ~= nil) then
					    	linkStatus4 = nimfStatusConfig["NimfStatus.Linkstatus"] or ""
					    end    
                        
                        if (wanv4Config["NimfConf.ConnectionType"] == "threeg") then
					        linkStatus4 = ""
					    end

                        -- Getting NAT Mode Status.
					    if (db.getAttribute("NatTable", "_ROWID_", "1", "Enable") == "1") then
					    	natEnabled = "Enabled"
					    else
					    	natEnabled = "Disabled"
					    end
     resTab.insertField (resultTab, "MAC Address", wanMacAddr or '')
     local ipAddrStr = ""
	 local i = 0
     if(wanRows) then
	 for k,v in pairs(wanRows) do
	    i = i + 1
	    local row = wanRows[i]
	    ipAddrStr = ipAddrStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.subnetMask"]
	    if(i ~= #wanRows) then
	      	ipAddrStr = ipAddrStr .. ", "
	    end
	end
	end
    resTab.insertField (resultTab, "IPv4 Address", ipAddrStr or '')
    ipAddrStr = ""
	if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then							
		i = 0
    	if(wan6Rows) then
	    	for k,v in pairs(wan6Rows) do
		    	i = i + 1
			    local row = wan6Rows[i]
				ipAddrStr = ipAddrStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.ipv6PrefixLen"]
				if(i ~= #wan6Rows) then
				    ipAddrStr = ipAddrStr .. ", "
				end
			end
		end
	end
    resTab.insertField (resultTab, "IPv6 Address", ipAddrStr or '')
    if (wanState == "1") then
		wanState = "UP"
    elseif (wanState == "2") then
		wanState = "UP(IPv6)"
	elseif (wanState == "3") then
		wanState = "UP(IPv4 and IPv6)"
	elseif (wanState == "0") then
		wanState = "DOWN"
	end
    resTab.insertField (resultTab, "Wan State", wanState or '')
    resTab.insertField (resultTab, "NAT (IPv4 only)", natEnabled or '')
    resTab.insertField (resultTab, "IPv4 Connection Type", connetionType4 or '')
    resTab.insertField (resultTab, "IPv6 Connection Type", connectionType6 or '')
    resTab.insertField (resultTab, "IPv4 Connection State", connetionStatus4 or '')
    resTab.insertField (resultTab, "IPv6 Connection State", connetionStatus6 or '')
    resTab.insertField (resultTab, "Link State", linkStatus4 or '')

    	local wanMode = ""
					local wmodeConfig = db.getRow("WanMode", "_ROWID_", "1")
					if (wmodeConfig ~= nil) then
						local wan = ""
                        wmodeConfig["WanMode.FailoverKey"] = "0"
                        if(wmodeConfig["WanMode.Wanmode"] == "2") then
                            wmodeConfig["WanMode.FailoverKey"] = "1"       
                            wmodeConfig["WanMode.Wanmode"] = "0"
                        end
						if (wmodeConfig["WanMode.Wanmode"] == "0") then
					    	wan = "Primary Wan Mode Using "..wmodeConfig["WanMode.FailoverPriLogicalIfName"]
                            if (wmodeConfig["WanMode.FailoverKey"] == "1") then
                                wan = "Auto Rollover on Wan Port Using " .. wmodeConfig["WanMode.FailoverSecLogicalIfName"]
                            else
                                wan = "Auto Rollover is Disabled"
                            end
                        elseif (wmodeConfig["WanMode.Wanmode"] == "1") then
                            if (wmodeConfig["WanMode.LoadBalanceAlgo"] == "1") then
                                wan = "LOAD Balancing Using Weighted LB"
                            elseif (wmodeConfig["WanMode.LoadBalanceAlgo"] == "0") then
                                wan = "LOAD Balancing Using RoundRobin"  
                            end
                        end
                    end
    local wanSpeed = db.getRow ("WanSpeed","LogicalIfName",wanType)
    if (wanSpeed ~= nil) then
        resTab.insertField (resultTab, "Upload Connection Speed", wanSpeed["WanSpeed.UploadSpeed"] or '')
        resTab.insertField (resultTab, "Download Connection Speed", wanSpeed["WanSpeed.DownloadSpeed"] or '')
    end
					                                	
--    resTab.insertField (resultTab, "WAN Mode:", wanMode or '')
    local gwIps = ""
	    if(gwRows) then
		    local i = 0
		    for k,v in pairs(gwRows) do
			    i = i + 1        
			    local row = gwRows[i]
				if (row["defaultRouters.nextHopGateway"] ~= nil and row["defaultRouters.nextHopGateway"] ~= "") then
				   	if (gwIps == "") then
				   		gwIps = gwIps .. row["defaultRouters.nextHopGateway"]
				   	else
				   		gwIps = gwIps .. ", " .. row["defaultRouters.nextHopGateway"]
				   	end
				end               
			end
		end
    resTab.insertField (resultTab, "Gateway", gwIps or '')

    local priDNSIps = ""
    if(nsRows) then
        local i = 0
		for k,v in pairs(nsRows) do
		  	i = i + 1        
		    local row = nsRows[i]
		    if (row["resolverConfig.nameserver1"] ~= nil and row["resolverConfig.nameserver1"] ~= "") then
		       	if (priDNSIps == "") then
		       		priDNSIps = priDNSIps .. row["resolverConfig.nameserver1"]
		       	else
		       		priDNSIps = priDNSIps .. ", " .. row["resolverConfig.nameserver1"]
		       	end
		    end              
		end
	end
    resTab.insertField (resultTab, "Primary DNS", priDNSIps or '')

    local secDNSIps = ""
		if(nsRows) then
	    	local i = 0
		    for k,v in pairs(nsRows) do
		    	i = i + 1        
			    local row = nsRows[i]
			    if (row["resolverConfig.nameserver2"] ~= nil and row["resolverConfig.nameserver2"] ~= "") then
			       	if (secDNSIps == "") then
			       		secDNSIps = secDNSIps .. row["resolverConfig.nameserver2"]
			      	else
			       		secDNSIps = secDNSIps .. ", " .. row["resolverConfig.nameserver2"]
			     	end
			    end              
			end
		end
    resTab.insertField (resultTab, "Secondary DNS", secDNSIps or '')
    local wanIpv6Info = nimfView.wanStatusGetWrap (wanType,"10")
    if ( wanIpv6Info ~= nil ) then
	    resTab.insertField ( resultTab,"Gateway (IPv6)",wanIpv6Info["Wan.Gateway"] or '')
    end
    local resolverConfigRow = db.getRow("resolverConfig","addressFamily","10")
    if ( resolverConfigRow ~= nil ) then
	    resTab.insertField ( resultTab,"Primary DNS(IPv6)",resolverConfigRow["resolverConfig.nameserver1"] or '')
	    resTab.insertField ( resultTab,"Secondary DNS(IPv6)",resolverConfigRow["resolverConfig.nameserver2"] or '')
    end

    elseif (wanType == "DMZ") then

					--[[
                    local lanRows = db.getRowsWhere ("ipAddressTable", where)
    			    local dhcpConf = db.getRow("dhcp", "dhcp._ROWID_", "2")    
					local dhcpStatus = ""
					if (dhcpConf ~= nil) then
						if (dhcpConf["dhcp.dhcpEnabled"] == "1") then
							dhcpStatus = "Enabled"
						elseif (dhcpConf["dhcp.dhcpEnabled"] == "0") then
							dhcpStatus = "Disabled"
						end
					end					
					local dhcpRelayConf = db.getRow("dhcpRelay", "dhcpRelay.LogicalIfName", "DMZ")    
					local dhcpRelayStatus = ""
					if (dhcpRelayConf ~= nil) then
						if (dhcpRelayConf["dhcpRelay.DhcpRelayStatus"] == "1") then
							dhcpRelayStatus = "Enabled"
						elseif (dhcpRelayConf["dhcpRelay.DhcpRelayStatus"] == "0") then
							dhcpRelayStatus = "Disabled"
						end
					end					
					local row = db.getRow("networkInterface", "LogicalIfName", "DMZ")
					local lanMacAddr = db.getAttribute("ethernet", "ethernet.interfaceName", row["networkInterface.interfaceName"], "macAddress")
              resTab.insertField (resultTab, "MAC Address", lanMacAddr or '')

          local lanIPStr = ""
					local i = 0
					if(lanRows) then
					    for k,v in pairs(lanRows) do
						    i = i + 1
						    local row = lanRows[i]
						    lanIPStr = lanIPStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.subnetMask"]
					        if(i ~= #lanRows) then
					        	lanIPStr = lanIPStr ..  ", "
					        end
					    end
					end
              resTab.insertField (resultTab, "IP Address", lanIPStr or '')
              resTab.insertField (resultTab, "DHCP Server", dhcpStatus or '')
              resTab.insertField (resultTab, "DHCP Relay", dhcpRelayStatus or '')
]]
    
    local where = "LogicalIfName = 'DMZ' AND addressFamily = '10'"
    local dmz6Rows = db.getRowsWhere ("ipAddressTable",where)
    local ipAddrStr = ""
	local i = 0
      if (dmz6Rows) then
         for k,v in pairs(dmz6Rows) do
             i = i + 1
         local row = dmz6Rows[i]
         if(row["ipAddressTable.ipAddress"] ~= nil) then
         ipAddrStr = ipAddrStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.ipv6PrefixLen"]
            if(i ~= #dmz6Rows) then
            ipAddrStr = ipAddrStr .. ", "
            end
         end
      end
    end
   resTab.insertField (resultTab, "IPv6 Address", ipAddrStr or '')

local dmz6dhcp6Conf = db.getRow("dhcpv6s", "dhcpv6s._ROWID_", "2")
           local dmz6dhcp6Status = ""
            local dmz6PrimaryDNS = ""
            local dmz6SecondaryDNS = ""
        if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then
               if (dmz6dhcp6Conf ~= nil) then
                          if (dmz6dhcp6Conf["dhcpv6s.isEnabled"] == "1") then
                          dmz6dhcp6Status = "Enabled"
                          dmz6PrimaryDNS  = dmz6dhcp6Conf["dhcpv6s.primaryDNSServer"] or ''
                          dmz6SecondaryDNS  = dmz6dhcp6Conf["dhcpv6s.secondaryDNSServer"] or ''
                              elseif (dmz6dhcp6Conf["dhcpv6s.isEnabled"] == "0") then
                                                dmz6dhcp6Status = "Disabled"
                              end
                           end
                else
                                        dmz6dhcp6Status = "IPv6 is disabled"
         end
    resTab.insertField (resultTab, "DHCPv6 Server:", dmz6dhcp6Status or '')

    end
   return resultTab 
end

function lanStatusGet ()
    local resultTab = {} 
    local where = "LogicalIfName = 'LAN' AND addressFamily = '2'"
					local lanRows = db.getRowsWhere ("ipAddressTable", where)
					local where6 = "LogicalIfName = 'LAN' AND addressFamily = '10'"
					local lan6Rows = db.getRowsWhere ("ipAddressTable", where6)
					local dhcpConf = db.getRow("dhcp", "dhcp._ROWID_", "1")    
					local dhcpStatus = ""
					if (dhcpConf ~= nil) then
						if (dhcpConf["dhcp.dhcpEnabled"] == "1") then
							dhcpStatus = "Enabled"
						elseif (dhcpConf["dhcp.dhcpEnabled"] == "0") then
							dhcpStatus = "Disabled"
						end
					end
					
					local dhcpRelayConf = db.getRow("dhcpRelay", "dhcpRelay.LogicalIfName", "LAN")    
					local dhcpRelayStatus = ""
					if (dhcpRelayConf ~= nil) then
						if (dhcpRelayConf["dhcpRelay.DhcpRelayStatus"] == "1") then
							dhcpRelayStatus = "Enabled"
						elseif (dhcpRelayConf["dhcpRelay.DhcpRelayStatus"] == "0") then
							dhcpRelayStatus = "Disabled"
						end
					end
			
					local dhcp6Conf = db.getRow("dhcpv6s", "dhcpv6s._ROWID_", "1")    
					local dhcp6Status = ""
					if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then
						if (dhcp6Conf ~= nil) then
							if (dhcp6Conf["dhcpv6s.isEnabled"] == "1") then
								dhcp6Status = "Enabled"
							elseif (dhcp6Conf["dhcpv6s.isEnabled"] == "0") then
								dhcp6Status = "Disabled"
							end
						end
					else
						dhcp6Status = "IPv6 is disabled"
					end
			
					local row = db.getRow("networkInterface", "LogicalIfName", "LAN")
					local lanMacAddr = db.getAttribute("ethernet", "interfaceName", "eth0.1", "macAddress")
    resTab.insertField (resultTab, "MAC Address", lanMacAddr or '')

    local lanIPStr = ""
					local i = 0
					if(lanRows) then
					    for k,v in pairs(lanRows) do
						    i = i + 1
						    local row = lanRows[i]
						    lanIPStr = lanIPStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.subnetMask"]
					        if(i ~= #lanRows) then
					        	lanIPStr = lanIPStr ..  ", "
					        end
					    end
					end
   -- resTab.insertField (resultTab, "IP Address:", lanIPStr or '')

    lanIPStr = ""
						if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then							
							i = 0
							if(lan6Rows) then
					    	    for k,v in pairs(lan6Rows) do
						    	    i = i + 1
							        local row6 = lan6Rows[i]
									lanIPStr = lanIPStr ..  row6["ipAddressTable.ipAddress"] .. " / " .. row6["ipAddressTable.ipv6PrefixLen"]
				   			        if(i ~= #lan6Rows) then
				    					lanIPStr = lanIPStr ..  ", "
					    	        end
								end
							end
						end
    resTab.insertField (resultTab, "IPv6 Address:", lanIPStr or '')

    resTab.insertField (resultTab, "DHCP Server:", dhcpStatus or '')
    resTab.insertField (resultTab, "DHCP Relay:", dhcpRelayStatus or '')
    resTab.insertField (resultTab, "DHCPv6 Server:", dhcp6Status or '')
    
    return resultTab
end

function wlanStatusGet ()
    local resultTab = {}

    local cardConfig = db.getRow ("dot11Card", "_ROWID_", "1")
			local wlInfo = {}
			if (cardConfig ~= nil) then
				wlInfo = db.getRow ("dot11Radio", "cardNo", cardConfig["dot11Card.cardNo"])
				wlInfo["dot11GlobalConfig.country"] = db.getAttribute ("dot11GlobalConfig", "_ROWID_", "1", "country") or ''
				wlInfo["dot11GlobalConfig.country"] = db.getAttribute ("dot11Countries", "countrycode", wlInfo["dot11GlobalConfig.country"], "country") or wlInfo["dot11GlobalConfig.country"]
				wlInfo["dot11Radio11n.puren"] = db.getAttribute ("dot11Radio11n", "radioNo", wlInfo["dot11Radio.radioNo"], "puren") or ''
		   	end
		   	if (cardConfig == nil) then
		   		cardConfig = {}
		   	end
		   	if (wlInfo == nil) then
		   		wlInfo = {}
		   	end
		   	local opFrq = ""
		   	if (wlInfo["dot11Radio.band"] == "b") then
		   		opFrq = "2.4GHz"
		   	elseif (wlInfo["dot11Radio.band"] == "a") then
		   		opFrq = "5GHz"
		   	end
		   		
      		if(wlInfo ["dot11Radio.opMode"] == "ng40-" or wlInfo ["dot11Radio.opMode"] == "ng") then
      			wlInfo ["dot11Radio.opMode"] = "NG";
      		elseif(wlInfo ["dot11Radio.opMode"] == "b and g") then	
      			wlInfo ["dot11Radio.opMode"] = "BG";
      		elseif(wlInfo ["dot11Radio.opMode"] == "g only") then	
      			wlInfo ["dot11Radio.opMode"] = "G Only";
      		end

      		if(wlInfo ["dot11Radio.opMode"] == "na40-" or wlInfo ["dot11Radio.opMode"] == "na") then
      			wlInfo ["dot11Radio.opMode"] = "NA";
      		elseif(wlInfo ["dot11Radio.opMode"] == "a only") then	
      			wlInfo ["dot11Radio.opMode"] = "A Only";
      		end

		if (wlInfo["dot11Radio11n.puren"] == "1") then
            		wlInfo ["dot11Radio.opMode"] = "N Only"; 
      		end
          local configuredRegion = db.getAttribute ("unitInfo", "rowid", 1, "sku_reginfo") or 0;
	  if ( configuredRegion ~= nil or configuredRegion ~= "")then
		  if(configuredRegion == "1") then region = "Africa"
		  elseif(configuredRegion == "2") then region = "Asia"
		  elseif(configuredRegion == "3") then region = "Europe"
		  elseif(configuredRegion == "4") then region = "Middle East"
		  elseif(configuredRegion == "5") then region = "North America"
		  elseif(configuredRegion == "6") then region = "Oceana"
		  end
	  end
	  local channelRow = db.getRow("dot11Radio","rowid","1")
	  if (channelRow ~= nil ) then channel = channelRow["dot11Radio.configuredChannel"] .. "-" .. tostring(tonumber(channelRow["dot11Radio.currentChannel"]/1000)).. " GHz" end
--        print ("Operating Frequency: "..opFrq)
--        print ("Mode: "..wlInfo["dot11Radio.opMode"])
--        print ("Channel: "..wlInfo["dot11Radio.configuredChannel"])

        local i = 0
						local table = db.getTable("dot11VAP")
						for k,v in pairs(table) do
							i = i + 1							
							local row = table[i]
							local profileConfig = db.getRow ("dot11Profile", "profileName", row["dot11VAP.profileName"])
							if (profileConfig == nil) then
								profileConfig = {}
							end
							local encryption = profileConfig["dot11Profile.pairwiseCiphers"] or ""
							local authentication = profileConfig["dot11Profile.authMethods"] or ""
							if (profileConfig["dot11Profile.security"] == "OPEN")  then
								encryption = "NONE"
								authentication = "NONE"
						    elseif (profileConfig["dot11Profile.security"] == "WEP") then
						        encryption = profileConfig["dot11Profile.groupCipher"] or ""
						        authentication = profileConfig["dot11Profile.wepAuth"] or ""
							end							
							if( row["dot11VAP.vapEnabled"] == "1") then
                                
                                resTab.insertField (resultTab, "Wireless Status",'Enable')

                                resTab.insertField (resultTab, "SSID", profileConfig["dot11Profile.ssid"] or '')
                                resTab.insertField (resultTab, "Mode", wlInfo ["dot11Radio.opMode"] or '')
                                resTab.insertField (resultTab, "Security Setting", profileConfig["dot11Profile.security"] or '')

                                resTab.insertField (resultTab, "Region", region or '')
                                resTab.insertField (resultTab, "Channel", channel or '')
--                                resTab.insertField (resultTab, "ENCRYPTION", encryption or '')
--                                resTab.insertField (resultTab, "AUTHENTICATION", authentication or '')
				local VapMac = db.getAttribute("dot11Interface","vapName", row["dot11VAP.vapName"],"macAddress")
				if(VapMac ~= nil or VapMac ~= "") then
	                                resTab.insertField (resultTab, "AP MAC Address", VapMac or '')
				end
                            end
			          end
    return resultTab
end

--
-- function to print system logs
--
function sysLogsGet()
	local i = 0
	local table = {}
	facilityID = "-1"
	local printCount = 0 
	LOGGING_DB_FILE_NAME="'/tmp/logging.db'"
	db.execute("ATTACH " .. LOGGING_DB_FILE_NAME .. " AS loggingDB")
	if (facilityID ~= "-1") then
		table = db.getRowsWhere("eventLog", "facilityId = " .. facilityID)
	else
		table = db.getRowsWhere("eventLog", "component <> 'UMI_COMP_SSLVPN'")
	end
	if(table ~= nil) then
		i = #table						
		for k,v in pairs(table) do						        
			local row = table[i]
			local s = row["eventLog.textMessage"]
			-- remove "\n" before displaying
			if (string.find (s,"\n", -2)) then
				s = string.sub (s, 1, -2)
			end
			printCount = printCount + 1 
			if (printCount <= 100) then
				print (s)
			end
			i = i - 1
		end
	end
	return true
end
--
--function to print dns logs 
--
function sysDnsLogsGet()
	logging.logsOperation("dnsmasq","refresh")
	local outputFile = db.getAttribute("environment", "name", "DNS_LOG_FILE_NAME", "value") or "/var/log/dnsmasq.log"
    outputText = util.fileToString(outputFile)
    realoutputText=""
    sep = "\n"
    fields = {}
    outputText:gsub("([^".. sep .."]*)"..sep, function(c) table.insert(fields, c) end)
    for i,k in ipairs(fields) do realoutputText ="".. k.. sep .. realoutputText end
    outputText ="" .. realoutputText .. ""
	print(outputText)	
	return true
end

--
--function to print ntp logs
--
function sysNtpLogsGet()
    local outputFile = db.getAttribute("environment", "name", "NTP_LOG_FILE_NAME", "value")
    local outputText = ""
    if ( db.getAttribute("OtherSystemLogs","rowid", "1", "ChangeOfTimeByNTPLogs") == "1" ) then
    	outputText = util.fileToString(outputFile)
	else
    	outputText = "" 
	end

    local realoutputText=""
    sep = "\n"
    fields = {}
    outputText:gsub("([^".. sep .."]*)"..sep, function(c) table.insert(fields, c) end)
    for i,k in ipairs(fields) do realoutputText ="".. k.. sep .. realoutputText end
    outputText ="" .. realoutputText .. ""
    print(outputText)
	return true
end

--
-- main function to get system status 
-- 
function sysStatusGet (args)
    printLabel ("System Info")    
    resultTab = sysInfoGet ()
    resTab.print (resultTab, 0)
    -- run program to NIMF Status and update DB
	local prog = db.getAttribute("environment", "name", "NIMFSTATUS_PROGRAM", "value")
	if (prog ~= nil) then
        util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " WAN1 2"))
        util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " WAN1 10"))
    end
--    prog = db.getAttribute("environment", "name", "LINKSTATUS_PROGRAM", "value")
--	if (prog ~= nil) then
--		util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " WAN1"))
--        util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " DMZ"))
--	end
	
	-- If IP Mode is IPv4/IPv6 then we need to call this binary.
	local ipMode = db.getAttribute ("networkInfo", "_ROWID_", "1", "netWorkMode") or '0'
	if (ipMode == "3") then
	    prog = db.getAttribute("environment", "name", "IPV6_GW_UPDATE", "value")
		if (prog ~= nil) then
			util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME))
		end
	end
--[[    printLabel ("WAN1 Information")
    resultTab = statusGet("WAN1")
    resTab.print (resultTab, 0)
--    printLabel ("WAN2 Information")
--    local configurablePort = db.getAttribute("ConfigPort","_ROWID_","1","LogicalIfName")
--	if(configurablePort == "WAN2") then
--        resultTab = statusGet ("WAN2")
--        resTab.print (resultTab, 0)
--    else
--    
--    ]]
    local vlanMacOffsetRow = {}
    local portmacAddr = {}
    local Port = {}
    local vlanMacoffSetType = db.getAttribute("vlanMac", "_ROWID_", "1", "vlanMacOffsetType") or '0'

        local file = io.open("/tmp/vlanmac.txt","r")
        if(file == nil) then
    	    util.appendDebugOut("Exec = " .. os.execute("sh /pfrm2.0/bin/getVlanMac.sh"))
	        local file = io.open("/tmp/vlanmac.txt","r")
	        local i = 0
    	    for line in io.lines("/tmp/vlanmac.txt") do 
		        i = i + 1
		        Port[i] = line
    	    end
            file:close()
        end
    
        if(file ~= nil) then
	        local i = 0
    	    for line in io.lines("/tmp/vlanmac.txt") do 
		        i = i + 1
		        Port[i] = line
    	    end
            file:close()
        end
    if(tonumber(vlanMacoffSetType) ~= 0) then        
        local m = 0
        for i=0,3 do
            j = i + 1
            local defaultPVID = db.getAttribute("vlanPortMgmt","PortNumber", i, "DefaultPVID")
            vlanMacOffsetRow = db.getRowWhere("vlan","vlanId=" .. defaultPVID .. " and portName='port10'")
            local vlanOffset = vlanMacOffsetRow["vlan.vlanMacOffset"] or '0'
            if (tonumber(vlanOffset) == 0) then
                m = 1
            else
                m = tonumber(vlanOffset) - 1
            end

            if (tonumber(defaultPVID) == 4094) then
                m = 17
                portmacAddr[i] = Port[m]
            else
                portmacAddr[i] = Port[m]
            end
        end
    else
        local lanMacAddr = db.getAttribute("ethernet", "interfaceName", "eth0", "macAddress")
        for i=0,3 do
            local defaultPVID = db.getAttribute("vlanPortMgmt","PortNumber", i, "DefaultPVID")
            if (tonumber(defaultPVID) == 4094) then
                m = 17
                portmacAddr[i] = Port[m]
            else
                portmacAddr[i] = Port[1]
            end
	 end
    end

    local vlanMac = {}
    local vlanBaseMac = db.getAttribute("defaultMac", "logicalName", "LAN", "macAddress") or '0'
    for n = 1,4 do
        local tmpDPVID = db.getAttribute("VlanPortMgmt","PortNumber",(n - 1),"DefaultPVID")
	    local wherePrt = "portName = 'port10' AND vlanId = '" .. tmpDPVID .. "'"
	    local vlanName = db.getRowWhere("vlan",wherePrt)
	    vlanMac[n] = ifDevLib.macIncrement(vlanBaseMac, vlanName["vlan.vlanMacOffset"])
    end

    for i =1,3 do 
	printLabel ("Lan Port " ..tostring(i).. " Information")
	local tmpPort1Addr = ''
        local tmpDPVID = db.getAttribute("VlanPortMgmt","PortNumber",tostring(tostring(i)-1),"DefaultPVID")
	local wherePrt1 = "portName = 'dummy' AND vlanId = '" .. tmpDPVID .. "'"
	vlanName = db.getRowWhere("vlan",wherePrt1)
						    
	where = "LogicalIfName='LAN' and addressFamily='2'"
	where2 = "LogicalIfName='LAN" .. tmpDPVID .. "' and addressFamily='2'"
	if(tmpDPVID == "1") then
	        tmpPort1Addr = db.getRowWhere("ipAddressTable",where)
	else
	        tmpPort1Addr = db.getRowWhere("ipAddressTable",where2)
	end
						    
	if (tmpDPVID == "1") then
	     dhcpRelay = db.getRowWhere("dhcpRelay", "logicalIfName = 'LAN'")
	     dhcpConf = db.getRowWhere("dhcp","logicalIfName = 'LAN'")
	else
	     dhcpRelay = db.getRowWhere("dhcpRelay", "logicalIfName = 'LAN" .. tmpDPVID .. "'")
	     dhcpConf = db.getRowWhere("dhcp","logicalIfName = 'LAN" .. tmpDPVID .. "'")
	end
	if(dhcpRelay ~= nil and dhcpRelay["dhcpRelay.DhcpRelayStatus"] == "1") then
	      dhcpStatus = "DHCP Relay Mode"
	elseif (dhcpConf ~= nil) then 
	    if (dhcpConf["dhcp.dhcpEnabled"] == "1") then
		        dhcpStatus = "Enabled"
	    elseif (dhcpConf["dhcp.dhcpEnabled"] == "0") then
		        dhcpStatus = "Disabled"
	    end
	end

	print("VLAN Profile:",vlanName["vlan.vlanName"] or '')
	print("VLAN ID:",tmpDPVID or '')
	print("MAC Address:",vlanMac[i] or '')
	print("IP Address:",tmpPort1Addr["ipAddressTable.ipAddress"] or '')
	print("Subnet Mask:",tmpPort1Addr["ipAddressTable.subnetMask"] or '')
	print("DHCP Status:",dhcpStatus or '')
    end

	printLabel ("Lan Port 4/DMZ  Information")
	local tmpPort8Addr = ''
        local tmpDPVID7 = db.getAttribute("VlanPortMgmt","PortNumber","3","DefaultPVID")
	local wherePrt8 = "portName = 'dummy' AND vlanId = '" .. tmpDPVID7 .. "'"
	vlanName = db.getRowWhere("vlan",wherePrt8)
				    
	where = "LogicalIfName='LAN' and addressFamily='2'"
	where2 = "LogicalIfName='DMZ' and addressFamily='2'"
	where3 = "LogicalIfName='LAN" .. tmpDPVID7 .. "' and addressFamily='2'"
	if(tmpDPVID7 == "1") then
	       tmpPort8Addr = db.getRowWhere("ipAddressTable",where)
	elseif(tmpDPVID7 == "4094") then
               vlanName["vlan.vlanName"] = "DMZ"
	       tmpPort8Addr = db.getRowWhere("ipAddressTable",where2)
	else
	       tmpPort8Addr = db.getRowWhere("ipAddressTable",where3)
	end
--[[						    
	if (tmpDPVID7 == "1") then
	     dhcpRelay = db.getRowWhere("dhcpRelay", "logicalIfName = 'LAN'")
	     dhcpConf = db.getRowWhere("dhcp","logicalIfName = 'LAN'")
	else
	     dhcpRelay = db.getRowWhere("dhcpRelay", "logicalIfName = 'LAN" .. tmpDPVID7 .. "'")
	     dhcpConf = db.getRowWhere("dhcp","logicalIfName = 'LAN" .. tmpDPVID7 .. "'")
	end
	if(dhcpRelay ~= nil and dhcpRelay["dhcpRelay.DhcpRelayStatus"] == "1") then
	      dhcpStatus = "DHCP Relay Mode"
	elseif (dhcpConf ~= nil) then 
	    if (dhcpConf["dhcp.dhcpEnabled"] == "1") then
		        dhcpStatus = "Enabled"
	    elseif (dhcpConf["dhcp.dhcpEnabled"] == "0") then
		        dhcpStatus = "Disabled"
	    end
	 end
]]
	if (tmpDPVID7 == "1") then
		dhcpRelay = db.getRowWhere("dhcpRelay", "logicalIfName = 'LAN'")
		dhcpConf = db.getRowWhere("dhcp","logicalIfName = 'LAN'")
	elseif (tmpDPVID7 == "4094") then
		dhcpRelay = db.getRowWhere("dhcpRelay","logicalIfName = 'DMZ'")
		dhcpConf = db.getRowWhere("dhcp","logicalIfName = 'DMZ'")
	else
	       dhcpRelay = db.getRowWhere("dhcpRelay", "logicalIfName = 'LAN" .. tmpDPVID7 .. "'")
	       dhcpConf = db.getRowWhere("dhcp","logicalIfName = 'LAN" .. tmpDPVID7 .. "'")
	end
	if(dhcpRelay ~= nil and dhcpRelay["dhcpRelay.DhcpRelayStatus"] == "1") then
	      dhcpStatus = "DHCP Relay Mode"
	elseif (dhcpConf ~= nil) then 
	      if (dhcpConf["dhcp.dhcpEnabled"] == "1") then
		        dhcpStatus = "Enabled"
	      elseif (dhcpConf["dhcp.dhcpEnabled"] == "0") then
		        dhcpStatus = "Disabled"
	      end
	end
	print("VLAN Profile:",vlanName["vlan.vlanName"] or '')
	print("VLAN ID:",tmpDPVID7 or '')
	print("MAC Address:",vlanMac[4] or '')
	print("IP Address:",tmpPort8Addr["ipAddressTable.ipAddress"] or '')
	print("Subnet Mask:",tmpPort8Addr["ipAddressTable.subnetMask"] or '')
	print("DHCP Status:",dhcpStatus or '')
    printLabel ("DMZ IPv6 Information")
    resultTab = statusGet ("DMZ")
    resTab.print (resultTab, 0)
--    end
    printLabel ("LAN IPv6 Information")
    resultTab = lanStatusGet()
    resTab.print (resultTab, 0)

    for i = 1,2 do
        printLabel ("Broadband Information for WAN"..i)
        resultTab = statusGet("WAN"..i)
        resTab.print (resultTab, 0)
    end

    --printLabel ("Wireless LAN Information")
    --resultTab = wlanStatusGet ()
    --resTab.print (resultTab, 0)
end   

--
-- main function to get SNMP configuration information
-- 
function sysBOOTPGet (args)
    local resultTab1 = {}
    local resultTab2 = {}

    row = db.getRow ("bootpc", "bootpc._ROWID_", 1)
    if (row ~= nil) then
       resTab.insertYNField (resultTab1, row["bootpc.bootpUpgrade"] == "1",
			    "Enable Bootp upgrade")
       resTab.insertYNField (resultTab1, row["bootpc.bootTimeCheck"] == "1",
			    "Disable bootp check at boot time")
       resTab.insertField (resultTab1, "Bootp Server", row["bootpc.bootpServer"])
    else
       resTab.insertField (resultTab1, "", "BOOTP disabled")
    end
    
    cmd = db.getAttribute("environment","name","BOOTPC_DB_PROGRAM","value")
    cmd = cmd .. " " .. DB_FILE_NAME .. " " .. "bootpc"
    schedGet (resultTab2, cmd)

    printLabel ("Bootp Configuration")
    resTab.print (resultTab1, 0)
    printLabel ("Bootp Schedule Configuration")
    resTab.print (resultTab2, 0)
end

--
--
--
function sysUsageRepGet (args)
    local resultTab = {}

    schedGet (resultTab, "/pfrm2.0/bin/ifDevUsageRecord " .. DB_FILE_NAME)
    printLabel ("Usage Reports Schedule Configurations")
    resTab.print (resultTab, 0)    	
end

--
-- main function to get SNMP configuration information
-- 
function sysSNMPGet ()
    local resultTab = {}
    local resultTabACL = {}
    sysCfgTab = sysSNMPConfigGet()
    if (sysCfgTab ~= nil) then
       printLabel ("SNMP System Configuration")
       resTab.print (sysCfgTab, 0)
    end
       
    if (agentIP == nil) then
        local snmpTab = db.getTable("snmpTrap")
	local idx = 0
	for k,v in pairs(snmpTab) do
	    idx = idx + 1
	    local row = snmpTab[idx]
	    buildSNMP (resultTab, row)
        end
    else 
	local row = db.getRow ("snmpTrap", "snmpTrap.ipAddr", agentIP)	
	buildSNMP (resultTab, row)
    end
    printLabel ("Trap Agent IP Address")
    resTab.print (resultTab, 0)
    local snmpACL = db.getTable ("snmpAccessControl")
    local idxi = 0
    for k,v in pairs(snmpACL) do
        idxi = idxi + 1
        local row = snmpACL[idxi]
        buildSNMPACL(resultTabACL, row)
    end
    printLabel ("Access Control List")
    resTab.print (resultTabACL, 0)
end

--
-- main function to get System information
-- 
function sysManInfoGet()
    local getmdOutput = ""
    local file = io.open("/tmp/getmddump.txt","r")
    if(file == nil) then
    	util.appendDebugOut("Exec = " .. os.execute("sh /pfrm2.0/bin/sysinfo.sh"))
	    file = io.open("/tmp/getmddump.txt","r")
    end
    if(file) then
        print("\nSystem - Manufacturer Information")
        print("**************************")
        for line in io.lines("/tmp/getmddump.txt") do
            print(line)
        end
        print("**************************")
    end

    file:close()
end

function scheduleReboot ()
    local scheRebootRow = db.getRow("scheduleReboot","_ROWID_","1")
    local status = ""
    local minute = ""
    local hour = ""
    if (tonumber(scheRebootRow["scheduleReboot.rebootStatus"]) == 0 )  then
        status = "Disable"
    else
        status = "Enable"
        minute = scheRebootRow["scheduleReboot.minute"] or ''
        hour   = scheRebootRow["scheduleReboot.hour"] or ''
    end
    print ("Status : ".. status)
    print ("Minute : ".. minute or '')
    print ("Hour   : ".. hour or '')
end


function sysSchRebootCfgInputVal (configRow)
    if (configRow["scheduleReboot.rebootStatus"] == nil) then
        printCLIError ("Invalid rebootStatus.")
        return false
    end
    if (configRow["scheduleReboot.minute"]== nil or configRow["scheduleReboot.minute"]== "") 
        then
        printCLIError ("Invalid minute.")
        return false
    end
    if (configRow["scheduleReboot.hour"]== nil or configRow["scheduleReboot.hour"] == "" ) 
        then
        printCLIError ("Invalid hour.")
        return false
    end

end

function sysSchRebootCfgSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    if (db.existsRow("scheduleReboot","_ROWID_","1")) then
        errorFlag, statusCode = db.update("scheduleReboot",configRow,"1")
    else
        errorFlag, statusCode = db.insert("scheduleReboot",configRow)
    end
    errorFlag, statusCode = nimfView.connResetConfig (configRow, "1", "edit")
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode

    return errorFlag, statusMessage
end

function sysSchRebootCfgInit(args)
    local primKey = args[1] -- Profile name
    local rowId = -1

    configRow = {}
    configRow["scheduleReboot.rebootStatus"] = db.getAttribute ("scheduleReboot", "_ROWID_", "1", "rebootStatus") or "0"
    configRow["scheduleReboot.minute"] = db.getAttribute ("scheduleReboot", "_ROWID_", "1", "minute") or ""
    configRow["scheduleReboot.hour"] = db.getAttribute ("scheduleReboot", "_ROWID_", "1", "hour") or ""
    if configRow == nil then
       configRow = {} 
       return rowId, configRow       
    end
    return rowId, configRow       
end
